/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.text.NumberFormat;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JFormattedTextField;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import outils.Outil;

import objets.CListeObjets;
import objets.CPrototype;

import mathgraph32.MtgFrame;

@SuppressWarnings("serial")
public class ConstructionIterativeDlg extends MtgDlg {
	public int nombreIt, nbSources, choixLigne;
	public boolean adapterStyle;
	public ListePrototypesPourRecurOuIter listePrototypes;
	private JScrollPane scrollPaneListePrototypes;
  private MtgButton btnCancel;
  private MtgButton btnOK;
  private MtgLabel label0, label1, label3;
  private MtgRadioButton radioSansLigne, radioAvecPolygone, radioAvecLigneBrisee;
  private javax.swing.JFormattedTextField editeurNombreIterations, editeurNbSources;
  private MtgCheckBox checkAdapterCouleurEtStyle;
  public ConstructionIterativeDlg(MtgFrame frameInit) {
    super (getString("iterationConstructionDlg1"), true, frameInit);
    initComponents();
    editeurNombreIterations.requestFocus();
    getRootPane().setDefaultButton(btnOK);
  }
  private void initComponents() {
    btnOK = new MtgButton(frame, getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed(evt);
      }
    });    
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed(evt);
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());  
    label0 = new MtgLabel(frame, getString("macroConstructionIterativeDlg2"));
    label0.setRequestFocusEnabled(false);
    label1 = new MtgLabel(frame, getString("iterationConstructionDlg2"));
    label1.setRequestFocusEnabled(false);
    label3 = new MtgLabel(frame, getString("iterationConstructionDlg4"));
    label3.setRequestFocusEnabled(false);
	  listePrototypes = new ListePrototypesPourRecurOuIter(frame, frame.cadreActif().document);
	  scrollPaneListePrototypes = new JScrollPane(listePrototypes,
    		JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    NumberFormat format;
    format = NumberFormat.getInstance();
    format.setParseIntegerOnly(true);
    format.setGroupingUsed(false);
    editeurNombreIterations = new JFormattedTextField(format);
    editeurNombreIterations.setColumns(4);
    editeurNbSources = new JFormattedTextField(format);
    editeurNbSources.setColumns(2);
    editeurNbSources.setText("0");
	  radioSansLigne = new MtgRadioButton(frame, getString("recursionConstructionDlg5"));
	  radioAvecPolygone = new MtgRadioButton(frame, getString("iterationConstructionDlg6"));
	  radioAvecLigneBrisee = new MtgRadioButton(frame, getString("iterationConstructionDlg7"));
	  radioSansLigne.setSelected(true);
	  ButtonGroup group = new ButtonGroup();
	  group.add(radioSansLigne);
	  group.add(radioAvecPolygone);
	  group.add(radioAvecLigneBrisee);
    checkAdapterCouleurEtStyle = new MtgCheckBox(frame, getString("iterationConstructionDlg5"));
    checkAdapterCouleurEtStyle.setSelected(true);
    JPanel panePrincipal = new JPanel();
    JPanel paneLeft = new JPanel();
    JPanel paneRight = new JPanel();
    panePrincipal.setLayout(new FlowLayout());
    paneLeft.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 0;
    constraints.insets.top = 0;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    
    setConstraints(constraints, 0, 0, 1, 1, 50, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(label0, constraints);	
    setConstraints(constraints, 1, 0, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(scrollPaneListePrototypes, constraints);
    setConstraints(constraints, 0, 1, 1, 1, 50, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(label1, constraints);	
    setConstraints(constraints, 1, 1, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(editeurNombreIterations, constraints);
    constraints.insets.bottom = 5;
    setConstraints(constraints, 0, 2, 1, 1, 50, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(label3, constraints);	
    setConstraints(constraints, 1, 2, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(editeurNbSources, constraints);
	  setConstraints(constraints, 0, 3, 2, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  paneLeft.add(radioSansLigne, constraints);
	  setConstraints(constraints, 0, 4, 2, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  paneLeft.add(radioAvecPolygone, constraints);
	  setConstraints(constraints, 0, 5, 2, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  paneLeft.add(radioAvecLigneBrisee, constraints);
    setConstraints(constraints, 0, 6, 2, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(checkAdapterCouleurEtStyle, constraints);
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
    panePrincipal.add(paneLeft);
    panePrincipal.add(paneRight);
    setContentPane(panePrincipal);
    pack();
  }
  private void btnOKActionPerformed(java.awt.event.ActionEvent evt) {
  	CListeObjets liste = frame.cadreActif().listePr;
  	try {
  		nombreIt = Integer.parseInt(editeurNombreIterations.getText());
  	}
		catch (NumberFormatException e) {
			dialogueAvertissement(getString("ch71"));
			editeurNombreIterations.requestFocus();
			return;
		}
 	
    if (nombreIt < 2 ) {
    	dialogueAvertissement(getString("ch71"));
    	editeurNombreIterations.requestFocus();
    	return;
    }
  	try {
  		nbSources = Integer.parseInt(editeurNbSources.getText());
  	}
		catch (NumberFormatException e) {
			dialogueAvertissement(getString("ch71"));
			editeurNbSources.requestFocus();
			return;
		}
    if (nbSources < 0) {
    	dialogueAvertissement(getString("ch71"));
    	editeurNbSources.requestFocus();
    	return;
    }
    CPrototype proto = listePrototypes.prototypeChoisi();
    String chErr = proto.implementationPossible(liste);
    if (chErr.length() != 0) {
    	frame.dialogueAvertissement(getString("ch142") + "\n" + chErr);
    	return;
    }
		if (!proto.estIterable(nbSources)) {
			frame.dialogueAvertissement(getString("ch197"));
			return;
		}
    double n = (double)(proto.size()-proto.nbObjSources-nbSources)*(nombreIt);
		if (n > (double) Outil.nombreMaxObjets) {
			frame.dialogueAvertissement(getString("ch198"));
			return;
		}
    adapterStyle = checkAdapterCouleurEtStyle.isSelected();
	  choixLigne = (radioSansLigne.isSelected() ? Outil.sansLigne : 
	  	(radioAvecPolygone.isSelected() ? Outil.avecPolygone : Outil.avecLigneBrisee));
    setRetourOK(true);
    dispose();
  }
  
  private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {                                          
    dispose();
  }       
}
