/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;
import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import objets.CListeObjets;
import objets.CPointDansRepere;
import objets.CValeurAssocieeAVariable;

import mathgraph32.MtgFrame;


@SuppressWarnings("serial")
public class CoordonneesDlg extends MtgDlg implements ActionListener {
	private boolean modification; // True si on modifie les coordonnes d'un point dj cr
	private CPointDansRepere pointAssocie;
	private ListeReperes listeReperes;
  private EditeurValeurDynamique editeurAbs;
  private EditeurValeurDynamique editeurOrd;
  private MtgButton btnOK, btnCancel;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsAbs;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsOrd;
  private CValeurAssocieeAVariable valAbs;
  private CValeurAssocieeAVariable valOrd;
  private MtgFocusTraversalPolicy mtgFocusTraversalPolicy;
  
  public CoordonneesDlg(MtgFrame frameInit, CPointDansRepere pointAssocieInit, boolean modificationInit) {
    super(getString("coordonneesDlg1"), true, frameInit);
    CListeObjets liste = frame.cadreActif().listePr;
    setResizable(false);
    pointAssocie = pointAssocieInit;
    modification = modificationInit;
    valAbs = new CValeurAssocieeAVariable(pointAssocieInit.abs);
    valOrd = new CValeurAssocieeAVariable(pointAssocieInit.ord);
    MtgLabel label1 = new MtgLabel(frame, getString("choixRepere"));
    MtgLabel label2 = new MtgLabel(frame, getString("coordonneesDlg3"));
    MtgLabel label3 = new MtgLabel(frame, getString("coordonneesDlg4"));
    editeurAbs = new EditeurValeurDynamique(frame, valAbs, pointAssocie);
    if (modification) editeurAbs.setText(pointAssocieInit.abs.chaineInfo());
    editeurOrd = new EditeurValeurDynamique(frame, valOrd, pointAssocie);
    if (modification) editeurOrd.setText(pointAssocieInit.ord.chaineInfo());
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnOK = new MtgButton(frame, getString("btnOK"));
    btnOK.setPreferredSize(btnCancel.getPreferredSize());
    int indmax;
    if (modification) indmax = liste.indexOf(pointAssocie) - 1;
    else indmax = liste.size() - 1;
    paneListeValeurEtFonctionsAbs = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurAbs, true, indmax, false);
    paneListeValeurEtFonctionsOrd = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurOrd, true, indmax, false);
    int indiceObj;
    if (modification) indiceObj = liste.indexOf(pointAssocie);
    else indiceObj = -1;
    listeReperes = new ListeReperes(frame, frame.cadreActif().listePr, indiceObj);
    if (modification) listeReperes.setSel(pointAssocie.rep);
    listeReperes.setBorder(BorderFactory.createEmptyBorder(1,5,1,5));
    JScrollPane scrollList = new JScrollPane(listeReperes, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

    JPanel pane = new JPanel();
    pane.setLayout(new GridBagLayout());

    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 2;
    constraints.insets.top = 0;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 40, 15, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label2, constraints);
    setConstraints(constraints, 1, 0, 1, 1, 30, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(editeurAbs, constraints);
    setConstraints(constraints, 2, 0, 1, 1, 30, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(btnOK, constraints);
    setConstraints(constraints, 1, 1, 1, 1, 0, 15, GridBagConstraints.NONE, GridBagConstraints.EAST);
    pane.add(paneListeValeurEtFonctionsAbs, constraints);

    setConstraints(constraints, 0, 2, 1, 1, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label3, constraints);
    setConstraints(constraints, 1, 2, 1, 1, 30, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(editeurOrd, constraints);
    setConstraints(constraints, 2, 1, 1, 1, 30, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(btnCancel, constraints);
    setConstraints(constraints, 1, 3, 1, 1, 0, 15, GridBagConstraints.NONE, GridBagConstraints.EAST);
    pane.add(paneListeValeurEtFonctionsOrd, constraints);
    
    setConstraints(constraints, 0, 4, 1, 1, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label1, constraints);
    setConstraints(constraints, 1, 4, 1, 1, 0, 40, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(scrollList, constraints);

     
    // On change l'ordre par dfaut des composants
    Vector<Component> order = new Vector<Component>(6);
    order.add(editeurAbs);
    order.add(editeurOrd);
    order.add(btnOK);
    order.add(paneListeValeurEtFonctionsAbs);
    order.add(paneListeValeurEtFonctionsOrd);
    order.add(btnCancel);
    mtgFocusTraversalPolicy = new MtgFocusTraversalPolicy(order);
    setFocusTraversalPolicy(mtgFocusTraversalPolicy);
    
    pane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
    setContentPane(pane);    
    
    btnOK.addActionListener(this);
    btnCancel.addActionListener(this);
    getRootPane().setDefaultButton(btnOK);  
    editeurAbs.setSelectionStart(0);
    editeurAbs.setSelectionEnd(editeurAbs.getText().length());
    pack();
  }

	public void actionPerformed(ActionEvent evt) {
    if (evt.getSource()== btnOK) {
      // Si la rponse est valide, on ferme la boite de dialogue sans la dtruire.
      // VrifieValidite se charge de changer l'angle de la rotation si la rponse est
      // correcte.
      if (editeurAbs.verifieValidite()) {
      	if (editeurOrd.verifieValidite()) {
	        setRetourOK(true);
	        // On abandonne les objets CValeurAssocieeAVariable de pointAssocie pour les remplacer par les locaaux
	        pointAssocie.abs = valAbs;
	        pointAssocie.ord = valOrd;
	        pointAssocie.rep = listeReperes.repereChoisi();
	        dispose();
      	}
      }
    }
    else {
      if (evt.getSource() == btnCancel) {
        dispose();
      }
    }
    // La variable retourOK a t initialie  false donc tous les autres cas n'on pas  tre traits
  }
}
