
/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.util.Vector;
import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CDroiteParEq;
import objets.CListeObjets;
import types.Int;

@SuppressWarnings("serial")
public class DroiteParEquationDlg extends MtgDlg {
	private boolean modification;
	private ListeReperes listeReperes;
  private MtgTextField editeurFormule;
  private MtgButton btnOK, btnCancel;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctions;
  private CDroiteParEq droiteAssociee;
  private MtgFocusTraversalPolicy mtgFocusTraversalPolicy;  
  
  public DroiteParEquationDlg(MtgFrame frameInit, CDroiteParEq droiteAssocieeInit, boolean modificationInit) {
  	super(getString("droiteParEquationDlg1"), true, frameInit);
    CListeObjets liste = frame.cadreActif().listePr;
  	droiteAssociee = droiteAssocieeInit;
  	modification = modificationInit;
    MtgLabel label1 = new MtgLabel(frame, getString("choixRepere"), SwingConstants.LEFT);
    MtgLabel label2 = new MtgLabel(frame, getString("droiteParEquationDlg2"), SwingConstants.LEFT);
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });    
    btnOK = new MtgButton(frame, getString("btnOK"));
    btnOK.setPreferredSize(btnCancel.getPreferredSize());
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        btnOKActionPerformed();
      }
    });
    int indmax;
    if (modification) indmax = liste.indexOf(droiteAssociee) - 1;
    else indmax = liste.size() - 1;
    editeurFormule = new MtgTextField(frame, 30);
    editeurFormule.setAutoscrolls(true);
    paneListeValeurEtFonctions = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurFormule, true, indmax, false);
    listeReperes = new ListeReperes(frame, frame.cadreActif().listePr, indmax);
    listeReperes.setBorder(BorderFactory.createEmptyBorder(1,5,1,5));
    JScrollPane scrollList = new JScrollPane(listeReperes, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    if (modification) {
    	String[] parametre = new String[2];
    	parametre[0] = "x";
    	parametre[1] = "y";
      editeurFormule.setText(droiteAssociee.equation.chaineCalculSansPar(parametre));
      listeReperes.setSel(droiteAssociee.rep);
    }
    
    JPanel pane = new JPanel();
    pane.setLayout(new GridBagLayout());

    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 2;
    constraints.insets.top = 0;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 40, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label2, constraints);
    setConstraints(constraints, 1, 0, 1, 1, 30, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(editeurFormule, constraints);
    setConstraints(constraints, 2, 0, 1, 1, 30, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(btnOK, constraints);
    setConstraints(constraints, 1, 1, 1, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.EAST);
    pane.add(paneListeValeurEtFonctions, constraints);

    setConstraints(constraints, 2, 1, 1, 1, 30, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(btnCancel, constraints);
    
    setConstraints(constraints, 0, 2, 1, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label1, constraints);
    setConstraints(constraints, 1, 4, 1, 1, 0, 40, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(scrollList, constraints);

     
    // On change l'ordre par dfaut des composants
    Vector<Component> order = new Vector<Component>(4);
    order.add(editeurFormule);
    order.add(btnOK);
    order.add(paneListeValeurEtFonctions);
    order.add(btnCancel);
    mtgFocusTraversalPolicy = new MtgFocusTraversalPolicy(order);
    setFocusTraversalPolicy(mtgFocusTraversalPolicy);
    
    getRootPane().setDefaultButton(btnOK);  
    pane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
    setContentPane(pane);
    pack();    
    
  }
  
  private void btnCancelActionPerformed() {
  	dispose();
  }
  
  private void btnOKActionPerformed() {
  	Int indiceErreur = new Int(0);
  	if (droiteAssociee.verifieFormules(editeurFormule.getText(), indiceErreur)) {
  		droiteAssociee.rep = listeReperes.repereChoisi();
  		// On regarde si la droite existe bien. Sinon on demande confirmation
  		MtgInternalFrame cadre = frame.cadreActif();
  		droiteAssociee.positionne(cadre.infRand, cadre.dimf);
  		if (droiteAssociee.existe) {
	  		setRetourOK(true);
	  		dispose();
  		}
  		else {
    		frame.dialogueAvertissement(getString("ch181"));
    		editeurFormule.requestFocus();
  		}
  	}
  	else {
  		frame.dialogueAvertissement(getString("ch180"));
  		editeurFormule.setCaretPosition(indiceErreur.getValue());
  		editeurFormule.requestFocus();
  	}
  	
  }
  
}
