/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import objets.CCalculBase;
import objets.CElementBase;
import objets.CListeObjets;
import objets.CValeurAssocieeComp;
import types.Complexe;
import types.ErreurCalculException;
import types.Int;

import mathgraph32.MtgFrame;

@SuppressWarnings("serial")
public class EditeurValeurComplexe extends MtgTextField {
  protected MtgFrame frame;
  private CValeurAssocieeComp val; // L'objet CValeurDynamique que ce JTextField doit modifier.
  private CElementBase objet; // L'objet qui contient val et qui doit tre modifi
  
  /** Creates a new instance of EditeurValeur */
  public EditeurValeurComplexe(MtgFrame frameInit, CValeurAssocieeComp valInit, CElementBase objetInit) {
    super(frameInit, 30);
    frame = frameInit;
    val = valInit;
    objet = objetInit;
    setEditable(true); 
  }
  
  /**
   * Ajout version 3.2.8.2
   * @param valeur
   */
  public final void associeA(CValeurAssocieeComp valeur) {
  	val = valeur;
  }
  
  /* Vrification de la validit de la rponse contenue dans le champ d'dition
   * du JTextField
  */
  public final boolean verifieValidite() {
    Complexe v = new Complexe();
    int indice;
    // Pour la version applet on ne doit pouvoir utiliser que des objets
    // crs par l'utilisateur
    CListeObjets liste = frame.cadreActif().listePr;
    String rep = (String) getText();
    // Dans le cas ou on cre un nouvel objet, celui-ci n'a pas encore t ajout
    //  la liste des objets de l'applet.
    int indobj = liste.indexOf(objet);
    if (indobj == -1) indice = liste.size() - 1;
    else indice = indobj - 1;
	  Int inderr = new Int();
	  // On vrifie d'abor la syntaxe de la rponse
	  // La seule valeur qu'on a le droit d'utiliser est pi qui est toujours
	  // le premier objet de la liste
	  if (CCalculBase.verifieSyntaxeComplexe(liste, rep, inderr, indice, null)){
	  
	    CCalculBase calc = CCalculBase.creeCalculBaseComplexe(rep ,liste, 0, rep.length()-1, null);
	    try {
        calc.listeProprietaire.initialiseNombreIterations();
	      calc.resultatComplexe(false, v);
	    }
	    catch (ErreurCalculException e) {
      	if (e.depassementCapacite)
      		frame.dialogueAvertissement(MtgFrame.getString("ch179"));
      	else frame.dialogueAvertissement(MtgFrame.getString("ch44"));
	      return false;
	    }
	
      if(!calc.existe(false)) {
      	frame.dialogueAvertissement(MtgFrame.getString("ch44"));
        requestFocus();
        return false;
      }
	    
	    // Si tout est correct, on affecte le rsultat du calcul propos
	    if (valeurAcceptee(v))  {
	      val.donneCalcul(calc);
	      return true;
	    }
	    else return false;
	  }
	  else {
	    // On place le curseur  l'endroit de l'erreur de syntaxe
	    this.select(inderr.getValue(),  inderr.getValue());
	    frame.dialogueAvertissement(MtgFrame.getString("ch5"));
	    requestFocus();
	    return false;
	  }
  }
  
  /*
   * Fonction charge de vrifier si la rponse, quand elle est correcte, est valide
   * pour le type de valeur attendu. Par exemple, le rayon d'un cercle doit tre non nul.
   * Renvoie true par dfaut et doit tre redfinie pour tous les descendants.
   *
  */
  public boolean valeurAcceptee(Complexe v) {
    return true;
  }
}
