/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.File;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import objets.CCalculBase;
import objets.CListeObjets;
import types.ErreurCalculException;
import types.Int;
import mathgraph32.Global;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

@SuppressWarnings("serial")
public class ExporterPNGAvecUniteDlg extends MtgDlg {
	public JFileChooser fileChooser;
	public MtgTextField editeurLongueurUniteCM;
	MtgLabel label1, label2, labelTaille;
	public MtgCheckBox cbAttenuation; // Ajout version 4.6.4
	public ListeChoixDPI listeChoixDPI;
	public double unite; // le rsultat de l'entre dans editeurLongueurUniteCM
	// Modifi version 4.6. La mme bote de dialogue servira pour exporter avec unit et sans
	// Quand on exporte sans tenir compte de l'unit de longueur on remplacera l'intitul du champ d'dition
	// par chelle alors que sinon ce sera Longueur de l'unit en cm
	private boolean avecUnite;
	public int widthPixel; // Largeur de la figure en pixel
	public int heightPixel; // Hauteur de la figure en pixel
	private double widthcm;
	private double heightcm;
	private boolean uniteOK;
	private MtgTextField infoTaille;
	
	class MyFilter extends javax.swing.filechooser.FileFilter {
		public boolean accept(File file) {
			if (file.isDirectory())
				return true;
			String filename = file.getName().toLowerCase();
			return filename.endsWith(Global.GLOBAL_COMMA_PNG);
		}

		public String getDescription() {
			return Global.GLOBAL_STAR_COMMA_PNG;
		}
	}
	
  public ExporterPNGAvecUniteDlg(MtgFrame frameInit, boolean avecUniteInit) {
    super (getString("exporterPNGAvecUniteDlg1"), true, frameInit);
    avecUnite = avecUniteInit;
    uniteOK = true;
    initComponents();
    verifieUnite();
		fileChooser.setFileFilter(new MyFilter());
    // fileChooser.setAcceptAllFileFilterUsed(true);
    fileChooser.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
          if (evt.getActionCommand().equals(JFileChooser.CANCEL_SELECTION)){
              dispose();
          }else if (evt.getActionCommand().equals(JFileChooser.APPROVE_SELECTION)){
          		verifieUnite();
          	 	if (uniteOK) {
          	 		// Correction version 4.7.5.2
          	 		// if ((unite >= 0.1) && (unite <= 4)) {
          	 		if (avecUnite || ((unite >= 0.1) && (unite <= 4))) {
	          	 		setRetourOK(true);
	          	 		dispose();
          	 		}
          	 		else {
	            		dialogueAvertissement(getString("ch194"));
	                editeurLongueurUniteCM.requestFocus();	
          	 		}
          	 	}
          	 	else {
            		dialogueAvertissement(getString("ch71"));
                editeurLongueurUniteCM.requestFocus();
          	 	}
          }
      }
  });          
  }
  
  private void initComponents() {
  	label1 = new MtgLabel(frame, getString("exporterPNGAvecUniteDlg3"));
  	if (avecUnite) label2 = new MtgLabel(frame, getString("exporterPNGAvecUniteDlg4"));
  	else label2 = new MtgLabel(frame, getString("exporterPNGAvecUniteDlg5"));
		cbAttenuation = new MtgCheckBox(frame, MtgFrame.getString("coeffMultPressePapier3"), frame.pref_attenuation);
  	labelTaille = new MtgLabel(frame, getString("editeurFormuleDlg3"));
  	infoTaille = new MtgTextField(frame, 25);
  	infoTaille.setEditable(false);
  	editeurLongueurUniteCM = new MtgTextField(frame, 6);
  	editeurLongueurUniteCM.setText("1"); // 1 cm par dfaut pour la longeur unit
  	editeurLongueurUniteCM.addKeyListener(new KeyListener() {
			
			public void keyTyped(KeyEvent arg0) {
				// TODO Auto-generated method stub
				
			}
			
			public void keyReleased(KeyEvent arg0) {
				verifieUnite();
				
			}
			
			public void keyPressed(KeyEvent arg0) {
				// TODO Auto-generated method stub
				
			}
		});
  	fileChooser = new JFileChooser(frame.repertoireFichiers);
  	fileChooser.setDialogType(JFileChooser.SAVE_DIALOG);
  	listeChoixDPI = new ListeChoixDPI();
  	
    JPanel panePrincipal = new JPanel();
    JPanel paneBas = new JPanel();
    JPanel paneTaille = new JPanel();
    paneTaille.setLayout(new BoxLayout(paneTaille, BoxLayout.Y_AXIS));
    panePrincipal.setLayout(new GridBagLayout());
    // paneBas.setLayout(new GridBagLayout());
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 100, 60, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(fileChooser, constraints);
    
    paneBas.add(label1);
    paneBas.add(listeChoixDPI);
    paneBas.add(Box.createRigidArea(new Dimension(15,0)));
    paneBas.add(label2);
    paneBas.add(editeurLongueurUniteCM);
    paneBas.setBorder(BorderFactory.createTitledBorder(getString("exporterPNGAvecUniteDlg2")));
    paneTaille.add(labelTaille);
    paneTaille.add(infoTaille);
    // Modification version 4.7.0.3
    // paneTaille.add(cbAttenuation);
    if (!avecUnite) paneTaille.add(cbAttenuation);
   
    setConstraints(constraints, 0, 1, 2, 1, 100, 20, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER);
    panePrincipal.add(paneBas, constraints);
    setConstraints(constraints, 0, 2, 2, 1, 100, 20, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER);
    panePrincipal.add(paneTaille, constraints);
    
    setContentPane(panePrincipal);
    pack();
  }
  
  private void verifieUnite() {
  	MtgInternalFrame cadre = frame.cadreActif();
    CListeObjets liste = cadre.listePr;
    String rep = editeurLongueurUniteCM.getText();
    Int inderr = new Int();    
  	if (CCalculBase.verifieSyntaxe(liste, rep, inderr, 0, null)) {
      CCalculBase calc = CCalculBase.creeCalculBase(rep ,liste, 0, rep.length()-1, null);
      try {
        calc.listeProprietaire.initialiseNombreIterations();
        unite = calc.resultat(false);
      }
      catch (ErreurCalculException e) {
      	infoTaille.setText("");
      	uniteOK = false;
        return;
      }
      uniteOK = true;
			float dpi = listeChoixDPI.resolutionDPI();
      if (avecUnite) {
				double unitePixels = liste.pointeurLongueurUnite.rendLongueur();
				double pixelsParUnite = unite/2.54*dpi;
				widthPixel = Math.round((float) ((float)cadre.dimf.longueurAbscisses/unitePixels*pixelsParUnite));
				heightPixel = Math.round((float) ((float)cadre.dimf.longueurOrdonnees/unitePixels*pixelsParUnite));     	
      }
      else {
				float screendpi = Toolkit.getDefaultToolkit().getScreenResolution();
				// Dans ce cas unite reprente en fait l'chelle demande
				widthPixel = Math.round((float) ((float)cadre.dimf.longueurAbscisses/screendpi*dpi*unite));
				heightPixel = Math.round((float) ((float)cadre.dimf.longueurOrdonnees/screendpi*dpi*unite));      	
      }
			widthcm = ((float)widthPixel)/dpi*2.54f;
			heightcm = ((float)heightPixel)/dpi*2.54;
			String ch = glob.Kernel.chaineNombre(widthcm, 2) + " cm x " + glob.Kernel.chaineNombre(heightcm, 2) + " cm";
			ch = ch + " , " + Integer.toString(widthPixel) + " pixels x " + Integer.toString(heightPixel) + " pixels";
			infoTaille.setText(ch);
      return;
  	}
  	else {
  		uniteOK = false;
  		infoTaille.setText("");
  		return;
  	}
  }
}
