/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.io.IOException;
import javax.swing.Box;
import javax.swing.BoxLayout;
import objets.CMathGrafDoc;
import types.NatureObjet;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import mathgraph32.MtgMenuFileItem;

@SuppressWarnings("serial")
public class FigureBaseDlg extends MtgDlg {
  private MtgButton btnCancel;
  private MtgButton btnOK;
  private javax.swing.JPanel paneLeft;
  private javax.swing.JPanel paneRight;
  private javax.swing.JPanel panePrincipal;  
  private javax.swing.ButtonGroup btnGroup;
  private MtgRadioButton radiobtnTriangleQuelconque;
  private MtgRadioButton radiobtnTriangleRectangle;
  private MtgRadioButton radiobtnTriangleIsocele;
  private MtgRadioButton radiobtnTriangleRectangleIsocele;
  private MtgRadioButton radiobtnTriangleEquilateral;
  private MtgRadioButton radiobtnCarre;
  private MtgRadioButton radiobtnRectangle;
  private MtgRadioButton radiobtnParallelogramme;
  private MtgRadioButton radiobtnLosange;
  private MtgRadioButton radiobtnTrapeze;
  private MtgRadioButton radiobtnTrapezeRectangle;
  private MtgRadioButton radiobtnTrapezeIsocele;
  private MtgRadioButton radiobtnHexagoneRegulier;  
  private MtgRadioButton radiobtnPentagoneRegulier;  
  
  
 
	
  public FigureBaseDlg(MtgFrame frameInit) {
    super (getString("menuFigureBase"), true, frameInit);
    initComponents();
    getRootPane().setDefaultButton(btnOK);
  }
  
  private void initComponents()  {
    btnGroup = new javax.swing.ButtonGroup();
    radiobtnTriangleQuelconque = new MtgRadioButton(frame, getString("triangleQuelconque"));
    radiobtnTriangleRectangle = new MtgRadioButton(frame, getString("triangleRectangle"));
    radiobtnTriangleIsocele = new MtgRadioButton(frame, getString("triangleIsocele"));
    radiobtnTriangleRectangleIsocele = new MtgRadioButton(frame, getString("triangleRectangleIsocele"));
    radiobtnTriangleEquilateral = new MtgRadioButton(frame, getString("triangleEquilateral"));
    radiobtnCarre = new MtgRadioButton(frame, getString("carre"));
    radiobtnRectangle = new MtgRadioButton(frame, getString("rectangle"));
    radiobtnParallelogramme = new MtgRadioButton(frame, getString("parallelogramme"));
    radiobtnLosange = new MtgRadioButton(frame, getString("losange"));
    radiobtnTrapeze = new MtgRadioButton(frame, getString("trapeze"));
    radiobtnTrapezeRectangle = new MtgRadioButton(frame, getString("trapezeRectangle"));
    radiobtnTrapezeIsocele = new MtgRadioButton(frame, getString("trapezeIsocele"));
    radiobtnHexagoneRegulier = new MtgRadioButton(frame, getString("hexagoneRegulier"));
    radiobtnPentagoneRegulier = new MtgRadioButton(frame, getString("pentagoneRegulier"));
    btnGroup.add(radiobtnTriangleQuelconque);
    btnGroup.add(radiobtnTriangleRectangle);
    btnGroup.add(radiobtnTriangleIsocele);
    btnGroup.add(radiobtnTriangleRectangleIsocele);
    btnGroup.add(radiobtnTriangleEquilateral);
    btnGroup.add(radiobtnCarre);
    btnGroup.add(radiobtnRectangle);
    btnGroup.add(radiobtnParallelogramme);
    btnGroup.add(radiobtnLosange);
    btnGroup.add(radiobtnTrapeze);
    btnGroup.add(radiobtnTrapezeRectangle);
    btnGroup.add(radiobtnTrapezeIsocele);
    btnGroup.add(radiobtnHexagoneRegulier);
    btnGroup.add(radiobtnPentagoneRegulier);
    paneLeft = new javax.swing.JPanel();
    paneRight = new javax.swing.JPanel();
    panePrincipal = new javax.swing.JPanel();
    btnOK = new MtgButton(frame, getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed(evt);
      }
    });    
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed(evt);
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());
    paneLeft.setLayout(new BoxLayout(paneLeft, BoxLayout.Y_AXIS));
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    paneLeft.add(radiobtnTriangleQuelconque);
    paneLeft.add(radiobtnTriangleRectangle);
    paneLeft.add(radiobtnTriangleIsocele);
    paneLeft.add(radiobtnTriangleRectangleIsocele);
    paneLeft.add(radiobtnTriangleEquilateral);
    paneLeft.add(radiobtnCarre);
    paneLeft.add(radiobtnRectangle);
    paneLeft.add(radiobtnParallelogramme);
    paneLeft.add(radiobtnLosange);
    paneLeft.add(radiobtnTrapeze);
    paneLeft.add(radiobtnTrapezeRectangle);
    paneLeft.add(radiobtnTrapezeIsocele);
    paneLeft.add(radiobtnHexagoneRegulier);
    paneLeft.add(radiobtnPentagoneRegulier);
    radiobtnTriangleQuelconque.setSelected(true);
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    panePrincipal.setLayout(new FlowLayout());
    panePrincipal.add(paneLeft);
    panePrincipal.add(paneRight);
    setContentPane(panePrincipal);
    pack();
  }
  
  private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {
    dispose ();
  }                                         

  private void btnOKActionPerformed(java.awt.event.ActionEvent evt) {
		// Modification version 3.5 : Si la prcdente fentre a t maximise, on maximise la nouvelle
  	String ch = null;
  	if (radiobtnTriangleQuelconque.isSelected()) { 
   		ch = "triangleQuelconque";
  	}
  	else
  	if (radiobtnTriangleRectangle.isSelected()) {
  		ch = "triangleRectangle";
  	}
  	else
  	if (radiobtnTriangleIsocele.isSelected()) {
  		ch = "triangleIsocele";
  	}
  	else
  	if (radiobtnTriangleRectangleIsocele.isSelected()) {
  		ch = "triangleRectangleIsocele";
  	}
  	else
    if (radiobtnTriangleEquilateral.isSelected()) {
  		ch = "triangleEquilateral";
    }
  	else
  	if (radiobtnCarre.isSelected()) {
  		ch = "carre";  		
  	}
  	else
  	if (radiobtnRectangle.isSelected()) {
  		ch = "rectangle";
  	}
  	else
  	if (radiobtnParallelogramme.isSelected()) {
  		ch = "parallelogramme";
  	}
  	else
  	if (radiobtnLosange.isSelected()) {
  		ch = "losange";
  	}
  	else
   	if (radiobtnTrapeze.isSelected()) {
  		ch = "trapeze";
   	}
  	else
  	if (radiobtnTrapezeRectangle.isSelected()) {
  		ch = "trapezeRectangle";
  	}
  	else
  	if (radiobtnTrapezeIsocele.isSelected()) {
  		ch = "trapezeIsocele";
  	}
  	else
  	if (radiobtnHexagoneRegulier.isSelected()) {
  		ch = "hexagoneRegulier";
  	}
  	else // Ajout version 4.9.6.1
  	if (radiobtnPentagoneRegulier.isSelected()) {
  		ch = "pentagoneRegulier";
  	}
  	if (MtgFrame.screenIsHighResolution) ch = ch + "HD";
  	CMathGrafDoc doc = frame.chargeDocumentDansDossierMtgFrame("constructions/"  + ch + ".mgj");
		MtgInternalFrame cadre = new MtgInternalFrame(frame, "MathGraph" + Integer.toString(frame.instanceNouvelleFenetre()), doc, doc.sizeWindowx, doc.sizeWindowy);
		try {
			cadre.gestionnaire.initialise();
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		frame.mtgDesktopPane.add(cadre);
		frame.mtgDesktopPane.getDesktopManager().activateFrame(cadre);
		try {
			cadre.setSelected(true);
		} catch (java.beans.PropertyVetoException err) {
		};
		frame.preparePositionPourFenenetre(new Dimension(cadre.getWidth(), cadre.getHeight()));
		cadre.setLocation(frame.xfenetre, frame.yfenetre);
		// On cre un item de menu dans le menu Fenetres pour activer cete fentre
		MtgMenuFileItem item = new MtgMenuFileItem(cadre);
		frame.menuFenetre.add(item);
		frame.metAJourBoutonsBarreHorizontale(cadre.document);
		if (cadre.listePr.nombreObjetsParNature(NatureObjet.NMacro) != 0) frame.activeOutilExecutionMacro();
		else frame.activeOutilCapture();
		frame.metAJourBoutonsBarreOutils(doc);
		frame.validate();
		frame.activeOutilCapture();
    dispose();
    setRetourOK(true);
  }       
}
