/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/**
 * Prvisualisateur d'image
 */ 

package dialogs;

import javax.swing.*;
import java.beans.*;
import java.awt.*;
import java.io.File;

/** Prvisualisateur d'image */
@SuppressWarnings("serial")
public class ImagePreview extends JComponent
                          implements PropertyChangeListener {
    ImageIcon thumbn = null;
    File file = null;

    public ImagePreview(JFileChooser fc) {
        setPreferredSize(new Dimension(100, 50));
        fc.addPropertyChangeListener(this);
    }

    public void loadImage() {
        if (file == null) {
            thumbn = null;
            return;
        }

        ImageIcon tmpIcon = new ImageIcon(file.getPath());
        if (tmpIcon != null) {
            if (tmpIcon.getIconWidth() > 90) {
                thumbn = new ImageIcon(tmpIcon.getImage().getScaledInstance(90, -1,
                                                      Image.SCALE_DEFAULT));
            } else { // pas besoin de miniaturiser
                thumbn = tmpIcon;
            }
        }
    }

    public void propertyChange(PropertyChangeEvent ev) {
        boolean update = false;
        String prop = ev.getPropertyName();

        // Si le rpertoire a chang on ne montre as d'image
        if (JFileChooser.DIRECTORY_CHANGED_PROPERTY.equals(prop)) {
            file = null;
            update = true;

        // Si un fichier a t slectionn, on dtermine lequel
        } else if (JFileChooser.SELECTED_FILE_CHANGED_PROPERTY.equals(prop)) {
            file = (File) ev.getNewValue();
            update = true;
        }

        //Update the preview accordingly.
        if (update) {
            thumbn = null;
            if (isShowing()) {
                loadImage();
                repaint();
            }
        }
    }

    protected void paintComponent(Graphics g) {
        if (thumbn == null) {
            loadImage();
        }
        if (thumbn != null) {
            int x = getWidth()/2 - thumbn.getIconWidth()/2;
            int y = getHeight()/2 - thumbn.getIconHeight()/2;

            if (y < 0) {
                y = 0;
            }

            if (x < 5) {
                x = 5;
            }
            thumbn.paintIcon(this, g, x, y);
        }
    }
}
