/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import javax.swing.JLabel;
import objets.CAffichageLieAPoint;
import objets.CLieuObjetAncetre;
import mathgraph32.MtgFrame;

@SuppressWarnings("serial")
/**
 * Modification version 4.5 : Si on clique sur un lieu d'objet gnr par un CCommentaire ou CLatex
 * on donne la possibilit de modifier le mode d'affichage et la taille du commentaire oue le latex gnrant le lieu (utile pour les graduations)
 * @author Yves
 *
 */
public class LieuObjetDlg extends MtgDlg {
	private CLieuObjetAncetre lieuAModifier;
	private boolean modification;
  private MtgButton btnCancel;
  private MtgButton btnOK;
  private MtgLabel label1;
  private MtgLabel label2;
  private MtgLabel label3;
  private EditeurNombrePositionsPourLieuObjet editeur;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctions;
  // Ahout version 4.5
  private PaneTaillePolice paneTaillePolice;
  private PaneModeAffichage paneModeAffichage;
  private boolean estElementFinal; // Ajout version 5.4

  public LieuObjetDlg(MtgFrame frameInit, CLieuObjetAncetre lieuAModifierInit, boolean modificationInit) {
    super (getString("lieuObjetDlg1"), true, frameInit);
    lieuAModifier = lieuAModifierInit;
    modification = modificationInit;
    initComponents();
    getRootPane().setDefaultButton(btnOK);   
  }

  private void initComponents() {
    btnOK = new MtgButton(frame, getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed(evt);
      }
    });    
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed(evt);
      }
    });
    label1 = new MtgLabel(frame, getString("lieuObjetDlg2"), JLabel.RIGHT);
    label1.setRequestFocusEnabled(false);
    label2 = new MtgLabel(frame, getString("lieuObjetDlg3"), JLabel.RIGHT);
    label2.setRequestFocusEnabled(false);
    label3 = new MtgLabel(frame, getString("lieuObjetDlg4"), JLabel.RIGHT);
    label3.setRequestFocusEnabled(false);
    editeur=new EditeurNombrePositionsPourLieuObjet(frame, lieuAModifier.nombreTraces, lieuAModifier);
    editeur.setColumns(12);
    editeur.setText(lieuAModifier.nombreTraces.chaineInfo());
  	editeur.setSelectionStart(0);
  	editeur.setSelectionEnd(editeur.getText().length());
  	int indmax;
  	if (modification) indmax = frame.cadreActif().listePr.indexOf(lieuAModifier);
  	else indmax = frame.cadreActif().listePr.size() - 1;  	
    //////////////////////////////////////////////////////
    // Modification version 5.4 
    estElementFinal = lieuAModifier.estElementFinal;
    //////////////////////////////////////////////////////
    paneListeValeurEtFonctions = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeur, true, indmax, false);
   
    getContentPane().setLayout(new GridBagLayout());
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    
    
    setConstraints(constraints, 0, 0, 1, 1, 30, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    getContentPane().add(label1, constraints);    
    setConstraints(constraints, 0, 1, 1, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    getContentPane().add(label2, constraints);    
    setConstraints(constraints, 0, 2, 1, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    getContentPane().add(label3, constraints);    
    setConstraints(constraints, 1, 0, 1, 1, 40, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    getContentPane().add(editeur, constraints);
    setConstraints(constraints, 1, 1, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    getContentPane().add(paneListeValeurEtFonctions, constraints);
    if (estElementFinal) {
    	editeur.setEditable(false);
    	paneListeValeurEtFonctions.setVisible(false);
    }

    if (estElementFinal) {
    	editeur.setEditable(false);
    	paneListeValeurEtFonctions.setVisible(false);
    }
    setConstraints(constraints, 2, 0, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    getContentPane().add(btnOK, constraints);    
    setConstraints(constraints, 2, 1, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    getContentPane().add(btnCancel, constraints);
    if (modification && lieuAModifier.estCapturableSouris()) {
    	CAffichageLieAPoint aff = (CAffichageLieAPoint) lieuAModifier.elementAssocie;
    	paneTaillePolice = new PaneTaillePolice(aff.indiceTaillePolice);
    	paneModeAffichage = new PaneModeAffichage(frame, aff, true); // Modifi version 4.5.2
      setConstraints(constraints, 0, 3, 2, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
      getContentPane().add(paneModeAffichage, constraints); 
      setConstraints(constraints, 2, 3, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
      getContentPane().add(paneTaillePolice, constraints); 
    }
    pack();
  }
  
  private void btnOKActionPerformed(java.awt.event.ActionEvent evt) {
    if (estElementFinal || editeur.verifieValidite()) { // Modifi version 5.4
      if (modification && lieuAModifier.estCapturableSouris()) {
      	CAffichageLieAPoint aff = (CAffichageLieAPoint) lieuAModifier.elementAssocie;
    		// aff.nombreDecimales = paneListeDecimales.nombreDecimales();
    		aff.indiceTaillePolice = paneTaillePolice.indiceTaille();
    		aff.encadrement = paneModeAffichage.styleEncadrement();
    		aff.effacementFond = paneModeAffichage.effacementFond();
    		aff.couleurFondAffichage = paneModeAffichage.couleurFond();
    		aff.alignementHorizontal = paneModeAffichage.alignementHorizontal();
    		aff.alignementVertical = paneModeAffichage.alignementVertical();      	
      }
      setRetourOK(true);
      dispose();
    }
    
  }
  
  private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {                                          
    dispose();
  }       
}