/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.Vector;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CFonc;
import objets.CListeObjets;
import objets.CMaxFonc;
import objets.CValeurAssocieeAVariable;

@SuppressWarnings("serial")
public class MaximumMinimumDlg extends MtgDlg {
	private CMaxFonc max; // L'objet qu'on modifie
	private boolean modification; // True si on modifie un objet dja cr
	private EditeurNomCalcul editeurNom;
	private ListeFonctionsReelles listeFonctions;
	private MtgButton btnOK, btnCancel;
	private EditeurValeurDynamique editeurValeura;
	private EditeurValeurDynamique editeurValeurb;
	private EditeurValeurDynamique editeurValeure;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsa;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsb;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionse;
	private MtgTextArea info;
	// On utilise des copies des valeurs dynamiques en cas d'abandon aprs avoir valid
	private CValeurAssocieeAVariable valeura, valeurb, valeure;
  private MtgFocusTraversalPolicy mtgFocusTraversalPolicy;
	
  public MaximumMinimumDlg(MtgFrame frameInit, CMaxFonc maximumInit, boolean modificationInit, boolean bMaximumInit) {
    super(bMaximumInit ? getString("maximumMinimumDlg0") : getString("maximumMinimumDlg1"), true, frameInit);
    max = maximumInit;
    modification = modificationInit;
    setResizable(false);
    String ch = " " + (bMaximumInit ? getString("maximum") : getString("minimum")) + " ";
    MtgLabel label1 = new MtgLabel(frame, getString("maximumMinimumDlg2") + ch + getString("maximumMinimumDlg2Bis"), SwingConstants.LEFT);
    MtgLabel label2 = new MtgLabel(frame, getString("nomDu") + ch, SwingConstants.LEFT);
    MtgLabel label3 = new MtgLabel(frame, getString("maximumMinimumDlg3"), SwingConstants.LEFT);
    MtgLabel label4 = new MtgLabel(frame, getString("solutionEquationDlg5"), SwingConstants.LEFT);
    MtgLabel label5 = new MtgLabel(frame, getString("solutionEquationDlg6"), SwingConstants.LEFT);
    MtgLabel label6 = new MtgLabel(frame, getString("solutionEquationDlg7"), SwingConstants.LEFT);
    MtgLabel label7 = new MtgLabel(frame, getString("integraleDlg8"), SwingConstants.LEFT);
    editeurNom = new EditeurNomCalcul(frame);
    info = new MtgTextArea(frame);
    info.setEditable(false);
    info.setColumns(35);
    info.setRows(1);
    // JScrollPane scrollPaneInfo = new JScrollPane(info, JScrollPane.VERTICAL_SCROLLBAR_NEVER, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    // Modifi version 4.8.0
    JScrollPane scrollPaneInfo = new JScrollPane(info, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    CListeObjets liste = frame.cadreActif().listePr;
    valeura = new CValeurAssocieeAVariable(max.a);
    valeurb = new CValeurAssocieeAVariable(max.b);
    valeure = new CValeurAssocieeAVariable(max.incertitude);
    editeurValeura = new EditeurValeurDynamique(frame, valeura, max);
    editeurValeurb = new EditeurValeurDynamique(frame, valeurb, max);
    editeurValeure = new EditeurValeurDynamique(frame, valeure, max);
    int indmax;
    if (modification) indmax = liste.indexOf(max) - 1;
    else indmax = liste.size() - 1;
    listeFonctions = new ListeFonctionsReelles(frame, liste, indmax, false);
    listeFonctions.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent evt) {
        	listeFonctionsSelectionChanged();
        }
      });
    if (modification) {
    	listeFonctions.setSel(max.fonctionAssociee);
    	editeurNom.setText(max.nom());
    	editeurValeura.setText(max.a.chaineInfo());
    	editeurValeurb.setText(max.b.chaineInfo());
    	editeurValeure.setText(max.incertitude.chaineInfo());
    }    
    JScrollPane scrollPane = new JScrollPane(listeFonctions, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

    
    btnOK = new MtgButton(frame, getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed();
      }
    });    
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
    btnOK.setPreferredSize(btnCancel.getMaximumSize());
    paneListeValeurEtFonctionsa = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurValeura, true, indmax, false);
    paneListeValeurEtFonctionsb = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurValeurb, true, indmax, false);
    paneListeValeurEtFonctionse = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurValeure, true, indmax, false);
    
    JPanel pane = new JPanel();
    pane.setLayout(new GridBagLayout());

    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 2;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 4, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label1, constraints);
    setConstraints(constraints, 0, 1, 1, 1, 25, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label2, constraints);
    setConstraints(constraints, 1, 1, 1, 1, 50, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    pane.add(editeurNom, constraints);
    setConstraints(constraints, 3, 1, 1, 1, 25, 0, GridBagConstraints.NONE, GridBagConstraints.EAST);
    pane.add(btnOK, constraints);
    setConstraints(constraints, 0, 2, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label3, constraints);
    setConstraints(constraints, 1, 2, 1, 7, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    pane.add(scrollPane, constraints);
    setConstraints(constraints, 3, 2, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.EAST);
    pane.add(btnCancel, constraints);
    setConstraints(constraints, 2, 3, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label4, constraints);
    setConstraints(constraints, 3, 3, 1, 1, 0, 10, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    pane.add(editeurValeura, constraints);
    constraints.insets.top = 0;
    setConstraints(constraints, 3, 4, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(paneListeValeurEtFonctionsa, constraints);
    constraints.insets.top = 5;
    setConstraints(constraints, 2, 5, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label5, constraints);
    setConstraints(constraints, 3, 5, 1, 1, 0, 10, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    pane.add(editeurValeurb, constraints);
    constraints.insets.top = 0;
    setConstraints(constraints, 3, 6, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(paneListeValeurEtFonctionsb, constraints);
    constraints.insets.top = 5;

    setConstraints(constraints, 2, 7, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label6, constraints);
    setConstraints(constraints, 3, 7, 1, 1, 0, 10, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    pane.add(editeurValeure, constraints);
    constraints.insets.top = 0;
    setConstraints(constraints, 3, 8, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(paneListeValeurEtFonctionse, constraints);
    constraints.insets.top = 5;

    setConstraints(constraints, 0, 9, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label7, constraints);
    setConstraints(constraints, 0, 10, 4, 1, 0, 10, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    pane.add(scrollPaneInfo, constraints);
    
    listeFonctionsSelectionChanged();
    setContentPane(pane);

    // On change l'ordre par dfaut des composants
    Vector<Component> order = new Vector<Component>(7);
    order.add(editeurNom);
    order.add(listeFonctions);
    order.add(editeurValeura);
    order.add(editeurValeurb);
    order.add(editeurValeure);
    order.add(btnOK);
    order.add(btnCancel);
    mtgFocusTraversalPolicy = new MtgFocusTraversalPolicy(order);
    setFocusTraversalPolicy(mtgFocusTraversalPolicy);
    getRootPane().setDefaultButton(btnOK);    
    pack();
  }

  public void btnCancelActionPerformed() {
  	dispose();
  }
  
  public void btnOKActionPerformed() {
    MtgInternalFrame cadre = frame.cadreActif();
    CListeObjets listePr = cadre.listePr;
    String chNom = editeurNom.getText();
    // On regarde d'abord si le nom propos est correct et non vide en cas de cration d'un calcul
    if (!modification) {
	    // On vrifie d'abord la validit du nom de la fonction
	    if (!listePr.validationNomCalcul(frame, chNom)) {
	      editeurNom.requestFocus();
	      return;
	    }
    }
    if (editeurValeura.verifieValidite()) {
    	if (editeurValeurb.verifieValidite()) {
      	if (editeurValeure.verifieValidite()) {
	        setRetourOK(true);
	        if (!modification)max.donneNom(chNom);
	        // On abandonne les objets CValeurAssocieeAVariable de pointAssocie pour les remplacer par les locaaux
	        max.a = valeura;
	        max.b = valeurb;
	        max.incertitude = valeure;
	        max.fonctionAssociee = listeFonctions.fonctionSelectionnee();
	        dispose();
      	}
    	}
    }
  }
  
  public void listeFonctionsSelectionChanged() {
  	CFonc fonc = listeFonctions.fonctionSelectionnee();
  	String ch = fonc.infoFormule();
  	info.setText(ch);
  	info.setCaretPosition(0);
  	pack();
  }
  
}
