/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import mathgraph32.MtgFrame;
import objets.CListeObjets;

@SuppressWarnings("serial")
public class MesureCoefficientDirecteurDlg extends MtgDlg {
	private MtgLabel label1, label2;
	private MtgButton btnOK, btnCancel;
	public ListeReperes listeReperes;
	private JScrollPane scrollPaneListeReperes;
	public MtgTextField editeurNom;
	
  public MesureCoefficientDirecteurDlg(MtgFrame frameInit) {
    super(getString("mesureCoefficientDirecteurDlg1"), true, frameInit);
    initComponents();
    getRootPane().setDefaultButton(btnOK);
  }
  
  private void initComponents() {
		CListeObjets liste = frame.cadreActif().listePr;
		int indmax = liste.size() - 1;
  	label1 = new MtgLabel(frame, getString("mesureCoefficientDirecteurDlg2"));
  	label2 = new MtgLabel(frame, getString("pointSurCourbeDlg2"));
  	
		btnOK = new MtgButton(frame, getString("btnOK"));
		btnOK.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				btnOKActionPerformed();
			}
		});
		btnCancel = new MtgButton(frame, getString("btnCancel"));
		btnCancel.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				btnCancelActionPerformed();
			}
		});
		btnOK.setMaximumSize(btnCancel.getPreferredSize());

		listeReperes = new ListeReperes(frame, liste, indmax);
		scrollPaneListeReperes = new JScrollPane(listeReperes, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
				
    editeurNom = new MtgTextField(frame, 8);
		
		JPanel panePrincipal = (JPanel)getContentPane();
		panePrincipal.setLayout(new GridBagLayout());
		JPanel paneRight = new JPanel();
		paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
		
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 5;
		constraints.insets.top = 5;
		constraints.insets.left = 5;
		constraints.insets.right = 5;

		setConstraints(constraints, 0, 0, 1, 1, 20, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label1, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 50, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		panePrincipal.add(editeurNom, constraints);
		setConstraints(constraints, 0, 1, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label2, constraints);
		setConstraints(constraints, 1, 1, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		panePrincipal.add(scrollPaneListeReperes, constraints);

		
		paneRight.add(btnOK);
		paneRight.add(Box.createRigidArea(new Dimension(0, 5)));
		paneRight.add(btnCancel);

		setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.NONE, GridBagConstraints.NORTH);
		panePrincipal.add(paneRight, constraints);
		pack();
  }
	
  private void btnOKActionPerformed() {
  	// On vrifie si le nom choisi pour la mesure est correct
    if (!frame.cadreActif().listePr.validationNomCalcul(frame, editeurNom.getText())) {
      editeurNom.requestFocus();
      return;
    }
  	setRetourOK(true);
  	dispose();
  }
	
  private void btnCancelActionPerformed() {
  	dispose();
  }  
  
}
