/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.BorderLayout;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import types.FonteNom;

@SuppressWarnings("serial")
public class OptionsFigureDlg extends MtgDlg {
	private JTabbedPane tabbedPane;
	private PaneOptionsFigureChoixUniteAngle paneOptionsFigureChoixUniteAngle;
	private PaneOptionFigureElementsFiges paneOptionFigureElementsFiges;
	// private PaneOptionsFigureImageFond paneOptionsFigureImageFond;
	private PaneOptionsFigureMenu paneOptionsFigureMenu;
	private PaneOptionsFigureCouleurFond paneOptionsFigureCouleurFond;
	private PaneOptionsFigureMacroDemarrage paneOptionsFigureMacroDemarrage; // Ajout version 4.8
	private PaneOptionsFigureChampsEdition paneOptionsFigureChampsEdition;
	private MtgButton btnOK;
	private MtgButton btnCancel;
  public OptionsFigureDlg(MtgFrame frameInit) {
    super(getString("dialogueOptions"),true, frameInit);
    MtgInternalFrame cadre = frame.cadreActif();
    tabbedPane = new JTabbedPane();
		tabbedPane.setFont(FonteNom.getFonteNom(MtgFrame.sizeFontLabel, FonteNom.Normal,
				(double)frame.pref_addSystemFont/MtgFrame.sizeFontLabel + MtgFrame.coefMultForSystemFonts));		
    paneOptionsFigureChoixUniteAngle = new PaneOptionsFigureChoixUniteAngle(cadre);
    // Ligne suivante modifi version 4.3
    // if (frame.pref_niveau >= MtgFrame.niveauMoyen) paneOptionsFigureChoixUniteAngle.radioRadian.setEnabled(false);
    if (!frame.pref_NiveauPersonnalise && (frame.pref_niveau >= MtgFrame.niveauMoyen)) paneOptionsFigureChoixUniteAngle.radioRadian.setEnabled(false);
    paneOptionFigureElementsFiges = new PaneOptionFigureElementsFiges(frame, cadre);
    // paneOptionsFigureImageFond = new PaneOptionsFigureImageFond(cadre); Supprim version 4.4
    paneOptionsFigureMenu = new PaneOptionsFigureMenu(cadre);
    paneOptionsFigureCouleurFond = new PaneOptionsFigureCouleurFond(cadre);
    paneOptionsFigureMacroDemarrage = new PaneOptionsFigureMacroDemarrage(cadre); // Ahjout version 4.8
    paneOptionsFigureChampsEdition = new PaneOptionsFigureChampsEdition(cadre);
    tabbedPane.addTab(paneOptionsFigureChoixUniteAngle.titreOnglet(), null, paneOptionsFigureChoixUniteAngle, null);
    tabbedPane.addTab(paneOptionFigureElementsFiges.titreOnglet(), null, paneOptionFigureElementsFiges, null);
    //tabbedPane.addTab(paneOptionsFigureImageFond.titreOnglet(), null, paneOptionsFigureImageFond, null); Supprim version 4.4
    tabbedPane.addTab(paneOptionsFigureMenu.titreOnglet(), null, paneOptionsFigureMenu, null);
    tabbedPane.addTab(paneOptionsFigureCouleurFond.titreOnglet(), null, paneOptionsFigureCouleurFond, null);
    tabbedPane.addTab(paneOptionsFigureMacroDemarrage.titreOnglet(), null, paneOptionsFigureMacroDemarrage, null); // Ajout version 4.8
    tabbedPane.addTab(paneOptionsFigureChampsEdition.titreOnglet(), null, paneOptionsFigureChampsEdition, null);
	  btnOK = new MtgButton(frame, getString("btnOK"));
	  btnOK.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnOKActionPerformed();
	    }
	  });    
	  btnCancel = new MtgButton(frame, getString("btnCancel"));
	  btnCancel.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnCancelActionPerformed();
	    }
	  });
	  btnOK.setPreferredSize(btnCancel.getPreferredSize());
    JPanel paneSouth = new JPanel();
    paneSouth.add(btnOK);
    paneSouth.add(btnCancel);
    getContentPane().setLayout(new BorderLayout());
    getContentPane().add("North", tabbedPane);
    getContentPane().add("South", paneSouth);
    getRootPane().setDefaultButton(btnOK);    
    pack();
  }
  
  private void btnCancelActionPerformed() {
  	dispose();
  }
  
  private void btnOKActionPerformed() {
  	MtgInternalFrame cadre = frame.cadreActif();
  	// On regarde si le chemin d'accs pour l'image de fond est valide
  	/* Supprim version 4.4
  	if (!paneOptionsFigureImageFond.donneesValides()) {
  		tabbedPane.setSelectedIndex(2);
  		return;
  	}
  	*/
  	paneOptionFigureElementsFiges.sauveDonnees();
  	boolean modificationUniteAngle = paneOptionsFigureChoixUniteAngle.sauveDonnees();;
  	boolean modification = modificationUniteAngle;
  	modification = modification || paneOptionsFigureChoixUniteAngle.sauveDonnees();
  	// modification = modification || paneOptionsFigureImageFond.sauveDonnees(); Supprim version 4.4
  	modification = modification || paneOptionsFigureCouleurFond.sauveDonnees();
  	modification = modification || paneOptionsFigureChampsEdition.sauveDonnees();
  	modification = modification || paneOptionsFigureMacroDemarrage.sauveDonnees();
  	setRetourOK(true);
  	// Si on a chang l'unit de la figure, dans positionne(), les calculs constants ne seront pas recalculs.
  	// Pour qu'ils le soient on sauvegarde la figure dans un flux et on le recharge de faon que les
  	// constantes soient recalcules.
	  if (modificationUniteAngle) {
	  	try {
		    // Ligne suivante ajoute  la version 4.7.1 our corriger un bug qui faisait que si une figure contenait des diteurs de
		    // formules ceux-ci n'taient plus associ au bon objet aprs reconstruction de la liste
		    cadre.retireComposants();
				ByteArrayOutputStream sauvegarde = new ByteArrayOutputStream();
				MathGraphDataOutputStream oups = new MathGraphDataOutputStream(sauvegarde);
		    cadre.listePr.write(oups);
		    cadre.listePr.retireTout();
		    oups.close();
		    ByteArrayInputStream ba = new ByteArrayInputStream(sauvegarde.toByteArray());
		    MathGraphDataInputStream inps = new MathGraphDataInputStream(ba);
		    cadre.listePr.read(inps);
		    inps.close();
		    // Les deux lignes suivantes ajoute  la version 4.7.1 our corriger un bug qui faisait que si une figure contenait des diteurs de
		    // formules ceux-ci n'taient plus associ au bon objet aprs reconstruction de la liste
		    cadre.listePr.retablitComposants();
		    cadre.creePaneAffichageVariables();
	  	}
	    catch (IOException err) {
	  		 System.out.println("Error : Internal in-out exception");
	    }			
  	}
	  	
  	if (modification) {
  		cadre.repaintFigureAvecCalcul();
      cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil111"));  
  	}
  	if (paneOptionsFigureMenu.sauveDonnees()) {
  		frame.metAJourIconesEtMenus(cadre.document);
  	}
  	dispose();

  }
}
