/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import mathgraph32.MtgFrame;

@SuppressWarnings("serial")
public class PanePreferencesNiveau extends PaneOptionsPreferences {
	
	private MtgLabel label1;
	private MtgRadioButton radioNiveauAvanceAvecComplexes;
	private MtgRadioButton radioNiveauAvanceSansComplexes;
	private MtgRadioButton radioNiveauMoyen;
	private MtgRadioButton radioNiveauElementaire;
	private ButtonGroup btnGroup;
	public int niveauChoisi;
	
	public PanePreferencesNiveau(MtgFrame frameInit) {
		super(frameInit);
		niveauChoisi = frame.pref_niveau;
		initComponents(frame.pref_NiveauPersonnalise);
	}
	
	public void initComponents(boolean personnalise) {
		removeAll(); // Car peut tre appel par le dialogue parent
		label1 = new MtgLabel(frame, MtgDlg.getString("niveau"));
		// Modifi version 3.2.4. les descripteurs de niveau peuvent tre personnaliss
		// Attention : le niveau le plus lev est le niveau 1 mais pour les fichiers contenant les outils par niveaux c'est l'inverse
		radioNiveauAvanceAvecComplexes = new MtgRadioButton(frame, 
				personnalise ? frame.descripteurNiveauPersonnalise[1] : frame.descripteurNiveauPredefini[1]);
		radioNiveauAvanceAvecComplexes.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setNiveauChoisi();
			}
		});
		radioNiveauAvanceSansComplexes =  new MtgRadioButton(frame, 
				personnalise ? frame.descripteurNiveauPersonnalise[2] : frame.descripteurNiveauPredefini[2]);
		radioNiveauAvanceSansComplexes.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setNiveauChoisi();
			}
		});
		radioNiveauMoyen = new MtgRadioButton(frame, 
				personnalise ? frame.descripteurNiveauPersonnalise[3] : frame.descripteurNiveauPredefini[3]);
		radioNiveauMoyen.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setNiveauChoisi();
			}
		});
		radioNiveauElementaire = new MtgRadioButton(frame, 
				personnalise ? frame.descripteurNiveauPersonnalise[4] : frame.descripteurNiveauPredefini[4]);
		radioNiveauElementaire.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setNiveauChoisi();
			}
		});
		btnGroup= new ButtonGroup();
		btnGroup.add(radioNiveauAvanceAvecComplexes);
		btnGroup.add(radioNiveauAvanceSansComplexes);
		btnGroup.add(radioNiveauMoyen);
		btnGroup.add(radioNiveauElementaire);
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		add(label1);
		add(radioNiveauElementaire);
		add(radioNiveauMoyen);
		add(radioNiveauAvanceSansComplexes);
		add(radioNiveauAvanceAvecComplexes);
		setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		setNiveauChoisi();
		radioNiveauAvanceAvecComplexes.setSelected(niveauChoisi == MtgFrame.niveauAvanceAvecComplexes);
		radioNiveauAvanceSansComplexes.setSelected(niveauChoisi == MtgFrame.niveauAvanceSansComplexes);
		radioNiveauMoyen.setSelected(niveauChoisi == MtgFrame.niveauMoyen);
		radioNiveauElementaire.setSelected(niveauChoisi == MtgFrame.niveauElementaire);
	}
	
	public void sauveDonnees() {

		if (radioNiveauAvanceAvecComplexes.isSelected()) frame.pref_niveau = MtgFrame.niveauAvanceAvecComplexes;
		if (radioNiveauAvanceSansComplexes.isSelected()) frame.pref_niveau = MtgFrame.niveauAvanceSansComplexes;
		if (radioNiveauMoyen.isSelected()) frame.pref_niveau = MtgFrame.niveauMoyen;
		if (radioNiveauElementaire.isSelected()) frame.pref_niveau = MtgFrame.niveauElementaire;
		frame.setDescripteursNiveau();
		frame.setDocPreferencesNiveau();
		// On charge aussi le document qui spcifie les outils permis ou interdits nomm
		// default1.mgj pour le niveau lmentaire
		// default2.mgj pour le niveau collge
		// default3.mg pour le niveau avanc sans complexes
		// default4.mgj pour le niveau avanc avec complexes
		// On le cherche dans le dossier contenant le jar excutable.
			
	}
	
	public String titreOnglet() {
		return MtgDlg.getString("ongletNiveau");
	}	
	
	private void setNiveauChoisi() {
		if (radioNiveauAvanceAvecComplexes.isSelected()) niveauChoisi = MtgFrame.niveauAvanceAvecComplexes;
		if (radioNiveauAvanceSansComplexes.isSelected()) niveauChoisi = MtgFrame.niveauAvanceSansComplexes;
		if (radioNiveauMoyen.isSelected()) niveauChoisi = MtgFrame.niveauMoyen;
		if (radioNiveauElementaire.isSelected()) niveauChoisi = MtgFrame.niveauElementaire;
	}
	
}
