/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.io.File;
import javax.swing.JFileChooser;
import mathgraph32.MtgFrame;

@SuppressWarnings("serial")
public class PanePreferencesRepertoireFichiers extends PaneOptionsPreferences {
	private MtgLabel label1;
	private MtgButton btnParcourir;
	private MtgTextField editeurRepertoire;
	
	public PanePreferencesRepertoireFichiers(MtgFrame frameInit) {
    super(frameInit);
    initComponents();
    editeurRepertoire.setText(frame.pref_fileDirectory);
	}
	
	public void initComponents() {
		label1 = new MtgLabel(frame, MtgDlg.getString("repertoireFichiers"));
    editeurRepertoire = new MtgTextField(frame, 60);
	  btnParcourir = new MtgButton(frame, MtgDlg.getString("parcourir"));
    btnParcourir.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnParcourirActionPerformed();
      }
    });
    setLayout(new GridBagLayout());

		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 5;
		constraints.insets.top = 5;
		constraints.insets.left = 5;
		constraints.insets.right = 5;

		MtgDlg.setConstraints(constraints, 0, 0, 1, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		add(label1, constraints);
		MtgDlg.setConstraints(constraints, 0, 1, 1, 1, 100, 10, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		add(editeurRepertoire, constraints);
		MtgDlg.setConstraints(constraints, 0, 2, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		add(btnParcourir, constraints);
	}	

  private void btnParcourirActionPerformed() {
		JFileChooser fc = new JFileChooser(frame.pref_fileDirectory);
		fc.setApproveButtonText(MtgDlg.getString("btnChoisir"));
		fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		// On ajoute un panneau de visualisation des fichiers image
		fc.setAccessory(new ImagePreview(fc));
		fc.showOpenDialog(frame);
		File file = fc.getSelectedFile();
		if (file != null) {
			if (file.isDirectory()) {
				editeurRepertoire.setText(file.getAbsolutePath());
			}
		}
  }
  
	/**
	 * Fonction renvoyant true si les donnes de ce panneau sont valides.
	 * True par dfaut.
	 * Sera redfini pour la panneau de choix d'image de fond pour la figure
	 */
	public boolean donneesValides() {
		String ch = editeurRepertoire.getText();
		if (ch.length() == 0) return true;
		File file = new File(ch);
		return file.isDirectory();
	}
	
  public void sauveDonnees() {
  	frame.pref_fileDirectory = editeurRepertoire.getText();
  	frame.repertoireFichiers = "";
  }
	
	public String titreOnglet() {
		return MtgDlg.getString("ongletRepertoireOuvertureFichiers");
	}	
  
  
}
