/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CCalcul;
import objets.CFonc;
import objets.CFoncNVar;
import objets.CListeObjets;
import objets.CTestDependanceVariable;
import types.FonteNom;
import types.NatureCalcul;

@SuppressWarnings("serial")
public class TestDependanceVariableDlg extends MtgDlg {
	private CTestDependanceVariable testDependance;
	private boolean modification;
	// Modifi version 5.3 pour pouvoir faire des test de dpendance fonction complexe - variable.
	// private ListeFonctionsReellesNVariables listeFonctions;
	private ListeFonctionsReellesOuComplexesNVariables listeFonctions;
	private MtgLabel label1, label2, label3, label4;
	private MtgTextField editeurNom , info;
  private MtgButton btnOK, btnCancel;
  private JList<String> listeVariables;
	// private Vector<String> tab; // Liste des variables partielles
	private DefaultListModel<String> listModel;

  public TestDependanceVariableDlg(MtgFrame frameInit, CTestDependanceVariable testDependanceInit, boolean modificationInit) {
  	super(getString("menuCreerTestDependanceVariable"), true, frameInit);
  	modification = modificationInit;
  	testDependance = testDependanceInit;
    listModel = new DefaultListModel<String>();
    listeVariables = new JList<String>();
		listeVariables.setFont(FonteNom.getFonteEdition(MtgFrame.sizeFontList, FonteNom.Normal,
				(double)frame.pref_addSystemFont/MtgFrame.sizeFontList + MtgFrame.coefMultForDlgBoxIcons));		
    listeVariables.setModel(listModel);
    initComponents();
  	listeFonctionsSelectionChanged();
    if (modification) {
    	listeVariables.setSelectedIndex(testDependance.indiceVariable);
    }
    else {
    	listeVariables.setSelectedIndex(0);
    }
    getRootPane().setDefaultButton(btnOK);      	
  }

  private void initComponents() {
    label1 = new MtgLabel(frame, getString("testDependanceVariableDlg2"), JLabel.RIGHT);
    label1.setRequestFocusEnabled(false);
    label2 = new MtgLabel(frame, getString("suiteRecReelleDlg3"), JLabel.RIGHT);
    label2.setRequestFocusEnabled(false);
    label3 = new MtgLabel(frame, getString("informationFonction"), JLabel.RIGHT);
    label3.setRequestFocusEnabled(false);  
    label4 = new MtgLabel(frame, getString("testDependanceVariableDlg4"), JLabel.RIGHT);
    label4.setRequestFocusEnabled(false);
    editeurNom = new MtgTextField(frame,8);
    info = new MtgTextField(frame);
    info.setEditable(false);
    info.setColumns(40);
    CListeObjets liste = frame.cadreActif().listePr;
    int indmax;
    if (modification) {
    	indmax = liste.indexOf(testDependance) - 1;
    }
    else {
    	indmax = liste.size() - 1;
    }
    listeFonctions = new ListeFonctionsReellesOuComplexesNVariables(frame, true, liste, indmax, false);
    listeFonctions.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent evt) {
      	listeFonctionsSelectionChanged();
      }
    });
    if (modification)  {
    	editeurNom.setEditable(false);
    	editeurNom.setText(testDependance.nom());
    	listeFonctions.setSel(testDependance.fonctionAssociee);
    }
    JScrollPane scrollPaneListe = new JScrollPane(listeFonctions, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    
    // tab = new Vector<String>();
    listeVariables.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    btnOK = new MtgButton(frame, getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed();
      }
    });    
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());
    JPanel panePrincipal = new JPanel();
    JPanel paneRight = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 40, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label1, constraints);
    
    setConstraints(constraints, 1, 0, 1, 1, 40, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(editeurNom, constraints);    
    
    setConstraints(constraints, 0, 1, 1, 1, 0, 50, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label2, constraints);

    setConstraints(constraints, 1, 1, 1, 1, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    panePrincipal.add(scrollPaneListe, constraints);

    setConstraints(constraints, 0, 2, 1, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label4, constraints);
    setConstraints(constraints, 1, 2, 1, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(listeVariables, constraints);

    setConstraints(constraints, 0, 3, 2, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label3, constraints);
    setConstraints(constraints, 0, 4, 2, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(info, constraints);
    
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
    setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.NONE, GridBagConstraints.NORTH);
    panePrincipal.add(paneRight, constraints);
    
    setContentPane(panePrincipal);
    pack(); 
  }

  private void listeFonctionsSelectionChanged() {
  	// Vector<String> tab = new Vector<String>();
  	listModel.removeAllElements();
  	CCalcul fonc = listeFonctions.fonctionSelectionnee();
  	// boolean fonc1var = fonc.getNatureCalcul() == NatureCalcul.NFonction; // Modifi version 5.1.2
  	boolean fonc1var = fonc.estDeNatureCalcul(NatureCalcul.NFonctionReelle1Variable|NatureCalcul.NFonctionComplexe) ; // Modifi version 5.3
  	// if (fonc1var) listModel.add(0,((CFonc)fonc).chaineVariable); // Modifi version 5.1.1
  	 if (fonc1var) listModel.add(0,((CFonc)fonc).chaineVariable());
  	else {
  		CFoncNVar foncnvar = (CFoncNVar) fonc;
	  	for (int i = 0; i < foncnvar.nbVar; i ++) {
	  		listModel.add(i," " + foncnvar.chaineVariable()[i]+ " ");
	  		// tab.add(fonc.chaineVariable[i]);
	  	}
  	}
  	listeVariables.setSelectedIndex(0);
  	int i = listeFonctions.getSelectedIndex();
  	if (i != -1) {
  		if (fonc1var) info.setText(((CFonc)fonc).infoFormule());
  		else info.setText(((CFoncNVar)fonc).infoFormule());
  	}
		pack();
  }

  private void btnOKActionPerformed() {                                      
    MtgInternalFrame cadre = frame.cadreActif();
    CListeObjets listePr = cadre.listePr;
    String chNom = editeurNom.getText();
    if (!modification) {
	    // Modifi version 4.6
	    if (!listePr.validationNomCalcul(frame, chNom)) {
	      editeurNom.requestFocus();
	      return;
	    }
    }
    CCalcul fonc = listeFonctions.fonctionSelectionnee();
    if (modification) {
    	testDependance.fonctionAssociee = fonc;
    	testDependance.indiceVariable = listeVariables.getSelectedIndex();
    }
    else {
    	CTestDependanceVariable test = new CTestDependanceVariable(listePr, null, false, chNom, fonc, listeVariables.getSelectedIndex());
	    frame.ajouteElement(test);
	    test.positionne(frame.cadreActif().infRand, frame.cadreActif().dimf);
	  	cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil156"));
    }
  	setRetourOK(true);
  	dispose();
  }

  public void btnCancelActionPerformed() {
  	dispose();
  }

}
