/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package mathgraph32;

/**
 *
 * @author Yves
 */
import java.io.IOException;
import objets.*;

/**
 * Classe servant  grer les annulations refaire sur la figure
 * 
 * @author Yves
 */
public class GestionnaireEdition {
	static int nombreMaxiAnnulations = 50; // Le nombre maxi d'annulations
																					// autorises
	// Tableau qui recevra les sauvegardes de la figure
	// ByteArrayOutputStream[] sauvegarde = new
	// ByteArrayOutputStream[nombreMaxiAnnulations];
	CListeObjets[] sauvegarde = new CListeObjets[nombreMaxiAnnulations];
	// Tableau qui recevra des chaines indiquant quel est le type d'action ayant
	// men  la sauvegarde d'indice l'ndice dans le tableau
	String natureSauvegarde[] = new String[nombreMaxiAnnulations]; // Recevra une
																																	// chaine
																																	// disant quel
																																	// est le type
																																	// d'action
	// qui a men  la sauvegarde suivant son indice
	MtgInternalFrame cadre; // La fentre sur laquelle agit le gestionnaire
	int indiceDerniereSauvegarde;
	int indiceFigureEnCours; // Indice de la figure en cours dans le tableau de
														// sauvegardes

	/**
	 * Constructeur
	 * 
	 * @param documentInit
	 *          le document que gre le gestionnaire
	 */
	public GestionnaireEdition(MtgInternalFrame fenetreAssocieeInit) {
		cadre = fenetreAssocieeInit;
	}

	/**
	 * Sauvegarde la figure dans le fichier mmoire d'indice 0
	 */
	public void initialise() throws IOException {
		// sauvegarde[0] = new ByteArrayOutputStream();
		sauvegarde[0] = new CListeObjets(cadre.listePr);
		/*
		 * MathGraphDataOutputStream oups = new
		 * MathGraphDataOutputStream(sauvegarde[0]); cadre.listePr.write(oups);
		 * oups.close();
		 */
		cadre.listePr.setCopy(sauvegarde[0]);
		indiceFigureEnCours = 0;
		indiceDerniereSauvegarde = 0;
	}

	/**
	 * Sauvegarde de l'tat actuel de la figure
	 */
	public void enregistreFigureEnCours(String nature) {
		cadre.document.setDirty(true);
		if (indiceFigureEnCours < nombreMaxiAnnulations - 1) {
			indiceFigureEnCours++;
			natureSauvegarde[indiceFigureEnCours] = nature;
			// sauvegarde[indiceFigureEnCours] = new ByteArrayOutputStream(1000); //
			// Modifi version 4.4 pour acclrer le transfert
			sauvegarde[indiceFigureEnCours] = new CListeObjets();
			cadre.listePr.setCopy(sauvegarde[indiceFigureEnCours]);
			/*
			 * MathGraphDataOutputStream oups = new
			 * MathGraphDataOutputStream(sauvegarde[indiceFigureEnCours]); // On
			 * enregistre la liste d'objets principale dans l'archive
			 * cadre.listePr.write(oups); oups.close();
			 */
			indiceDerniereSauvegarde = indiceFigureEnCours;
			// Il faut mettre  jour les liste maintenues de faon internes par les
			// macros
			// quand la figure a t modifie
			cadre.listePr.etablitListesInternesMacros();

		} else // Il faut tout dcaler et on perd alors la premire sauvegarde
		{
			for (int i = 1; i < nombreMaxiAnnulations; i++) {
				natureSauvegarde[i - 1] = natureSauvegarde[i];
				sauvegarde[i - 1] = sauvegarde[i];
			}
			natureSauvegarde[nombreMaxiAnnulations - 1] = nature;
			// sauvegarde[nombreMaxiAnnulations - 1] = new ByteArrayOutputStream();
			sauvegarde[nombreMaxiAnnulations - 1] = new CListeObjets();
			cadre.listePr.setCopy(sauvegarde[nombreMaxiAnnulations - 1]);
			/*
			 * MathGraphDataOutputStream oups = new
			 * MathGraphDataOutputStream(sauvegarde[nombreMaxiAnnulations - 1]); // On
			 * enregistre la liste d'objets principale dans l'archive
			 * cadre.listePr.write(oups); oups.close();
			 */
			// Il faut mettre  jour les liste maintenues de faon internes par les
			// macros
			// cadre.listePr.etablitListesInternesMacros();
		}
		metAJourTipIconesAnnulerRefaire();
		cadre.listePr.initialiseDependances(); // Ajout version 4.9.2. Ds qu'on modifie une figure on rinitialise les dpendances
	}

	/**
	 * Ajout version 4.8. Quand on fait des zooms successifs on n'enregistre ue la
	 * dernire modification Attention de ne pas modifier les identifiants
	 * chaineOutil94 et chaineOutil95
	 * Version 5.0 : Idem pour les zoom pour les tailles de police
	 * @param nature
	 */
	public void enregistreFigureEnCoursPourZoom(String nature) {
		if (indiceFigureEnCours == 0) {
			enregistreFigureEnCours(nature);
			return;
		}
		String ch = natureSauvegarde[indiceFigureEnCours];
		if (ch.equals(MtgFrame.getString("chaineOutil94")) || ch.equals(MtgFrame.getString("chaineOutil95")) ||
				ch.equals(MtgFrame.getString("chaineOutil163")) || (ch.equals(MtgFrame.getString("chaineOutil164")))) {
			if (indiceFigureEnCours < nombreMaxiAnnulations - 1) {
				indiceFigureEnCours--;
				enregistreFigureEnCours(nature);
			} else
				enregistreFigureEnCours(nature);
		} else
			enregistreFigureEnCours(nature);

	}

	public boolean annulationPossible() {
		return (indiceFigureEnCours > 0);
	}

	public boolean refairePossible() {
		return (indiceFigureEnCours < indiceDerniereSauvegarde);
	}

	public void annuleAction() throws IOException {
 		cadre.document.setDirty(true);
		indiceFigureEnCours--;
		// Version 4.5.2. Il faut dtruire les MtgFormulaEditor associs  des
		// CEditeurFormule
		cadre.retireComposants();
		// Fin version 4.5.2
		cadre.listePr.retireTout();
		cadre.listePr = new CListeObjets();
		// A noter : Les diteurs de formule CEditeurFormule vont tre recrs et
		// donc aussi les MtgFormulaEditor
		sauvegarde[indiceFigureEnCours].setCopy(cadre.listePr);
		cadre.listePr.initialiseDependances(); // Ajout version 4.9.7 pour contrer bug alatoire sur macos d'animation
		cadre.listePr.setClone(sauvegarde[indiceFigureEnCours]);
		cadre.document.listePrincipale = cadre.listePr;
		// cadre.listePr.documentProprietaire = cadre.document; // Modifi version 5.0
		cadre.listePr.associeA(cadre.document);
		// Il faut mettre  jour les liste maintenues de faon internes par les
		// macros
		cadre.listePr.etablitListesInternesMacros();
		// Si la figure contient un affichage de variables en bas et  droite il
		// faut en tenir compte
		cadre.creePaneAffichageVariables();
		metAJourTipIconesAnnulerRefaire();
		cadre.videListesPourConstructions();
	}

	public void refaitAction() throws IOException {
		cadre.document.setDirty(true);
		indiceFigureEnCours++;
		// Version 4.5.2. Il faut dtruire les MtgFormulaEditor associs  des
		// CEditeurFormule
		cadre.retireComposants();
		// Fin version 4.5.2
		/*
		 * ByteArrayInputStream ba = new ByteArrayInputStream
		 * (sauvegarde[indiceFigureEnCours].toByteArray()); MathGraphDataInputStream
		 * inps = new MathGraphDataInputStream(ba); cadre.listePr = new
		 * CListeObjets();
		 */
		cadre.listePr.retireTout();
		cadre.listePr = new CListeObjets();
		sauvegarde[indiceFigureEnCours].setCopy(cadre.listePr);
		cadre.listePr.setClone(sauvegarde[indiceFigureEnCours]);
		cadre.document.listePrincipale = cadre.listePr;
		// cadre.listePr.documentProprietaire = cadre.document; // Modifi version 5.0
		cadre.listePr.associeA(cadre.document);

		/*
		 * cadre.listePr.read(inps); inps.close();
		 */
		// Il faut mettre  jour les liste maintenues de faon internes par les
		// macros
		cadre.listePr.etablitListesInternesMacros();
		// Si la figure contient un affichage de variables en bas et  droite il
		// faut en tenir compte
		// cadre.supprimeDialogueAffichageVariables();
		// cadre.creeDialogueAffichageVariables();
		cadre.creePaneAffichageVariables();
		metAJourTipIconesAnnulerRefaire();
		cadre.videListesPourConstructions();
	}

	public String chaineNatureAnnulationEnCours() {
		return natureSauvegarde[indiceFigureEnCours];
	}

	public String chaineNatureRefaireEnCours() {
		return natureSauvegarde[indiceFigureEnCours + 1];
	}

	public void metAJourTipIconesAnnulerRefaire() {
		String ch;
		if (annulationPossible())
			ch = MtgFrame.getString("ch13") + " " + chaineNatureAnnulationEnCours();
		else
			ch = "";
		cadre.frame.buttonAnnuler.setToolTipText(ch);
		if (ch.isEmpty())
			cadre.frame.menuItemEditionAnnuler.setText(MtgFrame.getString("ch13"));
		else
			cadre.frame.menuItemEditionAnnuler.setText(ch);
		if (refairePossible())
			ch = MtgFrame.getString("ch14") + " " + chaineNatureRefaireEnCours();
		else
			ch = "";
		cadre.frame.buttonRefaire.setToolTipText(ch);
		if (ch.isEmpty())
			cadre.frame.menuItemEditionRefaire.setText(MtgFrame.getString("ch14"));
		else
			cadre.frame.menuItemEditionRefaire.setText(ch);
	}
}