/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package mathgraph32;

import java.awt.Graphics2D;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.RenderingHints;
import java.awt.Transparency;
import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.net.URL;

import javax.swing.ImageIcon;

/**
 * Classe hritant de ImageIcon servant  grer les images hautes dfinitions.
 * @author yvesb
 *
 */
@SuppressWarnings("serial")
public class MtgImageIcon extends ImageIcon {
	double coef;
	public MtgImageIcon(double coefInit, URL adr) {
		super(getImageIcon(adr, coefInit));
		coef = coefInit;
	}
	public MtgImageIcon(double coefInit, ImageIcon imic) {
		super(getImageIcon(imic, coefInit));
		coef = coefInit;
	}
  private static final Image getImageIcon(URL url, double coef) {
		ImageIcon imi = new ImageIcon(url);
		int w = imi.getIconWidth();
		int h = imi.getIconHeight();
		Image img = imi.getImage();
    GraphicsEnvironment env = GraphicsEnvironment.getLocalGraphicsEnvironment();
    GraphicsDevice device = env.getDefaultScreenDevice();
    GraphicsConfiguration config = device.getDefaultConfiguration();
    BufferedImage bi = config.createCompatibleImage(w, h, Transparency.TRANSLUCENT);
		Graphics2D g = bi.createGraphics();
		g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
    g.setRenderingHint(RenderingHints.KEY_RENDERING,  RenderingHints.VALUE_RENDER_SPEED);
    g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF );
    g.drawImage(img, 0, 0 , null);
		g.dispose();
    BufferedImage after = config.createCompatibleImage((int)(w*coef),(int)(h*coef), Transparency.TRANSLUCENT);
    AffineTransform at = new AffineTransform();
    at.scale(coef, coef);
    AffineTransformOp scaleOp = new AffineTransformOp(at, AffineTransformOp.TYPE_BILINEAR);
    after = scaleOp.filter(bi, after);
		return after;
		// return img.getScaledInstance(w, h,  java.awt.Image.SCALE_SMOOTH); // Ne pas utiliser getScaledInstance qui est trop lent
  }
  private static final Image getImageIcon(ImageIcon imi, double coef) {
		int w = imi.getIconWidth();
		int h = imi.getIconHeight();
		Image img = imi.getImage();
    GraphicsEnvironment env = GraphicsEnvironment.getLocalGraphicsEnvironment();
    GraphicsDevice device = env.getDefaultScreenDevice();
    GraphicsConfiguration config = device.getDefaultConfiguration();
    BufferedImage bi = config.createCompatibleImage(w, h, Transparency.TRANSLUCENT);
		// BufferedImage bi = new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);
		Graphics2D g = bi.createGraphics();
		g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
    g.setRenderingHint(RenderingHints.KEY_RENDERING,  RenderingHints.VALUE_RENDER_SPEED);
    g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF );
    g.drawImage(img, 0, 0 , null);
		g.dispose();
    BufferedImage after = config.createCompatibleImage((int)(w*coef),(int)(h*coef), Transparency.TRANSLUCENT);
    AffineTransform at = new AffineTransform();
    at.scale(coef, coef);
    AffineTransformOp scaleOp = new AffineTransformOp(at, AffineTransformOp.TYPE_BILINEAR);
    after = scaleOp.filter(bi, after);
		return after;
		// return img.getScaledInstance(w, h,  java.awt.Image.SCALE_SMOOTH); // Ne pas utiliser getScaledInstance qui est trop lent
  }
}