/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package mathgraph32;

import java.awt.Insets;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JMenu;
import javax.swing.JSeparator;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;
import javax.swing.tree.DefaultMutableTreeNode;

import actions.MtgAction;

import objets.CMathGrafDoc;
import types.FonteNom;

@SuppressWarnings("serial")
public class MtgMenu extends JMenu implements MenuListener{
	// Inset pour les items de menus
	public static final Insets insetMenuItem = new Insets(0, 3, 0, 3);
	public MtgFrame frame; // Ajout version 5.0
	
	public MtgMenu(String ch, MtgFrame frameInit) { // Modifi vetsion 5.0
		super(ch);
		frame = frameInit;
		// Ajout version 5.0
		setFont(FonteNom.getFonteNom(MtgFrame.sizeFontMenuItem, FonteNom.Normal,
				(double)frame.pref_addSystemFont/MtgFrame.sizeFontMenuItem + MtgFrame.coefMultForSystemFonts));
		// Car l'augmentation des polices systme pas toujours effective
		// int k = (int) (2*MtgFrame.coefMultForDlgBoxIcons);
		// setBorder(BorderFactory.createEmptyBorder(0, k, 0, k));
		setMargin(insetMenuItem);
		addMenuListener(this);
	}
	
	public MtgMenu(MtgMenu menuInit, MtgFrame frameInit) {
		super(menuInit.getText());
		frame = frameInit;
		// Ajout version 5.0
		setFont(FonteNom.getFonteNom(MtgFrame.sizeFontMenuItem, FonteNom.Normal,
				(double)frame.pref_addSystemFont/MtgFrame.sizeFontMenuItem + MtgFrame.coefMultForDlgBoxIcons));
		// Car l'augmentation des polices systme pas toujours effective
		setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 5));
		for (int i = 0; i < menuInit.getMenuComponentCount(); i++) {
			JComponent comp = (JComponent) menuInit.getMenuComponent(i);
			if (comp instanceof MtgMenuItem) {
  			MtgMenuItem mtgitem = (MtgMenuItem) comp;
  			// Modification version 5.0 pour viter de redimensionner l'icne ce qui ne sert  rien ici
  			// if (mtgitem.getAction() != null) add (new MtgMenuItem((MtgAction) mtgitem.getAction()));
  			if (mtgitem.getAction() != null) 	add (new MtgMenuItem(mtgitem));
  			else add (new MtgMenuItem(mtgitem.frame));
			}
			else {
				if (comp instanceof MtgMenu){
					add(new MtgMenu((MtgMenu) comp,((MtgMenu) comp).frame));
				}
				else { // On a forcment un Jseparator
					add (new JSeparator());
				}
			}
		}
	}

  public final boolean adapteADocument(CMathGrafDoc doc, CMathGrafDoc docNiveau) {
  	boolean auMoinsUnItemVisible = false;
  	boolean bSeparator = false;
		for (int i = 0; i < getMenuComponentCount(); i++) {
			JComponent comp = (JComponent) getMenuComponent(i);
			if (comp instanceof MtgMenuItem) {
  			MtgMenuItem mtgitem = (MtgMenuItem) comp;
  			boolean b1 = mtgitem.adapteADocument(doc, docNiveau);
  			auMoinsUnItemVisible = auMoinsUnItemVisible || b1;
  			bSeparator = !b1;
			}
			else {
				if (comp instanceof MtgMenu){
					boolean b2 = ((MtgMenu) comp).adapteADocument(doc, docNiveau);
					auMoinsUnItemVisible = auMoinsUnItemVisible || b2;
					bSeparator = !b2;
				}
				else { // On a forcment un Jseparator
					if (bSeparator) comp.setVisible(false);
					bSeparator = true;
				}
			}
		}
		if (auMoinsUnItemVisible) setVisible(true);
		else setVisible(false);
		return auMoinsUnItemVisible;
  }
  
  
  /**
   * Fonction rcursive adaptant le document aux items cochs ou non
   * @param doc
   * @return
   */
  public final void adapteDocument(CMathGrafDoc doc) {
		for (int i = 0; i < getMenuComponentCount(); i++) {
			JComponent comp = (JComponent) getMenuComponent(i);
			if (comp instanceof MtgCheckBoxMenuItem) {
				MtgCheckBoxMenuItem mtgitem = (MtgCheckBoxMenuItem) comp;
  			mtgitem.adapteDocument(doc);
			}
			else {
				if (comp instanceof MtgMenu){
					((MtgMenu) comp).adapteDocument(doc);
				}
			}
		}
  }
  
  public final void ajouteTousItemsADocument(CMathGrafDoc doc) {
		for (int i = 0; i < getMenuComponentCount(); i++) {
			JComponent comp = (JComponent) getMenuComponent(i);
			if (comp instanceof MtgCheckBoxMenuItem) {
				MtgCheckBoxMenuItem mtgitem = (MtgCheckBoxMenuItem) comp;
				doc.addItem(mtgitem.actionAssociee.idMenu());
  			mtgitem.adapteDocument(doc);
			}
			else {
				if (comp instanceof MtgMenu){
					((MtgMenu) comp).ajouteTousItemsADocument(doc);
				}
			}
		}
  	
  }
  
  /**
   * Fonction slectionnant tous les items ayant une case  cocher. Modifi version 4.3
   */
  public final void selectAll(boolean bPreserveHorToolbar) {
		for (int i = 0; i < getMenuComponentCount(); i++) {
			JComponent comp = (JComponent) getMenuComponent(i);
			if (comp instanceof MtgCheckBoxMenuItem) {
				MtgCheckBoxMenuItem mtgitem = (MtgCheckBoxMenuItem) comp;
  			mtgitem.setSelected(bPreserveHorToolbar ? !mtgitem.actionAssociee.estDansBarreHorizontale() : true);
			}
			else {
				if (comp instanceof MtgMenu){
					((MtgMenu) comp).selectAll(bPreserveHorToolbar);
				}
			}
		} 	
  }
  
  /**
   * Fonction slectionnant tous les items ayant une case  cocher. MOdifi version 4.3
   */
  public final void deselectAll(boolean bPreserveHorToolbar) {
		for (int i = 0; i < getMenuComponentCount(); i++) {
			JComponent comp = (JComponent) getMenuComponent(i);
			if (comp instanceof MtgCheckBoxMenuItem) {
				MtgCheckBoxMenuItem mtgitem = (MtgCheckBoxMenuItem) comp;
  			mtgitem.setSelected(bPreserveHorToolbar ? mtgitem.actionAssociee.estDansBarreHorizontale() : false);
			}
			else {
				if (comp instanceof MtgMenu){
					((MtgMenu) comp).deselectAll(bPreserveHorToolbar);
				}
			}
		} 	
  }
  
  
  public final void addCloneAvecCheck(MtgMenu cible, CMathGrafDoc docNiveau, CMathGrafDoc docReference) {
		for (int i = 0; i < getMenuComponentCount(); i++) {
			JComponent comp = (JComponent) getMenuComponent(i);
			if ((comp.isVisible() && comp instanceof MtgMenuItem)) {
				MtgMenuItem item = (MtgMenuItem) comp;
				MtgAction action = (MtgAction)item. getAction();
				boolean b = (docNiveau == null) ? true : docNiveau.itemPermis(action.idMenu());
				if (b && (action.idMenu() != 0)) {
					MtgCheckBoxMenuItem mtgCheckItem = new MtgCheckBoxMenuItem (item, docReference);
					cible.add(mtgCheckItem);
				}
 			}
			else {
				if (comp instanceof MtgMenu){
					if (comp.isVisible()) {
						MtgMenu menu = new MtgMenu(((MtgMenu) comp).getText(), ((MtgMenu) comp).frame);
						cible.add(menu);
						((MtgMenu) comp).addCloneAvecCheck(menu, docNiveau, docReference);
					}
				}
			}
		}
  }
  
  /**
   * Fonction renvoyant true si au moins un des item avec checkBox contenu dans le menu
   * ou un sous-menu a t modifi
   * @return
   */
  public final boolean modifie() {
  	boolean res = false;
		for (int i = 0; i < getMenuComponentCount(); i++) {
			JComponent comp = (JComponent) getMenuComponent(i);
			if (comp instanceof MtgCheckBoxMenuItem) {
				MtgCheckBoxMenuItem mtgitem = (MtgCheckBoxMenuItem) comp;
  			res = res || mtgitem.modifie();
			}
			else {
				if (comp instanceof MtgMenu){
					res = res || ((MtgMenu) comp).modifie();
				}
			}
		}
  	return res;
  }
  
  public final void ajouteNoeud(DefaultMutableTreeNode node) {
		for (int i = 0; i < getMenuComponentCount(); i++) {
			JComponent comp = (JComponent) getMenuComponent(i);
			if (comp instanceof MtgCheckBoxMenuItem) {
				MtgCheckBoxMenuItem item = (MtgCheckBoxMenuItem) comp;
				item.ajouteNoeud(node);
 			}
			else {
				if (comp instanceof MtgMenu){
					MtgMenu menu = (MtgMenu) comp;
					DefaultMutableTreeNode newnode = new DefaultMutableTreeNode(menu.getText());
					node.add(newnode);
					menu.ajouteNoeud(newnode);
				}
			}
		}
  }

	@Override
	public void menuCanceled(MenuEvent e) {
	// TODO Auto-generated method stub
	
	}

	@Override
	public void menuDeselected(MenuEvent e) {
	// TODO Auto-generated method stub
	
	}

	/**
	 * Ajout version 5.0 pour que, si une barre de menu horizontale est droule et qu'on slectionne
	 * un menu, la barre horizontale disparaisse.
	 * @param e
	 */
	@Override
	public void menuSelected(MenuEvent e) {
		if (frame != null) frame.fermeMenusHorizontaux();
	}
  
}