/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.Color;

import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;
import mathgraph32.MtgFrame;

public class CBissectrice extends CDroite {
	CPointAncetre  B;	
	CPointAncetre  A;
	CPointAncetre  C;
	// Variable rajoute pour la version 2.1.1 pour corriger un bug du au passage en haute rsolution
	// car la longueur du vecteur directeur est fixe (0.1 par dfaut) mais doit tre pass  1
	// quand on passe en haute rsolution
	private double longueurVecteurDirecteur;

  /** Creates new CBissectrice */
  public CBissectrice() {
    super(); // Pour que le vecteur soit cr
    longueurVecteurDirecteur = 0.1;
  }
  
  /** Constructeur pour clonage */
  public CBissectrice(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit, boolean masqueInit,
    String nomInit, int indiceTailleNomInit, StyleTrait styleInit, double abscisseNomInit, 
    CPointAncetre BInit, CPointAncetre AInit, CPointAncetre CInit) {
      super(liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
        dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit, styleInit, abscisseNomInit);
      B = BInit;
      A = AInit;
      C = CInit;
      longueurVecteurDirecteur = 0.1;      
  }
  /** Constructeur plus court pour objets visuels */
  public CBissectrice(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, StyleTrait styleInit, CPointAncetre BInit, CPointAncetre AInit, CPointAncetre CInit, int indiceTailleNomInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, false, styleInit, indiceTailleNomInit);
      B = BInit;
      A = AInit;
      C = CInit;
      longueurVecteurDirecteur = 0.1;      
  }
   
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(B);
    int ind2 = listeSource.indexOf(A);   
    int ind3 = listeSource.indexOf(C);   
    int ind4 = listeSource.indexOf(impProto);    
    CElementBase ptelb =  new CBissectrice(listeCible, (CImplementationProto)(listeCible.get(ind4, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom, indiceTailleNom,
      style, abscisseNom, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
      (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)),
      (CPointAncetre)(listeCible.get(ind3, CPointAncetre.class)));
    return ptelb;
    
  }
  
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
  	return nom() + " : " + getString("chinfo7")  + " " + getString("ch31") + " " + B.nom() + A.nom() + C.nom() ;
	}

  /**
   * Fonction ajoutant  list les lments qui ont gnr l'objet.
   * @param liste
   */
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(B);
    liste.add(A);
    liste.add(C);
  }  
  
  @Override
  public void adaptePourHauteResolution(double coef) {
  	super.adaptePourHauteResolution(coef);
		// longueurVecteurDirecteur = 1; // Bug corrig version 4.7.5
  	longueurVecteurDirecteur *= coef;
  }

 // Dans cette version de l'applet, je corrige le bug de la bissectrice
  // qui n'existait plus quand l'angle est plat mais je ne corrige pas le bug
  // qui peut concerner les points lis  une bissectrice (priorit  la rapidit)
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    VecteurDirecteur u = new VecteurDirecteur();
    VecteurDirecteur v = new VecteurDirecteur();
    VecteurDirecteur u1 = new VecteurDirecteur();
    VecteurDirecteur v1 = new VecteurDirecteur();

		existe = (A.existe) && (B.existe) && (C.existe)	&& (!A.egal(B)) && (!A.egal(C));
		if (!existe) return;
		point_x = A.x;
		point_y = A.y;
		u.setVecteur(A,B);
		v.setVecteur(A,C);
    // Correction d'un bug figurant jusque dans la version 2.5 du C++
		if (Kernel.zeroAngle(Math.abs(u.mesureAngleVecteurs(v)) - Math.PI))
		//if (Glob.zeroAngle(u.mesureAngleVecteurs(v) - Math.PI))
		{
			u.tourne(Math.PI/2, v);
			// Modification version java 2.2.1
			// v.vecteurColineaire(1, vecteur);
			v.vecteurColineaire(longueurVecteurDirecteur, vecteur);
		}
    else
    {
      // Attention : dans la version C++, toutes les coordonnes de points
      // taient multiplies par 10 et donc il faut adapter le vecteur directeur
      // en consquence (par exemple quand un point est li  une  bissectrice)
      // Ancienne version C++:
      //u.vecteurColineaire(1,u1);
      //v.vecteurColineaire(1,v1);
      // Version rectifie
      u.vecteurColineaire(1,u1);
      v.vecteurColineaire(1,v1);
      VecteurDirecteur w = new VecteurDirecteur();
      w.x = u1.x + v1.x;
      w.y = u1.y + v1.y;
      w.vecteurColineaire(longueurVecteurDirecteur, vecteur);
    }
		super.positionne(infoRandom, dimfen);
  }

  public boolean confonduAvec(CElementBase p) {
    CBissectrice pt;
    if (p.getClass() == this.getClass()) {
      pt = (CBissectrice) p;
      return ((B==pt.B) && (A==pt.A) && (C==pt.C))
              || ((B==pt.C) && (A==pt.A) && (C==pt.B));
    }
    else return false;
  }  
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || B.dependDe(p) || A.dependDe(p) || C.dependDe(p));
  }

  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || B.dependDePourReclassement(p)
    	|| A.dependDePourReclassement(p) || C.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || B.dependDePourBoucle(p) || A.dependDePourBoucle(p)
      || C.dependDePourBoucle(p));
  }

  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (A == ancienPoint) A = nouveauPoint;
    if (B == ancienPoint) B = nouveauPoint;
    if (C == ancienPoint) C = nouveauPoint;
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(A) || A.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(B) || B.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(C) || C.estDefiniParObjetDans(listeOb));
	}
    
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
     super.chargeDepuisFlux(inps, list);
     int ind1 = inps.readCInt();
     int ind2 = inps.readCInt();
     int ind3 = inps.readCInt();
     B = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
     A = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
     C = (CPointAncetre) (list.get(ind3,CPointAncetre.class));
  }
  /* Enregistrement dans un flux java */  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
     int ind1 = inps.readInt();
     int ind2 = inps.readInt();
     int ind3 = inps.readInt();
     B = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
     A = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
     C = (CPointAncetre) (list.get(ind3,CPointAncetre.class));
  }  
  /* Enregistrement dans un flux java */
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(B);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(A);
    oups.writeInt(ind2);
    int ind3 = list.indexOf(C);
    oups.writeInt(ind3);
  }
  /**
   * Fonction rajoute pour la version 2.1 et servant  savoir si un point
   * est par dfinition sur une droite. Renverra true par exemple
   * si la droite est dfinie par deux points et le point est un de ces deux points
   * ou si le point est li  la droite
   * A t rajout pour amliorer l'outil d'intersection
   * A redfinir pour les descendants quand ncessaire
   * */
  public boolean contientParDefinition(CPointAncetre po)
  {
    return (A == po);
  }
}
