/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;
import types.*;
import dialogs.CalculComplexeDlg;
import dialogs.MtgDlg;
import mathgraph32.MtgFrame;

/**
 *
 * @author Yves
 */
public class CCalculComplexe extends CCalcul {
  public Complexe resultatComplexe = new Complexe();
  
  /** Creates a new instance of CCalculComplexe */
  public CCalculComplexe() {
  }
  /* Constructeur pour outil de cration */
  public CCalculComplexe(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit) {
    super(liste, impProtoInit, estElementFinalInit);
  }
  // Constructeur pour clonage
  public CCalculComplexe(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      String nomCalculInit, String chaineCalculInit, CCalculBase calculInit) {
    super(liste, impProtoInit, estElementFinalInit, nomCalculInit, chaineCalculInit, calculInit);
  }
  
  @Override  
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(impProto);        
    CCalculBase calculClone = calcul.getClone(listeSource, listeCible);
    return new CCalculComplexe(listeCible, (CImplementationProto)(listeCible.get(ind1, CImplementationProto.class)),
      estElementFinal, nomCalcul, chaineCalcul, calculClone);
  }
  
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NCalculComplexe;
  }
  
  @Override
  public String infoPourHistorique() {
    String ch =  nom() + " : " + getString("ch101") + "\n" + nom() + " = " + chaineCalcul;
    if (existe) ch = ch + "\n" + getString("chinfo97") + " " + rendChaineValeurComplexe(12);
    else ch = ch + "\n" + getString("ch18");
    return ch;
  }
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
		return nom() + " = " + chaineCalcul + " (" + getString("ch101") + ")";
	}
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = calcul.existe(infoRandom);
    if (!existe) return;
    // A revoir en java pour la gestion des erreurs de calcul
    // AnnuleErreurDeCalcul();	// Pour dtecter une ventuelle erreur de
    //                         // calcul lors de l'appel
    try {
      listeProprietaire.initialiseNombreIterations();
      calcul.resultatComplexe(infoRandom, resultatComplexe);
    }
    catch (ErreurCalculException e) {
      existe = false;
      resultat = 0;
    }
    if (Double.isNaN(resultatComplexe.x) || Double.isInfinite(resultatComplexe.x)
      || Double.isNaN(resultatComplexe.y) || Double.isInfinite(resultatComplexe.y)) {
       existe = false;
    }
  }
 
  @Override
  public void rendValeurComplexe(Complexe zRes) {
    zRes.set(resultatComplexe);
  }
 
  // La fonction suivante ne sera redfinie que pour CCalculComplexe
  // Sert dans les commentaires dynamiques
  // Modification version 5.0.3 : Sera aussi redfinie pour CMesureAffixe qui possde aussi un nom depuis la version 5
  public String rendChaineValeurPourCommentaire(int nbDecimales)
  {
    return rendChaineValeurComplexe(nbDecimales);
  }
  
  @Override
  public boolean modifiableParMenu() {
  	// Modifi version 3.5
  	// return true;
  	return !estElementFinal;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new CalculComplexeDlg(frame, this, true);
  }
  
   // Inutile de redfinir informationModification() pour les objets de type calcul
  
}