/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;

import java.awt.Color;

import types.*;

import dialogs.MtgDlg;
import dialogs.RayonDlg;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

import mathgraph32.MtgFrame;

/**
 * Clasee modifie pour la version 4.6.3 : On rajoute un champ spcifiant si le rayon est donn
 * en unit de longueur (par dfaut) ou en pixels (nouveaut))
 * @author Yves
 *
 */
public class CCercleOR extends CCercle {
  public boolean inPixels; // Ajout version 4.6.3. Sera false par dfaut. Si true, le rayon sera en pixels.
  private CPointAncetre O;
  public CValeurAssocieeAVariable R; // public pour modification par un dialogue
  // Ajout version 4.7 : un coefficient multiplicateur utili en haute rsolution quand le rayon est en pixels
  private double coefMult;
  
  /** Creates new CCercleOR */
  public CCercleOR() {
  	coefMult = 1;
  }
  
  // Pour la version 4.6.3, introduction d'un champ qui, s'il est  true, indique que le rayon est en pixels.
  // d'o changement de version
  @Override
  public int numeroVersion() {
  	return 2;
  }  

  // Constructeur pour clonage
  public CCercleOR(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, StyleTrait styleInit,
    CPointAncetre OInit, CValeurAssocieeAVariable RInit, boolean inPixelsInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit);
    O = OInit;
    R = RInit;
    inPixels = inPixelsInit;
    coefMult = 1;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(O);
    int ind2 = listeSource.indexOf(impProto);
    CValeurAssocieeAVariable RClone = (CValeurAssocieeAVariable) R.getClone(listeSource, listeCible);
    CElementBase ptelb = new CCercleOR(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, masque, style, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)), RClone, inPixels);
    return ptelb;
  }
  
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
  	return nom() + " : " + getString("chinfo195") + " " + O.nom() + " " +
  			getString("chinfo197") + " " + R.chaineInfo();
	}

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(O);
  } 
  
  // Ajout version 4.8
  @Override
  public void initialisePourDependance() {
  	super.initialisePourDependance();
  	R.initialisePourDependance();
  }

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || O.dependDe(p) || R.dependDe(p) || listeProprietaire.pointeurLongueurUnite.dependDe(p));
  }
  
  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || O.dependDePourReclassement(p)
    	|| R.dependDePourReclassement(p) || listeProprietaire.pointeurLongueurUnite.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
   return ((p == this) || O.dependDePourBoucle(p) || R.dependDePourBoucle(p))
    	|| listeProprietaire.pointeurLongueurUnite.dependDePourBoucle(p);
  }
  
  @Override
  public void adaptePourHauteResolution(double coef) {
  	// TODO Auto-generated method stub
  	if (inPixels) coefMult = coef;
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	R.positionne(infoRandom, dimfen);
  	CLongueur plongUnite = listeProprietaire.pointeurLongueurUnite;
  	// Modification version 4.6.3
    // existe = O.existe && R.existe() && plongUnite.existe;
  	existe = O.existe && R.existe();
  	if (!inPixels) existe = existe && plongUnite.existe;
    if (existe) {
      centre_x = O.x;
      centre_y = O.y;
      // Modification version 4.6.2
      // rayon = R.rendValeur(infoRandom)*listeProprietaire.pointeurLongueurUnite.rendLongueur();
      if (inPixels) rayon = R.rendValeur()*coefMult;
      else rayon = R.rendValeur()*listeProprietaire.pointeurLongueurUnite.rendLongueur();
      super.positionne(infoRandom, dimfen);
    }
  }
  
  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne.
   **/
  @Override
  public boolean confonduAvec(CElementBase p) {
    CCercleOR pt;
    if (p.getClass() == this.getClass()) {
      pt = (CCercleOR) p;
      return (O == pt.O) && R.confonduAvec(pt.R);
    } else return false;
  }

  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (O == ancienPoint) O = nouveauPoint;
  }

  @Override
  public boolean modifiableParMenu() {
  	// Modifi version 3.5
  	// return true;
  	return !estElementFinal;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new RayonDlg(frame, this, true);
  }
  
   @Override
  public String informationModification() {
  	return getString("chaineOutil57");
  }

  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(O) || O.estDefiniParObjetDans(listeOb))
		&&
		(R.estDefiniParObjetDans(listeOb));
	}
  
  // Modifi version 4.6.3.
  // Un cercle dont le rayon est donn en pixels n'utilise pas la longueur unit
  @Override
  public boolean utiliseLongueurUnite() {
  	// return true;
  	return !inPixels;
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    O = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    R = new CValeurAssocieeAVariable();
    R.chargeDepuisFlux(inps, list);
    inPixels = false;
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    O = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    R = new CValeurAssocieeAVariable();
    R.read(inps, list);
    // Ajout version 4.6.3
    if(nVersion >= 2) inPixels = inps.readBoolean();
    else inPixels = false;
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(O);
    oups.writeInt(ind1);
    R.write(oups, list);
    // Ajout version 4.6.3
    oups.writeBoolean(inPixels);
  }
  
  @Override
  public boolean contientParDefinition(CPointAncetre po) {
  	if (O == po) return true;
  	if (po.estPointImage()) {
  		CPointImage ptim = (CPointImage) po;
  		if (ptim.transformation.natureTransformation() == CTransformation.rotation) {
  			CRotation rot = (CRotation) ptim.transformation;
  			return ptim.antecedent.appartientParDefinition(this) && (rot.centre == O);
  		}
  		else return false;
  	}
  	else return false;
  }
  
}
