/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;


import java.io.*;

import mathgraph32.MtgFrame;
import dialogs.DeriveeDlg;
import dialogs.MtgDlg;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

/**
 *
 * @author Yves
 */
public class CDerivee extends CFonc {
  public CFonc fonctionAssociee;
  
  /** Creates a new instance of CDerivee */
  public CDerivee() {
    chaineCalcul = "";
    chaineVariable = "";
    calcul = null;
  }
  
  /**
   * Constructeur
   */
  public CDerivee(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      String nomInit, CFonc fonctionAssocieeInit) {
  	super(liste, impProtoInit, estElementFinalInit, nomInit, "", null, "");
  	fonctionAssociee = fonctionAssocieeInit;
  	creeCalcul();
  }
  
  /**
   * Constructeur pour clonage
   */
  public CDerivee(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      String nomInit, CCalculBase calculInit, CFonc fonctionAssocieeInit) {
  	super(liste, impProtoInit, estElementFinalInit, nomInit, "", calculInit, "");
  	fonctionAssociee = fonctionAssocieeInit;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(impProto);
    // Modifi version 4.2
    // CCalculBase calculClone = calcul.getClone(listeSource, listeCible);
    int ind2 = listeSource.indexOf(fonctionAssociee);
    // CElementBase ptelb = new CDerivee(listeCible, (CImplementationProto)(listeCible.get(ind1, CImplementationProto.class)),
    //    estElementFinal, nomCalcul, calculClone, (CFonc)(listeCible.get(ind2, CFonc.class)));
    CDerivee ptelb = new CDerivee(listeCible, (CImplementationProto)(listeCible.get(ind1, CImplementationProto.class)),
      estElementFinal, nomCalcul, null, (CFonc)(listeCible.get(ind2, CFonc.class)));
    if (listeCible.getClass() != CPrototype.class) ptelb.creeCalcul();
    return ptelb;    
  }

  // Ajout version 4.6. Pas de chaine de calcul pour une drive
  @Override
  public void reconstruitChaineCalcul() {
  	chaineCalcul = "";	
  }

  @Override
  public void metAJour() {
		if (! fonctionAssociee.deriveePossible(0)) 
			calcul = new COperation(listeProprietaire, new CConstante(listeProprietaire, 1),
					new CConstante(listeProprietaire, 0), Operateur.Divi);
		else creeCalcul();  	
  }
  
  /**
   * Constructeur spcial dans le cas d'une implmentation de prototype car le calcul
   * de la drive doit tre refait
   */
  /* Supprim version 4.2
  public final CDerivee getClonePourImplementationProto(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(impProto);        
    int ind2 = listeSource.indexOf(fonctionAssociee);
    CDerivee ptelb = new CDerivee(listeCible, (CImplementationProto)(listeCible.get(ind1)),
      estElementFinal, nomCalcul, null, (CFonc)(listeCible.get(ind2)));
    return ptelb;      	
  }
  */

  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NDerivee;
  }
  

  @Override
  public String infoPourHistorique() {
    return info();
  }
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
  	return nom() + " : " + getString("ch112") + " " + fonctionAssociee.nom();
	}
    
  /**
   * Fonction renvoyant une chaine du type f(t) = .... pour certain champs de botes de dialogues
   * @return
   */
  @Override
  public String infoFormule() {
  	return nom() + " : " + getString("ch112") + " " + fonctionAssociee.nom();
  }
  
  @Override
  public boolean modifiableParMenu() {
  	// Modifi version 3.5
  	// return true;
  	return !estElementFinal;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new DeriveeDlg(frame, this, true);
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || fonctionAssociee.dependDe(p));
  }

  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || fonctionAssociee.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || fonctionAssociee.dependDePourBoucle(p);
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = fonctionAssociee.existe;
  }
  
  public void creeCalcul()
  {
    calcul = CCalculBase.creeDerivee(fonctionAssociee.calcul, 0);
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return listeOb.contains(fonctionAssociee) || fonctionAssociee.estDefiniParObjetDans(listeOb);
	}      
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFluxSansCalcul(inps,list);
    int ind1 = inps.readCInt();
    fonctionAssociee = (CFonc) (list.get(ind1,CFonc.class));
    // Modification pour la version 2.5
    // creeCalcul();
    if (list.getClass() != CPrototype.class) creeCalcul();
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.readSansCalcul(inps, list);
    int ind1 = inps.readInt();
    fonctionAssociee = (CFonc) (list.get(ind1,CFonc.class));
    if (list.getClass() != CPrototype.class) {
			if (! fonctionAssociee.deriveePossible(0)) 
				calcul = new COperation(listeProprietaire, new CConstante(listeProprietaire, 1),
						new CConstante(listeProprietaire, 0), Operateur.Divi);
			else creeCalcul();
		}
  }

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.writeSansCalcul(oups, list);
    int ind1 = list.indexOf(fonctionAssociee);
    oups.writeInt(ind1);
  }
  
  // Ajout version 5.1.2 pour pouvoir calculer le test de dpendance d'une fonction drive par rapport  sa variable
  // Sera redfini pour CDerivee
  /**
   * Fonction renvoyant la chane rersentant la variable de la fonction
   * @return
   */
  @Override
  public String chaineVariable() {
  	return fonctionAssociee.chaineVariable();
  }

}