/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.Color;

import types.DimensionFenetre;
import types.StyleTrait;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.Kernel;

public abstract class CElementLigne extends CElementGraphique {
  public StyleTrait style; // Le style de trait
  
	/** Creates new CElementLigne */
  public CElementLigne() {
  }
  // Constructeur pour clonage
  public CElementLigne(CListeObjets liste,CImplementationProto impProtoInit,boolean estElementFinalInit,
    Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit,
    boolean masqueInit, String nomInit, int indiceTailleNomInit,
    StyleTrait styleInit) {
      super (liste,impProtoInit,estElementFinalInit,couleurInit, nomMasqueInit,
        dec_xInit, dec_yInit, masqueInit,nomInit, indiceTailleNomInit);
      style = styleInit;
  }
  // Rajout pour la version 1.8 : un constructeur qui sera rserv aux lments de type
  // ligne qui ne peuvent pas tre nomms, cad tous sauf les droites
  public CElementLigne(CListeObjets liste,CImplementationProto impProtoInit,boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, StyleTrait styleInit) {
      super (liste, impProtoInit,estElementFinalInit,couleurInit, false, 0, 0, masqueInit,"", 0);
      style = styleInit;
  }
  
  /**
   * Constructeur pour les lments ayant besoin d'tre nomms avec un nom vide;
   */
   public CElementLigne(CListeObjets liste,CImplementationProto impProtoInit,boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, StyleTrait styleInit, int indiceTailleNomInit) {
      super (liste, impProtoInit,estElementFinalInit,couleurInit, false, 0, 0, masqueInit,"", indiceTailleNomInit);
      style = styleInit;
  }
 
  /**
   * Fonctin renvoyant une chane prcisant quelle est la anture de l'objet auquel
   * est li un  point. Sert pour les points lis  des objets pour dire
   * quel est le type de l'objet auquel le point est li.
   * A redfinir pour les objets auxquels peut tre li un point
   * @return
   */
  public String complementInfo() {
    return "";
  }
  
  public void donneStyle(StyleTrait nouveauStyle)
  {
     style = nouveauStyle;
  }

  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    // style = inps.readByte(); Modifi version 5.0
    style = new StyleTrait();
    style.readOld(inps, true);
  }
  
  // Version Java :  cause d'une erreur dasn la version C++, le type de repre qui hrite de CElementLigne
  // appelait la srialization de CElementGraphique.
  // D'o l'utilit de la mthode suivante.
  final void chargeDepuisFluxSansStyleTrait(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    /* Modifi version 5.0
    style = inps.readByte();
    */
    style = new StyleTrait();
    if (list.numeroVersion < 15) style.readOld(inps, true);
    else style.read(inps);
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    // Modifi version 5.0
    // oups.writeByte(style);
    oups.writeByte(style.style);
    oups.writeByte(style.epaisseur );
    
  }
  
  /**
   * Ajout version 4.8.1
   * Modifi version 5.0
   * @param dimf
   * @return
   */
  public final String tikzLineStyle(DimensionFenetre dimf, double coefmult) {
  	double ep; // Epaisseur
  	// Si on rduit la figure on rduit l'paisseur de trait mais si on l'agrandit on garde la mme
  	double coef = (coefmult > 1) ? 1 : coefmult;
  	// Modification version 5.0 : Sur un cran haute rsolution les traits sont trop pais.
  	// coef /= MtgFrame.coefMultForPointsAndMarks(dimf.longueurAbscisses, dimf.longueurOrdonnees);
  	switch (style.style) {
			case StyleTrait.styleTraitContinu:
				ep = 0.3*coef*style.epaisseur;
				return "[color=" + tikzCouleur() + " , " + "line width = " + Kernel.chaineNombre(ep, 2) + "]";
			case StyleTrait.styleTraitPointille:
				ep = 0.3*coef*style.epaisseur;
				return "[color=" + tikzCouleur() + " , dotted, line width = "  + Kernel.chaineNombre(ep, 2) + "]";
			case StyleTrait.styleTraitTrait:
				ep = 0.3*coef*style.epaisseur;
				return "[color=" + tikzCouleur() + " , dashed, line width = " + Kernel.chaineNombre(ep, 2) + "]";
			case StyleTrait.styleTraitPointPoint:
				ep = 0.3*coef*style.epaisseur;
				return "[color=" + tikzCouleur() + " , " +
						"dash pattern=on 3pt off 1pt on 1pt off 1pt on 1pt off 1pt, line width = " + Kernel.chaineNombre(ep, 2) + "]";
			case StyleTrait.styleTraitPointPointPoint:
				ep = 0.3*coef*style.epaisseur;
				return "[color=" + tikzCouleur() + " , " +
						"dash pattern=on 3pt off 1pt on 1pt off 1pt on 1pt off 1pt on 1pt off 1pt, line width = " + Kernel.chaineNombre(ep, 2) + "]";
			case StyleTrait.styleTraitTraitLongTrait:
				ep = 0.3*coef*style.epaisseur;
				return "[color=" + tikzCouleur() + " , " +
						"dash pattern=on 3pt off 0.9pt on 1.5pt off 0.9pt, line width = " + Kernel.chaineNombre(ep, 2) + "]";
			default:
				return "";
		}
  }
  /**
   * Ajout version 5.0 pour le trac des flches de vecteurs.
   * Mme si le style de trc n'est pas continu on trace les flches avec un style de trait continu
   * Renvoie un style de trait continu avec la mme paisseur
   * @param dimf
   * @param coefmult
   * @return
   */
  public final String tikzLineStyleContinu(DimensionFenetre dimf, double coefmult) {
  	double ep; // Epaisseur
  	// Si on rduit la figure on rduit l'paisseur de trait mais si on l'agrandit on garde la mme
  	double coef = (coefmult > 1) ? 1 : coefmult;
		ep = 0.3*coef*style.epaisseur;
		return "[color=" + tikzCouleur() + " , " + "line width = " + Kernel.chaineNombre(ep, 2) + "]";
  }
  
  /**
   * Ajout version 4.8.1
   * @param dimf
   * @return
   */
  public final String tikzdrawFillStyle(DimensionFenetre dimf, double coefmult) {
  	double ep; // Epaisseur
  	switch (style.style) {
			case StyleTrait.styleTraitContinu:
					ep = 0.3*coefmult*style.epaisseur;
					return "[draw=" + tikzCouleur() + " , " + "fill=" + tikzCouleur() + ",line width = " + Kernel.chaineNombre(ep, 3) + "]";
			case StyleTrait.styleTraitPointille:
				ep = 0.3*coefmult*style.epaisseur;
				return "[draw=" + tikzCouleur() + " , " + "fill=" + tikzCouleur() + ", dotted, " + "line width = " + Kernel.chaineNombre(ep, 3) + "]";
			case StyleTrait.styleTraitTrait:
				ep = 0.3*coefmult*style.epaisseur;
				return "[draw=" + tikzCouleur() + " , " + "fill=" + tikzCouleur()+ ", dashed, " + "line width = " + Kernel.chaineNombre(ep, 3) + "]";
			case StyleTrait.styleTraitPointPoint:
				ep = 0.3*coefmult*style.epaisseur;
				return "[draw=" + tikzCouleur() + " , " + "fill=" + tikzCouleur() + ", dash pattern=on 3pt off 1pt on 1pt off 1pt on 1pt off 1pt, " +
						"line width = " + Kernel.chaineNombre(ep, 3) + "]";
			case StyleTrait.styleTraitTraitLongTrait:
				ep = 0.3*coefmult*style.epaisseur;
				return "[draw=" + tikzCouleur() + " , " + "fill=" + tikzCouleur() + ", dash pattern=on 9pt off 3pt on 5pt off 3pt, " +
						"line width = " + Kernel.chaineNombre(ep, 3) + "]";
			default:
				return "";
		}
  }
}
