/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package objets;

import mathgraph32.MtgFrame;
import dialogs.FonctionComplexe2VarDlg;
import dialogs.FonctionComplexe3VarDlg;
import dialogs.MtgDlg;
import types.NatureCalcul;

public class CFoncComplexeNVar extends CFoncNVar {

	/** Constructeur */
	public CFoncComplexeNVar() {
	}
	
	/** Constructeur pour outil de cration */
	public CFoncComplexeNVar(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
			int nbVarInit) {
	  super(liste, impProtoInit, estElementFinalInit, nbVarInit);
	}

	// Constructeur pour clonage
	public CFoncComplexeNVar(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
	    String nomInit,String chaineCalculInit, CCalculBase calculInit, String[] chaineVariableInit) {
	  super(liste, impProtoInit, estElementFinalInit, nomInit, chaineCalculInit, calculInit, chaineVariableInit);
	}

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(impProto);        
    CCalculBase calculClone = calcul.getClone(listeSource, listeCible);
    CElementBase ptelb = new CFoncComplexeNVar(listeCible, (CImplementationProto)(listeCible.get(ind1, CImplementationProto.class)),
      estElementFinal, nomCalcul, chaineCalcul, calculClone, chaineVariable);
    return ptelb;    
  }
	
  @Override
  public long getNatureCalcul() {
  	switch (nbVar) {
		case 2 :
			return NatureCalcul.NFonctionComplexe2Var;
		case 3 :
			return NatureCalcul.NFonctionComplexe3Var;
		default:
			return NatureCalcul.NFonctionComplexe2Var;
		}
  }

  /* Version 4.9.7 : Transfr dans CFoncNVar
  public void rendValeurFonctionComplexe(boolean infoRandom, Complexe[] parametre, Complexe zRes)
  throws ErreurCalculException {
    calcul.resultatFonctionComplexe(infoRandom, parametre, zRes);
  }
  */
  
  @Override
  public String infoPourHistorique() {
    String ch = nom() + " : " + getString("ch102") + " " + getString("ch87") + "\n" + nom() 
  	+ "(";
	  for (int i = 0; i < nbVar; i++) {
	  	ch = ch + chaineVariable[i];
	  	if (i != nbVar - 1) ch = ch + ',';
	  }
	  ch = ch + ")" + " = " + chaineCalcul;
	  if (!existe) ch = ch + "\n" + getString("ch18");
	  return ch;
  }
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
  	String ch = nom() + "(";
	  for (int i = 0; i < nbVar; i++) {
	  	ch = ch + chaineVariable[i];
	  	if (i != nbVar - 1) ch = ch + ',';
	  }
	  ch += ")" + " = " + chaineCalcul;
	  ch += " (" + getString("ch102") + ")";
    return ch;
	}

  // A revoir version 3.0
  @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	switch (nbVar) {
  	case 2 : return new FonctionComplexe2VarDlg(frame, this, true);
  	case 3 : return new FonctionComplexe3VarDlg(frame, this, true);
  	default : return null;
  	}
  }  
}
