/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import glob.*;

import java.awt.*;
import types.*;

public class CGrandArcDeCercle extends CArcDeCercle {
  
  /** Creates new CGrandArcDeCercle */
  public CGrandArcDeCercle() {
  }
  
  // Constructeur pour clonage
  public CGrandArcDeCercle(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, StyleTrait styleInit,
    CPointAncetre OInit, CPointAncetre AInit, CPointAncetre pointFinArcInit,
    CValeurAngle angleAuCentreInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit, OInit, AInit,
      pointFinArcInit, angleAuCentreInit);
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(O);
    int ind2 = listeSource.indexOf(A);
    CPointAncetre ptFinArc = null;
    CValeurAngle angleAuCentreClone = null;
    if (pointFinArc == null)
      angleAuCentreClone = (CValeurAngle) angleAuCentre.getClone(listeSource, listeCible);
    else {
      int ind3 = listeSource.indexOf(pointFinArc);
      ptFinArc = (CPointAncetre)(listeCible.get(ind3, CPointAncetre.class));
    }
    int ind4 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CGrandArcDeCercle(listeCible, (CImplementationProto)(listeCible.get(ind4, CImplementationProto.class)),
      estElementFinal, couleur, masque, style, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
      (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)), ptFinArc, angleAuCentreClone);
    return ptelb;
  }
  
  @Override
  public String infoPourHistorique() {
  	String ch = nom() + " : " + getString("chinfo201");
  	ch += "\n" + getString("chinfo202") + " "+ O.nom() + " " + getString("chinfo203") + " " + A.nom();
  	if (pointFinArc != null) ch += " " + getString("chinfo204") + " " + pointFinArc.nom();
  	else ch += getString("chinfo205") + " " + angleAuCentre.chaineInfo();
  	return ch;
  }
	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
  	String ch = nom() + " : " + getString("chinfo201");
  	ch += " " + getString("chinfo202") + " "+ O.nom() + " " + getString("chinfo203") + " " + A.nom();
  	if (pointFinArc != null) ch += " " + getString("chinfo204") + " " + pointFinArc.nom();
  	else ch += getString("chinfo205") + " " + angleAuCentre.chaineInfo();
  	return ch;
	}    
  
  // Modifi pour la version 1.9.5 car surArc ne doit plus utiliser de pointeurs
  // sur les objets sources
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
  	StyleTrait styleTrait;
    if (horsFenetre) return;
    if (masque) styleTrait = StyleTrait.traitPointille;
    else styleTrait = style;
    // Rajout en java par rapport  la version C++
    if (Kernel.testToile(centre_x, centre_y) && Kernel.testToile(origine_x, origine_y)) {
      pDC.setStroke(Pinceau.CreePinceau(styleTrait, listeProprietaire));
      pDC.setColor(couleur);
      Kernel.secteurRentrant(pDC, origine_x, origine_y, centre_x, centre_y, rayon, valeurAngleAuCentre);
    }
  }
  
  public boolean surArc(double xt, double yt) {
    double ang,ang1aux,ang2aux,dif;
    
    VecteurDirecteur w = new VecteurDirecteur(centre_x, centre_y, xt, yt);
    ang = w.angleRad();
    ang1aux = ang1;
    ang2aux = ang2;
    // On ordonne ang1Aux et ang2Aux dans l'ordre croissant
    if (ang1aux > ang2aux) {
      double d = ang2aux;
      ang2aux = ang1aux;
      ang1aux = d;
    }
    dif = ang2aux - ang1aux;
    if (Kernel.zeroAngle(dif - Math.PI)) {
      if (ang1 >= ang2) {
        return ((ang >= ang2) && (ang <= ang1));
      } else {
        return ((ang >= ang2) || (ang <= ang1));
      }
    } else {
      if (dif < Math.PI) {
        return ((ang <= ang1aux) || (ang >= ang2aux));
      } else {
        return ((ang >= ang1aux) && (ang <= ang2aux));
      }
    }
  }
  
  public double abscisseCurviligne(double ang) {
    double difang;
    
    if (valeurAngleAuCentre == 0) return 0;
    if (valeurAngleAuCentre >= 0) {
      if (ang <= ang1)
        difang = ang1 - ang;
      else difang = 2*Math.PI - ang + ang1;
    } else {
      if (ang >= ang1) difang = ang - ang1;
      else difang = 2*Math.PI - ang1 + ang;
    }
    return difang/(2*Math.PI-Math.abs(valeurAngleAuCentre));
  }
  
  
  // Mthode dfinie pour la version 1.8
  public int natureArc() {
    return NatureArc.NatureArcGrandArc;
  }
  
	/**
	 * Ajout version 4.8.1
	 */
	@Override
	public String tikz(DimensionFenetre dimf, boolean nomaff, double coefmult, boolean bu) {
		double x1 = listeProprietaire.tikzXcoord(A.x, dimf, bu);
		double y1 = listeProprietaire.tikzYcoord(A.y, dimf, bu);
		double ray = listeProprietaire.tikzLongueur(rayon, dimf, bu);
    VecteurDirecteur u = new VecteurDirecteur(centre_x, centre_y, origine_x, origine_y);
    ang1 = u.angleRad();
    ang2 = ang1 + valeurAngleAuCentre;
    if (ang2 < 0) ang2 = ang2 + 2*Math.PI;
    if (ang2 >= 2*Math.PI) ang2 = ang2 - 2*Math.PI;
    double dif = Kernel.mesurePrincipale(ang2-ang1);
    if (dif > 0) {
    	if (ang2 > ang1) ang2 = ang2 - 2*Math.PI;
    }
    else {
    	if (ang2 < ang1) ang2 = ang2 + 2*Math.PI;
    }
    /* Modifi version 5.1.1
    ang1 = ang1*Kernel.ConvRadDeg;
    ang2 = ang2*Kernel.ConvRadDeg;
    */
    ang1 = Math.toDegrees(ang1);
    ang2 = Math.toDegrees(ang2);
		return "\\draw " + tikzLineStyle(dimf, coefmult) + "(" + Kernel.chaineNombre(x1, 3) +  "," + Kernel.chaineNombre(y1, 3) +") arc " +
		"(" + Kernel.chaineNombre(ang1, 3) + " : " + Kernel.chaineNombre(ang2, 3) + " : " + Kernel.chaineNombre(ray, 3) + ");";
	}

  
}
