/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.*;


import types.*;

import dialogs.LieuDlg;
import dialogs.MtgDlg;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.Kernel;

import mathgraph32.MtgFrame;

// Modification pour la version 1.9.5
// A la suite du rajout des lieux d'objets cette classe n'est plus abstract
public class CLieuDeBase extends CElementLigne {
	// Ajout version 4.9.9 pour gnrer un nom
	public static int indice = 0;
	//
  public CPointAncetre pointATracer;
  public InfoLieu infoLieu;
  CSousListeObjets listeElementsAncetres;
  int[] abscissesCoordonneesPoints;
  int[] ordonneesCoordonneesPoints;
  // Rajout version 4.8.1 pour exportation tikz
  double[] xCoord;
  double[] yCoord;
  //
  int nombreLignes;
  int nombreLignesDePlusieursPoints;
  InfoLigneLieu[] infoLignes;
  int indicePremiereLigne;
  // Le java ne permet pas de tracer une ligne  partir d'un offset dans un tableau d'int.
  // Il faut donc maintenir par rapport au C++ deux tableaux supplmentaires pour contenir les
  // coordonnes de chaque ligne constituant le lieu.
  int[] abscissesPourTrace;
  int[] ordonneesPourTrace;
  
  /** Creates new CLieuDeBase */
  public CLieuDeBase() {
  }
  
  /** Constructeur pour clonage */
  public CLieuDeBase(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, StyleTrait styleInit, CPointAncetre pointATracerInit, InfoLieu infoLieuInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit);
    pointATracer = pointATracerInit;
    infoLieu = infoLieuInit;
    // Il faut crer les tableaux de coordonnes
    // On rserve une coordonne supplmentaire pour le cas d'un lieu ferm
    abscissesCoordonneesPoints = new int[infoLieu.nombreDePoints + 1];
    ordonneesCoordonneesPoints = new int[infoLieu.nombreDePoints + 1];
    abscissesPourTrace = new int[infoLieu.nombreDePoints + 1];
    ordonneesPourTrace = new int[infoLieu.nombreDePoints + 1];
    int nb = infoLieu.nombreDePoints/2+2;
    infoLignes = new InfoLigneLieu[nb];
    // Il faut initialiser chaque lment du tableau
    for (int i = 0; i < nb; i++) {
      infoLignes[i] = new InfoLigneLieu();
    }
    // Il faut crer la liste des lments dont dpend le point  tracer
    // Cette liste sera finie d'tre valide dans setClone()
    listeElementsAncetres = new CSousListeObjets();
  }
  
  /**
   * Fonction appele lorsque le nombre de points du lieu a chang
   */
  public void changeCaracteristiques() {
    abscissesCoordonneesPoints = new int[infoLieu.nombreDePoints + 1];
    ordonneesCoordonneesPoints = new int[infoLieu.nombreDePoints + 1];
    abscissesPourTrace = new int[infoLieu.nombreDePoints + 1];
    ordonneesPourTrace = new int[infoLieu.nombreDePoints + 1];
    int nb = infoLieu.nombreDePoints/2+2;
    infoLignes = new InfoLigneLieu[nb];
    // Il faut initialiser chaque lment du tableau
    for (int i = 0; i < nb; i++) {
      infoLignes[i] = new InfoLigneLieu();
    }  	
  }

  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    nombreLignes = ((CLieuDeBase) ptel).nombreLignes;
    infoLieu.setCopy(((CLieuDeBase)ptel).infoLieu); // Il faut copier toutes les coordonnes
    for (int j = 0; j < nombreLignes; j++) {
      infoLignes[j].setCopy( ((CLieuDeBase)ptel).infoLignes[j]);
      for (int k = 0; k < infoLignes[j].nombrePoints; k++) {
        abscissesCoordonneesPoints[infoLignes[j].indicePremierPoint+k] =
          ((CLieuDeBase)ptel).abscissesCoordonneesPoints[infoLignes[j].indicePremierPoint+k];
        ordonneesCoordonneesPoints[infoLignes[j].indicePremierPoint+k] =
          ((CLieuDeBase)ptel).ordonneesCoordonneesPoints[infoLignes[j].indicePremierPoint+k];
      }
    }
  }

  @Override
  public void setCloneTikz(CElementBase ptel) {
    super.setCloneTikz(ptel);
    xCoord = new double [infoLieu.nombreDePoints + 1];
    yCoord = new double [infoLieu.nombreDePoints + 1];    
    nombreLignes = ((CLieuDeBase) ptel).nombreLignes;
    infoLieu.setCopy(((CLieuDeBase)ptel).infoLieu); // Il faut copier toutes les coordonnes
    for (int j = 0; j < nombreLignes; j++) {
      infoLignes[j].setCopy( ((CLieuDeBase)ptel).infoLignes[j]);
      for (int k = 0; k < infoLignes[j].nombrePoints; k++) {
        xCoord[infoLignes[j].indicePremierPoint+k] =
          ((CLieuDeBase)ptel).xCoord[infoLignes[j].indicePremierPoint+k];
        yCoord[infoLignes[j].indicePremierPoint+k] =
          ((CLieuDeBase)ptel).yCoord[infoLignes[j].indicePremierPoint+k];
      }
    }
  }
  
  @Override
  public String chaineDesignation() {
    return getString("designationLieuPoint");
  }
  
  // A revoir version 4.7.6
  @Override
  /* Modifi version 4.9.9
  public boolean dependDe(CElementBase p) {
    return super.dependDe(p) || pointATracer.dependDe(p);
  }
  */
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || pointATracer.dependDe(p));
  }
 
  @Override
  /* Modifi version 4.9.9
  public boolean dependDePourReclassement(CElementBase p) {
    return super.dependDePourReclassement(p) || pointATracer.dependDePourReclassement(p);
  }
  */
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || pointATracer.dependDePourReclassement(p));
  }

  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || pointATracer.dependDePourBoucle(p));
  }
  
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
     
  	StyleTrait styleTrait;
    if (masque) styleTrait = StyleTrait.traitPointille;
    else styleTrait = style;
    pDC.setStroke(Pinceau.CreePinceau(styleTrait, listeProprietaire));
    pDC.setColor(couleur);
    for (int j = 0; j < nombreLignes; j++) {
      int nombrePoints = infoLignes[j].nombrePoints;
      // Optimis version 3.9 On trace maintenant les points isols du lieu
      int indPremPoint = infoLignes[j].indicePremierPoint;
      if (nombrePoints == 1) {
      	pDC.fillRect(abscissesCoordonneesPoints[indPremPoint], ordonneesCoordonneesPoints[indPremPoint], 1, 1);
      }
      else {
        // Il faut transfrer les coordonnes de chaque ligne pour pouvoir la tracer en java
        for (int k = 0; k < nombrePoints; k++) {
        	int k1 = indPremPoint + k;
          abscissesPourTrace[k] = abscissesCoordonneesPoints[k1];
          ordonneesPourTrace[k] = ordonneesCoordonneesPoints[k1];
        }
        pDC.drawPolyline(abscissesPourTrace, ordonneesPourTrace, nombrePoints);
      }
    }
  }
  
  public boolean contientUneSeuleLigne() {
    return (nombreLignesDePlusieursPoints == 1);
  }
  
  public boolean contientAuMoinsUneLigne() {
    return (nombreLignesDePlusieursPoints >= 1);
  }
  
  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (pointATracer== ancienPoint) pointATracer = nouveauPoint;
  }

  @Override
  public long getNature() {
    return NatureObjet.NLieu;
  }

  @Override
  // Version 3.5 : les lieux qui sont des objets finaux restent modifiables
  public boolean modifiableParMenu() {
  	return true;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new LieuDlg(frame, this);
  }
  
  @Override
  public String informationModification() {
  	return getString("chaineOutil59");
  }
  
  @Override
  public double distancePoint(double xp, double yp, boolean masquage) {
  	 if (!(existe) || (masquage && masque)) return -1;
  	 else return distancePointPourSurface(xp,yp,masquage);
  }
  
  @Override
  public double distancePointPourSurface(double xp,double yp, boolean masquage) {
  	double d;
  	int x1,y1,x2,y2,x1ord,y1ord,x2ord,y2ord; //coordonnes d'un segment de la courbe
  	VecteurDirecteur vec = new VecteurDirecteur();

  	if (!existe) return -1;
		else {
			// On regarde pour chaque ligne dont le lieu est form
			for (int j = 0; j < nombreLignes; j++)
			{
				for (int k = infoLignes[j].indicePremierPoint; k <= infoLignes[j].indicePremierPoint + 
						infoLignes[j].nombrePoints-2;k++)
				{
					x1 = abscissesCoordonneesPoints[k];
					y1 = ordonneesCoordonneesPoints[k];
					x2 = abscissesCoordonneesPoints[k+1];
					y2 = ordonneesCoordonneesPoints[k+1];
					if (x1 > x2) {
						x1ord = x2;
						x2ord = x1;
					}
					else
					{
						x1ord = x1;
						x2ord = x2;
					}
					if (y1 > y2) {
						y1ord = y2;
						y2ord = y1;
					}
					else {
						y1ord = y1;
						y2ord = y2;
					}
					d = Kernel.distancePointPoint(xp,yp,x1,y1);
					// Distance d'abord aux points calculs
					if (d < listeProprietaire.distanceMiniPourEtreProche) return d;
					// puis aux segments
					if ((xp>=x1ord) && (xp<=x2ord) && (yp>=y1ord) && (yp<=y2ord))
					{
						vec.x = x2 - x1;
						vec.y = y2 - y1;
						if ((vec.x != 0) || (vec.y != 0))
						{
							d = Kernel.distancePointDroite(xp,yp,x1,y1,vec);
							if (d < listeProprietaire.distanceMiniPourEtreProche) return d;
						}
					}
				} // for k
			} // for j
			return -1;			
		}
  }
  
  /**
   * Fonction renvoyant true si le lieu existe et au moins un des
   * points de ce lieu est visible. Utilis pour le trac automatique de courbes
   * Optimis version 4.7.6
   */
  public boolean dansFenetre(DimensionFenetre dimfen) {
		if (!existe) return false;
		boolean resultat = false;
		for (int j = 0; (j < nombreLignes)  && !resultat; j++)
		{
			for (int k = infoLignes[j].indicePremierPoint; k <= infoLignes[j].indicePremierPoint + 
					infoLignes[j].nombrePoints-2;k++) {
				resultat = resultat || dimfen.dansFenetre(abscissesCoordonneesPoints[k],ordonneesCoordonneesPoints[k]);
			}
		}
		return resultat;
	}
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return listeOb.contains(pointATracer) || pointATracer.estDefiniParObjetDans(listeOb);
	}        
  
  // C++ : La classe InfoLieu a t modifie pour la version 2.5 en lui rajoutant un membre GestionDiscontinuite
  // d'o la ncessit d'un paramtre supplmentaire
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list,
    boolean nouvelleVersion) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    pointATracer = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    infoLieu = new InfoLieu();
    if (nouvelleVersion) infoLieu.chargeDepuisFlux(inps, list);
    else infoLieu.chargeAncienDepuisFlux(inps, list);
    // Il faut crer les tableaux de coordonnes
    // On rserve une coordonne supplmentaire pour le cas d'un lieu ferm
    abscissesCoordonneesPoints = new int[infoLieu.nombreDePoints + 1];
    ordonneesCoordonneesPoints = new int[infoLieu.nombreDePoints + 1];
    abscissesPourTrace = new int[infoLieu.nombreDePoints + 1];
    ordonneesPourTrace = new int[infoLieu.nombreDePoints + 1];
    int nb = infoLieu.nombreDePoints/2+2;
    infoLignes = new InfoLigneLieu[nb];
    // Il faut initialiser chaque lment du tableau
    for (int i = 0; i < nb; i++) {
      infoLignes[i] = new InfoLigneLieu();
    }
    // Il faut crer la liste des lments dont dpend le point  tracer
    // Pour cette liste, on fait pointer la longueur unit ventuelle sur la lsite mre
    // Ligne suivante  revoir version 3.4.2
    // listeElementsAncetres = new CListeObjets(list.uniteAngle, list.pointeurLongueurUnite);
    listeElementsAncetres = new CSousListeObjets();
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    pointATracer = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    infoLieu = new InfoLieu();
    infoLieu.read(inps, list);
    // Il faut crer les tableaux de coordonnes
    // On rserve une coordonne supplmentaire pour le cas d'un lieu ferm
    abscissesCoordonneesPoints = new int[infoLieu.nombreDePoints + 1];
    ordonneesCoordonneesPoints = new int[infoLieu.nombreDePoints + 1];
    // Ajout version 4.8.1
    xCoord = new double[infoLieu.nombreDePoints + 1];
    yCoord = new double[infoLieu.nombreDePoints + 1];
    //
    abscissesPourTrace = new int[infoLieu.nombreDePoints + 1];
    ordonneesPourTrace = new int[infoLieu.nombreDePoints + 1];
    int nb = infoLieu.nombreDePoints/2+2;
    infoLignes = new InfoLigneLieu[nb];
    // Il faut initialiser chaque lment du tableau
    for (int i = 0; i < nb; i++) {
      infoLignes[i] = new InfoLigneLieu();
    }
    // Il faut crer la liste des lments dont dpend le point  tracer
    // Pour cette liste, on fait pointer la longueur unit ventuelle sur la lsite mre
    // Ligne suivante  revoir version 3.4.2
    // listeElementsAncetres = new CListeObjets(list.uniteAngle, list.pointeurLongueurUnite);
    listeElementsAncetres = new CSousListeObjets();
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(pointATracer);
    oups.writeInt(ind1);
    infoLieu.write(oups,list);
  }
  
  /**
   * Ajout version 4.8.1
   * Doit tre redfini par les descendants
   * @param dimf
   * @param nomaff
   * @param coefmult
   * @return
   */
  
  public String tikzOnePath(DimensionFenetre dimf, boolean nomaff, double coefmult, boolean bu) {
  	String ch = "";
    int ideb = infoLignes[indicePremiereLigne].indicePremierPoint;
    int nombrePointsLigne = infoLignes[indicePremiereLigne].nombrePoints;
    for (int i = 0; i < nombrePointsLigne; i++) {
    	ch += "(" + Kernel.chaineNombre(listeProprietaire.tikzXcoord(xCoord[ideb + i], dimf, bu), 3) + "," +
    			Kernel.chaineNombre(listeProprietaire.tikzYcoord(yCoord[ideb + i], dimf, bu), 3) + ")";
    	if (i < nombrePointsLigne - 1) ch += "--";
    }
    if (infoLieu.fermeOuNon) 
    	ch += "(" + Kernel.chaineNombre(listeProprietaire.tikzXcoord(xCoord[ideb], dimf, bu), 3) + "," +
    			Kernel.chaineNombre(listeProprietaire.tikzYcoord(yCoord[ideb], dimf, bu), 3) + ")";
    return ch;
  }
  
  
  @Override
  public String tikz(DimensionFenetre dimf, boolean nomaff, double coefmult, boolean bu) {
  	String ch = "";
    for (int j = 0; j < nombreLignes; j++) {
      int nombrePoints = infoLignes[j].nombrePoints;
      // Optimis version 3.9 On trace maintenant les points isols du lieu
      int indPremPoint = infoLignes[j].indicePremierPoint;
      if (nombrePoints == 1) {
      	double x = xCoord[indPremPoint];
      	double y = yCoord[indPremPoint];
      	if (dimf.dansFenetre(x, y))
      		ch += "\\draw [" + tikzCouleur() +"](" + Kernel.chaineNombre(listeProprietaire.tikzXcoord(x, dimf, bu), 3) +
      			"," + Kernel.chaineNombre(listeProprietaire.tikzYcoord(y, dimf, bu), 3) + ")circle(" +
      			Kernel.chaineNombre(listeProprietaire.tikzLongueur(2, dimf, bu), 3) + ");" + System.lineSeparator();
      }
      else {
        // Il faut transfrer les coordonnes de chaque ligne pour pouvoir la tracer en java
      	ch += "\\draw " + tikzLineStyle(dimf, coefmult);
        for (int k = 0; k < nombrePoints; k++) {
        	int k1 = indPremPoint + k;
        	ch += "(" + Kernel.chaineNombre(listeProprietaire.tikzXcoord(xCoord[k1], dimf, bu), 3) + "," +
        			Kernel.chaineNombre(listeProprietaire.tikzYcoord(yCoord[k1], dimf, bu), 3) + ")";
        	if (k < nombrePoints - 1) ch += "--";
        	else {
        		if (infoLieu.fermeOuNon) ch += "--cycle";
        		ch += ";" + System.lineSeparator();
        	}
        }
      }
    }
    return ch;
  }

  @Override
  public void positionneTikz(boolean infoRandom, DimensionFenetre dimfen) {
    xCoord = new double [infoLieu.nombreDePoints + 1];
    yCoord = new double [infoLieu.nombreDePoints + 1];
  }
  // Ajout version 4.9.9
  public String genereNom() {
  	indice++;
  	return getString("rlieu") + indice;
  }
}
