/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.util.*;
import java.awt.*;

import dialogs.MacroBoucleAvecTraceDlg;
import dialogs.MtgDlg;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class CMacroBoucleAvecTrace extends CMacroAvecListe {
	public int frequenceAnimation;
	public boolean animationVisible;
	public CVariableBornee variableAssociee;
	public CMacro macroAvantBoucles;
	public CMacro macroFinBoucle;
	private double pas;
	private double valeurMinimale;
	private double valeurMaximale;
	private double valeurCourante;
	private CListeObjets listeElementsDependants;
	private CListeObjets listeElementsDependantMacroFin;
	private CListeObjets listeVariablesModifiees;

	/** Creates new CMacroBoucleAvecTrace */
	public CMacroBoucleAvecTrace() {
	}

	// Constructeur pour clonage
	public CMacroBoucleAvecTrace(CListeObjets liste, CImplementationProto impProtoInit,
			boolean estElementFinalInit, Color couleurInit, double x_nomInit, double y_nomInit, double dec_xInit,
			double dec_yInit, boolean masqueInit, CPointAncetre pointLieInit, int indiceTaillePoliceInit,
			int encadrementInit, boolean effacementFondInit, Color couleurFondInit, String intituleInit,
			String commentaireMacroInit, CSousListeObjets listeAssocieeInit, int frequenceAnimationInit,
			boolean animationVisibleInit, CVariableBornee variableAssocieeInit, CMacro macroAvantBouclesInit,
			CMacro macroFinBoucleInit, int alignementHorizontalInit, int alignementVerticalInit) {
		super(liste, impProtoInit, estElementFinalInit, couleurInit, x_nomInit, y_nomInit, dec_xInit, dec_yInit,
				masqueInit, pointLieInit, indiceTaillePoliceInit, encadrementInit, effacementFondInit,
				couleurFondInit, intituleInit, commentaireMacroInit, listeAssocieeInit, alignementHorizontalInit,
				alignementVerticalInit);
		animationVisible = animationVisibleInit;
		frequenceAnimation = frequenceAnimationInit;
		variableAssociee = variableAssocieeInit;
		macroAvantBoucles = macroAvantBouclesInit;
		macroFinBoucle = macroFinBoucleInit;
		listeElementsDependants = new CListeObjets(liste);
		listeElementsDependantMacroFin = new CListeObjets(liste);
		listeVariablesModifiees = new CListeObjets(liste);
	}

	@Override
	public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
		CMacro mac2, mac3;
		CSousListeObjets listeAssocieeClone = new CSousListeObjets();
		listeAssocieeClone.setImage(listeAssociee, listeSource, listeCible);
		int ind1 = listeSource.indexOf(variableAssociee);
		if (macroAvantBoucles == null)
			mac2 = null;
		else {
			int ind2 = listeSource.indexOf(macroAvantBoucles);
			mac2 = (CMacro) (listeCible.get(ind2, CMacro.class));
		}
		if (macroFinBoucle == null)
			mac3 = null;
		else {
			int ind3 = listeSource.indexOf(macroFinBoucle);
			mac3 = (CMacro) (listeCible.get(ind3, CMacro.class));
		}
		int ind4 = listeSource.indexOf(pointLie);
		int ind5 = listeSource.indexOf(impProto);
		CElementBase ptelb = new CMacroBoucleAvecTrace(listeCible, (CImplementationProto) (listeCible.get(ind5, CImplementationProto.class)),
				estElementFinal, couleur, x_nom, y_nom, dec_x, dec_y, masque, (CPointAncetre) listeCible.get(ind4, CPointAncetre.class),
				indiceTaillePolice, encadrement, effacementFond, couleurFondAffichage, intitule, commentaireMacro,
				listeAssocieeClone, frequenceAnimation, animationVisible, (CVariableBornee) listeCible.get(ind1, CVariableBornee.class),
				mac2, mac3, alignementHorizontal, alignementVertical);
		((CMacroBoucleAvecTrace) ptelb).listeElementsDependants = new CListeObjets(listeCible);
		((CMacroBoucleAvecTrace) ptelb).listeElementsDependantMacroFin = new CListeObjets(listeCible);
		((CMacroBoucleAvecTrace) ptelb).listeVariablesModifiees = new CListeObjets(listeCible);
		return ptelb;
	}

	@Override
	public boolean agitSurVariable(CVariableBornee var) {
		return (macroAvantBoucles.agitSurVariable(var) || macroFinBoucle.agitSurVariable(var));
	}

	// Modifi pour la version 1.9.5
	@Override
	public int typeAnimation() {
		if ((animationVisible) && (frequenceAnimation != 0))
			return CMacro.animationParTimer;
		else
			return CMacro.animationParThread;
	}

	@Override
	public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
		boolean res = false;
		if (macroAvantBoucles != null)
			res |= macroAvantBoucles.dependDe(p);
		if (macroFinBoucle != null)
			res |= macroFinBoucle.dependDe(p);
		return memoriseDependance(res || super.dependDe(p));
	}

	// Modifi version 4.8
	@Override
	public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    boolean res = false;
		if (macroAvantBoucles != null)
			res |= macroAvantBoucles.dependDePourReclassement(p);
		if (macroFinBoucle != null)
			res |= macroFinBoucle.dependDePourReclassement(p);
		return memoriseDependancePourRec(res || super.dependDePourReclassement(p));
	}
	
	@Override
	public boolean dependDePourBoucle(CElementBase p) {
		boolean res = false;
		if (macroAvantBoucles != null)
			res |= macroAvantBoucles.dependDePourBoucle(p);
		if (macroFinBoucle != null)
			res |= macroFinBoucle.dependDePourBoucle(p);
		return res || super.dependDePourBoucle(p);
	}

  @Override
	public void metAJour() {
		super.metAJour();
		etablitListesInternes();
	}

	/**
	 * Fonction prparant les listes dont la macro a besoin pour s'excuter
	 */
	@Override
	public void etablitListesInternes() {
		listeElementsDependants.retireTout();
		listeElementsDependantMacroFin.retireTout();
		listeVariablesModifiees.retireTout();
		ListIterator<CElementBase> li1 = listeProprietaire.listIterator();
		while (li1.hasNext()) {
			CElementBase ptb1 = li1.next();
			if (ptb1.dependDePourBoucle(variableAssociee)) {
				listeElementsDependants.add(ptb1);
			}
		}
		if (macroFinBoucle != null) {
			ListIterator<CElementBase> li2 = listeProprietaire.listIterator();
			while (li2.hasNext()) {
				CElementBase ptb1 = li2.next();
				if (ptb1.getNatureCalcul() == NatureCalcul.NVariable) {
					CVariableBornee ptv1 = (CVariableBornee) ptb1;
					if ((ptv1 != variableAssociee) && macroFinBoucle.agitSurVariable(ptv1))
						listeVariablesModifiees.add(ptv1);
				}
			}
			// Il faut aussi crer la liste des lments dpendants
			// d'une variable sur laquelle agit la macro
			// excute  la fin de chaque liste
			ListIterator<CElementBase> li3 = listeProprietaire.listIterator();
			while (li3.hasNext()) {
				CElementBase ptb1 = li3.next();
				if (ptb1.getNature() != NatureObjet.NMacro) {
					ListIterator<CElementBase> li4 = listeVariablesModifiees.listIterator();
					while (li4.hasNext()) {
						CVariableBornee ptv2 = (CVariableBornee) (li4.next());
						if (ptb1.dependDe(ptv2)) { // Modifi par rapport au C++
							if (listeElementsDependantMacroFin.indexOf(ptb1) == -1)
              	if (ptb1.estDeNatureCalcul(NatureCalcul.NTouteValeurReelleSaufConst)) {
              		if (!macroFinBoucle.affecteValeurAVariable((CValeurDynamique) ptb1))
              			listeElementsDependantMacroFin.add(ptb1);
              	}
              	else listeElementsDependantMacroFin.add(ptb1);
						}
					}
				}
			}
		}
	}

	@Override
	// Modifi version 4.7.7. On peut vouloir tracer laisser les traces pour un seul objet qui peut ne pas exister
	// au dbut.
	/*
	public boolean executionPossible() {
		boolean res;
		if (macroFinBoucle == null)
			res = super.executionPossible();
		else {
			if (macroFinBoucle instanceof CMacroTraceAutoParVariable)
				res = true;
			else
				res = super.executionPossible();
		}
		return res;
	}
	*/
	public boolean executionPossible() {
		return true;
	}

	@Override
	public void actionAvantExecution(MtgInternalFrame cadre) {
		super.actionAvantExecution(cadre); // Met executioEnCours  true
		// Rajout pour version 1.8
		cadre.initialisetbuffer();

		// Avant le dbut de la boucle, on excute d'abord la macro
		// MacroAvantBoucles
		if (macroAvantBoucles != null) {
			if (macroAvantBoucles.executionPossible())
				macroAvantBoucles.executeSimple(cadre);
		}
		// Control.InvalideControl();
		pas = variableAssociee.valeurPas;
		valeurMinimale = variableAssociee.valeurMini;
		valeurMaximale = variableAssociee.valeurMaxi;
		// Ligne suivante modifie version 2.5.7
		// valeurCourante = valeurMinimale - pas;
		valeurCourante = valeurMinimale;
		// On permet le tirage alatoire par la fonction Rand()
		cadre.infRand = true;
	}

	// Si lors de actionDansPaint(), appel dans paint(), la macro affiche d'abord
	// l'image de fond, cette procdure renvoie true (c'est le dfaut).
	// Renverra fausse pour les macros de trace et de trace automatique
	// ainsi que pour les macros de boucle avec trace d'objet
	@Override
	public boolean affichageSurImFond() {
		return false;
	}

	// Modification version 2.0. Etait vide avant.
	@Override
	public void actionDansPaint(MtgInternalFrame cadre) {
		if (animationVisible) {
			cadre.transferetbufferversgbuffer();
			cadre.afficheToutDansgbuffer();
		}
	}

	@Override
	public void termineAction(MtgInternalFrame cadre) {
		super.termineAction(cadre);
		// S'il s'agit d'une animation par Thread, on peut avoir saut des
		// affichages.
		// Il faut donc afficher une dernire fois la figure pour tre sur de ne
		// rien avoir manqu
		// Modifi version 2.0
		cadre.transferetbufferversgbuffer();
		cadre.afficheToutDansgbuffer();
		cadre.paneFigure.repaint();
		
		// On ne permet plus le tirage alatoire par la fonction Rand()
		cadre.infRand = false;
		// Control->AfficheControl();
		// executionEnCours = false;
		// applet.repaint(); // Parce que certains affichages ont pu tre sauts
		//  vitesse maximale
	}

	@Override
	public void executePourThread(MtgInternalFrame cadre) {
		// On raffiche d'abord la figure
		// cadre.repaint();
		// Ncessaire puisque paint n'est pas appel
		cadre.setBackground(cadre.couleurFond);
		// Couleur.setCouleurFond(cadre.couleurFond);
		actionAvantExecution(cadre);
		// Il est ncessaire d'ausculter executionEnCours pour que la boucle ne
		// continue
		// pas alors qu'on a appuy sur la touche Escape
		while ((valeurCourante <= valeurMaximale) && executionEnCours) {
			variableAssociee.donneValeur(valeurCourante);
			listeElementsDependants.positionne(cadre.infRand, cadre.dimf);
			CMathGrafDoc doc = cadre.document;
			listeAssociee.afficheToutMemeMasquesPourTrace(cadre.tbuffer, cadre.dimf, doc.couleurFond, doc.transparencePourSurfaces, cadre.frame.pref_antialiasing);
			if ((macroFinBoucle != null) && (valeurCourante <= valeurMaximale)) {
				if (macroFinBoucle.executionPossible())
					macroFinBoucle.executeSimple(cadre);
				listeElementsDependantMacroFin.positionne(cadre.infRand, cadre.dimf);
			}
			if (cadre.runner != null)
				Thread.yield();
			if (animationVisible) {
				// Modifi version 2.0. les deux lignes suivantes seront excutes dans actionDansPaint()				
				// cadre.transferetbufferversgbuffer();
				// cadre.afficheToutDansgbuffer();
				cadre.paneFigure.repaint();
			}
			valeurCourante += pas;
		}
		cadre.runner = null;
		termineAction(cadre);
		// if (!animationVisible) cadre.paneFigure.repaint();
	}
	@Override
	public void executePourTimer(MtgInternalFrame cadre, boolean repaint) {
		if (!executionEnCours) return;
		variableAssociee.donneValeur(valeurCourante);
		listeElementsDependants.positionne(cadre.infRand, cadre.dimf);
		CMathGrafDoc doc = cadre.document;
		listeAssociee.afficheToutMemeMasquesPourTrace(cadre.tbuffer, cadre.dimf, doc.couleurFond, doc.transparencePourSurfaces,
				cadre.frame.pref_antialiasing);
		if ((macroFinBoucle != null) && (valeurCourante <= valeurMaximale)) {
			if (macroFinBoucle.executionPossible())
				macroFinBoucle.executeSimple(cadre);
			listeElementsDependantMacroFin.positionne(cadre.infRand, cadre.dimf);
		}
		// Modifi version 2.0. Les deux lignes suivantes seront excutes dans actionDansPaint()
		// cadre.transferetbufferversgbuffer();
		// cadre.afficheToutDansgbufferSansImFond();
		if (repaint)
			cadre.paneFigure.repaint();
		valeurCourante += pas;
		if (valeurCourante > valeurMaximale) {
			termineAction(cadre);
			return;
		}
		Thread.yield(); // Pour viter les ConcurrentModificationException
	}	

	@Override
	public int frequenceAnimation() {
		return frequenceAnimation;
	}

  @Override
  public String infoPourHistorique() {
    String ch = nom() + " : " + getString("chinfo131") + "\n" + getString("chInfo130Bis") + " " + variableAssociee.nom();
    ch += "\n" + getString("macroApparitionDlg2") + " " + intitule;
  	if (pointLie != null) ch += "\n" + getString("chinfo200") + " " + pointLie.nom();
  	if (effacementFond) ch += "\n" + getString("paneAffichage1");
  	ch += "\n" + getString("chinfo231") + listeAssociee.listeNoms();
  	return ch;
  }
  
 	/**
 	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
 	 * A redfinir pour les descendants.
 	 * @return
 	 */
   @Override
 	public String info() {
     return nom() + " : " + getString("chinfo131") + " " + getString("chInfo130Bis") + " " + variableAssociee.nom();
 	}

	@Override
	public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
		liste.ajouteElementsDe(listeAssociee);
	}

  // Suprim version 2.0 car provoques des stackoverflow

	/*
	@Override
	public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
		listeElementsDependants.remplacePoint(ancienPoint, nouveauPoint);
		listeElementsDependantMacroFin.remplacePoint(ancienPoint, nouveauPoint);
	}
	*/

	@Override
	public boolean modifiableParMenu() {
  	// Modification version 3.5
  	// return true;
  	return !estElementFinal;
	}

	@Override
	public MtgDlg dialogueModification(MtgFrame frame) {
		return new MacroBoucleAvecTraceDlg(frame, this, true);
	}

	@Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
		return super.estDefiniParObjetDans(listeOb)
				&& (listeOb.contains(variableAssociee) || variableAssociee.estDefiniParObjetDans(listeOb))
				&& (listeOb.contains(macroAvantBoucles) || macroAvantBoucles.estDefiniParObjetDans(listeOb))
				&& (listeOb.contains(macroFinBoucle) || macroFinBoucle.estDefiniParObjetDans(listeOb));
	}

	@Override
	public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
		super.chargeDepuisFlux(inps, list);
		if (nVersion == 1)
			animationVisible = true;
		else
			animationVisible = inps.readBoolean();
		frequenceAnimation = inps.readCInt();
		// Frquence  vitesse maxi remplace par 1/1000 de seconde
		// A revoir pour la vcersion logiciel
		// if (frequenceAnimation == 0) frequenceAnimation = 1;
		int ind1 = inps.readCInt();
		variableAssociee = (CVariableBornee) (list.get(ind1, CVariableBornee.class));
		int ind2 = inps.readCInt();
		if (ind2 == -1)
			macroAvantBoucles = null;
		else
			macroAvantBoucles = (CMacro) (list.get(ind2, CMacro.class));
		int ind3 = inps.readCInt();
		if (ind3 == -1)
			macroFinBoucle = null;
		else
			macroFinBoucle = (CMacro) (list.get(ind3, CMacro.class));
		if (list.getClass() != CPrototype.class) {
			listeElementsDependants = new CListeObjets(listeProprietaire);
			listeElementsDependantMacroFin = new CListeObjets(listeProprietaire);
			listeVariablesModifiees = new CListeObjets(listeProprietaire);
		}
    nVersion = numeroVersion(); // On remet tout  zro pour la version java
	}

	@Override
	public void read(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
		super.read(inps, list);
		animationVisible = inps.readBoolean();
		frequenceAnimation = inps.readInt();
		// Frquence  vitesse maxi remplace par 1/1000 de seconde
		// Modifi version 4.7.7. Seulement pour l'applet
		// if (frequenceAnimation == 0)
		// frequenceAnimation = 1;
		int ind1 = inps.readInt();
		variableAssociee = (CVariableBornee) (list.get(ind1, CVariableBornee.class));
		int ind2 = inps.readInt();
		if (ind2 == -1)
			macroAvantBoucles = null;
		else
			macroAvantBoucles = (CMacro) (list.get(ind2, CMacro.class));
		int ind3 = inps.readInt();
		if (ind3 == -1)
			macroFinBoucle = null;
		else
			macroFinBoucle = (CMacro) (list.get(ind3, CMacro.class));
		if (list.getClass() != CPrototype.class) {
			listeElementsDependants = new CListeObjets(listeProprietaire);
			listeElementsDependantMacroFin = new CListeObjets(listeProprietaire);
			listeVariablesModifiees = new CListeObjets(listeProprietaire);
		}
	}

	@Override
	public void write(MathGraphDataOutputStream oups, CListeObjets list) throws IOException {
		super.write(oups, list);
		oups.writeBoolean(animationVisible);
		oups.writeInt(frequenceAnimation);
		int ind1 = list.indexOf(variableAssociee);
		oups.writeInt(ind1);
		int ind2;
		if (macroAvantBoucles == null)
			ind2 = -1;
		else
			ind2 = list.indexOf(macroAvantBoucles);
		oups.writeInt(ind2);
		int ind3;
		if (macroFinBoucle == null)
			ind3 = -1;
		else
			ind3 = list.indexOf(macroFinBoucle);
		oups.writeInt(ind3);
	}
}
