/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.Color;

import dialogs.MacroClignotementDlg;
import dialogs.MtgDlg;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class CMacroClignotement extends CMacroAvecListe {
	public int dureeAnimation; // Dure en secondes (0 si animation
											// jusqu' clic de souris)
	boolean nombrePairClignotements;

  /** Creates new CMacroClignotement */
  public CMacroClignotement() {
  }
  
  // Constructeur pour clonage
  public CMacroClignotement(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, double x_nomInit, double y_nomInit, double dec_xInit, double dec_yInit, boolean masqueInit,
      CPointAncetre pointLieInit, int indiceTaillePoliceInit, int encadrementInit,
      boolean effacementFondInit, Color couleurFondInit, String intituleInit, String commentaireMacroInit,
      CSousListeObjets listeAssocieeInit, int dureeAnimationInit, int alignementHorizontalInit, int alignementVerticalInit) {
        super (liste, impProtoInit, estElementFinalInit, couleurInit, x_nomInit, y_nomInit,
          dec_xInit, dec_yInit, masqueInit, pointLieInit, indiceTaillePoliceInit, encadrementInit,
          effacementFondInit, couleurFondInit, intituleInit, commentaireMacroInit, listeAssocieeInit,
          alignementHorizontalInit, alignementVerticalInit);
        dureeAnimation = dureeAnimationInit;      
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(impProto);        
    CSousListeObjets listeAssocieeClone = new CSousListeObjets();
    listeAssocieeClone.setImage(listeAssociee, listeSource, listeCible);
    CElementBase ptelb = new CMacroClignotement(listeCible, (CImplementationProto)(listeCible.get(ind1, CImplementationProto.class)),
      estElementFinal, couleur, x_nom, y_nom, dec_x, dec_y, 
      masque, pointLie, indiceTaillePolice, encadrement,
      effacementFond, couleurFondAffichage, intitule, commentaireMacro, listeAssocieeClone, dureeAnimation,
      alignementHorizontal, alignementVertical);
    return ptelb;
   } 

  @Override
  public int typeAnimation() {
    return CMacro.animationParTimer;
  }

  @Override
  public int frequenceAnimation() {
    return 500; // Frquence de clignotement de une demi seconde
  }
  
  @Override
  public long dureePourAnimation() {
    return dureeAnimation;
  }
  
  @Override
  public boolean arretParClic() {
  	return (dureeAnimation == 0);
  }
  
  @Override
  public void actionAvantExecution(MtgInternalFrame cadre) {
  	super.actionAvantExecution(cadre);  // Met executioEnCours  true
    nombrePairClignotements = true;
    timeDebut = System.currentTimeMillis();
    if (dureeAnimation != 0) {
      timeFin = timeDebut + dureeAnimation*100; // La dure est en diximes de secondes
    }
  }
   
  @Override
  public void actionDansPaint(MtgInternalFrame cadre) {
  	// Modifi pour la version 2.0
    /*
     * if (!nombrePairClignotements) listeAssociee.effaceTout(cadre.gbuffer, cadre.dimf, true);
    listeAssociee.montreTout(nombrePairClignotements);
    cadre.afficheToutDansgbuffer();
    */
  	cadre.effacegbuffer();
  	if (nombrePairClignotements) cadre.afficheToutDansgbuffer();
  	else {
  		cadre.afficheToutDansgbufferSaufElementsDe(listeAssociee);
  	}
  }

  @Override
  public void executePourTimer(MtgInternalFrame cadre, boolean repaint) {
    if (!executionEnCours) return;
    //TRES IMPORTANT : c'est ici que doit se trouver cette instruction et non dans actionDansPaint() car
    // si deux fentres comportant des clignotements se recouvrent, le prochain repaint se fera pour
    // la zne de recouvrement et le clignotement ne se fait que dans la zne de recouvrement
    nombrePairClignotements = !nombrePairClignotements;
    if (repaint) cadre.paneFigure.repaint();
  }
  
  @Override
  public void termineAction(MtgInternalFrame cadre) {
    super.termineAction(cadre);
    cadre.repaintFigureAvecCalcul();
    if (!nombrePairClignotements) {
      listeAssociee.montreTout(true);
      cadre.afficheToutDansgbuffer();
      cadre.paneFigure.repaint();
    }
  }

  @Override
  public String infoPourHistorique() {
    String ch = nom() + " : " + getString("chinfo132") + " " + getString("ch31") + "\n" + listeAssociee.listeNoms();
    ch += "\n" + getString("macroApparitionDlg2") + " " + intitule;
  	if (pointLie != null) ch += "\n" + getString("chinfo200") + " " + pointLie.nom();
  	if (effacementFond) ch += "\n" + getString("paneAffichage1");
  	return ch;
  }
  
 	/**
 	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
 	 * A redfinir pour les descendants.
 	 * @return
 	 */
   @Override
 	public String info() {
     return nom() + " : " + getString("chinfo132") + " " + listeAssociee.listeNoms();
 	}

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.ajouteElementsDe(listeAssociee);
  }
  
  @Override
  public boolean modifiableParMenu() {
  	// Modification version 4.5.1
  	// return !estElementFinal;
  	return true;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new MacroClignotementDlg(frame, this, true);
  }
  

  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    dureeAnimation = inps.readCInt();
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    dureeAnimation = inps.readInt(); 
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeInt(dureeAnimation);
  }
}
