/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.Color;

import dialogs.MacroTraceAutoParVariableDlg;
import dialogs.MtgDlg;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class CMacroTraceAutoParVariable extends CMacroAvecListe {
	// Frquence. Utile pour les macros excutes dans uns macro
	// excutant une suite d'autres macros
  public int frequenceAnimationMacro;
	public CVariableBornee variableAssociee;
  private double valeur; // l'abscisse en cours du point li qui bouge
  private double valeurMini;
  private double valeurMaxi;
  private double pas;
  private double valeurInitiale;
  CListeObjets listeDep;

  /** Creates new CMacroTraceAutoParVariable */
  public CMacroTraceAutoParVariable() {
  }

  // Constructeur pour clonage
  public CMacroTraceAutoParVariable(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, double x_nomInit, double y_nomInit, double dec_xInit, double dec_yInit, boolean masqueInit,
      CPointAncetre pointLieInit, int indiceTaillePoliceInit, int encadrementInit,
      boolean effacementFondInit, Color couleurFondInit, String intituleInit, String commentaireMacroInit,
      CSousListeObjets listeAssocieeInit, CVariableBornee variableAssocieeInit, int frequenceAnimationMacroInit,
    	int alignementHorizontalInit, int alignementVerticalInit) {
    super (liste, impProtoInit, estElementFinalInit, couleurInit, x_nomInit, y_nomInit,
      dec_xInit, dec_yInit, masqueInit, pointLieInit, indiceTaillePoliceInit, 
      encadrementInit, effacementFondInit, couleurFondInit, intituleInit, commentaireMacroInit, 
      listeAssocieeInit, alignementHorizontalInit, alignementVerticalInit);
   variableAssociee = variableAssocieeInit;
   frequenceAnimationMacro = frequenceAnimationMacroInit;
   listeDep = new CListeObjets(liste);    
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    CSousListeObjets listeAssocieeClone = new CSousListeObjets();
    listeAssocieeClone.setImage(listeAssociee, listeSource, listeCible);
    int ind1 = listeSource.indexOf(variableAssociee);    
    int ind2 = listeSource.indexOf(pointLie); 
    int ind3 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CMacroTraceAutoParVariable(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, x_nom, y_nom, dec_x, dec_y, masque,
      (CPointAncetre)listeCible.get(ind2, CPointAncetre.class), indiceTaillePolice, encadrement,
      effacementFond, couleurFondAffichage, intitule, commentaireMacro, listeAssocieeClone,
      (CVariableBornee) (listeCible.get(ind1, CVariableBornee.class)), frequenceAnimationMacro,
      alignementHorizontal, alignementVertical);
   ((CMacroTraceAutoParVariable) ptelb).listeDep = new CListeObjets(listeCible.uniteAngle, listeCible.pointeurLongueurUnite);
     return ptelb;
  }

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || variableAssociee.dependDe(p));
  }
  
  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || variableAssociee.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return super.dependDePourBoucle(p) || variableAssociee.dependDePourBoucle(p);
  }

  @Override
  public boolean executionPossible() {
   	// Modification version 3.2.6
    // return variableAssociee.existe && listeAssociee.existeAuMoinsUnElement();
  	return variableAssociee.existe;
  }

  /**
   * Fonction prparant les listes dont la macro a besoin pour s'excuter
   */
  @Override
  public void etablitListesInternes() {
  	listeDep.retireTout();
    listeDep.ajouteObjetsDependantsSauf(variableAssociee, listeProprietaire, this);
  }
  // Ajout version 4.7.8.1
  @Override
  public void metAJour() {
  	super.metAJour();
  	etablitListesInternes();
  }  
  @Override
  public void actionAvantExecution(MtgInternalFrame cadre) { 
  	super.actionAvantExecution(cadre);  // Met executioEnCours  true
    // Rajout pour version 1.8
    cadre.initialisetbuffer();   
    pas = variableAssociee.valeurPas;
    valeurMini = variableAssociee.valeurMini;
    valeurMaxi = variableAssociee.valeurMaxi;
    valeurInitiale = variableAssociee.valeurActuelle;
    valeur = valeurMini - pas;
    listeDep.ajouteObjetsDependantsSauf(variableAssociee, listeProprietaire, this);
    timeDebut = System.currentTimeMillis();
    timeFin = timeDebut + 100000000; // Ce n'est pas timeFin qui compte mais le nombre de boucles
  }

  // Si lors de actionDansPaint(), appel dans paint(), la macro affiche d'abord
  // l'image de fond, cette procdure renvoie true (c'est le dfaut).
  // Renverra fausse pour les macros de trace et de trace automatique
  // ainsi que pour les macros de boucle avec trace d'objet
  @Override
  public boolean affichageSurImFond() {
    return false;
  }
  
  @Override
  public synchronized void actionDansPaint(MtgInternalFrame cadre) {
    boolean infor = cadre.infRand;
    DimensionFenetre dimfen = cadre.dimf;
    if (frequenceAnimationMacro == 0) {
      valeur= valeur + pas;
      while (valeur <= valeurMaxi)
      {
        variableAssociee.donneValeur(valeur);
        listeDep.positionne(infor, dimfen);
        CMathGrafDoc doc = cadre.document;
        listeAssociee.afficheToutMemeMasquesPourTrace(cadre.tbuffer, dimfen, doc.couleurFond, doc.transparencePourSurfaces,
        		cadre.frame.pref_antialiasing);
        valeur = valeur + pas;
     }
      cadre.transferetbufferversgbuffer();
      cadre.afficheToutDansgbuffer();
      executionEnCours = false;
    }
    else {
      // Le timer a dj chang l'abscisse
      cadre.transferetbufferversgbuffer();
      cadre.afficheToutDansgbuffer();
    }     
  }
  
  @Override
  public long dureePourAnimation() {
    return 1;
  }

  @Override
  public synchronized void executePourTimer(MtgInternalFrame cadre, boolean repaint) {
    if (!executionEnCours) return;
    DimensionFenetre dimfen = cadre.dimf;
    valeur = valeur + pas;
    if (valeur > valeurMaxi) {
      // On leurre paint() dans l'applet en changeant le temps de dbut d'affichage
      timeFin = timeDebut;
      valeur = valeurMaxi;
    }
    else {
      variableAssociee.donneValeur(valeur);
      listeDep.positionne(cadre.infRand, dimfen);
      CMathGrafDoc doc = cadre.document;
      listeAssociee.afficheToutMemeMasquesPourTrace(cadre.tbuffer, dimfen, doc.couleurFond, doc.transparencePourSurfaces,
      		cadre.frame.pref_antialiasing);
    }
    if (repaint) cadre.paneFigure.repaint();
  }

  @Override
  public void termineAction(MtgInternalFrame cadre) {
    super.termineAction(cadre);
    variableAssociee.donneValeur(valeurInitiale);
    listeDep.positionne(cadre.infRand, cadre.dimf);
    listeDep.retireTout();
    cadre.transferetbufferversgbuffer();
    cadre.afficheToutDansgbuffer();
  }

  @Override
  public int typeAnimation() {
    if (frequenceAnimationMacro == 0) return CMacro.sansAnimation;
    else return CMacro.animationParTimer;
  }

  @Override
  public int frequenceAnimation() {
    return frequenceAnimationMacro;
  }
  
  @Override
  public String infoPourHistorique() {
    String ch = nom() + " : " + getString("chinfo137") + " " + variableAssociee.nom();
		ch += "\n" + getString("macroApparitionDlg2") + " " + intitule;
		if (pointLie != null) ch += "\n" + getString("chinfo200") + " " + pointLie.nom();
		ch += "\n" + getString("chinfo231") + " " + listeAssociee.listeNoms();
		if (effacementFond) ch += "\n" + getString("paneAffichage1");
		return ch;
  }
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
    return nom() + " : " + getString("chinfo137") + " " + variableAssociee.nom();
	}

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.ajouteElementsDe(listeAssociee);
  }
  
  @Override
  public boolean modifiableParMenu() {
  	// Modification version 4.5.1
  	// return !estElementFinal;
  	return true;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new MacroTraceAutoParVariableDlg(frame, this, true);
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
		return 	super.estDefiniParObjetDans(listeOb)
			&& (listeOb.contains(variableAssociee) || variableAssociee.estDefiniParObjetDans(listeOb));
	}
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    frequenceAnimationMacro = inps.readCInt();   
    int ind1 = inps.readCInt();
    variableAssociee = (CVariableBornee) (list.get(ind1,CVariableBornee.class));
    if (list.getClass() != CPrototype.class) {
    	listeDep = new CListeObjets(listeProprietaire);
    }
  }  

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    frequenceAnimationMacro = inps.readInt();   
    int ind1 = inps.readInt();
    variableAssociee = (CVariableBornee) (list.get(ind1,CVariableBornee.class));
    if (list.getClass() != CPrototype.class) {
    	listeDep = new CListeObjets(listeProprietaire);
    }
  }
  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeInt(frequenceAnimationMacro);
    int ind1 = list.indexOf(variableAssociee);
    oups.writeInt(ind1);
  }
}
