/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;

import mathgraph32.MtgFrame;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

/**
 * Classe ajoute pour la version 2.0
 * @author Yves
 */
// Version 5.0. L'objet descend maintenant de CCalculAncetre
// public class CMesureAffixe extends CValeurDynamique {
public class CMesureAffixe extends CCalculAncetre {
  Complexe affixe = new Complexe();
  CRepere repereAssocie;
  CPointAncetre pointMesure;
  
  /** Creates a new instance of CMesureAffixe */
  public CMesureAffixe() {
  }
  
  /**
   * Version 5.0. Lors de la cration, l'utilisateur affecte un nom  la mesure d'o un changement de version
   */
  @Override
  public int numeroVersion() {
  	return 2;
  }  

  // Constructeur pour clonage
  /* Modifi version 5.0. Un paramtre de plus pour le nom
	/*
  public CMesureAffixe(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      CRepere repereAssocieInit, CPointAncetre pointMesureInit) {
    super(liste, impProtoInit, estElementFinalInit);
    repereAssocie = repereAssocieInit;
    pointMesure = pointMesureInit;
  }
  */
  public CMesureAffixe(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      CRepere repereAssocieInit, CPointAncetre pointMesureInit, String nomCalculInit) {
    super(liste, impProtoInit, estElementFinalInit, nomCalculInit);
    repereAssocie = repereAssocieInit;
    pointMesure = pointMesureInit;
  }
 
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(repereAssocie);
    int ind2 = listeSource.indexOf(pointMesure);
    int ind3 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CMesureAffixe(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, (CRepere)(listeCible.get(ind1, CRepere.class)),
      (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)), nomCalcul);
    return ptelb;    
  }
  
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NMesureAffixe;
  }
  
  /* Supprim version 5.0
  @Override
  public String nom() {
    return getString("chinfo82") + "(" + pointMesure.nom() + "," + repereAssocie.O.nom() + "," + repereAssocie.I.nom() + "," + 
    	repereAssocie.J.nom() + ")";
  }
  */

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || repereAssocie.dependDe(p) || pointMesure.dependDe(p));
  }

  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || repereAssocie.dependDePourReclassement(p) || pointMesure.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || repereAssocie.dependDePourBoucle(p) || pointMesure.dependDePourBoucle(p);
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = repereAssocie.existe && pointMesure.existe;
    if (!existe) return;
    // L'orientation sur l'cran est l'inverse de celle du repre
		// choisi.Il faut donc permuter les rles de u et v}
    VecteurDirecteur u = new VecteurDirecteur(repereAssocie.O,repereAssocie.I);
    VecteurDirecteur v = new VecteurDirecteur(repereAssocie.O,repereAssocie.J);
    // Calcul des	abscisses et ordonnes de l'origine relle du repre dans le repre de base
    double absOrigine = repereAssocie.valAbscisseOrigine/repereAssocie.unitex;
    double ordOrigine = repereAssocie.valOrdonneeOrigine/repereAssocie.unitey;
    double xno = repereAssocie.O.x - u.x*absOrigine - v.x*ordOrigine;
    double yno = repereAssocie.O.y - u.y*absOrigine - v.y*ordOrigine;
    // Calcul de l'abscisse et de l'ordonne par rsolution d'un systme linaire
    double det = u.x*v.y-v.x*u.y;
    affixe.x = (v.y*(pointMesure.x-xno)-v.x*(pointMesure.y-yno))/det*repereAssocie.unitex;
    affixe.y = (u.x*(pointMesure.y-yno)-u.y*(pointMesure.x-xno))/det*repereAssocie.unitey;  
  }
  
  @Override
  public boolean nomIndispensable(CElementGraphique el) {
    return (el == pointMesure);
  }  
  
  // Inutilis mais pour le compilateur
  @Override
  public double rendValeur() {
    return 0;
  }

  @Override
  public void rendValeurComplexe(Complexe zRes) {
    zRes.set(affixe.x, affixe.y);
  }
  
  // Ajout version 5.0.3 pour que la mesure puisse tre affiche dans un affichage dynamique dde texte ou CLatex
  public String rendChaineValeurPourCommentaire(int nbDecimales)
  {
    return rendChaineValeurComplexe(nbDecimales);
  }

  @Override
  public boolean confonduAvec(CElementBase p) {
    CMesureAffixe pt;
    if (p.getClass() == this.getClass()) {
      pt = (CMesureAffixe) p;
      return (repereAssocie == pt.repereAssocie) && (pointMesure == pt.pointMesure);
    }
    else return false;
  }
  
  @Override
  public String infoPourHistorique() {
    String ch = nom() + " : " + getString("chinfo108") + " " + getString("chinfo109") +
      	"  " + pointMesure.nom() + "\n" + getString("chinfo110") + " " + repereAssocie.nom();
    if (existe) ch = ch + "\n" + getString("chinfo97") + affixe.chaineNombre(8);
    else ch = ch + "\n" + getString("ch18");
    return ch;
  }
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
  	return nom() + " : " + getString("chinfo108") + " " + getString("chinfo109") +
      	" " + pointMesure.nom() + " " + getString("chinfo110") + " " + repereAssocie.nom();
	}
  
  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(pointMesure);
  } 

  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(repereAssocie) || repereAssocie.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(pointMesure) || pointMesure.estDefiniParObjetDans(listeOb));
	}
  
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (pointMesure == ancienPoint) pointMesure = nouveauPoint;
  }
  
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
     super.chargeDepuisFlux(inps, list);
     int ind1 = inps.readCInt();
     int ind2 = inps.readCInt();
     repereAssocie = (CRepere) (list.get(ind1,CRepere.class));
     pointMesure = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
   	 nomCalcul = listeProprietaire.genereNomPourCalcul("aff", true);
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
  	// Modifi version 5.0
    if (nVersion>=2) super.read(inps, list);
    else {
    	super.readSansNom(inps, list);
    	nomCalcul = listeProprietaire.genereNomPourCalcul("aff", true);
    }
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    repereAssocie = (CRepere) (list.get(ind1,CRepere.class));
    pointMesure = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(repereAssocie);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(pointMesure);
    oups.writeInt(ind2);
  }  
}