/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Ajout version 4.8. Utile pour les constructions itratives et rcursives
 */

package objets;

import java.awt.Color;
import java.io.IOException;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import mathgraph32.MtgFrame;
import types.DimensionFenetre;
import types.NatureObjet;

public class CPointClone extends CPointAncetre {
	private CPointAncetre point; // Le point clon

  /** Constructeur */
  public CPointClone() {
  }
  
  public CPointClone(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit,
    boolean masqueInit, String nomInit, int indiceTailleNomInit, int motifInit, boolean marquePourTraceInit, CPointAncetre pointInit) {
      super (liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
        dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit, motifInit, marquePourTraceInit);
      point = pointInit;
  }
  
  @Override  
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(impProto);
    int ind2 = listeSource.indexOf(point);
    CElementBase ptelb = new CPointClone(listeCible, (CImplementationProto)(listeCible.get(ind1, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom,
      indiceTailleNom, motif, marquePourTrace, (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)));
    return ptelb;
  }

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || point.dependDe(p));
  }

  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || point.dependDePourReclassement(p));
  }

  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || point.dependDePourBoucle(p));
  }

	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return listeOb.contains(point) || point.estDefiniParObjetDans(listeOb);
	}    

  @Override  
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	// Modifi version 4.9.8.3 pour qu'on puisse dplacer le nom d'un point clobe et le grer comme un autre
  	// Il faut mmoriser si le'objet est masqu ou non car setClone va le rinitialiser  la valeur du point copi
  	/*
  	boolean b = masque;
  	setClone(point);
  	positionneNom(); // Rajout version 4.9.8.3 pour pouvoir dplacer le nom d'un point clone
  	masque = b;
  	*/
  	existe = point.existe;
  	x = point.x;
  	y = point.y;
  	super.positionne(infoRandom, dimfen);
  }
  
  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (point == ancienPoint) point = nouveauPoint;
  }


  // Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
    return nom() + " : " + getString("natPointClone") + " " + getString("ch31") + " " + point.nom();
	}
  
  @Override
  public long getNature() {
    return NatureObjet.NPointClone;
  }

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(point);
  }    

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    point = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(point);
    oups.writeInt(ind1);
  }

}
