/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package objets;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.Kernel;
import java.awt.Color;
import java.awt.geom.Point2D;
import java.io.IOException;
import types.DimensionFenetre;
import types.Doub;
import types.NatureObjet;
import types.VecteurDirecteur;

import mathgraph32.MtgFrame;

public class CPointInterieurCercle extends CPointAncetrePointsMobiles {
	CCercle cercleAssocie;
	private double rapportHomothetie;
	private double anglePolaire;
	
  /** Creates new CPointInterieurCercle */
  public CPointInterieurCercle() {
  }
  
  // Constructeur pour clonage
  public CPointInterieurCercle(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit,
      boolean masqueInit, String nomInit, int indiceTailleNomInit, int motifInit, boolean pointLibreInit, CCercle cercleInit,
      double distanceCentreInit, double rapportHomothetieInit, boolean marquePourTraceInit) {
     super (liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
       dec_xInit, dec_yInit, masqueInit, nomInit,
      indiceTailleNomInit, motifInit, pointLibreInit, marquePourTraceInit);
     cercleAssocie = cercleInit;
     rapportHomothetie = distanceCentreInit;
     anglePolaire = rapportHomothetieInit;
  }
  
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(cercleAssocie);
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CPointInterieurCercle(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom,
      indiceTailleNom, motif, pointLibre, (CCercle)(listeCible.get(ind1, CCercle.class)), rapportHomothetie,
      anglePolaire, marquePourTrace);
    return ptelb;
  }
  
  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(cercleAssocie);
  }
  
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
  	return nom() + " : " + getString("chinfo169") + " " + cercleAssocie.nom();
	}
	
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || cercleAssocie.dependDe(p));
  }
  
  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || cercleAssocie.dependDePourReclassement(p));
  }

  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || cercleAssocie.dependDePourBoucle(p);
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	existe = cercleAssocie.existe;
  	if (!existe) return;
  	if (rapportHomothetie == 0) super.placeEn(cercleAssocie.centre_x, cercleAssocie.centre_y);
  	else {
  		VecteurDirecteur vec = new VecteurDirecteur();
  		vec.x = rapportHomothetie*cercleAssocie.rayon;
  		vec.y = 0;
  		VecteurDirecteur vec2 = new VecteurDirecteur();
  		vec.tourne(anglePolaire, vec2);
  		super.placeEn(cercleAssocie.centre_x + vec2.x, cercleAssocie.centre_y + vec2.y);
  	}
  	super.positionne(infoRandom, dimfen); // Ajout version 4.6.4
  }
  
  @Override
  public void placeEn(double xn, double yn) {
  	super.placeEn(xn, yn);
  	VecteurDirecteur vec = new VecteurDirecteur(cercleAssocie.centre_x, cercleAssocie.centre_y, xn, yn);
  	if (vec.Nul()) {
  		rapportHomothetie = 0;
  		anglePolaire = 0;
  	}
  	else {
  		anglePolaire = vec.angleRad();
  		rapportHomothetie = vec.norme()/cercleAssocie.rayon;
  	}
  }
  
  @Override
  public boolean testDeplacement(DimensionFenetre dimfen, double xtest, double ytest,
        Point2D.Double pointr, Doub abscr) {
  	// On regarde si le point test est bien intrieur au polygone
  	if (Kernel.distancePointPoint(xtest, ytest, cercleAssocie.centre_x, cercleAssocie.centre_y) <= cercleAssocie.rayon) {
  		pointr.x = xtest;
  		pointr.y = ytest;
  		return true;
  	}
  	else return false;
  }
  
  @Override
  public long getNature() {
    return NatureObjet.NPointInterieurCercle;
  }  
  
  @Override
  public boolean confonduAvec(CElementBase p) {
    if (p.getClass() == this.getClass()) {
	  	CPointInterieurCercle pt = (CPointInterieurCercle) p;
	  	if (Kernel.zero(pt.x-x)&& Kernel.zero(pt.y-y)) return (cercleAssocie == pt.cercleAssocie);
	  	else return false;
    }
    else return false;
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return listeOb.contains(cercleAssocie) || cercleAssocie.estDefiniParObjetDans(listeOb);
	}  
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    cercleAssocie = (CCercle) (list.get(ind1,CCercle.class));
    rapportHomothetie = inps.readDouble();
    anglePolaire = inps.readDouble();
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(cercleAssocie);
    oups.writeInt(ind1);
    oups.writeDouble(rapportHomothetie);
    oups.writeDouble(anglePolaire);

  }   
}
