/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package objets;

import types.Complexe;
import types.ErreurCalculException;

public class CProduitDansFormule extends CSommeDansFormule {
	
	public CProduitDansFormule() {
		
	}
	
	public CProduitDansFormule(CListeObjets liste, CCalculBase calculASommerInit, CCalculBase indiceDebutInit,
			CCalculBase indiceFinInit, CCalculBase pasInit, String nomVariableInit) {
		super(liste, calculASommerInit, indiceDebutInit, indiceFinInit, pasInit, nomVariableInit);
	}
	
	// Ajout version 5.1.4
  @Override
  protected long nature() {
  	return natProduitDansFormule;
  }  

	@Override
	CCalculBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
		return new CProduitDansFormule(listeCible, calculASommer.getClone(listeSource, listeCible), indiceDebut.getClone(listeSource, listeCible),
				indiceFin.getClone(listeSource, listeCible), pas.getClone(listeSource, listeCible), nomVariable);
	}

	@Override
	public CCalculBase getCopie() {
		return new CProduitDansFormule(listeProprietaire, calculASommer.getCopie(), indiceDebut.getCopie(),
				indiceFin.getCopie(), pas.getCopie(), nomVariable);
	}	
	
  @Override
  public double resultat(boolean infoRandom) throws ErreurCalculException {
  	double[] val = new double[1];
  	double dinddeb = indiceDebut.resultat(infoRandom);
  	if ((dinddeb > Integer.MAX_VALUE) || (dinddeb < Integer.MIN_VALUE) || (dinddeb != Math.floor(dinddeb))) throw new ErreurCalculException();
  	double dindfin = indiceFin.resultat(infoRandom);
  	if ((dindfin > Integer.MAX_VALUE) || (dindfin < Integer.MIN_VALUE) || (dindfin != Math.floor(dindfin))
  			|| (dindfin < dinddeb)) throw new ErreurCalculException();
  	double dpas = pas.resultat(infoRandom);
  	if ((dpas > Integer.MAX_VALUE) || (dpas < Integer.MIN_VALUE)
  			|| (dpas != Math.floor(dpas)) || (dpas <= 0)) throw new ErreurCalculException();
  	listeProprietaire.nombreIterations += (dindfin-dinddeb)/dpas;
  	if (listeProprietaire.nombreIterations > CCalculBase.nombreMaxiIterations) throw new ErreurCalculException(true);
  	int inddeb = (int) dinddeb;
  	int indfin = (int) dindfin;
  	int valpas = (int) dpas;
  	double somme = 1;
  	for (int i = inddeb; i <= indfin; i+= valpas) {
    	val[0] = i;
    	somme *= calculASommer.resultatFonction(infoRandom, val);
  	}
  	return somme;
  }
  
  @Override
  public void resultatComplexe(boolean infoRandom, Complexe zRes)  throws ErreurCalculException {
  	Complexe[] val = new Complexe[1];
  	val[0] = new Complexe();
  	Complexe inddeb = new Complexe();
  	indiceDebut.resultatComplexe(infoRandom, inddeb);
  	if (inddeb.y != 0) throw new ErreurCalculException();
  	double dinddeb = inddeb.x;
  	if ((dinddeb > Integer.MAX_VALUE) || (dinddeb < Integer.MIN_VALUE) || (dinddeb != Math.floor(dinddeb))) throw new ErreurCalculException();
  	Complexe indfin = new Complexe();
  	indiceFin.resultatComplexe(infoRandom, indfin);
  	if (inddeb.y != 0) throw new ErreurCalculException();
  	double dindfin = indfin.x;
  	if ((dindfin > Integer.MAX_VALUE) || (dindfin < Integer.MIN_VALUE) || (dindfin != Math.floor(dindfin))
  			|| (dindfin < dinddeb)) throw new ErreurCalculException();
  	Complexe valpas = new Complexe();
  	pas.resultatComplexe(infoRandom, valpas);
  	if (valpas.y != 0) throw new ErreurCalculException();
  	double dvalpas = valpas.x;
  	if ((dvalpas > Integer.MAX_VALUE) || (dvalpas < Integer.MIN_VALUE)
  			|| (dvalpas != Math.floor(dvalpas)) || (dvalpas <= 0)) throw new ErreurCalculException();
  	listeProprietaire.nombreIterations += (dindfin-dinddeb)/dvalpas;
  	if (listeProprietaire.nombreIterations > CCalculBase.nombreMaxiIterations) throw new ErreurCalculException(true);
  	zRes.x = 1;
  	zRes.y = 0;
  	Complexe z = new Complexe(0, 0);
  	for (int i = (int)inddeb.x; i <= (int) indfin.x; i+= (int)valpas.x) {
    	val[0].x = i;
    	val[0].y = 0;
    	calculASommer.resultatFonctionComplexe(infoRandom, val, z);
    	zRes.produitComp(z, zRes);
  	}
  	return;  
  }

  public double resultatFonction(boolean infoRandom, double valeurParametre) throws ErreurCalculException {
  	double[] val = new double[2];
  	val[0] = valeurParametre;
  	double dinddeb = indiceDebut.resultatFonction(infoRandom, valeurParametre);
  	if ((dinddeb > Integer.MAX_VALUE) || (dinddeb < Integer.MIN_VALUE) || (dinddeb != Math.floor(dinddeb))) throw new ErreurCalculException();
  	double dindfin = indiceFin.resultatFonction(infoRandom, valeurParametre);
  	if ((dindfin > Integer.MAX_VALUE) || (dindfin < Integer.MIN_VALUE) || (dindfin != Math.floor(dindfin))
  			|| (dindfin < dinddeb)) throw new ErreurCalculException();
  	double dpas = pas.resultatFonction(infoRandom, valeurParametre);
  	if ((dpas > Integer.MAX_VALUE) || (dpas < Integer.MIN_VALUE)
  			|| (dpas != Math.floor(dpas)) || (dpas <= 0)) throw new ErreurCalculException();
  	listeProprietaire.nombreIterations += (dindfin-dinddeb)/dpas;
  	if (listeProprietaire.nombreIterations > CCalculBase.nombreMaxiIterations) throw new ErreurCalculException(true);
  	int inddeb = (int) dinddeb;
  	int indfin = (int) dindfin;
  	int valpas = (int) dpas;
  	double somme = 1;
  	for (int i = inddeb; i <= indfin; i+= valpas) {
    	val[1] = i;
    	somme *= calculASommer.resultatFonction(infoRandom, val);
  	}
  	return somme;
  }
	
  // calculASommer pointe sur une fonction de n+1 variables, n tant la  dmiension de valeurParamre
  // La dernire variable sert  appeler la fonction pour les diffrentes valeurs de l'indice.
  @Override
  public double resultatFonction(boolean infoRandom, double[] valeurParametre) throws ErreurCalculException {
  	int n = valeurParametre.length;
  	double[] val = new double[n + 1];
  	for (int i = 0; i < n; i++) val[i] = valeurParametre[i];
  	double dinddeb = indiceDebut.resultatFonction(infoRandom, valeurParametre);
  	if ((dinddeb > Integer.MAX_VALUE) || (dinddeb < Integer.MIN_VALUE) || (dinddeb != Math.floor(dinddeb))) throw new ErreurCalculException();
  	double dindfin = indiceFin.resultatFonction(infoRandom, valeurParametre);
  	if ((dindfin > Integer.MAX_VALUE) || (dindfin < Integer.MIN_VALUE) || (dindfin != Math.floor(dindfin))
  			|| (dindfin < dinddeb)) throw new ErreurCalculException();
  	double dpas = pas.resultatFonction(infoRandom, valeurParametre);
  	if ((dpas > Integer.MAX_VALUE) || (dpas < Integer.MIN_VALUE)
  			|| (dpas != Math.floor(dpas)) || (dpas <= 0)) throw new ErreurCalculException();
  	listeProprietaire.nombreIterations += (dindfin-dinddeb)/dpas;
  	if (listeProprietaire.nombreIterations > CCalculBase.nombreMaxiIterations) throw new ErreurCalculException(true);
  	int inddeb = (int) dinddeb;
  	int indfin = (int) dindfin;
  	int valpas = (int) dpas;
  	double somme = 1;
  	for (int i = inddeb; i <= indfin; i+= valpas) {
    	val[n] = i;
    	somme *= calculASommer.resultatFonction(infoRandom, val);
  	}
  	return somme;
  }
  
  void  resultatFonctionComplexe(boolean infoRandom, Complexe valeurParametre, Complexe zRes)
  throws ErreurCalculException
  {
  	Complexe[] val = new Complexe[2];
  	val[0] = valeurParametre;
  	val[1] = new Complexe();
  	Complexe inddeb = new Complexe();
  	indiceDebut.resultatFonctionComplexe(infoRandom, valeurParametre, inddeb);
  	if (inddeb.y != 0) throw new ErreurCalculException();
  	double dinddeb = inddeb.x;
  	if ((dinddeb > Integer.MAX_VALUE) || (dinddeb < Integer.MIN_VALUE) || (dinddeb != Math.floor(dinddeb))) throw new ErreurCalculException();
  	Complexe indfin = new Complexe();
  	indiceFin.resultatFonctionComplexe(infoRandom, valeurParametre, indfin);
  	if (indfin.y != 0) throw new ErreurCalculException();
  	double dindfin = indfin.x;
  	if ((dindfin > Integer.MAX_VALUE) || (dindfin < Integer.MIN_VALUE) || (dindfin != Math.floor(dindfin))
  			|| (dindfin < dinddeb)) throw new ErreurCalculException();
  	Complexe valpas = new Complexe();
  	pas.resultatFonctionComplexe(infoRandom, valeurParametre, valpas);
  	if (valpas.y != 0) throw new ErreurCalculException();
  	double dvalpas = valpas.x;
  	if ((dvalpas > Integer.MAX_VALUE) || (dvalpas < Integer.MIN_VALUE)
  			|| (dvalpas != Math.floor(dvalpas)) || (dvalpas <= 0)) throw new ErreurCalculException();
  	listeProprietaire.nombreIterations += (dindfin-dinddeb)/dvalpas;
  	if (listeProprietaire.nombreIterations > CCalculBase.nombreMaxiIterations) throw new ErreurCalculException(true);  	
  	zRes.x = 1;
  	zRes.y = 0;
  	Complexe z = new Complexe(0, 0);
  	for (int i = (int)inddeb.x; i <= (int) indfin.x; i+= (int)valpas.x) {
    	val[1].x = i;
    	val[1].y = 0;
    	calculASommer.resultatFonctionComplexe(infoRandom, val, z);
    	zRes.produitComp(z, zRes);
  	}
  	return;
  }
  
  void  resultatFonctionComplexe(boolean infoRandom, Complexe[] valeurParametre, Complexe zRes)
  throws ErreurCalculException
  {
  	int n = valeurParametre.length;
  	Complexe[] val = new Complexe[n + 1];
  	for (int i = 0; i < n; i++) val[i] = new Complexe(valeurParametre[i]);
  	val[n] = new Complexe();
  	Complexe inddeb = new Complexe();
  	indiceDebut.resultatFonctionComplexe(infoRandom, valeurParametre, inddeb);
  	if (inddeb.y != 0) throw new ErreurCalculException();
  	double dinddeb = inddeb.x;
  	if ((dinddeb > Integer.MAX_VALUE) || (dinddeb < Integer.MIN_VALUE) || (dinddeb != Math.floor(dinddeb))) throw new ErreurCalculException();
  	Complexe indfin = new Complexe();
  	indiceFin.resultatFonctionComplexe(infoRandom, valeurParametre, indfin);
  	if (indfin.y != 0) throw new ErreurCalculException();
  	double dindfin = indfin.x;
  	if ((dindfin > Integer.MAX_VALUE) || (dindfin < Integer.MIN_VALUE) || (dindfin != Math.floor(dindfin))
  			|| (dindfin < dinddeb)) throw new ErreurCalculException();
  	Complexe valpas = new Complexe();
  	pas.resultatFonctionComplexe(infoRandom, valeurParametre, valpas);
  	if (valpas.y != 0) throw new ErreurCalculException();
  	double dvalpas = valpas.x;
  	if ((dvalpas > Integer.MAX_VALUE) || (dvalpas < Integer.MIN_VALUE)
  			|| (dvalpas != Math.floor(dvalpas)) || (dvalpas <= 0)) throw new ErreurCalculException();
  	listeProprietaire.nombreIterations += (dindfin-dinddeb)/dvalpas;
  	if (listeProprietaire.nombreIterations > CCalculBase.nombreMaxiIterations) throw new ErreurCalculException(true);
  	zRes.x = 1;
  	zRes.y = 0;
  	Complexe z = new Complexe(0, 0);
  	for (int i = (int)inddeb.x; i <= (int) indfin.y; i+= (int)valpas.x) {
    	val[n].x = i;
    	val[n].y = 0;
    	calculASommer.resultatFonctionComplexe(infoRandom, val, z);
    	zRes.produitComp(z, zRes);
  	}
  	return; 
  }  
  
  @Override
	public String chaineCalcul(String[] varFor) {
  	String[] varFor2;
  	int n = 0;
  	if (varFor == null) varFor2 = new String[1];
  	else {
	  	n = varFor.length; 
	  	varFor2 = new String[n+1];
  	}
  	for (int i = 0; i < n; i++) varFor2[i] = varFor[i];
  	varFor2[n] = nomVariable;
  	return getString("produit") + "(" + calculASommer.chaineCalculSansPar(varFor2) + ","
  		+ nomVariable + "," + indiceDebut.chaineCalculSansPar(varFor) + ","
  		+ indiceFin.chaineCalculSansPar(varFor) + "," + pas.chaineCalculSansPar(varFor) + ")";
  }

  @Override
	public String chaineLatex(String[] varFor) {
  	String[] varFor2;
  	int n = 0;
  	if (varFor == null) varFor2 = new String[1];
  	else {
	  	n = varFor.length; 
	  	varFor2 = new String[n+1];
  	}
  	for (int i = 0; i < n; i++) varFor2[i] = varFor[i];
  	varFor2[n] = nomVariable;
  	// Modification version 4.7.7 : on renvoie un vrai symbole LaTeX de somme
  	String up = "";
  	if (pas.nature() == CCalculBase.natConstante) {
  		CConstante c = (CConstante) pas;
  		if (c.valeur == 1) up = indiceFin.chaineLatex(varFor);
  	}
  	if (up.length() == 0) up = indiceFin.chaineLatex(varFor) + "\\text{, " + getString("step") + " }"+pas.chaineLatex(varFor);
  	return "\\prod\\limits_{"+nomVariable + "=" + indiceDebut.chaineLatexSansPar(varFor) + "}" + "^{"+ up+"}"+ "{" + calculASommer.chaineLatex(varFor2) + "}";
  	/*
  	return getString("produit") + parouvLatex + calculASommer.chaineLatexSansPar(varFor2) + ","
  		+ nomVariable + "," + indiceDebut.chaineLatexSansPar(varFor) + ","
  		+ indiceFin.chaineLatexSansPar(varFor) + "," + pas.chaineLatexSansPar(varFor) + parferLatex;
  		*/
  }
  
  @Override
  public boolean deriveePossible(int indiceVariable) {
  	return !calculASommer.dependDeVariable(indiceVariable)
  		&& !indiceDebut.dependDeVariable(indiceVariable) && !indiceFin.dependDeVariable(indiceVariable)
  		&& !pas.dependDeVariable(indiceVariable);
  }
  
}
