/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.*;
import java.io.*;
import java.awt.geom.*;


import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

import mathgraph32.MtgFrame;


public class CSegment extends CDroite {
  public CPointAncetre point1, point2; // Les deux extrmits
  private Point2D.Double pointr = new Point2D.Double();
  // Ajout version 3.2.0
  protected double x1; // Pour mmoriser les coordonnes des extrmits qui ne coincident plus ncessairement
	protected double y1;
	protected double x2;
	protected double y2;
  // avec xext1, yext1, xext2, yext2
  /** Creates new CSegment */
  public CSegment() {
  }
  
  /* Constructeur pour les segments internes*/
  public CSegment(CListeObjets liste, CPointAncetre point1Init, CPointAncetre point2Init) {
    point1 = point1Init;
    point2 = point2Init;
    listeProprietaire = liste;
  }

  // Constructeur pour clonage
  public CSegment(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean masqueInit, StyleTrait styleInit,
      CPointAncetre point1Init, CPointAncetre point2Init) {
    super (liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit);
    point1 = point1Init;
    point2 = point2Init;
  }
  
  /* Fonction donnant une valeur aux pointeurs des extrmits (utilis dans l'outil
   * de cration d'image par une translation pour levecteur clignotatnt)
   */
  public void donneExtremites(CPointAncetre pt1, CPointAncetre pt2) {
    point1 = pt1;
    point2 = pt2;
  }
  
  /** Constructeur plus court pour certains objets visuels */
  public CSegment(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    CPointAncetre point1Init, CPointAncetre point2Init) {
    super(liste, impProtoInit, estElementFinalInit, Couleur.TraduitCouleur(1), false, StyleTrait.traitPointille);
    point1 = point1Init;
    point2 = point2Init;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(point1);
    int ind2 = listeSource.indexOf(point2);  
    int ind3 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CSegment(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, masque,style, 
      (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
      (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)));
    return ptelb;
  }
  
  // Ajoiut version 3.2.0
  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    x1 = ((CSegment)ptel).x1;
    y1 = ((CSegment)ptel).y1;
    x2 = ((CSegment)ptel).x2;
    y2 = ((CSegment)ptel).y2;
  }
  

  @Override
  public String complementInfo() {
    return getString("chinfo41");
  }
  
  // Inutile de redfinir setClone. Travail dj fait par CDroite
  
  // Ajout version 4.9.9
  @Override
  public String nom() {
  	return "[" + point1.nom() + point2.nom() + "]";
  }
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
  	return getString("chinfo9") + " " + nom();
	}
  

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(point1);
    liste.add(point2);
  }  
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
  	return memoriseDependance(super.dependDe(p) || point1.dependDe(p) || point2.dependDe(p));
  }
  
  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || point1.dependDePourReclassement(p) || point2.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
	return ((p ==this) || point1.dependDePourBoucle(p) || point2.dependDePourBoucle(p));
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = (point1.existe) && (point2.existe);
    if (!(existe)) return;
    // Ajout version 3.2.0. Utilis dans appartientA
    x1 = point1.x;
    y1 = point1.y;
    x2 = point2.x;
    y2 = point2.y;

    point_x = x1;
    point_y = y1;
    vecteur.setVecteur(point1, point2);
    // Modification version 4.7.7 : Si les deux points sont confondus, on considre le segment comme existant, mais hors-fentre
    if (vecteur.Nul()) {
    	existe = true;
    	horsFenetre = true;
    	return;
    }
    // Fin modif
  	if (point1.dansFenetre(dimfen) && point2.dansFenetre(dimfen)) {
      xext1 = x1;
      yext1 = y1;
      xext2 = x2;
      yext2 = y2;
      horsFenetre = false;
  	}
  	else {
  		// Au moins un des deux points est hors-fentre
    	super.positionne(infoRandom, dimfen);
    	// Si la droite est hors fenetre le segment l'est aussi
    	if (horsFenetre) return;
    	if (point1.dansFenetre(dimfen)) {
    		// point2 n'est pas dans la fentre
        VecteurDirecteur u1 = new VecteurDirecteur(point1, point2);
        VecteurDirecteur u2 = new VecteurDirecteur(x1, y1, xext1, yext1);
        if (u2.presqueNul()) {
        	horsFenetre = true;
        	return;
        }
        if (Kernel.colineairesMemeSens(u1, u2)) {
        	xext2 = x1;
        	yext2 = y1;
        }
        else {
        	xext1 = x1;
        	yext1 = y1;
        }
    	}
    	else {
      	if (point2.dansFenetre(dimfen)) {
      		// point1 n'est pas dans la fentre
          VecteurDirecteur u1 = new VecteurDirecteur(point2, point1);
          VecteurDirecteur u2 = new VecteurDirecteur(x2, y2, xext1, yext1);
          if (u2.presqueNul()) {
          	horsFenetre = true;
          	return;
          }         
          if (Kernel.colineairesMemeSens(u1, u2)) {
          	xext2 = x2;
          	yext2 = y2;
          }
          else {
          	xext1 = x2;
          	yext1 = y2;
          }
      	}
      	else {
      		// Si les deux points sont hors-fentre on regarde si un des points d'intersection
      		// de la droite avec le cadre de la fentre appartient au segment
      		// Correction d'un bug version 3.2.4 : il faut les deux tests car un des deux points peut tre presque sur le bord
          // if (!Kernel.appartientSegment(xext1, yext1, x1, y1, x2, y2)) horsFenetre = true;
          if (!Kernel.appartientSegment(xext1, yext1, x1, y1, x2, y2) || !Kernel.appartientSegment(xext2, yext2, x1, y1, x2, y2)) horsFenetre = true;	      		
      	}
    	}
  	}
  }
  
  /* fonction utilise pour les segments internes aux objets */
  public void positionne() {
  	horsFenetre = false;
    existe = (point1.existe) && (point2.existe);
    if (!(existe)) return;
    point_x = point1.x;
    point_y = point1.y;
    vecteur.setVecteur(point1, point2);
    xext1 = point1.x;
    yext1 = point1.y;
    xext2 = point2.x;
    yext2 = point2.y;
    x1 = xext1;
    y1 = yext1;
    x2 = xext2;
    y2 = yext2;
    
  }
  
  // Ajout version 3.2.2 car CDroiteAncetre ne renvoie plus la bonne valeur pour un segment
  public double abscisseMaximale() {
    // Cas d'un segment ayant ses extrmits confondues
    if ((vecteur.x == 0) && (vecteur.y == 0)) return 0;
    if (Math.abs(vecteur.x) >= Math.abs(vecteur.y))
      return (x2-point_x)/vecteur.x;
    else
      return (y2-point_y)/vecteur.y;
  }
  
  // Ajout version 3.2.2 car CDroiteAncetre ne renvoie plus la bonne valeur pour un segment
  public double abscisseMinimale() {
    // Cas d'un segment ayant ses extrmits confondues
    if ((vecteur.x == 0) && (vecteur.y == 0)) return 0;
    if (Math.abs(vecteur.x) >= Math.abs(vecteur.y))
      return (x1-point_x)/vecteur.x;
    else
      return (y1-point_y)/vecteur.y;
  }  
  
  // Ajout version 3.2.0
  // Pour ne rien faire lorsque CDroite.positionne appelle positionneNom()
  @Override
  public void positionneNom() {
  }
  

  public boolean appartientA(double x, double y) {
    if (!existe) return false;
    else return Kernel.appartientSegment(x,y,x1, y1, x2, y2);
  } 
  
  // Rajout pour la version 2.0

  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est confondu avec l'lment en question ou trs proche.
   * A t modifi par rapport  la version C++
  **/
  public boolean confonduAvec(CElementBase p) {
    CSegment pt;
    if (p.getClass() == this.getClass()) {
      pt = (CSegment) p;
      return (
        ((point1 == pt.point1) && (point2 == pt.point2))
        ||
        ((point1 == pt.point2)  && (point2 == pt.point1))
        );
    }
    else return false;
  }        
  /** distancePoint renvoie -1 dans le cas o le l'objet n'est pas dsignable 
   Modifie pour la version 1.9.5 car  cause des lieux d'objets,
   cette fonction ne doit plus utiliser de pointeurs sur point1 et point2
  **/
  @Override  
  public double distancePoint(double xp, double yp, boolean masquage) {
    if (!(existe) || (masquage && masque))
    return -1;
    else
    {
      VecteurDirecteur vec = new VecteurDirecteur(xext1,  yext1, xext2, yext2);
      double n = vec.norme();
      if (Kernel.zero(n)) return Kernel.distancePointPoint(xext1, yext1, xp, yp);
      else {
        Kernel.projetteOrtho(xp, yp, xext1, yext1, vec, pointr);
        if (appartientA(pointr.x,pointr.y)) {
          return Kernel.distancePointPoint(xp,yp,pointr.x, pointr.y);
        }
        else return -1;
      }      
    }
  }
  
  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (point1 == ancienPoint) point1 = nouveauPoint;
    if (point2 == ancienPoint) point2 = nouveauPoint;
  }
  
  @Override
  public boolean coincideAvec(CElementBase p) {
    if (p.getNature() != NatureObjet.NSegment) return false;
    CSegment seg = (CSegment) p;
    return ((point1.coincideAvec(seg.point1) && (point2.coincideAvec(seg.point2)))
      || (point1.coincideAvec(seg.point2) && (point2.coincideAvec(seg.point1))));
  }  

  @Override
  public long getNature() {
    return NatureObjet.NSegment;
  }
  
  /*
  @Override
  // Supprim version 3.2.4
  // CDroiteAncetre renvoie !horsFenetre qui est calcul dans positionne
  public boolean dansFenetre(DimensionFenetre dimf) {
  	return !horsFenetre;
  }
  */
  
  @Override
  public String chaineDesignation() {
    return getString("designationSegment");
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(point1) || point1.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(point2) || point2.estDefiniParObjetDans(listeOb));
	}
  

  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list)
    throws IOException {
     super.chargeDepuisFlux(inps, list);
     int ind1 = inps.readCInt();
     int ind2 = inps.readCInt();
     point1 = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
     point2 = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    point1 = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    point2 = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
  }
  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(point1);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(point2);
    oups.writeInt(ind2);
  }
  
  @Override
  public boolean contientParDefinition(CPointAncetre po)
  {
    return (point1 == po) || (point2 == po);
  }
  
  @Override
  public boolean estParalleleParDefinition(CDroiteAncetre dte){
  	return (dte == this) || (point1.appartientParDefinition(dte) && point2.appartientParDefinition(dte))
		|| dte.formentDroiteParallele(point1, point2);
  }
  
}
