/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package objets;

import java.io.IOException;

import mathgraph32.MtgFrame;
import types.DimensionFenetre;
import types.ErreurCalculException;
import types.NatureCalcul;
import dialogs.MtgDlg;
import dialogs.SuiteRecurrenteReelle3Dlg;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
// Suite rcurrente relle de la forme u(n+2)=f[n,u(n+1),u(n)] o f est une fonction relle de trois variables
public class CSuiteRec3 extends CSuiteRec {
	public CValeurAssocieeAVariable deuxiemeTerme;

	/** Creates new CSuiteRec */
	public CSuiteRec3() {
	}

	/**
	 * Constructeur pour cration d'objets
	 * @param liste
	 * @param nombreTermesInit
	 * @param premierTermeInit
	 */
	public CSuiteRec3(CListeObjets liste, CValeurAssocieeAVariable nombreTermesInit,
			CValeurAssocieeAVariable premierTermeInit, CValeurAssocieeAVariable deuxiemeTermeInit) {
		super(liste, nombreTermesInit, premierTermeInit);
		deuxiemeTerme = deuxiemeTermeInit;
	}

	/** Constructeur pour clonage
	 * 
	 * @param liste
	 * @param impProtoInit
	 * @param estElementFinalInit
	 * @param nomCalculInit
	 * @param fonctionAssocieeInit
	 * @param nombreTermesInit
	 * @param premierTermeInit
	 */
	public CSuiteRec3(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
			String nomCalculInit, CFoncNVar fonctionAssocieeInit,
			CValeurAssocieeAVariable nombreTermesInit, CValeurAssocieeAVariable premierTermeInit, CValeurAssocieeAVariable deuxiemeTermeInit) {
		super(liste, impProtoInit, estElementFinalInit, nomCalculInit, fonctionAssocieeInit, nombreTermesInit, premierTermeInit);
		deuxiemeTerme = deuxiemeTermeInit;
	}

	// Ajout version 4.8
	@Override
	public void initialisePourDependance() {
		super.initialisePourDependance();
		deuxiemeTerme.initialisePourDependance();
	}

	// Ajout version 4.8 (bug)
	@Override
	public boolean dependDe(CElementBase p) {
		if (elementTestePourDependDe == p) return dependDeElementTeste;
		// Corrig version 4.9.4
		// return memoriseDependance(super.dependDe(p) || premierTerme.dependDe(p) || fonctionAssociee.dependDe(p) || nombreTermes.dependDe(p));
		return memoriseDependance(super.dependDe(p) || deuxiemeTerme.dependDe(p));
	}

	// Ajout version 4.8 (bug)
	@Override
	public boolean dependDePourReclassement(CElementBase p) {
		if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
		return memoriseDependancePourRec(super.dependDePourReclassement(p) || deuxiemeTerme.dependDePourReclassement(p));
	}

	// Ajout version 4.8 (bug)
	@Override
	public boolean dependDePourBoucle(CElementBase p) {
		return super.dependDePourBoucle(p) || deuxiemeTerme.dependDePourBoucle(p);
	}

	@Override
	public boolean confonduAvec(CElementBase p) {
		boolean res = super.confonduAvec(p);
		if (!res) return false;
		CSuiteRec3 pt = (CSuiteRec3) p;
		return (deuxiemeTerme.confonduAvec(pt.deuxiemeTerme));
	}

	@Override
	public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
		int ind1 = listeSource.indexOf(fonctionAssociee);
		int ind2 = listeSource.indexOf(impProto);
		CValeurAssocieeAVariable premierTermeClone =
				(CValeurAssocieeAVariable) premierTerme.getClone(listeSource, listeCible);
		CValeurAssocieeAVariable deuxiemeTermeClone =
				(CValeurAssocieeAVariable) deuxiemeTerme.getClone(listeSource, listeCible);
		CValeurAssocieeAVariable nombreTermesClone =
				(CValeurAssocieeAVariable) nombreTermes.getClone(listeSource, listeCible);
		CElementBase ptelb = new CSuiteRec3(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
				estElementFinal, nomCalcul, (CFoncNVar)listeCible.get(ind1, CFoncNVar.class), nombreTermesClone,
				premierTermeClone, deuxiemeTermeClone);
		return ptelb;
	}

	@Override
	public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
		double u,v,w;
		nombreTermes.positionne(infoRandom, dimfen);
		premierTerme.positionne(infoRandom, dimfen);
		deuxiemeTerme.positionne(infoRandom, dimfen);
		existe = fonctionAssociee.existe && premierTerme.existe() && nombreTermes.existe();
		if (!existe) return;
		double dnbt = Math.floor(nombreTermes.rendValeur() + 0.5);
		existe = (dnbt >= 1) && (dnbt <= NombreMaxiTermesSuiteRec);
		if (!existe) return;
		if ((int) dnbt != nbt) {
			nbt = (int) dnbt;
			metAJourNombreTermes();
		}

		// Modifi version 3.1.2 pour autoriser la remise  jour des valeurs renvoyes par rand
		// u = premierTerme.rendValeur(infoRandom);
		u = premierTerme.rendValeur();
		valeurs[0] = u;
		v = deuxiemeTerme.rendValeur();
		valeurs[1] = v;
		indiceDernierTermeExistant = 0;
		int i = 2;
		while (i < nbt) {
			try {
				listeProprietaire.initialiseNombreIterations();
				// v = fonctionAssociee.calcul.resultatFonction(infoRandom, u);
				double [] arg = new double[3];
				arg[0] = i - 2;
				arg[1] = v;
				arg[2] = u;
				w = fonctionAssociee.calcul.resultatFonction(true, arg);
				if (Double.isNaN(v) || Double.isInfinite(v)) {
					break;
				}
				u = v;
				v = w;
				valeurs[i] = w;
				i++;
			} catch (ErreurCalculException e) {
				break;
			}
		}
		i--;
		indiceDernierTermeExistant = i;
	}

	// Modifi version 4.9.9
	@Override
	public String infoPourHistorique() {
		String ch = nomCalcul + " : " + getString("chinfo188") + "\n" + getString("chinfo84") + " " + fonctionAssociee.nom() + "\n" +
				getString("chinfo85") + " " + premierTerme.chaineInfo() + "\n" +
				getString("chinfo186") + " " + deuxiemeTerme.chaineInfo() + "\n" +
				getString("chinfo86") + " " + nombreTermes.chaineInfo();
		if (!existe) ch = ch + "\n" + getString("ch18");
		return ch;
	}
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
		return nomCalcul + " : " + getString("chinfo188") + " " + getString("chinfo84") + " " + fonctionAssociee.nom();
  }

	@Override
	public MtgDlg dialogueModification(MtgFrame frame) {
		return new SuiteRecurrenteReelle3Dlg(frame, this, true);
	}  

	@Override
	public long getNatureCalcul() {
		return NatureCalcul.NSuiteRecurrenteReelle3;
	}

	@Override
	public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
		super.read(inps, list);
		deuxiemeTerme = new CValeurAssocieeAVariable();
		deuxiemeTerme.read(inps, list);  	 
	}

	@Override
	public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
		super.write(oups, list);
		deuxiemeTerme.write(oups, list);  	 
	}

}
