/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package outils;

import objets.*;
import types.*;

import entreesSorties.MathGraphTimerTaskClig;

import java.awt.*;
import java.util.ListIterator;

import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

/**
 * Classe anctre de tous les outils de cration ou modification d'objets
 * @author Yves
 *
 */
public abstract class Outil {
	public MtgFrame frame;
	public boolean nbClignPair;
	private boolean avecClignotement;
	MathGraphTimerTaskClig timerTaskClignotement;
	public java.util.Timer timer;
	// Ajout version 4.8
	// Nombre maxi d'objets que l'on peut crer lors d'une cration itrative ou rcursive
	// public final static int nombreMaxObjets = 40000; // Modifi version 5.1
	public final static int nombreMaxObjets = 200000;
	// Ajout version 4.8 pour les outils de construction itrative et rcursive
	public final static int sansLigne = 0;
	public final static int avecPolygone = 1;
	public final static int avecLigneBrisee = 2;



	/** Creates a new instance of Outil */
	public Outil(MtgFrame frameInit, boolean avecClignotementInit) {
		frame = frameInit;
		avecClignotement = avecClignotementInit;
	}

	/** Appel lorsque l'outil est slectionn */
	public void select() {
		// Ajout version 4.9.9
		MtgInternalFrame cadre = frame.cadreActif();
		if (cadre!=null) {
			if ((frame.outilActif != this) && cadre.nameEditor.isVisible())
				cadre.nameEditor.cache();
		}
		//
		frame.outilActif.deselect();
		if (avecClignotement) {
			timerTaskClignotement = new MathGraphTimerTaskClig(frame);
			timer = new java.util.Timer();
			timer.schedule(timerTaskClignotement, 0l, 500);
			resetClignotement();
		}
		frame.listeExclusionDesignation.retireTout();
		frame.listeObjetsVisuels.retireTout();
		frame.outilActif = this;
	}

	/** Appel lorsque l'outil est dslectionn */
	// Moidfi version 4.9.9
	public void deselect() {
		MtgInternalFrame cadre = frame.cadreActif();
		if (cadre != null) {
			cadre.videDesignation();
			frame.outilPointageActif.designationDe(0);
			if (avecClignotement) {
				annuleClignotement();
				if (timer != null) timer.cancel();
				if (timerTaskClignotement != null) timerTaskClignotement.cancel();
				timerTaskClignotement = null;
				timer = null;
				// Modifi version 3.3.4
	      // La ligne suivante est indispensable car dans certains cas des points ont t nomms
	      // pour des outils de mesure et quand on annule la cration de mesure il faut que leur nom
	      // soit effac. Pour cela on efface toute la fentre.
				// frame.cadreActif().repaintFigureSansCalcul();
			}
			else annuleObjetsVisuels();
	    // System.gc(); Supprim version 5.0.0.5
		}
	}
	/**
	 * Ajout versions 5.0.
	 * Renverra true si on peut crer un point quand on clique sur un endroit vide de la figure.
	 * Devra tre redfini pour les outils pour lesquels ce n'est pas possible comme les outils de mesure.
	 * @return
	 */
	protected boolean creationPointPossible() {
		return false;
	}
	/**
	 * Mthode renvoyant true seulement si l'outil peut tre slectionn. A
	 * redfinir seulement pour les outils qui peuvent ventuellement tre
	 * impossibles  slectionner, par exemple l'outil de cration d'un cercle par
	 * son centre et son rayon si une longueur unit n'a pas t dfinie pour la
	 * figure
	 **/
	public boolean activationValide() {
		return (frame.cadreActif() != null);
	}
	
	/**
	 * Mthode renvoyant un message  afficher dans les cas o l'activation n'est pas valide
	 * @return
	 */
	public String messageActivationNonValide() {
		return null;
	}

	/** Appel pour initialiser le clignotement */
	public void resetClignotement() {
		nbClignPair = true;
		frame.outilPointageActif.aRedessinerDansPaint = true;
	}

	/** Appel pour arrter le clignotement */
	public void annuleClignotement() {
		frame.outilPointageActif.aRedessinerDansPaint = true;
		MtgInternalFrame cadre = frame.cadreActif();
		frame.listeClignotante.retireTout();
		frame.listeObjetsVisuels.retireTout();
		cadre.repaintFigureSansCalcul();
	}
	
	/** Appel pour mettre  zro la liste visuelle et rafficher la figure
	 */
	public void annuleObjetsVisuels() {
		frame.listeObjetsVisuels.retireTout();
		frame.cadreActif().repaintFigureSansCalcul();
	}

	/* Appel pour ajouter un lment graphique  la liste clignotante de l'applet */
	public void ajouteClignotementDe(CElementGraphique el) {
		frame.listeClignotante.add(el);
	}

	/**
	 * Appel pour exclure une lment des objets qui peuvent tre dsigns  la
	 * souris
	 */
	public void excluDeDesignation(CElementGraphique el) {
		frame.listeExclusionDesignation.add(el);
	}

	/*
	 * Appel lorsqu'un objet a t cliqu pour tre trait par l'outil par
	 * exemple le premier point pour crer une droite passant par deux points
	 */
	public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
	}

	/**
	 * Appel pour crer finalement l'objet une fois que tous les objets dont il
	 * dpend ont t dsigns
	 */
	public void creeObjet() {
	}
	
	/**
	 * A redfinir pour les objets qui crent l'objet aprs un clic droit
	 */
	public void creeObjetApresClicDroit() {
	}

	/*
	 * Verification de l'existence de l'objet point par pto Si l'objet existe
	 * dj, un message d'avertissement est affich et sinon l'objet pto est
	 * rajout  la liste principale de l'applet
	 */
	public void ajouteElement(CElementBase pto) {
		CListeObjets liste = frame.cadreActif().listePr;
		if (!liste.existeDeja(pto)) {
			frame.ajouteElement(pto);
			frame.positionne();
			frame.cadreActif().paneFigure.repaint();
		}
	}

	/**
	 * Mthode ajoutant les objets visuels  la liste listeObjetsVisuels de
	 * l'applet au fur et  mesure que l'on dsigne les obejts  la souris Doit
	 * tre redfini pour chaque outil utilisant une aide visuelle  la cration
	 */
	public void ajouteObjetsVisuels() {
	}

	/**
	 * Mthode ajoutant un rapporteur  la liste d'objets visuels. Utilise pour
	 * l'outil rapporteur et peut-tre plus tard les outils de cration d'arc de
	 * cercle. Renvoie un pointeur sur un point d'intersection entre le cercle
	 * extrieur du rapporteur et une demi droite joignant le centre de ce cercle
	 * avec le point suivant le pointeur souris
	 * Modifi version 5.0
	 */
	public CPointAncetre ajouteRapporteur(CPointAncetre origine) {
		MtgInternalFrame cadre = frame.cadreActif();
		CMathGrafDoc doc = cadre.document;
		double k = MtgFrame.coefMultForFonts(cadre.dimf.longueurAbscisses,  cadre.dimf.longueurOrdonnees);
		double coef = MtgFrame.screenIsHighResolution ? 2 : 1;
		// On cre d'abord un cercle de centre origine et de rayon fixe
		// qui servira de support extrieur au rapporteur
		StyleTrait styleTrait;
		CListeObjets liste = frame.listeObjetsVisuels;
		CCercle ptCercleRapExt = new CCercle(liste, null, false, Couleur
				.TraduitCouleur(1), false, StyleTrait.traitFin, origine.x, origine.y,
				 80*coef);
		frame.listeObjetsVisuels.add(ptCercleRapExt);
		// Puis un cercle intrieur
		CCercle ptCercleRapInt = new CCercle(liste, null, false, Couleur
				.TraduitCouleur(1), false, StyleTrait.traitFin, origine.x, origine.y,
				40*coef);
		frame.listeObjetsVisuels.add(ptCercleRapInt);
		// On cre un demi-droite de Origine vers PointSouris
		CDemiDroiteOA ptdemid = new CDemiDroiteOA(liste, null, false, Couleur
				.TraduitCouleur(1), false, StyleTrait.traitPointille, origine, frame
				.cadreActif().paneFigure.pointSouris);
		frame.listeObjetsVisuels.add(ptdemid);
		// On cre l'intersection de cette demi-droite avec le cercle extrieur
		// du rapporteur
		CIntDroiteCercle ptint = new CIntDroiteCercle(liste, null, false, ptdemid,
				ptCercleRapExt);
		frame.listeObjetsVisuels.add(ptint);
		CPointAncetre ptplb = new CPointLieBipoint(liste, null, false, Couleur
				.TraduitCouleur(1), false, 0d, 0d, false, "", 13, MotifPoint.Croix,
				ptint, 1, false);
		ptplb.cache();
		frame.listeObjetsVisuels.add(ptplb);
		// On rajoute des graduations au rapporteur
		// On cre d'abord une rotation d'angle 10
		CRotation ptrot10 = new CRotation(liste, null, false, origine, 10);
		frame.listeObjetsVisuels.add(ptrot10);
		// On cre une homothtie servant  crer les graduations de 10
		CHomothetie hom1 = new CHomothetie(liste, null, false, origine, 0.85);
		frame.listeObjetsVisuels.add(hom1);
		// Et une autre pour les graduations de 90
		CHomothetie hom2 = new CHomothetie(liste, null, false, origine, 0.5);
		frame.listeObjetsVisuels.add(hom2);
		// On cre les images par des rotations d'angle 10
		CPointAncetre ptp1 = ptplb;
		for (int i = 1; i <= 36; i++) {
			CPointImage ptim = new CPointImage(liste, null, false, Couleur
					.TraduitCouleur(3), MotifPoint.Pixel, 13, ptp1, ptrot10);
			ptim.cache();
			frame.listeObjetsVisuels.add(ptim);
			// On cre l'image du point cr par l'homothtie pour crer les
			// graduations
			// Pour les multiples de 90 elles sont rouges, sinon vertes
			CPointImage ptima = new CPointImage(liste, null, false, Couleur
					.TraduitCouleur(3), 13, MotifPoint.Pixel, ptim, hom1);
			ptima.cache();
			frame.listeObjetsVisuels.add(ptima);
			// On cre un segment graduation
			int coul;
			if ((i % 9) == 0) {
				coul = 4;
				styleTrait = StyleTrait.traitEpais;
			} else {
				coul = 2;
				styleTrait = StyleTrait.traitFin;
			}
			// On cre des graduations joignant le centre aux 90
			if ((i % 9) == 0) {
				CPointImage ptim2 = new CPointImage(liste, null, false, Couleur
						.TraduitCouleur(1), MotifPoint.Croix, 13, ptim, hom2);
				ptim2.cache();
				frame.listeObjetsVisuels.add(ptim2);
				CSegment ptseg90 = new CSegment(liste, null, false, origine, ptim2); // Segment
																																							// en
																																							// pointills
				frame.listeObjetsVisuels.add(ptseg90);
				CPointImage ptim3 = new CPointImage(liste, null, false, Couleur
						.TraduitCouleur(1), MotifPoint.Croix, 2, ptima, hom1);
				ptim3.cache();
				frame.listeObjetsVisuels.add(ptim3);
				String chai = new String();
				switch (i) {
				case 9:
				case 27:
					chai = "90";
					break;
				case 18:
					chai = "180";
					break;
				case 36:
					chai = "0";
					break;
				}
				/* Modifi version 5.0
				CCommentaire ptcom = new CCommentaire(liste, null, false, Couleur
						.TraduitCouleur(coul), 0d, 0d, -8d, -6d, false, ptim3, 0,
						StyleEncadrement.Sans, false, doc.couleurFond, chai,
						CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut);
				*/
				CCommentaire ptcom = new CCommentaire(liste, null, false, Couleur
						.TraduitCouleur(coul), 0d, 0d,(int)(-8d*k), (int)(-6d*k), false, ptim3, (int)(k*10),
						StyleEncadrement.Sans, false, doc.couleurFond, chai,
						CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut);
				// Version C++ : ptcom->DecaleNom(-75,-20);
				frame.listeObjetsVisuels.add(ptcom);
			}
			CSegment ptgrad = new CSegment(liste, null, false, Couleur
					.TraduitCouleur(coul), false, styleTrait, ptima, ptim);
			frame.listeObjetsVisuels.add(ptgrad);
			// Pour 10 et -10 on ajoute un commentaire de graduation
			if ((i == 1) || (i == 35)) {
				CPointImage ptim4 = new CPointImage(liste, null, false, Couleur
						.TraduitCouleur(1), MotifPoint.Croix, 13, ptima, hom1);
				ptim4.cache();
				frame.listeObjetsVisuels.add(ptim4);
				/* Modifi version 5.0
				CCommentaire ptcom = new CCommentaire(liste, null, false, Couleur
						.TraduitCouleur(coul), 0d, 0d, -10, -6, false, ptim4, 0,
						StyleEncadrement.Sans, false, doc.couleurFond, "10",
						CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut);
				*/
				CCommentaire ptcom = new CCommentaire(liste, null, false, Couleur
						.TraduitCouleur(coul), 0d, 0d, (int)(-10d*k), (int)(-6d*k), false, ptim4, (int)(k*10),
						StyleEncadrement.Sans, false, doc.couleurFond, "10",
						CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut);
				frame.listeObjetsVisuels.add(ptcom);
			}
			ptp1 = ptim;
		}
		return ptplb;
	}

	/**
	 * Fonction crant l'image de objet par la transformation ptTRans suivant la
	 * nature de cet objet. L'objet appartient  la liste d'objets liste
	 */
	CElementGraphique imageParTransformation(CListeObjets liste,
			CElementGraphique objet, CTransformation trans, Int nombreObjetsAjoutes) {
		nombreObjetsAjoutes.setValue(0);
		CElementGraphique elg = null;
		long nat = objet.getNature();
		Color coul = frame.couleurActive();
		if ((nat & NatureObjet.NToutPoint) != 0) {			
			elg = new CPointImage(liste, null, false, coul, false, 0d, 0d, false, frame.genereNomPourPoint(),
					frame.pref_indiceTaillePoliceNom, frame.stylePointActif(), (CPointAncetre) objet, trans, false);
		} else {
			if (nat == NatureObjet.NDroite) {
				elg = new CDroiteImage(liste, null, false, coul, false, 0d, 0d, false,
						"", frame.pref_indiceTaillePoliceNom, frame.styleTraitActif(),
						CDroite.abscisseNomParDefaut, (CDroite) objet, trans);
			} else {
				if (nat == NatureObjet.NDemiDroite) {
					elg = new CDemiDroiteImage(liste, null, false, coul, false, frame
							.styleTraitActif(), (CDemiDroite) objet, trans);
				} else {
					if (nat == NatureObjet.NCercle) {
						elg = new CCercleImage(liste, null, false, coul, false, frame
								.styleTraitActif(), (CCercle) objet, trans);
					} else {
						if (nat == NatureObjet.NArc) {
							elg = new CArcDeCercleImage(liste, null, false, coul, false,
									frame.styleTraitActif(), (CArcDeCercleAncetre) objet, trans);
						} else {
							if (nat == NatureObjet.NSegment) {
								CSegment seg = (CSegment) objet;
								CPointImage ptim1, ptim2;
								ptim1 = liste.pointImageDePar(seg.point1, trans);
								if (ptim1 == null) {
									ptim1 = new CPointImage(liste, null, false, coul, false, 0d, 0d, false, frame.genereNomPourPoint(),
											frame.pref_indiceTaillePoliceNom, frame.stylePointActif(), seg.point1, trans, false);
									frame.ajouteElement(ptim1);
									nombreObjetsAjoutes.setValue(nombreObjetsAjoutes.getValue() + 1);
								}
								ptim2 = liste.pointImageDePar(seg.point2, trans);
								if (ptim2 == null) {
									ptim2 = new CPointImage(liste, null, false, coul, false, 0d, 0d, false, frame.genereNomPourPoint(),
											frame.pref_indiceTaillePoliceNom, frame.stylePointActif(), seg.point2, trans, false);
									frame.ajouteElement(ptim2);
									nombreObjetsAjoutes.setValue(nombreObjetsAjoutes.getValue() + 1);
								}
								elg = new CSegment(frame.cadreActif().listePr, null, false,
										frame.couleurActive(), false, frame.styleTraitActif(),
										ptim1, ptim2);

							} else {
								if ((nat & (NatureObjet.NLigneBrisee | NatureObjet.NPolygone)) != 0) {
									CLignePolygonaleAncetre ligneanc = (CLignePolygonaleAncetre) objet;
									CPointImage ptim;
									if (nat == NatureObjet.NLigneBrisee)
										elg = new CLigneBrisee(liste, frame.couleurActive(), frame.styleTraitActif());
									else
										elg = new CPolygone(liste, frame.couleurActive(), frame.styleTraitActif());
									CLignePolygonaleAncetre ligne = (CLignePolygonaleAncetre) elg;
									ListIterator<CNoeudPointeurSurPoint> li = ligneanc.collectionPoints.listIterator();
									while (li.hasNext()) {
										CPointAncetre ant = (CPointAncetre) (li.next()).pointeurSurPoint;
										ptim = liste.pointImageDePar(ant, trans);
										if (ptim == null) {
											ptim = new CPointImage(liste, null, false, coul, false, 0d, 0d, false, frame.genereNomPourPoint(),
													frame.pref_indiceTaillePoliceNom, frame.stylePointActif(), ant, trans, false);
											// frame.ajouteElement(ptim);  Modifi version 5.0
											frame.ajouteElementSansEditionNom(ptim);
											nombreObjetsAjoutes.setValue(nombreObjetsAjoutes.getValue() + 1);
										}
										ligne.collectionPoints.add(new CNoeudPointeurSurPoint(liste, ptim));
									}
									// Il faut prparer les tableaux de coordonnes une fois que l'on connait le nombre de points de la ligne
									ligne.prepareTableauxCoordonnees();
								}
							}
						}
					}
				}
			}
		}
		return elg;
	}

	/**
	 * Fonction renvoyant la chane  afficher  l'tape i (i >= 1) comme
	 * indication Doit tre redfinie par les descendants devant crire plusieurs
	 * indications
	 * 
	 * @param i
	 *          : l'indice
	 * @return
	 */

	public String chaineIndication(int indEtape) {
		return "";
	}

	/**
	 * Renvoie le curseur utilis par l'outil quand le pointeur soursi entre dans
	 * une fentre MtgInternalFrame Est par dfaut une croix. A redfir pour les
	 * outils utilisant un autre curseur
	 * 
	 * @return
	 */
	public Cursor curseurUtilise() {
		return frame.outilPointageActif.curseurActif;
	}
	
  public static final String getString(String key) {
  	return MtgFrame.getString(key);
  }
	
}