/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Ajout version 4.8
 * Outil pour implmenter de faon rcursive une construction contenue dans le document de la figure dans la figure
 * @author Yves
 * La macro choisie doit tre compatible avec une implmentation rcursive.
 * Celle-ci sera carctrise par le nombre de fois que l'itration devra s'appliquer lors d'une itration et par le pas d'itration
 * qui sera le nombre d'objets finaux  sauter avant de reproduire la construction
 */

package outils;

import java.awt.Point;
import java.util.Vector;

import objets.CElementBase;
import objets.CElementGenerique;
import objets.CElementGraphique;
import objets.CElementLigne;
import objets.CImplementationProto;
import objets.CLigneBrisee;
import objets.CListeObjets;
import objets.CLongueur;
import objets.CMathGrafDoc;
import objets.CNoeudPointeurSurPoint;
import objets.CPointAncetre;
import objets.CPolygone;
import objets.CPrototype;
import objets.CSurfaceAncetre;
import types.NatureCalcul;
import types.NatureObjet;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import dialogs.ChoixObjetsNonGraphiquesPourImplementationProtoDlg;
import dialogs.ConstructionRecursiveDlg;

public class OutilConstructionRecursiveImplementationFigure extends Outil {
	public final static int sansLigne = 0;
	public final static int avecPolygone = 1;
	int pasRec = 1;
	int nbImp = 4;
	int profondMax = 2;
	int nbSourcesPourIter;
	int choixLigne;
	boolean adapterStyle;
	boolean creationNiveauMax;
	Vector<CPrototype> tableProto;
	int indiceElementSourceGraphique;
	int nombreObjetsCalculADetruire; // Ajout version 3.4.0.3 : il faut que, dans le cas o une bote de dialogue
	// de choix d'objets sources non graphiques a t utilise et des calculs y ont t crs, qu'ils soient dtruits dans deselect()
	// lorsque la construction ne va pas  son terme.
	
	public OutilConstructionRecursiveImplementationFigure(MtgFrame frameInit) {
    super(frameInit, true);
  }
	
	private boolean choixProto() {
    ConstructionRecursiveDlg dlg = new ConstructionRecursiveDlg(frame);
		dlg.setLocationRelativeTo(frame);
    dlg.setVisible(true);
    if (!dlg.retourOK()) {
			frame.activeOutilCapture();
			return false;
    }
    else {
			nbImp = dlg.nbImp;
			pasRec = dlg.pasRec;
			profondMax = dlg.profondMax;
			nbSourcesPourIter = dlg.nbSources;
			tableProto = dlg.tableProto;
			choixLigne = dlg.choixLigne;
			adapterStyle = dlg.adapterStyle;
			creationNiveauMax = dlg.creationNiveauMax;
		}
		return dlg.retourOK();
	}
	
  @Override
	public void select() {
    super.select();
    nombreObjetsCalculADetruire = 0;
    boolean choixProto = choixProto();
    if (!choixProto) {
    	frame.activeOutilCapture();
    	return;
    }
    CListeObjets liste = frame.cadreActif().listePr;
    // Il peut arriver que la construction n'utilise aucun objet source
    CPrototype proto = tableProto.get(0);
    if (proto.nbObjSources == 0) {
    	creeObjet();
    	frame.activeOutilCapture();
    	return;
    }
    if (proto.natureSources != CPrototype.ConstructionEltsGraphiquesSeuls) {
    	// Choix des objets sources de type calcul
    	// Modification version 3.5 : Si la macro ne possde qu'un seul objet source et que c'est un repre
    	// et qu'il n'y a qu'un repre dans la figure (qui ne soit pas objet final, on l'affecte automatiquement)
    	if ((proto.nombreSourcesTypeCalcul() == 1) && (proto.nombreObjetsSourcesCalcul(NatureCalcul.NRepere) == 1)
    			&& liste.nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NRepere) == 1) {
    		CElementGenerique gen = (CElementGenerique)proto.get(0);
    		gen.elementAssocie = liste.premierRepere();	    		
    	}
    	else {
	    	// ChoixObjetsNonGraphiquesPourImplementationProtoDlg dlg = new ChoixObjetsNonGraphiquesPourImplementationProtoDlg(frame, proto); // Modifi version 5.1
    		ChoixObjetsNonGraphiquesPourImplementationProtoDlg dlg = new ChoixObjetsNonGraphiquesPourImplementationProtoDlg(frame, proto, true);
	  		dlg.setLocationRelativeTo(frame);
	    	dlg.setVisible(true);
	    	if (dlg.retourOK()) {
	    		nombreObjetsCalculADetruire = dlg.nombreCalculsCrees;
	    	}
	    	else {
	      	frame.activeOutilCapture();
	      	return;    		
	    	}
	    }
    }
    if (proto.natureSources == CPrototype.ConstructionEltsCalculsSeuls) {
    	creeObjet();
    	return;
    }
    else {
  		frame.outilPointageActif = frame.outilPointageCre;
  		indiceElementSourceGraphique = proto.nombreSourcesTypeCalcul();
  		CElementGenerique gen = (CElementGenerique)proto.get(indiceElementSourceGraphique);
  		long nat = gen.natureGraphique;
  		if (indiceElementSourceGraphique == proto.nbObjSources) {
  			creeObjet();
  			return;
  		}
  		frame.outilPointageActif.designationDe(CElementBase.natureGraphiquePourImplementationProto(nat));
  		frame.indication(proto.chaineIndicationImplementationProto(indiceElementSourceGraphique + 1, nat)
  				+ ' ' + getString("ch158"));
    }
  }


	@Override
	public void deselect() {
		super.deselect();
		if (nombreObjetsCalculADetruire != 0) 
			frame.detruitNDerniersElements(nombreObjetsCalculADetruire);
	}

	public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
		CPrototype proto = tableProto.get(0);
		CElementGenerique gen = (CElementGenerique)proto.get(indiceElementSourceGraphique);
		gen.elementAssocie = elg;
		indiceElementSourceGraphique++;
		if (indiceElementSourceGraphique == proto.nbObjSources) creeObjet();
		else {
			CElementGenerique gen2 = (CElementGenerique)proto.get(indiceElementSourceGraphique);			
  		long natgraph = gen2.natureGraphique;
  		frame.outilPointageActif.designationDe(CElementBase.natureGraphiquePourImplementationProto(natgraph));
  		frame.indication(proto.chaineIndicationImplementationProto(indiceElementSourceGraphique+1, natgraph)
  				+ ' ' + getString("ch158"));
			frame.outilActif.ajouteClignotementDe(elg);
			frame.outilActif.resetClignotement();
		}
	}

	public void creeObjet() {
		MtgInternalFrame cadre = frame.cadreActif();
		CListeObjets liste = cadre.listePr;
    Vector<CNoeudPointeurSurPoint> listePoints = new Vector<CNoeudPointeurSurPoint> (); // Pour joindre ventuellement les points de dernire gnration
 		int initialSize = liste.size();
		CLongueur ancienneLongueurUnite = liste.pointeurLongueurUnite;
		CImplementationProto impProto = new CImplementationProto(liste, tableProto.get(0));
		// On cre une liste forme des objets sources fixes et une autre forme des objets sources variables.
		CListeObjets listeObjetsSourcesConst = new CListeObjets();
		CPrototype proto0 = tableProto.get(0);
		for (int i = 0; i < nbSourcesPourIter; i++) {
			CElementGenerique eg = (CElementGenerique)(proto0.get(i));
			listeObjetsSourcesConst.add(eg.elementAssocie);
		}
		CListeObjets listeObjetsSourcesVar = new CListeObjets();
		for (int i = nbSourcesPourIter; i < proto0.nbObjSources; i++) {
			CElementGenerique gen = (CElementGenerique) proto0.get(i);
			listeObjetsSourcesVar.add(gen.elementAssocie);
		}
		// int indprot = liste.indexOf(impProto); // Bug corrig version 4.9.2.1
		int indprot = liste.size(); // Pour corriger le bug
		int profond = 0;
		impProto.implementeRecursif(liste, tableProto, cadre, listeObjetsSourcesConst, listeObjetsSourcesVar,
				pasRec, nbImp, creationNiveauMax, profond, profondMax, listePoints);
		// liste.reconstruitChainesCalcul(indprot + 1, indprot + proto0.size() - proto0.nbObjSources);
		liste.traiteImplementationsProtoSuccessivesRecur(indprot, proto0, cadre);
		if (adapterStyle) {
	    for (int i= liste.indexOf(impProto)+1; i < liste.size(); i++) {
	    	CElementBase elb = liste.get(i);
	    	if (elb.estElementFinal && (elb.estDeNature(NatureObjet.NToutObjet))) {
	    		CElementGraphique elg = (CElementGraphique) elb;
	    		elg.donneCouleur(frame.couleurActive());
	    		if (elg.estDeNature(NatureObjet.NToutPoint)) ((CPointAncetre)elg).donneMotif(frame.stylePointActif());
	    		else
	    		if (elg.estDeNature(NatureObjet.NObjetLigne)) ((CElementLigne)elg).donneStyle(frame.styleTraitActif());
	    		else
	      	if (elg.estDeNature(NatureObjet.NSurface)) ((CSurfaceAncetre)elg).styleRemplissage = frame.styleRemplissageActif();    		
	    	}
	    }
    }
		// On regarde si on moins un des objets finaux crs existe
		// liste.positionne(false, cadre.dimf); Inutile car liste dj positionne dans CImplementationProto.implemente
		boolean auMoinsUnExiste = false;
		// Optimis version 3.4.0.1
		for (int i= liste.indexOf(impProto)+1; (i < liste.size()) && !auMoinsUnExiste; i++) {
			CElementBase elb = liste.get(i);
			auMoinsUnExiste = auMoinsUnExiste || elb.existe;
		}

		// Si aucun lment final existe on prvient l'utilisateur
		if (!auMoinsUnExiste) {
			if (!frame.dialogueConfirmation_YES_NO(getString("ch135"))) {
				liste.pointeurLongueurUnite = ancienneLongueurUnite;
				// Il faut retirer de la liste les lments qui ont t rajouts en cas de refus
				// Il y en a autant que d'objets finaux et intermdiaires plus l'implmentation de prototype elle-mme
				frame.detruitNDerniersElements(liste.size() - initialSize);
				frame.activeOutilCapture();
				return;    						
			}
		}
    // Si demand un polygone ou une ligne on joint tous les points crs.
    if (choixLigne != sansLigne) {
	    if (choixLigne == avecPolygone) {
				listePoints.add(new CNoeudPointeurSurPoint(liste, listePoints.firstElement().pointeurSurPoint));
				CPolygone poly = new CPolygone(liste, listePoints, frame.couleurActive(), frame.styleTraitActif());
				liste.add(poly);
				poly.positionne(false, frame.cadreActif().dimf);
	    }
	    else {
				CLigneBrisee ligne = new CLigneBrisee(liste, listePoints, frame.couleurActive(), frame.styleTraitActif());
				liste.add(ligne);
				ligne.positionne(false, frame.cadreActif().dimf);
	    }
    }
    
		// On sauvegarde l'tat de la figure
		cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil115"));
		// Version 4.5 : Inutile de recalculer la figure car cela a dj t fait dans CImplementationProoto.implemente
	  // cadre.repaintFigureAvecCalcul();
		// Erreur corrige version 4.9.2.1 : Il faut recalculer toute la figure car lors du choix des objets sources on a pu
		// entrer directement une formule et cet objet-l n'aura pas t positionn
		// cadre.repaintFigureSansCalcul();
		cadre.repaintFigureAvecCalcul();
    // Si tout s'est bien pass il ne faut pas que deselect dtruise les ventuels calculs crs dans
    // la bote de dialogue de choix d'objets sources graphiques.
    nombreObjetsCalculADetruire = 0;
	  frame.activeOutilCapture();
	}
	
  @Override
  public boolean activationValide() {
  	CMathGrafDoc doc = frame.cadreActif().document;
    if (doc.tablePrototypes.size() == 0) return false;
    return (doc.nombrePrototypesPourIterOuRecur() != 0);
  }
  
	@Override
	public String messageActivationNonValide() {
		CMathGrafDoc doc = frame.cadreActif().document;
		String ch = getString("messagePasDeConstruction");
		if (doc.tablePrototypes.size() == 0) return ch;
		else return ch + getString("utilisable");
	}	

}