/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;

import objets.CDroiteDirectionFixe;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CPointAncetre;
import types.NatureObjet;

import mathgraph32.MtgFigurePanel;
import mathgraph32.MtgFrame;

public class OutilDroiteHorizontale extends Outil {
  CPointAncetre point1;

	public OutilDroiteHorizontale(MtgFrame frameInit) {
		super(frameInit, false); // False car pas de clignotement ncessaire
	}
	/**
	 * Ajout versions 5.0.
	 * Renverra true si on peut crer un point quand on clique sur un endroit vide de la figure.
	 * Devra tre redfini pour les outils pour lesquels ce n'est pas possible comme les outils de mesure.
	 * @return
	 */
	protected boolean creationPointPossible() {
		return true;
	}
	
  @Override
  public void select() {
    super.select();
    point1 = null;
    frame.outilPointageCre.designationDe(NatureObjet.NToutPoint);
    frame.outilPointageActif = frame.outilPointageCre;
    frame.indication(getString("droiteHorizontale1"));
    ajouteObjetsVisuels();
  }

  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    if (point1 == null) {
    	// Ajout version 5.0
    	if (elg == null) {
    		point1 = frame.ajoutePoint(pointClique.getX(), pointClique.getY(), false);
    	}
    	//
    	else point1 = (CPointAncetre) elg;
      creeObjet();
    }
  }
  
  @Override
  public void creeObjet() {
  	CDroiteDirectionFixe pt = new CDroiteDirectionFixe(frame.cadreActif().listePr, 
      frame.couleurActive(), frame.styleTraitActif(), point1, true, frame.pref_indiceTaillePoliceNom);
   if (frame.ajouteElementAvecVerification(pt)) {
     frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil33"));
     frame.cadreActif().repaintFigureAvecCalcul();
   }
   annuleClignotement(); // Pour effacer les objets visuels			   
   select();
  }
  
  @Override
  public void ajouteObjetsVisuels() {
    CListeObjets liste = frame.listeObjetsVisuels;
    MtgFigurePanel pane = frame.cadreActif().paneFigure;
    CDroiteDirectionFixe dte = new CDroiteDirectionFixe(liste,frame.couleurActive(),
      frame.styleTraitActif(), pane.pointSouris, true, 13);
    liste.add(dte);
  }
}