/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package outils;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Point;
import dialogs.EquationDroiteDlg;
import objets.CAffichageLieAPoint;
import objets.CCommentaire;
import objets.CDroiteAncetre;
import objets.CElementGenerique;
import objets.CElementGraphique;
import objets.CImplementationProto;
import objets.CLatex;
import objets.CListeObjets;
import objets.CPrototype;
import objets.CRepere;
import types.NatureCalcul;
import types.NatureObjet;
import types.StyleEncadrement;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class OutilEquationDroite extends Outil {
	private CDroiteAncetre droite1;
	private int nbDecimales;
	private CRepere repere;
	private boolean effacementFond;
	private int styleEncadrement;
	private Color couleurFond;
	private int alignementHorizontal;
	private int alignementVertical;
	private int indiceTaillePolice;
  private Point pointDesigne = new Point();	
	
	public OutilEquationDroite(MtgFrame frameInit) {
    super(frameInit, true);
  }
	
  @Override
	public void select() {
    super.select();
    droite1 = null;
    frame.outilPointageActif = frame.outilPointageClic;
    frame.indication(getString("equationDroite1"));
  }
	
	
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
  	if (frame.outilPointageActif == frame.outilPointageClic) {
      int x = pointClique.x;
      int y = pointClique.y;
      pointDesigne.x = x;
      pointDesigne.y = y;
      CListeObjets liste = frame.cadreActif().listePr;
      // Avant l'affichage du dialogue on cre un message qui sera ajout  la liste des objets clignotants
      // pour matrialiser l'emplacement de l'affichage
      /* Modifi version 5.0
      CCommentaire comClignotant = new CCommentaire(liste, null, false, frame.couleurActive(), x, y, 0d, 0d, false, null, 2,
      		StyleEncadrement.Sans, false, frame.cadreActif().couleurFond, getString("ch80"),
  				CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut);
  		*/
      CCommentaire comClignotant = new CCommentaire(liste, null, false, frame.couleurActive(), x, y, 0d, 0d, false, null, (int)(MtgFrame.coefMultForSystemFonts*13),
      		StyleEncadrement.Sans, false, frame.cadreActif().couleurFond, getString("ch80"),
  				CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut);
      comClignotant.positionne(false, frame.cadreActif().dimf);
      ajouteClignotementDe(comClignotant);
      resetClignotement();
      frame.outilPointageActif = frame.outilPointageCre;
      frame.outilPointageActif.designationDe(NatureObjet.NDroite);
      frame.outilPointageActif.reset();
      frame.indication(getString("equationDroite2"));
  	}
  	else {
	    if (droite1 == null) {
	      droite1 = (CDroiteAncetre) elg;
	      ajouteClignotementDe(elg);
	      frame.cadreActif().videDesignation();
	    	frame.indication("");
	    	EquationDroiteDlg dlg = new EquationDroiteDlg(frame);
	  		dlg.setLocationRelativeTo(frame);
	    	dlg.setVisible(true);
	    	if (dlg.retourOK()) { 
		    	nbDecimales = dlg.paneListeDecimales.nombreDecimales();
		    	repere = dlg.listeReperes.repereChoisi();
		    	effacementFond = dlg.paneModeAffichage.effacementFond();
		    	styleEncadrement = dlg.paneModeAffichage.styleEncadrement();
		    	couleurFond = dlg.paneModeAffichage.couleurFond();
		    	alignementHorizontal = dlg.paneModeAffichage.alignementHorizontal();
		    	alignementVertical = dlg.paneModeAffichage.alignementVertical();
		    	indiceTaillePolice = dlg.paneTaillePolice.indiceTaille();
		    	creeObjet();
	    	}
	    	select();
	    }
  	}
  }	
  @Override
  public void creeObjet() {
  	MtgInternalFrame cadre = frame.cadreActif();
  	CPrototype proto = frame.chargeConstructionDansDossierMtgFrame("EquationDroite"+Integer.toString(nbDecimales) + "Dec.mgc");
  	CListeObjets liste = cadre.listePr;
  	//On implmente maintenant ce prototype dans la figure
		((CElementGenerique)proto.get(0)).elementAssocie = repere;
		((CElementGenerique)proto.get(1)).elementAssocie = droite1;
		CImplementationProto impProto = new CImplementationProto(liste, proto);
		impProto.implemente(liste, proto, cadre, 0); // Modifi version 4.8
		impProto.nomProto = getString("equationDroiteDlg1");
		// On modifie le dernier objet implment qui est un affichage LaTeX
		// suivant les choix de la bote de dialogue
		CLatex latex = (CLatex) liste.get(liste.size() - 1);
		latex.effacementFond = effacementFond;
		latex.encadrement = styleEncadrement;
		latex.couleurFondAffichage = couleurFond;
		latex.donneCouleur(frame.couleurActive());
		latex.placeEn(pointDesigne.x, pointDesigne.y);
		latex.indiceTaillePolice = indiceTaillePolice; // Ajout version 5.0
		// Correction version 5.0
		latex.alignementHorizontal = alignementHorizontal;
		latex.alignementVertical = alignementVertical;
		//
    cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil143"));
    cadre.repaintFigureAvecCalcul();
    select();
  }
	
  @Override
	public Cursor curseurUtilise() {
		if (frame.outilPointageActif == frame.outilPointageClic) return MtgFrame.cornerCursor;
		else return MtgFrame.crossHairCursor;
	}
  
  @Override
	public boolean activationValide() {
		CListeObjets liste = frame.cadreActif().listePr;
		return (liste.nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NRepere)!=0);
	}
	
	@Override
	public String messageActivationNonValide() {
		CListeObjets liste = frame.cadreActif().listePr;
		if (liste.nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NRepere)==0) return getString("messagePasDeRepere");
		else return "";
	}		

}