/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;
import mathgraph32.MtgFigurePanel;
import mathgraph32.MtgFrame;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CPointAncetre;
import objets.CPointImage;
import objets.CPointParSommeVect;
import objets.CTransParVect;
import objets.CVecteur;
import types.Couleur;
import types.MotifPoint;
import types.NatureObjet;
import types.StyleFleche;
import types.StyleTrait;

public class OutilPointParSommeVect extends Outil {
	private CVecteur vecteur1;
	private CVecteur vecteur2;
	private CPointAncetre point1;
	
  /** Creates a new instance of OutilPointParSommeVect */
  public OutilPointParSommeVect(MtgFrame frameInit) {
    super(frameInit, true);
  }

  @Override
  public void select() {
    super.select();
    vecteur1 = null;
    vecteur2 = null;
    point1 = null;
    frame.outilPointageCre.designationDe(NatureObjet.NVecteur);
    frame.outilPointageActif = frame.outilPointageCre;
    frame.indication(getString("pointParSommeVect1"));
  }
	
  @Override       
  public void creeObjet() {
    CPointParSommeVect pt = new CPointParSommeVect(frame.cadreActif().listePr, null, false, frame.couleurActive(),
    		false, 0, 3, false, frame.genereNomPourPoint(), frame.pref_indiceTaillePoliceNom,
    		frame.stylePointActif, point1, vecteur1, vecteur2, false);
    if (frame.ajouteElementAvecVerification(pt)) {
      frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil133"));
      frame.cadreActif().repaintFigureAvecCalcul();
    }
    // annuleClignotement();
    point1 = null;
  }

  @Override
  public void ajouteObjetsVisuels() {
    CListeObjets liste = frame.listeObjetsVisuels;
    MtgFigurePanel pane = frame.cadreActif().paneFigure;
    
    CTransParVect trans1 = new CTransParVect(liste, null, false, vecteur1);
    frame.ajouteObjetVisuel(trans1);
    CPointImage ptim1 = new CPointImage(liste, null, false, Couleur.TraduitCouleur(1),
       MotifPoint.PetitRond, 13, pane.pointSouris, trans1);
    ptim1.cache();
    frame.ajouteObjetVisuel(ptim1);
    CVecteur vect1 = new CVecteur(liste, null, false, vecteur1.couleur, false, StyleTrait.traitFin, pane.pointSouris, ptim1,
				StyleFleche.FlecheCourtePleine);
    frame.ajouteObjetVisuel(vect1);
    CTransParVect trans2 = new CTransParVect(liste, null, false, vecteur2);
    frame.ajouteObjetVisuel(trans2);
    CPointImage ptim2 = new CPointImage(liste, null, false, Couleur.TraduitCouleur(1),
       MotifPoint.PetitRond, 13, ptim1, trans2);
    frame.ajouteObjetVisuel(ptim2);
    CVecteur vect2 = new CVecteur(liste, null, false, vecteur2.couleur, false, StyleTrait.traitFin, ptim1, ptim2,
				StyleFleche.FlecheCourtePleine);
    frame.ajouteObjetVisuel(vect2);
  }
  
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    if (vecteur1 == null) {
      vecteur1 = (CVecteur) elg;
      excluDeDesignation(elg);
      ajouteClignotementDe(elg);
      resetClignotement();
      frame.indication(getString("pointParSommeVect2"));
    }
    else {
      if (vecteur2 == null) {
        vecteur2 = (CVecteur) elg;
        ajouteClignotementDe(elg);
        frame.indication(getString("pointParSommeVect3"));
        frame.outilPointageCre.designationDe(NatureObjet.NToutPoint);
        resetClignotement();
        ajouteObjetsVisuels();
      }
      else {
      	if (point1 == null) {
      		point1 = (CPointAncetre) elg;
      		creeObjet();
      	}
      }
    }
  }
  
  @Override
	public boolean activationValide() {
		return frame.cadreActif().listePr.nombreObjetsParNatureVisibles(NatureObjet.NVecteur)>0;
	}
	
	@Override
	public String messageActivationNonValide() {
		return getString("messagePasDeVecteur");
	}	
  
}