/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.*;

import objets.*;
import types.*;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

/**
 *
 * @author Yves
 */
public class OutilSup extends Outil {
  private CElementGraphique objet;
  private CElementBase elementEffectifASupprimer;
  
  /** Creates a new instance of OutilSup */
  public OutilSup(MtgFrame frameInit) {
    super(frameInit, true);
  }
  
  
  /** Appel lorsque l'outil est slectionn */
  @Override
  public void select() {
    super.select();
    objet = null;
    frame.outilPointageSup.designationDe(NatureObjet.NToutObjet);
    frame.outilPointageActif = frame.outilPointageSup;
    frame.indication(getString("suppression1"));
  }
  
	@Override
  public Cursor curseurUtilise() {
		return MtgFrame.supCursor;
	}
  
	// Modifi version 3.3.3.1
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    if (objet == null) {
      objet = elg;
      MtgInternalFrame cadre = frame.cadreActif();    
      CListeObjets liste = cadre.listePr;
      elementEffectifASupprimer = elg.antecedentDirect();
      ajouteClignotementDe(elg);
      resetClignotement();
      if (elementEffectifASupprimer.getNature() == NatureObjet.NTransformation) {
      	// Si l'objet antcdent direct est une transformation, on regarde s'il y a d'autres images par
      	// cette transformation  dtruire. Si oui, on ne dtruit pas la transformation.
      	// Sinon on dtruit la transformation et ses dpendants.
      	if (liste.nombreImagesParTransformationNonDepDe((CTransformation) elementEffectifASupprimer, elg) > 0) elementEffectifASupprimer = elg;
      }  
      if (elementEffectifASupprimer.getNature() == NatureObjet.NBipoint)  {
      	CBipoint bip = (CBipoint) elementEffectifASupprimer;
      	CPointLieBipoint ptlb1 = bip.pointReelDansListe(1);
      	// if (ptlb1 != elg) ajouteClignotementDe(ptlb1); // Modifi version 4.8
      	// Dans le cas d'un lment final de construction il se peut qu'un seul des deux points normalement
      	// associs au bipoint ait t cr
      	if ((ptlb1 != null) && (ptlb1 != elg)) ajouteClignotementDe(ptlb1);
      	CPointLieBipoint ptlb2 = bip.pointReelDansListe(2);
      	// if (ptlb2 != elg) ajouteClignotementDe(ptlb2); // Idem modifi version 4.8
      	if ((ptlb2 != null) && (ptlb2 != elg)) ajouteClignotementDe(ptlb2);
      	if ((ptlb1 != null) && (ptlb2 != null)) { // Ajout version 4.8
	        if (!frame.dialogueConfirmation_YES_NO(getString("ch38"))) {
	        	select();
	        	return;
	        }
      	}
      	// Modification version 5.1.4
        // if (frame.cadreActif().listePr.nombreDependants(elementEffectifASupprimer) >= 3) {
      	if (frame.cadreActif().listePr.nombreDependants(elementEffectifASupprimer) >= 4) {
          ajouteClignotementDe(elg);
          resetClignotement();
          if (!frame.dialogueConfirmation_YES_NO(getString("ch37"))) {
            select();
            return;
          }
         }
        creeObjet();        
      }
      else {
	      if (frame.cadreActif().listePr.nombreDependants(elg) >= 2) {
	        ajouteClignotementDe(elg);
	        resetClignotement();
	        if (!frame.dialogueConfirmation_YES_NO(getString("ch37"))) {
	          select();
	          return;
	        }
	       }
	     creeObjet();
      }
    }
  }
   
  @Override
  public void creeObjet() {
  	annuleClignotement();
  	MtgInternalFrame cadre = frame.cadreActif();
    frame.retireDependants(elementEffectifASupprimer);
    // Ajout version 4.8.0 : IL faut remettre  jour les index de cahque lment de la liste (CElementBase.index)
    cadre.listePr.updateIndexes();
    //
    cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil16"));
    cadre.repaintFigureAvecCalcul();
    cadre.videListesPourConstructions();  // On remet  jour une construction ventuellement commence
    // frame.activeOutilCapture();
    select();
  }
 
  
}
