/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package outils;
import java.awt.Color;
import java.util.Vector;

import objets.CCercleOA;
import objets.CElementGenerique;
import objets.CImplementationProto;
import objets.CIntCercleCercle;
import objets.CListeObjets;
import objets.CNoeudPointeurSurPoint;
import objets.CPointAncetre;
import objets.CPointLieBipoint;
import objets.CPolygone;
import objets.CPrototype;
import objets.CSegment;
import types.StyleTrait;
import mathgraph32.MtgFigurePanel;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;


public class OutilTriangleEquilateralDirect extends OutilPar2pts {

  /** Creates a new instance of OutilTriangleEquilateralDirect */
  public OutilTriangleEquilateralDirect(MtgFrame frameInit) {
    super(frameInit);
  }
	/**
	 * Ajout versions 5.0.
	 * Renverra true si on peut crer un point quand on clique sur un endroit vide de la figure.
	 * Devra tre redfini pour les outils pour lesquels ce n'est pas possible comme les outils de mesure.
	 * @return
	 */
	protected boolean creationPointPossible() {
		return true;
	}
 
  /* Appel pour crer finalement l'objet une fois que tous les objets dont il
   * dpend ont t dsigns */
  @Override
  public void creeObjet() {
		MtgInternalFrame cadre = frame.cadreActif();
		CListeObjets liste = cadre.listePr;      
		CPrototype proto = frame.chargeConstructionDansDossierMtgFrame("TriangleEquilateralDirect.mgc");
		((CElementGenerique)proto.get(0)).elementAssocie = point1;
		((CElementGenerique)proto.get(1)).elementAssocie = point2;
		CImplementationProto impProto = new CImplementationProto(liste, proto);
		impProto.implemente(liste, proto, cadre, 0); // Modifi version 4.8
		impProto.nomProto = getString("triangleEquilateral");  		
		CPointAncetre pt1 = (CPointAncetre) liste.get(liste.size() - 1);
		pt1.donneCouleur(frame.couleurActive());
		pt1.donneMotif(frame.stylePointActif);
		pt1.donneNom(frame.genereNomPourPoint()); // Ajout version 4.7.0.1
		pt1.indiceTailleNom = frame.pref_indiceTaillePoliceNom; // Ajout version 4.9.1.1
		CSegment seg1 = new CSegment(liste, null, false, frame.couleurActive(), false,
				frame.styleTraitActif(), point1, point2);
		if (!liste.existeDeja(seg1)) liste.add(seg1);
		CSegment seg2 = new CSegment(liste, null, false, frame.couleurActive(), false,
				frame.styleTraitActif(), point1, pt1);
		if (!liste.existeDeja(seg2)) liste.add(seg2);
		CSegment seg3 = new CSegment(liste, null, false, frame.couleurActive(), false,
				frame.styleTraitActif(), point2, pt1);
		if (!liste.existeDeja(seg3)) liste.add(seg3);
		// Version 5.0 : On rajoute un polygone
		Vector<CNoeudPointeurSurPoint> collectionPoints = new Vector<CNoeudPointeurSurPoint>();
		CNoeudPointeurSurPoint noeud = new CNoeudPointeurSurPoint(liste, point1);
		collectionPoints.add(noeud);
		noeud = new CNoeudPointeurSurPoint(liste, point2);
		collectionPoints.add(noeud);
		noeud = new CNoeudPointeurSurPoint(liste, pt1);
		collectionPoints.add(noeud);
		noeud = new CNoeudPointeurSurPoint(liste, point1);
		collectionPoints.add(noeud);
		CPolygone pol= new CPolygone(liste, collectionPoints, frame.couleurActive(), frame.styleTraitActif());
		if (!liste.existeDeja(pol)) liste.add(pol);
		//

		// Ajout version 4.9.9
		cadre.nameEditor.associeA(pt1);
		cadre.nameEditor.setPositionOnInternaleFrame();
		//
    frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil137"));
    annuleClignotement();
    cadre.repaintFigureAvecCalcul();    
    select();  	
  }
  
  @Override
  public void ajouteObjetsVisuels() {
    MtgFigurePanel pane = frame.cadreActif().paneFigure;
		CListeObjets liste = frame.listeObjetsVisuels;
  	CSegment seg1 = new CSegment(liste, null, true, frame.couleurActive(), false,
  			frame.styleTraitActif(), point1, pane.pointSouris);
  	frame.ajouteObjetVisuel(seg1);
  	CCercleOA cerc1 = new CCercleOA(liste, null, false, Color.black, true, StyleTrait.traitPointille, point1, pane.pointSouris);
  	frame.ajouteObjetVisuel(cerc1);
  	CCercleOA cerc2 = new CCercleOA(liste, null, false, Color.black, true, StyleTrait.traitPointille, pane.pointSouris, point1);
  	frame.ajouteObjetVisuel(cerc2);
  	CIntCercleCercle inter = new CIntCercleCercle(liste, null, false, cerc1, cerc2);
  	frame.ajouteObjetVisuel(inter);
  	// On ne se sert que d'un des deux points d'intersection qui est dans le bon sens.
    CPointLieBipoint ptl = new CPointLieBipoint(liste, null, false, frame.couleurActive(),
        false, 0d, 0d, false, "", 2, frame.stylePointActif(), inter, 2, false);
		frame.ajouteObjetVisuel(ptl);
  	CSegment seg2 = new CSegment(liste, null, true, frame.couleurActive(), false,
  			frame.styleTraitActif(), point1, ptl);
  	frame.ajouteObjetVisuel(seg2);
  	CSegment seg3 = new CSegment(liste, null, true, frame.couleurActive(), false,
  			frame.styleTraitActif(), ptl, pane.pointSouris);
  	frame.ajouteObjetVisuel(seg3);
  }
  
  public String chaineIndication (int indEtape) {
    switch(indEtape) {
      case 1 : return getString("triangleEquilateralDirect1");
      case 2 : return getString("triangleEquilateralDirect2");
      default: return "";
    }
  }
  
}
