/*
LINUXMathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package mathgraph32;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GraphicsDevice;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.beans.PropertyVetoException;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.JarURLConnection;
import java.net.URL;
import java.net.URLDecoder;
import java.util.Enumeration;
import java.util.Locale;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JColorChooser;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JSlider;
import javax.swing.JTextArea;
import javax.swing.JToolBar;
import javax.swing.ToolTipManager;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import outils.*;
import objets.*;
import types.DimensionFenetre;
import types.FonteNom;
import types.InfoProx;
import types.MotifPoint;
import types.NatureCalcul;
import types.NatureObjet;
import types.StyleFleche;
import types.StyleMarqueSegment;
import types.StyleRemplissage;
import types.StyleTrait;
import types.UniteAngle;
import dialogs.MtgDlg;
import dialogs.MtgLabel;
import dialogs.PaneInsertionCaractereFleche;
import dialogs.PaneInsertionCaractereGrec;
import dialogs.PaneInsertionCaractereMath;
import dialogs.PaneLatexInsertionCaractereFleche;
import dialogs.PaneLatexInsertionCaractereGrec;
import dialogs.PaneLatexInsertionCaractereMath;
import dialogs.WarningDlg;
import entreesSorties.MathGraphDataInputStream;
import glob.Kernel;
import actions.*;

/**
 * MtgPane est la fentre principale de MathGraph32
 */
@SuppressWarnings("serial")
// Version 5.0 N'implemente plus ActionListener : Inutile
public class MtgFrame extends JFrame implements WindowListener, ComponentListener {
	// Version 5.0.1 : Trois  variables globales
	public static boolean screenIsHighResolution; // Pour savoir si l'cran est haute rsolution
	public static double coefMultForDlgBoxIcons; // Coefficient multiplicateur pour certains lments de l'interface
	public static double coefMultForSystemFonts; // Ajout version 5.0.2 pour mieux grer les tailles de police systme suivant la rsolution horizontale de l'cran
	//
	private boolean premierDemEnCours = false; // Ajout version 5.0.0.5 pour viter dmarrage trs lent sous Linux
	public String path; // Ajout version 5.0. Sera nul pour un dmarrage normal et sinon contiendra le chemin d'accs
	//  une figure pour le dmarrage d'une figure en ligne de commande
	public String url; // Dans le cas o le logiciel est lanc en JNLP avec une URL qui donne l'adresse de post de la rponse
	//
	String versionLoaded; // Ajout version 5.0 : Vaudra "none" si les ficghiers d'aide et de construction en ligne n'ont pas pu tre chargs
	// lors des prcdentes utilisations et sinon contiendra le numro de version.
	public static final String numeroVersion = "5.4.0"; // Rajout version 4.4 pour export HTML
	// Ajouts version 5.0
	public static int sizeFontList;
	public static int sizeFontLabel;
	public static int sizeFontButton;
	public static int sizeFontMenuItem;
	public static int sizeFontEditor;
	public static int sizeFontTooltip;
	//
	public static final int nombreFenetresDansMenu = 10; // Nombre de fentres
																												// proposes dans le
																												// menu Fenetres
	// Constantes dfinissant les options de fonctionnement
	public static final int niveauAvanceAvecComplexes = 1;
	public static final int niveauAvanceSansComplexes = 2;
	public static final int niveauMoyen = 3;
	public static final int niveauElementaire = 4;
	private Locale locale; // Version 4.3 : Pour la gestion de la langue
	// public MtgImageIcon blankIcon = new MtgImageIcon(coefMultVertIcons(), MtgFrame.class.getResource("images/BlankIcon24.gif")); 	
	public static ResourceBundle messages; // Version 4.3 : Pour la gestion des messages suivant la langu=
	// Version 4.3 : un tableau contenant les documents pour les niveaux prdfinis et les descripteurs pour chaque niveau prdfini
	public CMathGrafDoc docPreferencesNiveauPredefini[] = new CMathGrafDoc[5];
	public String[] descripteurNiveauPredefini = new String[5];
	// Version 4.3 : un tableau contenant les documents pour les niveaux utilisateurs ventuels
	public CMathGrafDoc docPreferencesNiveauPersonnalise[] = new CMathGrafDoc[5];
	public String[] descripteurNiveauPersonnalise = new String[5];
	// Version 4.3 : un tableau contenant les documents pour les niveaux rellement utiliss suivant les prfrences et les descripteurs de niveau
	public CMathGrafDoc docPreferencesNiveau[] = new CMathGrafDoc[5];
	public String[] descripteurNiveau = new String[5];
	
	
	// Ajout version 3.2.4 : un tableau de 4 chanes contenant les descripteurs des 4 niveaux
	// Si le texte associ aux 4 figures default1  default4 contenues dans le mme rpertoire que
	// le Jar excutable n'est pas vide, les 50 premiers caractres seront le descripteur du niveau
	// sinon ce sont les chanes contenues dans getString("niveauAvanceAvecComplexes  niveau lmentaire qui seront utilises
	// public String[] descripteurNiveau = new String[4];
	// Proprits de l'application
	public Properties properties;
	// Variables correspondant aux prfrences contenues dans properties;
	public String pref_langage;
	// Ajout version 4.6.4 : Sert  savoir s'il faut attnuer l'paisseur des traits quand on copie-colle
	// avec un coefficient multiplicateur suprieur  1
	public boolean pref_attenuation;
	public int pref_angleUnit; // Nombre de points utiliss quand on utilise le
															// menu Mouvoir Animer un point li
	public int pref_moveNbPoints; // Frquence utilise quand on utilise le menu
																// Mouvoir Animer un point li
	public int pref_moveFreq; // Position de la prochaine fentre MtgInternalFrame
	public boolean pref_moveCyclic; // Animation cyclique ou non
	public String pref_fileDirectory; // / rpertoire par dfaut d'ouverture des
																		// figures
	public String pref_constDirectory; // / rpertoire par dfaut d'ouverture des
																			// cosntructions
	public int pref_niveau;
	public boolean pref_AideVisuelle; // Aide  la cration visuelle d'objets
	public boolean pref_decalageAffichagesLibres; // Dcalage des affichages
																								// libres lors des translations
																								// de la figure
	// Taille de police pour commentaires et affichages de valeur
	public int pref_indiceTaillePoliceNom; // Depuis la version 5.0 il ne s'agit plus d'un incide mais de la taille relle de la police
	public int pref_IndiceTaillePoliceAffichages; // Depuis la version 5.0 il ne s'agit plus d'un incide mais de la taille relle de la police
	public boolean pref_NomsAuto;
	// Ajout version 5.0 : Si true on cre des points lors d'un clic  un emplacement vide comme pou Geogebra
	public boolean pref_creationPointsAuto;
	//
	// Ajout version 3.2.4 possibilit de choisir un fonctionnement personnalis
	public boolean pref_NiveauPersonnalise;
	// Ajout version 3.2.4 possibilit de choisir l'affichage automatique des mesures en mode personnalis
	public boolean[] pref_affichageMesuresNiveau = new boolean[5];
	// Version 4.5.1 : Possibilit de choisir l'antialiasing active ou non
	public boolean pref_antialiasing;
	// Ajout version 4.6 : Facteur d'agrandissement des images quand on colle dans le presse-papier et quand on exporte en jpeg
	public double pref_ClipboardImagesRatio;
	// Ajout version 4.7.5.2 : Dcalage des polices systme pour pouvor les agrandir
	public int pref_addSystemFont;
	// Ajout pour version 2.0. Transparence par dfaut pour les nouvelles figures
	public static float transparenceParDefaut = 0.2f;
	public static final String nombrePointsPourLieuPointsParDefaut = "500"; // Pass de 300  500 pour version 4.5.1
	// Deux pointeurs servant  ractiver l'outil en cours prcdent
	public MtgAction actionEnCours;

	public int xfenetre = 5;
	public int yfenetre = 5;
	// Ajout version 4.6.3 : Une variable qui sera mis  false au dbut puis sera mis  true
	// dans MtgInteralFrame.paintcomponent uen fois que la barre d'outil de gauche a t affiche
	// une premirefois
	public boolean barreOutilsGaucheEtablie = false;
	// Numro d'instance pour la cration d'un nouveau document
	public int instanceNouvelleFenetre = 0;
	// Le rpertoire utilis par les documents. Sera actualis  l'ouverture ou
	// enregistrement d'un document
	public String repertoireFichiers = "";
	// Les panneaux contenant les composants et la figure
	JPanel paneLeft;
	JPanel paneSouth;
	JPanel paneRight;
	MtgPaneStylePoint paneStylePoint;
	MtgPaneStyleFleche paneStyleFleche;
	public JPanel paneRightHaut = new JPanel();
	MtgPaneStyleMarqueSegment paneStyleMarqueSegment;
	MtgPaneStyleRemplissage paneStyleRemplissage;
	MtgPaneChoixCouleur paneCouleurs;
	public MtgPaneTransparence paneTransparence;
	JPanel paneInfo = new JPanel(); // Pour mettre les labels indiquant les optios
																	// noms auto et aide visuelle
	// Une zne de texte de deux lignes en bas de la fentre pour les indications

	JTextArea indicationArea = new JTextArea(1, 60);

	public JLabel labelNomsAuto, labelAideVisuelle, labelUniteAngle;

	// Ajout version 5.0
	MtgIconButton[] tabIcones = null;
	JToolBar barreGauche, barreDroite;
	// Fin modif version 5.0
	
	// La barre d'outil horizontale
	public JToolBar barHor = new JToolBar(JToolBar.HORIZONTAL);
	// Ajout version 4.3 : Les deux MtgMenuItem qui servent  annuler refaire
	public MtgMenuItem menuItemEditionAnnuler = null;
	public MtgMenuItem menuItemEditionRefaire = null;
	// Les menus principaux qui seront modifis quand on utilise le menu Options Figure en cours Menu
	public MtgMenu menuFichier;
	public MtgMenu menuEdition;
	public MtgMenu menuCreer;
	public MtgMenu menuCalculs;
	public MtgMenu menuModifier;
	public MtgMenu menuMouvoir;
	public MtgMenu menuMacros;
	public MtgMenu menuFenetre;
	public MtgMenu menuHelp;
	public MtgMenu menuOptions;
	public MtgMenu menuConstructions;
	// Les menus qui eux ne seront pas modifis et seront utilsis pour initialiser le PaneOptionsFigureMenu dans le OptionsFigureDlg
	public MtgMenu menuFichierInitial;
	public MtgMenu menuEditionInitial;
	public MtgMenu menuCreerInitial;
	public MtgMenu menuCalculsInitial;
	public MtgMenu menuModifierInitial;
	public MtgMenu menuMouvoirInitial;
	public MtgMenu menuMacrosInitial;
	public MtgMenu menuHelpInitial;
	public MtgMenu menuOptionsInitial;
	public MtgMenu menuConstructionsInitial;
	
	// La barre le contenant
	JMenuBar menuBar = new JMenuBar();
	// curseurs pointeurs de souris
	public static Cursor handCursor = new Cursor(Cursor.HAND_CURSOR);
	public static Cursor crossHairCursor = new Cursor(Cursor.CROSSHAIR_CURSOR);
	public static Cursor defaultCursor = new Cursor(Cursor.DEFAULT_CURSOR);
	public static Cursor moveCursor = new Cursor(Cursor.MOVE_CURSOR);
	public static Cursor waitCursor = new Cursor(Cursor.WAIT_CURSOR);
	public static Cursor cornerCursor; // Curseur pour les affichages de valeurs ,
	// commentaires, macros, user defined
	public static Cursor macroCursor; // Curseur pour excution de macro
	public static Cursor supCursor; // Curseur poubelle
	public static Cursor modifCursor; // Curseur de modification d'objet graphique
	public static Cursor gommeCursor; // Curseur gomme
	public static Cursor paletteCursor; // Curseur de palette
	public static Cursor crayonCursor; // Curseur de cration de point libre
	// Les actions associess aux items de menus ou icones
	public ActionChoixFenetre actionChoixFenetre; // Sert  activer une fentre
																								// quand plus de 10 sont
																								// ouvertes
	public JMenuItem itemChoixFenetre; // L'item qui est rajout au menu Fenetre
																			// quand plus de 10 sont ouvertes
	public JCheckBoxMenuItem cbAideVisuelle, cbNomsAuto, cbModeTrace, cbImageFondVisible;
	public JCheckBoxMenuItem cbModeCreationPointAuto; // Ajout version 5.0
	MtgAction actionAnnuler;
	MtgAction actionRefaire;
	MtgAction actionCapture;
	MtgAction actionTraceObjets;
	MtgAction actionFichierOuvrir;
	MtgAction actionFichierEnregistrer;
	public MtgAction actionFichierEnregistrerSous; // public car appel par
	// ActionFichierEnregistrer
	ActionFichierExporterPNG actionFichierExporterPNG;
	ActionFichierExporterJPEG actionFichierExporterJPEG;
	ActionFichierExporterPNGAvecUnite actionFichierExporterPNGAvecUnite;
	// ActionFichierExporterHTML actionFichierExporterHTML; Plus support version 5.0
	ActionFichierExporterHTML5 actionFichierExporterHTML5; // Ajout version 4.7.7
	ActionFichierExporterTikz actionFichierExporterTikz; // Ajout version 4.8.1
	MtgAction actionFichierQuitter;	
	MtgAction actionTranslaterFigure;
	MtgAction actionCreerPointLibre;
	MtgAction actionCreerMilieu;
	MtgAction actionCreerPointLie;
	MtgAction actionCreerDroiteAB;
	MtgAction actionCreerDroiteParallele;
	MtgAction actionCreerDroitePerpendiculaire;
	MtgAction actionCreerDroiteBissectrice;
	MtgAction actionCreerDroiteMediatrice;
	MtgAction actionCreerSegment;
	MtgAction actionCreerDemiDroite;
	MtgAction actionCreerIntersection;
	MtgAction actionCreerCercleOA;
	MtgAction actionCreerCercleOR;
	MtgAction actionCreerCalcul;
	MtgAction actionModifierNom;
	MtgAction actionMesureLongueur;
	MtgAction actionMesureLongueurLigne;
	MtgAction actionMesureAirePolygone;
	MtgAction actionMesureAbscisse;
	MtgAction actionMesureAngleNonOriente;
	MtgAction actionMesureAngleOriente;
	MtgAction actionRapporteur;
	MtgAction actionSuppressionObjetGraphique;
	MtgAction actionCreerMarqueAngleNonOriente;
	MtgAction actionCreerMarqueAngleOriente;
	MtgAction actionCreerMarqueAngleOrienteSensDirect;
	MtgAction actionCreerMarqueAngleOrienteSensIndirect;
	MtgAction actionCreerImageParTranslation;
	MtgAction actionCreerImageParTranslationParCoord;
	MtgAction actionCreerImageParSymetrieAxiale;
	MtgAction actionCreerImageParSymetrieCentrale;
	MtgAction actionCreerImageParHomothetie;
	MtgAction actionCreerImageParRotation;
	MtgAction actionCreerImageParInversion;
	MtgAction actionCreerArcPetitParPointFinal;
	MtgAction actionCreerArcGrandParPointFinal;
	MtgAction actionCreerArcDirectParPointFinal;
	MtgAction actionCreerArcIndirectParPointFinal;
	MtgAction actionCreerArcPetitParAngle;
	MtgAction actionCreerArcGrandParAngle;
	MtgAction actionCreerArcDirectParAngle;
	MtgAction actionCreerArcIndirectParAngle;
	MtgAction actionCreerArcPetitParRapporteur;
	MtgAction actionCreerArcGrandParRapporteur;
	MtgAction actionPalette;
	MtgAction actionCreer;
	MtgAction actionCreerDroiteHorizontale;
	MtgAction actionCreerDroiteVerticale;
	MtgAction actionCreerMarqueSegment;
	MtgAction actionCreerPolygone;
	MtgAction actionCreerLigneBrisee;
	MtgAction actionCreerSurface;
	MtgAction actionCreerSurfaceLieuDroite;
	MtgAction actionCreerSurfaceLieuDeuxPoints;
	MtgAction actionCreerSurfaceDeuxLieux;
	MtgAction actionCreerProjeteOrthogonal;
	MtgAction actionCreerRepere;
	MtgAction actionCreerPointParCoordonnees;
	MtgAction actionModifierObjetGraphique;
	MtgAction actionCreerDroiteParCoef;
	MtgAction actionCreerCalculComplexe; // Pas d'icne pour cet outil
	MtgAction actionCreerAffichageValeur;
	MtgAction actionCreerFonctionReelle;
	MtgAction actionCreerFonctionComplexe; // Pas d'icne pour cet outil
	MtgAction actionCreerFonctionReelle2Var; // Pas d'icne pour cet outil
	MtgAction actionCreerFonctionReelle3Var; // Pas d'icne pour cet outil
	MtgAction actionCreerFonctionComplexe2Var; // Pas d'icne pour cet outil
	MtgAction actionCreerFonctionComplexe3Var; // Pas d'icne pour cet outil
	
	MtgAction actionCreerCentreGravite;
	MtgAction actionCreerPointParAbscisse;
	MtgAction actionCreerAffichageValeurLiePoint; // Pas d'icne pour cet outil
	MtgAction actionCreerImageLibre; // Pas d'icne pour cet outil
	MtgAction actionCreerImageLieePoint; // Pas d'icne pour cet outil
	MtgAction actionCreerCommentaire;
	MtgAction actionCreerCommentaireLiePoint; // Pas d'icne pour cet outil
	MtgAction actionCreerLatex;
	MtgAction actionCreerLatexLiePoint;
	MtgAction actionCreerVariable;
	MtgAction actionCreerLieuParPointLie;
	MtgAction actionCreerLieuParVariable;
	MtgAction actionCreerLieuDiscretParPointLie;
	MtgAction actionCreerLieuDiscretParVariable;
	MtgAction actionCreerLieuObjetParPointLie;
	MtgAction actionCreerLieuObjetParVariable;
	MtgAction actionCreerImageParSimilitude;
	MtgAction actionCreerBarycentre;
	MtgAction actionExecuterMacro;
	MtgAction actionGomme;
	MtgAction actionCreerVecteur;
	MtgAction actionMesureAbscisseDansRepere;
	MtgAction actionMesureOrdonneeDansRepere;
	MtgAction actionMesureAffixe;
	MtgAction actionMesureProduitScalaire;
	MtgAction actionCreerDemiPlan;
	MtgAction actionCreerDerivee;
	MtgAction actionCreerSuiteRecurrenteReelle;
	MtgAction actionCreerSuiteRecurrenteComplexe;
	MtgAction actionCreerPointParAffixe;
	MtgAction actionCreerGrapheSuiteRecurrenteReelle;
	MtgAction actionCreerGrapheSuiteRecurrenteComplexe;
	MtgAction actionFichierNouvelleFigure;
	MtgAction actionFichierNouvelleFigureSansLongueurUnite;
	MtgAction actionFichierNouvelleFigureAvecRepere;
	MtgAction actionFichierNouvelleFigureParCode; // Ajout version 4.7.7. Cration d'une figure  partir de code base64
	MtgAction actionModifierRepere;
	MtgAction actionCreerSolutionEquation;
	MtgAction actionCreerMaximum;
	MtgAction actionCreerMinimum;
	MtgAction actionCreerIntegrale;
	MtgAction actionRideau;
	MtgAction actionCreerTestExistence;
	MtgAction actionCreerPartieReelle;
	MtgAction actionCreerPartieImaginaire;
	MtgAction actionCreerModule;
	MtgAction actionCreerArgument;
	MtgAction actionCreerSommeIndicee;
	MtgAction actionCreerProduitIndice;
	MtgAction actionCreerMacroApparition;
	MtgAction actionCreerMacroDisparition;
	MtgAction actionCreerMacroAnimation;
	MtgAction actionCreerMacroAnimationAvecTrace;
	MtgAction actionCreerMacroAffectationValeurVariable;
	MtgAction actionCreerMacroModificationVariable;
	MtgAction actionCreerMacroJouantSon;
	MtgAction actionCreerMacroIncrementationVariable;
	MtgAction actionCreerMacroDecrementationVariable;
	MtgAction actionCreerMacroTraceAutoParPointLie;
	MtgAction actionCreerMacroTraceAutoParVariable;
	MtgAction actionCreerMacroAffectationPointLie;
	MtgAction actionCreerMacroClignotement;
	MtgAction actionCreerMacroAffichantTout;
	MtgAction actionCreerMacroPause;
	MtgAction actionCreerMacroBoucleAvecAnimation;
	MtgAction actionCreerMacroBoucleAvecTrace;
	MtgAction actionCreerMacroSuiteMacros;
	MtgAction actionEditionCopier;
	MtgAction actionEditionCopierCode; // Ajout version 4.7.7
	ActionFichierImprimer actionFichierImprimer;
	ActionZoomPlus actionZoomPlus;
	ActionZoomMoins actionZoomMoins;
	ActionModeTraceActivation actionModeTrace;
	ActionModePointAuto actionModePointAuto; // Ajout version 5.0
	ActionRecopierStyle actionRecopierStyle; // Ajout version 5.0
	MtgAction actionHelpAbout;
	MtgAction actionHelpNet;
	MtgAction actionHelpCodeHTML;
	MtgAction actionHelpHelp;
	MtgAction actionSupprimerLiaison;
	MtgAction actionCreerLiaison;
	MtgAction actionSupprimerLiaisonAffichage;
	MtgAction actionCreerLiaisonAffichage;
	MtgAction actionReclassementFinObjetGraphique;
	MtgAction actionReclassementFinValeur;
	MtgAction actionReclassementDebutObjetGraphique;
	MtgAction actionReclassementDebutValeur;
	MtgAction actionCreerPointLibreEnt;
	MtgAction actionPunaiser;
	MtgAction actionDepunaiser;
	MtgAction actionCreerObjetDuplique;
	MtgAction actionOptionsFigureEnCours;
	MtgAction actionOptionsPreferences;
	MtgAction actionSuppressionImageFond;
	MtgAction actionImageFondVisible;
	MtgAction actionChoixImageFond;
	MtgAction actionCreerCourbeFonction;
	MtgAction actionReafficherTout;
	MtgAction actionCreerCentreCercle;
	MtgAction actionConstructionImplementerDepuisFigure;
	MtgAction actionSuppressionValeur;
	MtgAction actionAjoutObjetsMacro;
	MtgAction actionRetraitObjetsMacro;
	MtgAction actionModificationMacro;
	MtgAction actionMouvoirAnimationPointLie;
	MtgAction actionRenommerCalcul;
	MtgAction actionConstructionEnregistrer;
	MtgAction actionIncorporerConstruction;
	MtgAction actionModifierConstruction;
	MtgAction actionCreerLongueurUnite;
	MtgAction actionChoixSourcesNonGraphiques;
	MtgAction actionChoixSourcesGraphiques;
	MtgAction actionChoixFinauxGraphiques;
	MtgAction actionChoixFinauxNonGraphiques;
	MtgAction actionConstructionCreer;
	MtgAction actionConstructionReinitialiser;
	MtgAction actionConstructionImplementerDepuisFichier;
	MtgAction actionConstructionImplementerPredefinie;
	MtgAction actionConstructionSupprimer;
	MtgAction actionConstructionFusionner;
	MtgAction actionCreationVisuelle;
	MtgAction actionNomsAuto;
	MtgAction actionAfficherObjetsNumeriques;
	MtgAction actionEditerTexteFigure;
	MtgAction actionCurseur;
	MtgAction actionPointSurCourbeParPointLie;
	MtgAction actionPointSurCourbeParAbscisse;
	MtgAction actionTangente;
	MtgAction actionMesureCoefficientDirecteur;
	MtgAction actionActiverOutilPrecedent;
	MtgAction actionCreerMacroApparitionAPartirAutreMacro;
	MtgAction actionCreerMacroDisparitionAPartirAutreMacro;
	MtgAction actionFichierNouvelleFigurePredefinie;
	MtgAction actionCreerPointParSommeVect;
	MtgAction actionCreerPointParMultVect;
	MtgAction actionCarreDirect;
	MtgAction actionTriangleEquilateralDirect;
	MtgAction actionParallelogramme;
	MtgAction actionPolygoneRegulier;
	MtgAction actionCreerPointInterieur;
	MtgAction actionCreationLiaisonPointInterieurPolygone;
	MtgAction actionCreationLiaisonPointInterieurCercle;
	ActionOptionPersonnaliserNiveau1 actionOptionPersonnaliserNiveau1;
	ActionOptionPersonnaliserNiveau2 actionOptionPersonnaliserNiveau2;
	ActionOptionPersonnaliserNiveau3 actionOptionPersonnaliserNiveau3;
	ActionOptionPersonnaliserNiveau4 actionOptionPersonnaliserNiveau4;
	ActionRelationEntreObjets actionRelationEntreObjets;
	MtgAction actionCreerMacroModeTraceActivation;
	MtgAction actionCreerMacroModeTraceDesactivation;
	MtgAction actionHelpTexteFigure;
	MtgAction actionCreerEquationDroite;
	MtgAction actionCreerEquationDroiteLiePoint;
	MtgAction actionCreerDroiteParEquation;
	MtgAction actionMarquerPointPourTrace;
	MtgAction actionDemarquerPointPourTrace;
	MtgAction actionCreerAbscisseOrigineRep;
	MtgAction actionCreerOrdonneeOrigineRep;
	MtgAction actionCreerUnitexRep;
	MtgAction actionCreerUniteyRep;
	MtgAction actionAjoutGraduationSimple;
	MtgAction actionAjoutGraduationTrigo;
	MtgAction actioncreerEditeurFormule;
	MtgAction actionCreerTestEquivalence; // Ajout version 4.6
	MtgAction actionCreerTestEqNatOp;
	MtgAction actionCreerDeriveePartielle; // Ajout version 4.6
	MtgAction actionCreerTestDependanceVariable; // Ajout version 4.6
	MtgAction actionCreerSegmentParLong; // Ajout version 4.6.3
	MtgAction actionCreerSuiteRecurrenteComplexe2; // Ajout version 4.7.5
	MtgAction actionCreerSuiteRecurrenteReelle2; // Ajout version 4.7.5
	MtgAction actionCreerSuiteRecurrenteComplexe3; // Ajout version 4.7.5
	MtgAction actionCreerSuiteRecurrenteReelle3; // Ajout version 4.7.5
	MtgAction actionCreerTestFact; // Ajout version 4.7.9
	MtgAction actionConstructionIterativeImplementerDepuisFigure; // Ajout version 4.8
	MtgAction actionConstructionRecursiveImplementerDepuisFigure; // Ajout version 4.8
	MtgAction actionCreerMacroConstructionIterative; // Ajout version 4.8
	MtgAction actionCreerMacroConstructionRecursive; // Ajout version 4.8
	MtgAction actionCreerObjetClone; // Ajout version 4.8
	MtgAction actionEditionCopierCodeTikz; // Ajout version 4.8.1
	MtgAction actionHTMLInfo; // Ajout version 4.9.2
	MtgAction actionProtocole; // Ajout version 4.9.9
	MtgAction actionHelpOnline; // Ajout version 5.0
	MtgAction actionAugmenterTailles; // Ajout version 5.0
	MtgAction actionDiminuerTailles; // Ajout version 5.0
	MtgAction actionChangerStyleTousPoints; // Ajout version 5.0
	// Ajout version 5.0 : Actions servant  charger des figures prdfinies.
	MtgAction actionFigureBaseCourbes;
	MtgAction actionFigureBaseCourbePolaire;
	MtgAction actionFigureBaseCourbeParam;
	MtgAction actionFigureBaseSuiteunfn;
	MtgAction actionFigureBaseSuiteRecParPtLie;
	MtgAction actionFigureBaseSuiteRecParPremierTerme;
	MtgAction actionFigureBaseLoiBinomiale;
	MtgAction actionFigureBaseLoiBinomialeIntervalleConfiance;
	MtgAction actionFigureBaseLoiBinomialeApproximationParLoiNormale;
	MtgAction actionFigureBaseLoiExponentielle;
	MtgAction actionFigureBaseLoiNormale;
	MtgAction actionFigureBaseLoiNormaleualpha;
	MtgAction actionFigureBaseLoiNormaleInverse;
	MtgAction actionFigureBaseEspaceCube;
	MtgAction actionFigureBaseEspaceParallelepipedeRect;
	MtgAction actionFigureBaseEspaceTetraedre;
	MtgAction actionFigureBaseEspaceTetraedreRegulier;
	MtgAction actionFigureBaseEspaceCylindre;
	MtgAction actionFigureBaseEspaceConeSimple;
	MtgAction actionFigureBaseEspaceConeDouble;
	MtgAction actionFigureBaseEspacePyramideBaseCarre;
	MtgAction actionFigureBaseEspacePyramideBaseRectangle;
	MtgAction actionFigureBaseEspacePyramideReguliereBaseCarre;
	MtgAction actionFigureBaseEspacePrismeBaseTriangulaire;
	MtgAction actionFigureBaseEspacePrismeBaseTrapeze;
	MtgAction actionFigureBaseEspaceSurfaceFonction2Var;
	MtgAction actionFigureBaseEspaceRepereOrthonormal; // Ajout version 5.1.2
	MtgAction actionFigureBaseEspaceRepereOrthogonal; // Ajout version 5.1.2
	MtgAction actionFigureBaseRepere;
	MtgAction actionFigureBaseRepereVect;
	MtgAction actionPostHtml;

	// Un groupe de boutons pour englober tous les boutons de la barre d'outil de
	// cration d'objet
	ButtonGroup btngroup1 = new ButtonGroup();
	// les boutons de la barre d'outils de cration d'objets
	MtgToggleButton buttonCapture;
	MtgToggleButton buttonModifierNom;
	MtgToggleButton buttonProtocole;
	MtgToggleButton buttonAfficherObjetsNumeriques;
	MtgToggleButton buttonSuppressionObjetGraphique;
	MtgToggleButton buttonSuppressionValeur;
	MtgToggleButton buttonExecuterMacro;
	MtgToggleButton buttonGomme;
	MtgToggleButton buttonRideau;
	MtgToggleButton buttonHelpTexteFigure;

	// Un groupe de boutons pour englober tous les boutons de la barre d'outil de
	// cration d'objet
	ButtonGroup btngroup2 = new ButtonGroup();
	// Les boutons de la barre d'outils du haut
	MtgActionButton buttonFichierOuvrir;
	MtgActionButton buttonFichierNouvelleFigure;
	MtgActionButton buttonFichierEnregistrer;
	MtgActionButton buttonEditionCopier;
	MtgActionButton buttonExporterUnite; // Ajout version 4.6.3
	MtgActionButton buttonFichierNouvelleFigureAvecRepereSansVecteurs; // Ajout version 4.6.3
	MtgActionButton buttonAnnuler;
	MtgActionButton buttonRefaire;
	MtgActionButton buttonZoomPlus;
	MtgActionButton buttonZoomMoins;
	MtgActionButton buttonReafficherTout;
	MtgActionButton buttonFichierImprimer;
	public MtgToggleButton buttonModeTrace;
	MtgToggleButton buttonTranslationFigure;
	MtgToggleButton buttonPalette;
	MtgToggleButton buttonModifierObjetGraphique;
	MtgToggleButton buttonRelationEntreObjets;
	// Ajout version 5.0 : Une icne  bascule pour activer ou dsactiver le mode de cration automatique de points.
	public MtgToggleButton buttonModePointAuto;
	// Ajout version 5.0 : Une icne pour recopier le tyle d'un objet
	MtgToggleButton buttonRecopierStyle;
	// Ajout version 5.0 : Deux boutons pour reclasser vers le dbut ou la fin un objet graphique
	MtgToggleButton buttonReclassementDebutObjetGraphique;
	MtgToggleButton buttonReclassementFinObjetGraphique;
	// Ajout version 5.0 : Deux boutons pour augenter et diminuer les taille de police
	MtgToggleButton buttonAugmenterTailles;
	MtgToggleButton buttonDiminuerTailles;

	// Le style de point actif qui est dtermin quand on presse un
	// MtgStylePointButton
	public int stylePointActif;
	// Le style de flche actif qui est dtermin quand on presse un
	// MtgStyleFlecheButton
	public int styleFlecheActif;
	// Le style de marque de segment actif qui est dtermin quand on presse un
	// MtgStyleMarqueSegmentButton
	public int styleMarqueSegmentActif;
	// Le style de marque de segment actif qui est dtermin quand on presse un
	// MtgStyleMarqueAngleButton
	public int styleMarqueAngleActif;
	// Le style de remplissage actif
	public int styleRemplissageActif;



	
	// Une liste contenant les styles de trait
	MtgListeStyleTrait listeStyleTrait;
	// Une liste contenant les styles de marque d'angle
	MtgPaneStyleMarqueAngle paneStyleMarqueAngle;
	// Ajout version 5.0 : un JSlider pour chosir l'paisseur de trait.
	JSlider sliderEpaisseur;
	JPanel paneSlider;
	JLabel labelEpTrait;


	// Bouton de choix de la couleur active
	public JColorChooser colorChooser;
	public Outil outilActif;
	public MtgDesktopPane mtgDesktopPane;
	public CListeObjets listeClignotante; // Liste pour faire clignoter
	public CListeObjets listeExclusionDesignation; // Liste des objets interdits
	// d'tre points  la souris
	public CListeObjets listeObjetsVisuels; // Liste des objets d'aide  la
	// cration visuelle
	// Les outils de pointage
	public OutilPointageCapt outilPointageCapture = new OutilPointageCapt(this);
	public OutilPointageClic outilPointageClic = new OutilPointageClic(this);
	public OutilPointageCre outilPointageCre = new OutilPointageCre(this);
	public OutilPointageIntersection outilPointageIntersection = new OutilPointageIntersection(this);
	
	public OutilPointageSup outilPointageSup = new OutilPointageSup(this);
	public OutilPointagePourRideau outilPointagePourRideau = new OutilPointagePourRideau(this);
	public OutilPointageRapporteur outilPointageRapporteur = new OutilPointageRapporteur(this);
	public OutilPointageObjetClignotant outilPointageObjetClignotant = new OutilPointageObjetClignotant(this);
	public OutilPointagePourNommer outilPointagePourNommer = new OutilPointagePourNommer(this);
	public OutilPointageDecalageFigure outilPointageDecalageFigure = new OutilPointageDecalageFigure(this);
	public OutilPointageModification outilPointageModification = new OutilPointageModification(this);
	public OutilPointageExecutionMacro outilPointageExecutionMacro = new OutilPointageExecutionMacro(this);
	public OutilPointageTraceObjet outilPointageTraceObjet = new OutilPointageTraceObjet(this);
	public OutilPointageCreationCurseur outilPointageCreationCurseur = new OutilPointageCreationCurseur(this);
	public OutilPointageCreationSegmentParLongueur outilPointageCreationSegmentParLongueur = new OutilPointageCreationSegmentParLongueur(this);
	// Pointeur sur l'outil de pointage actif qui est l'outil de capture au dpart
	// Ajout version 4.9.9
	public OutilPointageProtocole outilPointageProtocole = new OutilPointageProtocole(this);
	public OutilPointage outilPointageActif = outilPointageCapture;

	public InfoProx infoProx = new InfoProx(); // Utilis pour savoir les objets
	// proches du pointeur souris
	// Outils de cration et modification
	public OutilAnnuler outilAnnuler = new OutilAnnuler(this);
	public OutilRefaire outilRefaire = new OutilRefaire(this);
	public OutilCapture outilCapture = new OutilCapture(this);
	public OutilTraceObjets outilTraceObjets = new OutilTraceObjets(this);
	public OutilPtLib outilCreerPointLibre = new OutilPtLib(this);
	public OutilMilieu outilMilieu = new OutilMilieu(this);
	public OutilDtAB outilDtAB = new OutilDtAB(this);
	public OutilDtPar outilDtPar = new OutilDtPar(this);
	public OutilDtPer outilDtPer = new OutilDtPer(this);
	public OutilSegment outilSeg = new OutilSegment(this);
	public OutilDemiDt outilDemiDt = new OutilDemiDt(this);
	public OutilCerOA outilCerOA = new OutilCerOA(this);
	public OutilDtBis outilDtBis = new OutilDtBis(this);
	public OutilDtMed outilDtMed = new OutilDtMed(this);
	public OutilInt outilInt = new OutilInt(this);
	public OutilTranslation outilTrans = new OutilTranslation(this);
	public OutilSymAxiale outilSymAxiale = new OutilSymAxiale(this);
	public OutilSymCentrale outilSymCentrale = new OutilSymCentrale(this);
	public OutilRapporteur outilRapporteur = new OutilRapporteur(this);
	public OutilRot outilRot = new OutilRot(this);
	public OutilCerOR outilCerOR = new OutilCerOR(this);
	public OutilSup outilSup = new OutilSup(this);
	public OutilPtLie outilPtLie = new OutilPtLie(this);
	public OutilPtParAbs outilPtParAbs = new OutilPtParAbs(this);
	public OutilHom outilHom = new OutilHom(this);
	public OutilMesLong outilMesLong = new OutilMesLong(this);
	public OutilMesAngNor outilMesAngnor = new OutilMesAngNor(this);
	public OutilMesAngOr outilMesAngOr = new OutilMesAngOr(this);
	public OutilMesAbs outilMesAbs = new OutilMesAbs(this);
	public OutilMarqueAngleNonOriente outilMarqueAngleNonOriente = new OutilMarqueAngleNonOriente(this);
	public OutilMarqueAngleOriente outilMarqueAngleOriente = new OutilMarqueAngleOriente(this);
	public OutilMarqueAngleSensDirect outilMarqueAngleSensDirect = new OutilMarqueAngleSensDirect(this);
	public OutilMarqueAngleSensIndirect outilMarqueAngleSensIndirect = new OutilMarqueAngleSensIndirect(this);
	public OutilTranslation outilCreerImageParTranslation = new OutilTranslation(this);
	public OutilTranslationParCoord outilCreerImageParTranslationParCoord = new OutilTranslationParCoord(this);
	public OutilSymAxiale outilCreerImageParSymetrieAxiale = new OutilSymAxiale(this);
	public OutilSymCentrale outilCreerImageParSymetrieCentrale = new OutilSymCentrale(this);
	public OutilHom outilCreerImageParHomothetie = new OutilHom(this);
	public OutilArcPetitParPointFinal outilCreerArcPetitParPointFinal = new OutilArcPetitParPointFinal(this);
	public OutilArcGrandParPointFinal outilCreerArcGrandParPointFinal = new OutilArcGrandParPointFinal(this);
	public OutilArcDirectParPointFinal outilCreerArcDirectParPointFinal = new OutilArcDirectParPointFinal(this);
	public OutilArcIndirectParPointFinal outilCreerArcIndirectParPointFinal = new OutilArcIndirectParPointFinal(this);
	public OutilArcPetitParAngle outilCreerArcPetitParAngle = new OutilArcPetitParAngle(this);
	public OutilArcGrandParAngle outilCreerArcGrandParAngle = new OutilArcGrandParAngle(this);
	public OutilArcDirectParAngle outilCreerArcDirectParAngle = new OutilArcDirectParAngle(this);
	public OutilArcIndirectParAngle outilCreerArcIndirectParAngle = new OutilArcIndirectParAngle(this);
	public OutilArcPetitParRapporteur outilArcPetitParRapporteur = new OutilArcPetitParRapporteur(this);
	public OutilArcGrandParRapporteur outilArcGrandParRapporteur = new OutilArcGrandParRapporteur(this);
	public OutilDroiteHorizontale outilDroiteHorizontale = new OutilDroiteHorizontale(this);
	public OutilDroiteVerticale outilDroiteVerticale = new OutilDroiteVerticale(this);
	public OutilMarqueSegment outilMarqueSegment = new OutilMarqueSegment(this);
	public OutilPolygone outilPolygone = new OutilPolygone(this);
	public OutilLigneBrisee outilLigneBrisee = new OutilLigneBrisee(this);
	public OutilSurface outilSurface = new OutilSurface(this);
	public OutilSurfaceLieuDroite outilSurfaceLieuDroite = new OutilSurfaceLieuDroite(this);
	public OutilSurfaceLieuDeuxPoints outilSurfaceLieuDeuxPoints = new OutilSurfaceLieuDeuxPoints(this);
	public OutilSurfaceDeuxLieux outilSurfaceDeuxLieux = new OutilSurfaceDeuxLieux(this);
	public OutilProjeteOrthogonal outilProjeteOrthogonal = new OutilProjeteOrthogonal(this);
	public OutilRepere outilRepere = new OutilRepere(this);
	public OutilPointParCoordonnees outilPointParCoordonnees = new OutilPointParCoordonnees(this);
	public OutilModifierObjetGraphique outilModifierObjetGraphique = new OutilModifierObjetGraphique(this);
	public OutilDroiteParCoef outilDroiteParCoef = new OutilDroiteParCoef(this);
	public OutilAffichageValeur outilAffichageValeur = new OutilAffichageValeur(this);
	public OutilFonctionReelle outilFonctionReelle = new OutilFonctionReelle(this);
	public OutilFonctionComplexe outilFonctionComplexe = new OutilFonctionComplexe(this);
	public OutilFonctionReelle2Var  outilFonctionReelle2Var = new OutilFonctionReelle2Var(this);
	public OutilFonctionReelle3Var  outilFonctionReelle3Var = new OutilFonctionReelle3Var(this);
	public OutilFonctionComplexe2Var outilFonctionComplexe2Var = new OutilFonctionComplexe2Var(this);
	public OutilFonctionComplexe3Var outilFonctionComplexe3Var = new OutilFonctionComplexe3Var(this);
	public OutilCentreGravite outilCentreGravite = new OutilCentreGravite(this);
	public OutilPtParAbs outilPointParAbscisse = new OutilPtParAbs(this);
	public OutilAffichageValeurLiePoint outilAffichageValeurLiePoint = new OutilAffichageValeurLiePoint(this);
	public OutilCommentaire outilCommentaire = new OutilCommentaire(this);
	public OutilCommentaireLiePoint outilCommentaireLiePoint = new OutilCommentaireLiePoint(this);
	public OutilLatexLiePoint outilLatexLiePoint = new OutilLatexLiePoint(this);
	public OutilVariable outilVariable = new OutilVariable(this);
	public OutilLieuPointParPointLie outilLieuPointParPointLie = new OutilLieuPointParPointLie(this);
	public OutilLieuParVariable outilLieuParVariable = new OutilLieuParVariable(this);
	public OutilLieuDiscretParPointLie outilLieuDiscretParPointLie = new OutilLieuDiscretParPointLie(this);
	public OutilLieuDiscretParVariable outilLieuDiscretParVariable = new OutilLieuDiscretParVariable(this);
	public OutilLieuObjetParPointLie outilLieuObjetParPointLie = new OutilLieuObjetParPointLie(this);
	public OutilLieuObjetParVariable outilLieuObjetParVariable = new OutilLieuObjetParVariable(this);
	public OutilSimilitude outilSimilitude = new OutilSimilitude(this);
	public OutilPalette outilPalette = new OutilPalette(this);
	public OutilBarycentre outilBarycentre = new OutilBarycentre(this);
	public OutilModifierBarycentre outilModifierBarycentre = new OutilModifierBarycentre(this);
	public OutilExecutionMacro outilExecutionMacro = new OutilExecutionMacro(this);
	public OutilGomme outilGomme = new OutilGomme(this);
	public OutilVecteur outilVecteur = new OutilVecteur(this);
	public OutilMesureAbscisseDansRepere outilMesureAbscisseDansRepere = new OutilMesureAbscisseDansRepere(this);
	public OutilMesureOrdonneeDansRepere outilMesureOrdonneeDansRepere = new OutilMesureOrdonneeDansRepere(this);
	public OutilMesureAffixe outilMesureAffixe = new OutilMesureAffixe(this);
	public OutilInversion outilInversion = new OutilInversion(this);
	public OutilDemiPlan outilDemiPlan = new OutilDemiPlan(this);
	public OutilDerivee outilDerivee = new OutilDerivee(this);
	public OutilSuiteRecurrenteReelle outilSuiteRecurrenteReelle = new OutilSuiteRecurrenteReelle(this);
	public OutilSuiteRecurrenteComplexe outilSuiteRecurrenteComplexe = new OutilSuiteRecurrenteComplexe(this);
	public OutilPointParAffixe outilPointParAffixe = new OutilPointParAffixe(this);
	public OutilGrapheSuiteRecurrenteReelle outilGrapheSuiteRecurrenteReelle = new OutilGrapheSuiteRecurrenteReelle(this);
	public OutilGrapheSuiteRecurrenteComplexe outilGrapheSuiteRecurrenteComplexe = new OutilGrapheSuiteRecurrenteComplexe(this);
	public OutilModifierRepere outilModifierRepere = new OutilModifierRepere(this);
	public OutilSolutionEquation outilSolutionEquation = new OutilSolutionEquation(this);
	public OutilMaximum outilMaximum = new OutilMaximum(this);
	public OutilMinimum outilMinimum = new OutilMinimum(this);
	public OutilIntegrale outilIntegrale = new OutilIntegrale(this);
	public OutilRideau outilRideau = new OutilRideau(this);
	public OutilTestExistence outilTestExistence = new OutilTestExistence(this);
	public OutilPartieReelle outilPartieReelle = new OutilPartieReelle(this);
	public OutilPartieImaginaire outilPartieImaginaire = new OutilPartieImaginaire(this);
	public OutilModule outilModule = new OutilModule(this);
	public OutilArgument outilArgument = new OutilArgument(this);
	public OutilSomme outilSomme = new OutilSomme(this);
	public OutilProduit outilProduit = new OutilProduit(this);
	public OutilMacroApparition outilMacroApparition = new OutilMacroApparition(this);
	public OutilMacroDisparition outilMacroDisparition = new OutilMacroDisparition(this);
	public OutilMacroAnimation outilMacroAnimation = new OutilMacroAnimation(this);
	public OutilMacroAnimationAvecTrace outilMacroAnimationAvecTrace = new OutilMacroAnimationAvecTrace(this);
	public OutilMacroAffectationValeurVariable outilMacroAffectationValeurVariable = new OutilMacroAffectationValeurVariable(this);
	public OutilMacroJouantSon outilMacroJouantSon = new OutilMacroJouantSon(this);
	public OutilImageLibre outilImageLibre = new OutilImageLibre(this);
	public OutilImageLieePoint outilImageLieePoint = new OutilImageLieePoint(this);
	public OutilSuppressionLiaison outilSuppressionLiaison = new OutilSuppressionLiaison(this);
	public OutilCreationLiaison outilCreationLiaison = new OutilCreationLiaison(this);
	public OutilSuppressionLiaisonAffichage outilSuppressionLiaisonAffichage = new OutilSuppressionLiaisonAffichage(this);
	public OutilCreationLiaisonAffichage outilCreationLiaisonAffichage = new OutilCreationLiaisonAffichage(this);
	public OutilReclassementFinObjetGraphique outilReclassementFinObjetGraphique = new OutilReclassementFinObjetGraphique(this);
	public OutilReclassementFinValeur outilReclassementFinValeur = new OutilReclassementFinValeur(this);
	public OutilReclassementDebutObjetGraphique outilReclassementDebutObjetGraphique =
		new OutilReclassementDebutObjetGraphique(this);
	public OutilReclassementDebutValeur outilReclassementDebutValeur = new OutilReclassementDebutValeur(this);
	public OutilPointLibreEnt outilPointLibreEnt = new OutilPointLibreEnt(this);
	public OutilPunaiserPointMobile outilPunaiserPointMobile = new OutilPunaiserPointMobile(this);
	public OutilDepunaiserPointMobile outilDepunaiserPointMobile = new OutilDepunaiserPointMobile(this);
	public OutilMacroModificationVariable outilMacroModificationVariable = new OutilMacroModificationVariable(this);
	public OutilMacroIncrementationVariable outilMacroIncrementationVariable = new OutilMacroIncrementationVariable(this);
	public OutilMacroDecrementationVariable outilMacroDecrementationVariable = new OutilMacroDecrementationVariable(this);
	public OutilMacroTraceAutoParPointLie outilMacroTraceAutoParPointLie = new OutilMacroTraceAutoParPointLie(this);
	public OutilMacroTraceAutoParVariable outilMacroTraceAutoParVariable = new OutilMacroTraceAutoParVariable(this);
	public OutilMacroAffectationPointLie outilMacroAffectationPointLie = new OutilMacroAffectationPointLie(this);
	public OutilMacroClignotement outilMacroClignotement = new OutilMacroClignotement(this);
	public OutilMacroAffichantTout outilMacroAffichantTout = new OutilMacroAffichantTout(this);
	public OutilMacroPause outilMacroPause = new OutilMacroPause(this);
	public OutilMacroBoucleAvecAnimation outilMacroBoucleAvecAnimation = new OutilMacroBoucleAvecAnimation(this);
	public OutilMacroBoucleAvecTrace outilMacroBoucleAvecTrace = new OutilMacroBoucleAvecTrace(this);
	public OutilMacroSuiteMacros outilMacroSuiteMacros = new OutilMacroSuiteMacros(this);
	public OutilObjetDuplique outilObjetDuplique = new OutilObjetDuplique(this);
	public OutilOptionsFigure outilOptionsFigure = new OutilOptionsFigure(this);
	public OutilOptionsPreferences outilOptionsPreferences = new OutilOptionsPreferences(this);
	public OutilMesureLongueurLigne outilMesureLongueurLigne = new OutilMesureLongueurLigne(this);
	public OutilMesureAirePolygone outilMesureAirePolygone = new OutilMesureAirePolygone(this);
	public OutilCourbeFonction outilcourbeFonction = new OutilCourbeFonction(this);
	public OutilReafficherTout outilReafficherTout = new OutilReafficherTout(this);
	public OutilCentreCercle outilCentreCercle = new OutilCentreCercle(this);
	public OutilConstructionImplementationFigure outilConstructionImplementationFigure =
		new OutilConstructionImplementationFigure(this);
	public OutilConstructionImplementationPredefinie outilConstructionImplementationPredefinie =
		new OutilConstructionImplementationPredefinie(this);
	public OutilSuppressionValeur outilSuppressionValeur = new OutilSuppressionValeur(this);
	public OutilAjoutObjetMacro outilAjoutObjetMacro = new OutilAjoutObjetMacro(this);
	public OutilRetraitObjetsMacro outilRetraitObjetsMacro = new OutilRetraitObjetsMacro(this);
	public OutilModificationMacro outilModificationMacro = new OutilModificationMacro(this);
	public OutilMouvoirAnimationPointLie outilMouvoirAnimationPointLie =
		new OutilMouvoirAnimationPointLie(this);
	public OutilRenommerCalculDlg outilRenommerCalculDlg = new OutilRenommerCalculDlg(this);
	public OutilConstructionEnregistrer outilConstructionEnregistrer =
		new OutilConstructionEnregistrer(this);
	public OutilConstructionModifier outilModifierConstruction = new OutilConstructionModifier(this);
	public OutilCreerLongueurUnite outilCreerLongueurUnite = new OutilCreerLongueurUnite(this);
	public OutilConstructionChoixSourcesGraphiques outilConstructionChoixSourcesGraphiques = 
		new OutilConstructionChoixSourcesGraphiques(this);
	public OutilConstructionChoixFinauxGraphiques outilConstructionChoixFinauxGraphiques =
		new OutilConstructionChoixFinauxGraphiques(this);
	public OutilConstructionImplementationFichier outilConstructionImplementationFichier =
		new OutilConstructionImplementationFichier(this);
	public OutilConstructionSupprimer outilConstructionSupprimer = new OutilConstructionSupprimer(this);
	public OutilCurseur outilCurseur = new OutilCurseur(this);
	public OutilPointSurCourbeParPointLie outilPointSurCourbeParPointLie =
		new OutilPointSurCourbeParPointLie(this);
	public OutilPointSurCourbeParAbscisse outilPointSurCourbeParAbscisse = new OutilPointSurCourbeParAbscisse(this);
	public OutilTangente outilTangente = new OutilTangente(this);
	public OutilMesureCoefficientDirecteur outilMesureCoefficientDirecteur = new OutilMesureCoefficientDirecteur(this);
	public OutilMacroApparitionAPartirAutreMacro outilMacroApparitionAPartirAutreMacro =
		new OutilMacroApparitionAPartirAutreMacro(this);
	public OutilMacroDisparitionAPartirAutreMacro outilMacroDisparitionAPartirAutreMacro =
		new OutilMacroDisparitionAPartirAutreMacro(this);
	public OutilMesureProduitScalaire outilMesureProduitScalaire = new OutilMesureProduitScalaire(this);
	public OutilPointParSommeVect outilPointParSommeVect = new OutilPointParSommeVect(this);
	public OutilPointParMultVect outilPointParMultVect = new OutilPointParMultVect(this);
	public OutilCarreDirect outilCarreDirect = new OutilCarreDirect(this);
	public OutilTriangleEquilateralDirect outilTriangleEquilateralDirect = new OutilTriangleEquilateralDirect(this);
	public OutilParallelogramme outilParallelogramme = new OutilParallelogramme(this);
	public OutilPolygoneRegulier outilPolygoneRegulier = new OutilPolygoneRegulier(this);
	public OutilPointInterieur outilPointInterieur = new OutilPointInterieur(this);
	public OutilCreationLiaisonInterieurPolygone outilCreationLiaisonInterieurPolygone = 
		new OutilCreationLiaisonInterieurPolygone(this);
	public OutilCreationLiaisonInterieurCercle outilCreationLiaisonInterieurCercle = 
		new OutilCreationLiaisonInterieurCercle(this);
	public OutilLatex outilLatex = new OutilLatex(this);
	public OutilRelationEntreObjets outilRelationEntreObjets = new OutilRelationEntreObjets(this);
	public OutilMacroModeTraceActivation outilMacroModeTraceActivation = new OutilMacroModeTraceActivation(this);
	public OutilMacroModeTraceDesactivation outilMacroModeTraceDesactivation = new OutilMacroModeTraceDesactivation(this);
	public OutilEquationDroite outilEquationDroite = new OutilEquationDroite(this);
	public OutilEquationDroiteLiePoint outilEquationDroiteLiePoint =new OutilEquationDroiteLiePoint(this);
	public OutilDroiteParEquation outilDroiteParEquation = new OutilDroiteParEquation(this);
	public OutilMarquerPointPourTrace outilMarquerPointPourTrace = new OutilMarquerPointPourTrace(this);
	public OutilDemarquerPointPourTrace outilDemarquerPointPourTrace = new OutilDemarquerPointPourTrace(this);
	public OutilAbscisseOrigineRep outilAbscisseOrigineRep = new OutilAbscisseOrigineRep(this);
	public OutilOrdonneeOrigineRep outilOrdonneeOrigineRep = new OutilOrdonneeOrigineRep(this);
	public OutilUnitexRep outilUnitexRep = new OutilUnitexRep(this);
	public OutilUniteyRep outilUniteyRep = new OutilUniteyRep(this);
	public OutilAjoutGraduationSimple outilAjoutGraduationSimple = new OutilAjoutGraduationSimple(this);
	public OutilAjoutGraduationTrigo outilAjoutGraduationTrigo = new OutilAjoutGraduationTrigo(this);
	public OutilEditeurFormule outilEditeurFormule = new OutilEditeurFormule(this); // Ajout version 4.5.2
	public OutilTestEquivalence outilTestEquivalence = new OutilTestEquivalence(this); // Ajout version version 4.6
	public OutilTestEqNatOp outilTestEqNatOp = new OutilTestEqNatOp(this); // Ajout version version 4.7.2
	public OutilDeriveePartielle outilDeriveePartielle = new OutilDeriveePartielle(this); // Ajout version version 4.6
	public OutilTestDependanceVariable outilTestDependanceVariable = new OutilTestDependanceVariable(this);
	public OutilSegmentParLong outilSegmentParLong = new OutilSegmentParLong(this); // Ajout version 4.6.3
	public OutilSuiteRecurrenteComplexe2 outilSuiteRecurrenteComplexe2 = new OutilSuiteRecurrenteComplexe2(this); // Ajout version 4.7.5
	public OutilSuiteRecurrenteReelle2 outilSuiteRecurrenteReelle2 = new OutilSuiteRecurrenteReelle2(this); // Ajout version 4.7.5
	public OutilSuiteRecurrenteComplexe3 outilSuiteRecurrenteComplexe3 = new OutilSuiteRecurrenteComplexe3(this); // Ajout version 4.7.5
	public OutilSuiteRecurrenteReelle3 outilSuiteRecurrenteReelle3 = new OutilSuiteRecurrenteReelle3(this); // Ajout version 4.7.5
	public OutilTestFact outilTestFact = new OutilTestFact(this); // Ajout version version 4.7.9
	// Ajout version 4.8
	public OutilConstructionIterativeImplementationFigure outilConstructionIterativeImplementationFigure = new OutilConstructionIterativeImplementationFigure(this);
	// Ajout version 4.8
	public OutilConstructionRecursiveImplementationFigure outilConstructionRecursiveImplementationFigure = new OutilConstructionRecursiveImplementationFigure(this);
	// Ajout version 4.8
	public OutilMacroConstructionIterative outilMacroConstructionIterative = new OutilMacroConstructionIterative(this);
	// Ajout version 4.8
	public OutilMacroConstructionRecursive outilMacroConstructionRecursive = new OutilMacroConstructionRecursive(this);
	// Ajout version 4.8
	public OutilObjetClone outilObjetClone = new OutilObjetClone(this);
	// Ajout version 4.9.2
	public OutilHTMLInfo outilHTMLInfo = new OutilHTMLInfo(this);
	// Ajout version 4.9.9
	public OutilProtocole outilProtocole = new OutilProtocole(this);
	//
	// Outil servant  nommer un point ou une droite
	public OutilEntrerNom outilModifierNom = new OutilEntrerNom(this);
	// Outil servant  translater la figure
	public OutilTranslationFigure outilTranslationFigure = new OutilTranslationFigure(this);
	// Ajout version 5.0 Outil servant  recopier la couleur ou le style d'un objet
	public OutilRecopierStyle outilRecopierStyle = new OutilRecopierStyle(this);
	// Ajout version 5.0.0.4
	public OutilCalculReel outilCalculReel = new OutilCalculReel(this);
	public OutilCalculComplexe outilCalculComplexe = new OutilCalculComplexe(this);
	//
	// Panneaux servant dans le dialogue de choix de caractre spciaux pour les commentaires
	public PaneInsertionCaractereGrec paneInsertionCaractereGrec = new PaneInsertionCaractereGrec();
	public PaneInsertionCaractereMath paneInsertionCaractereMath = new PaneInsertionCaractereMath();
	public PaneInsertionCaractereFleche paneInsertionCaractereFleche = new PaneInsertionCaractereFleche();
	// Panneaux servant dans le dialogue de choix de caractre spciaux pour les affichages latex
	public PaneLatexInsertionCaractereGrec paneLatexInsertionCaractereGrec = new PaneLatexInsertionCaractereGrec();
	public PaneLatexInsertionCaractereMath paneLatexInsertionCaractereMath = new PaneLatexInsertionCaractereMath();
	public PaneLatexInsertionCaractereFleche paneLatexInsertionCaractereFleche = new PaneLatexInsertionCaractereFleche();

	public int instanceNouvelleFenetre() {
		instanceNouvelleFenetre++;
		return instanceNouvelleFenetre;
	}

	/**
	 * Constructeur
	 */
	public MtgFrame(String pathInit, String urlInit) throws IOException {
		path = pathInit;
		url = urlInit;
		// Nouveau version 5.0.1
		screenIsHighResolution = screenIsHighResolution();
		coefMultForDlgBoxIcons = coefMultForDlgBoxIcons();
		coefMultForSystemFonts = coefMultForSystemFonts();
		// Dplac ici version 5.0.1
		sizeFontList = screenIsHighResolution ? 12 : 11;
		sizeFontLabel  = screenIsHighResolution ? 13 :11;
		sizeFontButton = screenIsHighResolution ? 13 : 11;
		sizeFontMenuItem = screenIsHighResolution ? 14 : 12;
		sizeFontEditor = screenIsHighResolution ? 13 : 11;
		sizeFontTooltip = screenIsHighResolution ? 14 : 12;
		paneInsertionCaractereGrec = new PaneInsertionCaractereGrec();
		paneInsertionCaractereMath = new PaneInsertionCaractereMath();
		paneInsertionCaractereFleche = new PaneInsertionCaractereFleche();
		paneLatexInsertionCaractereGrec = new PaneLatexInsertionCaractereGrec();
		paneLatexInsertionCaractereMath = new PaneLatexInsertionCaractereMath();
		paneLatexInsertionCaractereFleche = new PaneLatexInsertionCaractereFleche();
		mtgDesktopPane = new MtgDesktopPane();
		//
		mtgDesktopPane = new MtgDesktopPane();
		mtgDesktopPane.setBackground(Color.white);
		paneLeft = new JPanel();
		paneSouth = new JPanel();
		paneRight = new JPanel();
		
		setVisible(false);
		// On met le dlai des info-bulles  1/4 s
		ToolTipManager.sharedInstance().setInitialDelay(250);
		// frameActif = this; Supprim version 4.8
	
		// On charge les prfrences
		chargePreferences();
		adaptePoliceSysteme(); // Ajout version 4.7.5.2

		setTitle("MathGraph32 Java");
		// On tablit l'icne de l'application
		URL adr = MtgFrame.class.getResource("images/logoMathGraph.png");
		MtgImageIcon imic = new MtgImageIcon(1, adr);
		Image imageic = imic.getImage();
		setIconImage(imageic);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		addWindowListener(this);
		addComponentListener(this); // Ajout version 4.6.3
		// Prparation des curseurs utilisateurs
		adr = MtgFrame.class.getResource("images/CurseurCoin.gif");
		ImageIcon icon = new ImageIcon(adr);
		Image img = icon.getImage();
		cornerCursor = Toolkit.getDefaultToolkit().createCustomCursor(img, new Point(0, 0), "");
		adr = MtgFrame.class.getResource("images/CurseurMacro.gif");
		icon = new ImageIcon(adr);
		img = icon.getImage();
		macroCursor = Toolkit.getDefaultToolkit().createCustomCursor(img, new Point(6, 0), "");
		adr = MtgFrame.class.getResource("images/CurseurSup.gif");
		icon = new ImageIcon(adr);
		img = icon.getImage();
		supCursor = Toolkit.getDefaultToolkit().createCustomCursor(img, new Point(0, 0), "");
		adr = MtgFrame.class.getResource("images/CurseurModif.gif");
		icon = new ImageIcon(adr);
		img = icon.getImage();
		modifCursor = Toolkit.getDefaultToolkit().createCustomCursor(img, new Point(0, 0), "");
		adr = MtgFrame.class.getResource("images/CurseurGomme.gif");
		icon = new ImageIcon(adr);
		img = icon.getImage();
		gommeCursor = Toolkit.getDefaultToolkit().createCustomCursor(img, new Point(0, 0), "");
		adr = MtgFrame.class.getResource("images/CurseurPalette.gif");
		icon = new ImageIcon(adr);
		img = icon.getImage();
		paletteCursor = Toolkit.getDefaultToolkit().createCustomCursor(img, new Point(0, 0), "");
		adr = MtgFrame.class.getResource("images/CurseurCrayon.gif");
		icon = new ImageIcon(adr);
		img = icon.getImage();
		crayonCursor = Toolkit.getDefaultToolkit().createCustomCursor(img, new Point(0, 0), "");

		listeClignotante = new CListeObjets();
		listeObjetsVisuels = new CListeObjets();
		listeExclusionDesignation = new CListeObjets();
		indicationArea.setEditable(false);
		indicationArea.setFont(FonteNom.getFonteNom(16, FonteNom.Normal, pref_addSystemFont/16 + coefMultForSystemFonts));		

		// On cre le dialogue de choix de couleurs
		colorChooser = new JColorChooser();
		colorChooser.setPreviewPanel(new JPanel()); // Pour qu'il n'y ait pas de
		// preview
		colorChooser.setColor(Color.black);
		Insets ins = new Insets(0, 0, 0, 0);
		barHor.setMargin(ins);
		barHor.setFloatable(false);

		// Cration des outils
		// creeBoutonsEtActions(); Supprim version 5.0
		// ajouteBoutonsBarreOutilsNiveauLycee();
		// metAJourBoutonsBarreHorizontale(null); Supprim version 5.0
		
		paneRight.setLayout(new BorderLayout());
		paneSouth.setLayout(new BorderLayout());
		paneSouth.add("East", paneInfo);
		paneSouth.add("Center", indicationArea);
		creeBarreDroite();

		stylePointActif = MtgFrame.screenIsHighResolution ? MotifPoint.Rond : MotifPoint.PetitRond; // Modifi version 5.0
		styleFlecheActif = StyleFleche.FlecheLonguePleine;
		styleMarqueSegmentActif = StyleMarqueSegment.marqueSegment1;
		styleRemplissageActif = StyleRemplissage.QuadrillagePleinTransp;
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add("North", barHor);
		getContentPane().add("West", paneLeft);
		getContentPane().add("Center", mtgDesktopPane);
		getContentPane().add("South", paneSouth);
		getContentPane().add("East", paneRight);

		// Le menu Fenetre ne doit pas tre mis  jour quand on change les options
		// de fonctionnement
		menuFenetre = new MtgMenu(getString("menuFenetre"), this);

		// activeOutilCapture();  // Modifi version 4.4.1.2
		// actionEnCours = actionCapture;
		setVisible(true);
		//
	}

	public int nombreCadresOuverts() {
		JInternalFrame[] tableCadres = mtgDesktopPane.getAllFrames();
		return tableCadres.length;
	}

	public void windowActivated(WindowEvent e) {
		// On tablit la variable globale frameActif
		// frameActif = this; Supprim version 4.8
	}

	public void windowClosed(WindowEvent e) {
		System.exit(0);
	}

	public void windowClosing(WindowEvent e) {
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		JInternalFrame[] table = mtgDesktopPane.getAllFrames();
		for (int i = 0; i < table.length; i++) {
			MtgInternalFrame cadre = (MtgInternalFrame) table[i];
			if (cadre.document.isDirty()) {
				int rep = dialogueConfirmation_YES_NO_CANCEL(getString("ch119") + " "
								+ cadre.nomDocument());
				if (rep == JOptionPane.CANCEL_OPTION) {
					setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
					return;
				}
				if (rep == JOptionPane.YES_OPTION)
					if (!((ActionFichierEnregistrer) cadre.frame.actionFichierEnregistrer)
							.enregistreFigure()) {
						setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
						return;						
					}
			}
		}
		if (cadreActif() != null) outilActif.deselect();
		// On sauve les prfrences de l'application
		sauvePreferences();
	}

	// Modifi version 5.0
	public void windowDeactivated(WindowEvent e) {
		if (tabIcones != null) fermeMenusHorizontaux();
	}

	public void windowDeiconified(WindowEvent e) {
	}

	public void windowIconified(WindowEvent e) {
		activeOutilCapture();
	}

	public void windowOpened(WindowEvent e) {
	}

	public void exit() {
		JInternalFrame[] table = mtgDesktopPane.getAllFrames();
		for (int i = 0; i < table.length; i++) {
			MtgInternalFrame cadre = (MtgInternalFrame) table[i];
			if (cadre.document.isDirty()) {
				int rep = dialogueConfirmation_YES_NO_CANCEL(getString("ch119") + " "
								+ cadre.nomDocument());
				if (rep == JOptionPane.CANCEL_OPTION) {
					return;
				}
				if (rep == JOptionPane.YES_OPTION)
					if (!((ActionFichierEnregistrer) cadre.frame.actionFichierEnregistrer).enregistreFigure()) return;
			}
		}
		// On sauve les prfrences de l'application
		sauvePreferences();
		dispose();
		System.exit(0);
	}
	
	/**
	 * Pointeur sur le cadre en cours d'activit
	 * 
	 * @return
	 */
	public MtgInternalFrame cadreActif() {
		return (MtgInternalFrame) mtgDesktopPane.getSelectedFrame();
	}

	/**
	 * Activation de la fentre dont le titre est titre
	 * 
	 * @param titre
	 */
	public void activeCadre(MtgInternalFrame cadre) {
		mtgDesktopPane.getDesktopManager().activateFrame(cadre);
		try {
			cadre.setSelected(true);
		} catch (PropertyVetoException e) {
		}
		// metAJourIconesEtMenus(cadre.document);  // Version 4.1.2 : Inutile car fait dans MtgInternalFrame.internalFrameActivated
		// cadre.montreDialogueAffichageVariables();
	}
	/* Supprim version 5.0
	public void actionPerformed(ActionEvent e) {
	}
	*/

	/**
	 * Renvoie la couleur active qui est donne par le bouton colorButton
	 * 
	 * @return
	 */
	public Color couleurActive() {
		return paneTransparence.couleurActive();
	}

	/**
	 * Redessine la figure sans la recalculer via paint de MtgFigurePanel
	 */
	public void repaintFigure() {
		cadreActif().repaintFigureSansCalcul();
	}


	/**
	 * Fonction rajoutant un objet  la liste de la fentre active avec ventuellement dition du nom si c'est un point ou une droite
	 */
	// Ajout version 4.9.9
	public final void ajouteElement(CElementBase pt) {
		MtgInternalFrame cadre = cadreActif();
		pt.positionne(cadre.infRand, cadre.dimf);
		if (pt.getNatureCalcul() == NatureCalcul.NVariable) {
			cadre.listePr.add(1, pt);
			// Important version 4.8.0 La variable tant rajoute en dbut de liste
			// tous les indices doivent tre recalculs
			cadre.listePr.updateIndexes();
		} else {
			cadre.listePr.add(pt);
		}
		// Ajout version 4.9.9
		if (!pref_NomsAuto && pt.estDeNature(NatureObjet.NToutPoint | NatureObjet.NDroite) && pt.existe && !(pt.getClass() == CPointLieBipoint.class)) {
			// Modifi version 5.0 : Pas d'dition si on a dj affect un nom  l'lment (outil rapporteur)
			if (!((CElementGraphique)pt).masque && ((CElementGraphique)pt).nom.isEmpty()) {
				MtgNameEditor ne = cadre.nameEditor;
				ne.associeA((CElementGraphique) pt);
				ne.setPositionOnInternaleFrame();
			}
		}
		//
		// Ajout version 4.9.8.2
		pt.metAJourPourCreation();
	}
	/**
	 * Ajout version 5.0
	 * Fonction rajoutant un objet  la liste de la fentre active sans dition du nom si c'est un point ou une droite
	 * @param pt
	 */
	public final void ajouteElementSansEditionNom(CElementBase pt) {
		MtgInternalFrame cadre = cadreActif();
		pt.positionne(cadre.infRand, cadre.dimf);
		if (pt.getNatureCalcul() == NatureCalcul.NVariable) {
			cadre.listePr.add(1, pt);
			// Important version 4.8.0 La variable tant rajoute en dbut de liste
			// tous les indices doivent tre recalculs
			cadre.listePr.updateIndexes();
		} else {
			cadre.listePr.add(pt);
		}
		//
		// Ajout version 4.9.8.2
		pt.metAJourPourCreation();
	}
	
	
	// Ajout version 4.9.9
	// Modifi version 5.3
	// public final void editeNomIntersection(CBipoint bp) {
	public final void editeNomIntersection(CPointAncetre pt1, CPointAncetre pt2) {
		MtgInternalFrame cadre = cadreActif();
		MtgNameEditor ne = cadre.nameEditor;
		if (pt1.existe && pt1.dansFenetre && pt1.rendNom().isEmpty()) { // Attention : Le point peut avoir nomm automatiquement
			ne.associeA((CElementGraphique) pt1);
			ne.setPositionOnInternaleFrame();		
			return;
		}
		else {
			if (pt2.existe && pt2.dansFenetre && pt1.rendNom().isEmpty()) {
				ne.associeA((CElementGraphique) pt2);
				ne.setPositionOnInternaleFrame();		
				return;
			}
		}
	}

	/**
	 * Fonction ajoutant l'lment pt avant l'lment d'indice indice et dcale pt
	 * et les suivants
	 * 
	 * @param pt
	 * @param indice
	 */
	public void insereElement(CElementBase pt, int indice) {
		MtgInternalFrame cadre = cadreActif();
		cadre.listePr.add(indice, pt);
		cadre.listePr.updateIndexes(); // Ajout version 4.9.2 pour corriger bug depuis gestion diffrentes de inexeOf
	}

	public void remplaceElement(CElementBase ancien, CElementBase nouveau) {
		MtgInternalFrame cadre = cadreActif();
		CListeObjets liste = cadre.listePr;
		int ind = liste.indexOf(ancien);
		liste.set(ind, nouveau);
		liste.positionne(cadre.infRand, cadre.dimf);
	}
	/**
	 * Ajout version 5.0
	 * Ajoute un point de coordonnes (x, y)
	 * @param x
	 * @param y
	 * @param nomAGenerer : Si true on gnrre un nom pour le point cr
	 */ 
	public final CPointAncetre ajoutePoint(double x, double y, boolean nomAGenerer) {
    CPointBase pt = new CPointBase(cadreActif().listePr, null, false,
        couleurActive(), false, 0, 3, false, (nomAGenerer || pref_NomsAuto) ? genereNomPourPoint() : "", pref_indiceTaillePoliceNom,
        stylePointActif(), true, x, y, false);
    ajouteElementSansEditionNom(pt);
    return pt;
	}

	/**
	 * Fonction rajoutant un objet  la liste principale de la fentre active Il
	 * faut pour cela mettre  zro la liste clone et la cloner  nouveau Renvoie
	 * true si l'lment n'a pas dj t cr N'est pas utilis dans le cas
	 * d'image par une transformation car dans ce cas il y a des lments qui ont
	 * t ventuellement ajouts et  dtruire
	 * Modifi version 3.9 : Vrifie que l'lment existe. Sinon demande confirmation de la cration
	 */
	public boolean ajouteElementAvecVerification(CElementBase pt) {
		if (!cadreActif().listePr.existeDeja(pt)) {
			if (pt.estDeNature(NatureObjet.NToutObjet)) {
				if (pt.antecedentDirect() != pt)
					pt.antecedentDirect().positionne(false, cadreActif().dimf);
				pt.positionne(false, cadreActif().dimf);
				if (pt.existe) {
					if (!((CElementGraphique) pt).dansFenetre(cadreActif().dimf)) {
						if (!dialogueConfirmation_YES_NO(getString("ch94")))
							return false;
					}
				}
				else {
	        if (!dialogueConfirmation_YES_NO(getString("ch85"))) {
	        	return false;
	        }
				}
			}
			ajouteElement(pt);
			positionne();
			return true;
		} else {
			dialogueAvertissement(getString("ch7"));
			return false;
		}
	}

	/**
	 * Fonction rajoutant un objet image par transformation  la liste principale
	 * de la fentre active Renvoie true si l'lment n'a pas dj t cr
	 * 
	 * @param nbEltsAjoutes
	 *          reprsente le nombre d'lments qui ont t ventuellement 	 *          ajouts
	 * @param trans
	 *          est la transformation lors de la cration de l'objet (par exemple
	 *          pour un segment image les points images des extrmits)
	 */
	public boolean ajouteElementsImageAvecVerification(CElementBase pt,
			int nbEltsAjoutes, CTransformation trans) {
		MtgInternalFrame cadre = cadreActif();
		CListeObjets liste = cadre.listePr;
		DimensionFenetre dimfen = cadre.dimf;
		if (!cadre.listePr.existeDeja(pt)) {
			// On commence par positionner la tansformation qui est le premier des
			// objets ajouts  la liste
			// avant les objets ajouts ventueks
			trans.positionne(false, dimfen);
			pt.antecedentDirect().positionne(false, dimfen);
			// Il faut positionner les ventuels lments ajouts
			for (int i = nbEltsAjoutes; i >= 1; i--) {
				CElementBase el = liste.get(liste.size() - i);
				el.positionne(false, dimfen);
			}
			// On positionne aussi l'image  rajouter
			pt.positionne(false, dimfen);
			if (!((CElementGraphique) pt).dansFenetre(dimfen)) {
				if (!dialogueConfirmation_YES_NO(getString("ch94")))
					return false;
			}
			ajouteElement(pt);
			positionne();
			return true;
		} else {
			if (nbEltsAjoutes != 0)
				detruitNDerniersElements(nbEltsAjoutes);
			dialogueAvertissement(getString("ch7"));
			return false;
		}
	}

	/**
	 * Fonction dtruisant le dernier objet de la liste du cadre actif et de sa
	 * liste clone listeClone
	 */
	public void detruitDernierElement() {
		cadreActif().listePr.detruitDernierObjet();
	}

	/**
	 * Fonction dtruisant les n derniers objets de la liste du cadre actif et de
	 * sa liste clone listeClone
	 */
	public void detruitNDerniersElements(int n) {
		for (int i = 0; i < n; i++) {
			detruitDernierElement();
		}
	}

	/**
	 * Retrait des listes des objets dpendant de pt Ces lments pourraont
	 * ensuite tre dtruits par le garbage collector
	 * 
	 * @param pt
	 */
	public void retireDependants(CElementBase pt) {
		MtgInternalFrame cadre = cadreActif();
		cadre.listePr.retireDependants(pt, cadre);
		cadre.videListesPourConstructions();
	}

	/**
	 * Fonction retirant de la liste les N derniers lments
	 * 
	 * @param N
	 */
	public void retireNDerniersElements(int N) {
		MtgInternalFrame cadre = cadreActif();
		cadre.listePr.retireNDerniersElements(N);
		cadre.videListesPourConstructions();
	}

	/** Fonction rajoutant un objet  la liste des objets visuels */
	public void ajouteObjetVisuel(CElementBase pt) {
		listeObjetsVisuels.add(pt);
	}

	/**
	 * Positionnement de la liste du cadre actif
	 */
	public final void positionne() {
		MtgInternalFrame cadre = cadreActif();
		cadre.listePr.positionne(cadre.infRand, cadre.dimf);
	}

	/** Fonction positionnat la liste pour la cration visuelle d'objets */
	public void positionneListeVisuelle() {
		listeObjetsVisuels.positionne(false, cadreActif().dimf);
	}

	public void indication(String ch) {
		indicationArea.setText(ch);
		indicationArea.repaint();
	}

	public final void activeOutilCapture() {
		if (buttonCapture == null) return;
		if (outilActif != null) outilActif.deselect();
		outilActif = outilCapture;
		buttonCapture.setSelected(true);
		MtgInternalFrame cadre = cadreActif();
		if (cadre != null) {
			outilActif.select();
			cadre.requestFocus();
		}
	}

	/**
	 * Activation de l'outil de capture
	 */
	public final void activeOutilExecutionMacro() {
		if (outilActif != null)
			outilActif.deselect();
		outilActif = outilExecutionMacro;
		buttonExecuterMacro.setSelected(true);
		if (cadreActif() != null) { // Pour ne pas appeler deselect qui arrtrait
			// une animation en cours
			outilPointageActif = outilPointageExecutionMacro;
			outilPointageActif.designationDe(NatureObjet.NMacro);
			indication(getString("executionMacro1"));
			repaintFigure();
		}
	}

	// Pour les quatre fonctions suivantes, il faut dans toutes la mesure du possible les appeler
	// plutt que d'appeler les mthodes de mme nom dans MTgDlg car alors les messages sont
	// centrs par rapport  la fentre de l'application et non par rapport  l'cran
	/** Fonction affichant un dialogue d'avertissement */
	public void dialogueAvertissement(String texte) {
		JOptionPane.showMessageDialog(this, texte, getString("avertissement"),
				JOptionPane.WARNING_MESSAGE);
	}
	
	public final void dialogueAvertissement(Component comp, String texte) {
		JOptionPane.showMessageDialog(comp, texte, getString("avertissement"),
				JOptionPane.WARNING_MESSAGE);
	}	

	/** Fonction affichant un dialogue d'information */
	public void dialogueInformation(String texte) {
		JOptionPane.showMessageDialog(this, texte, getString("information"),
				JOptionPane.WARNING_MESSAGE);
	}
	/**
	 * Fonction affichant un dialogue demandant une rponse oui-non et renvoyant
	 * true si la rponse est oui
	 */
	public boolean dialogueConfirmation_YES_NO(String texte) {
		return (JOptionPane.showConfirmDialog(this, texte, getString("confirmation"),
				JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION);
	}
	
  public int dialogueConfirmation_YES_NO_CANCEL(String texte) {
  	return JOptionPane.showConfirmDialog(this, texte, getString("confirmation"), JOptionPane.YES_NO_CANCEL_OPTION);
  }
  
  public void creeBarreDroite() {
  	paneRight.removeAll();
		paneRightHaut = new JPanel();
		paneRightHaut.setLayout(new GridBagLayout());  	
		JPanel paneRightHaut1 = new JPanel();
		JPanel paneRightHaut2 = new JPanel();
		indicationArea.setForeground(Color.blue);
		labelNomsAuto = new MtgLabel(this, getString("labelNomsAuto"));
		// labelNomsAuto.setFont(Font.decode("Dialog-11")); // Supprim version 4.7.5.2
		labelNomsAuto.setBorder(BorderFactory.createRaisedBevelBorder());
		labelAideVisuelle = new MtgLabel(this, getString("labelAideVisuelle"));
		// labelAideVisuelle.setFont(Font.decode("Dialog-11")); // Supprim version 4.7.5.2
		labelAideVisuelle.setBorder(BorderFactory.createRaisedBevelBorder());
		labelUniteAngle = (pref_angleUnit == UniteAngle.Degre) ? new MtgLabel(this, getString("radioDegre")) : new MtgLabel(this, getString("radioRadian"));
		// labelUniteAngle.setFont(Font.decode("Dialog-11")); // Supprim version 4.7.5.2
		labelUniteAngle.setBorder(BorderFactory.createRaisedBevelBorder());
		paneTransparence = new MtgPaneTransparence(this, transparenceParDefaut);
		paneCouleurs = new MtgPaneChoixCouleur(this);
		paneStylePoint = new MtgPaneStylePoint(this);
		paneStylePoint.setBorder(BorderFactory.createEtchedBorder());
		paneStyleMarqueSegment = new MtgPaneStyleMarqueSegment(this);
		paneStyleFleche = new MtgPaneStyleFleche(this);
		paneStyleRemplissage = new MtgPaneStyleRemplissage(this);

		listeStyleTrait = new MtgListeStyleTrait(this);
		paneStyleMarqueAngle = new MtgPaneStyleMarqueAngle(this);
		JScrollPane scrollPaneStyleTrait = new JScrollPane(listeStyleTrait,
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		scrollPaneStyleTrait.setBorder(BorderFactory.createEtchedBorder());		
		
		// Ajout version 5.0
		labelEpTrait = new JLabel("1");
		labelEpTrait.setFont(FonteNom.getFonteNom(14, FonteNom.Normal, coefMultForDlgBoxIcons));
		labelEpTrait.setToolTipText(getString("tipEpaisseur"));
		sliderEpaisseur = new JSlider(JSlider.HORIZONTAL, 1, 15, 1);
		sliderEpaisseur.setMajorTickSpacing(10);
		sliderEpaisseur.setMinorTickSpacing(1);
		sliderEpaisseur.setPaintTicks(true);
		sliderEpaisseur.setPaintLabels(true);
		sliderEpaisseur.setToolTipText(getString("tipEpaisseur"));
		double k = MtgFrame.coefMultForDlgBoxIcons;
		Font font = new Font("Serif", Font.ROMAN_BASELINE, (int)(k*11));
		sliderEpaisseur.setFont(font);
		sliderEpaisseur.setPreferredSize(new Dimension((int)(k*80), (int)(k*40)));
		sliderEpaisseur.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				labelEpTrait.setText(Integer.toString(sliderEpaisseur.getValue()));
				
			}
		});
		paneSlider = new JPanel();
		paneSlider.setLayout(new BoxLayout(paneSlider, BoxLayout.LINE_AXIS));
		paneSlider.add(sliderEpaisseur);
		paneSlider.add(labelEpTrait);
		paneSlider.setBorder(BorderFactory.createEmptyBorder(5,0,5,0));
		//
		
		JScrollPane scrollPaneListeStyleMarqueAngle = new JScrollPane(
				paneStyleMarqueAngle, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		scrollPaneListeStyleMarqueAngle.setBorder(BorderFactory.createEtchedBorder());		
		scrollPaneListeStyleMarqueAngle.getVerticalScrollBar().setUnitIncrement(16);

		// int a = listeStyleMarqueAngle.barStyleMarqueAngle1.getPreferredSize().width;
		Dimension d = scrollPaneListeStyleMarqueAngle.getPreferredSize();
		d.height = paneStyleMarqueAngle.taillePourScrollPane().height;
		d.width += 3;
		scrollPaneListeStyleMarqueAngle.setPreferredSize(d);
		scrollPaneListeStyleMarqueAngle.setBorder(BorderFactory.createEtchedBorder());
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 0;
		constraints.insets.top = 0;
		constraints.insets.left = 0;
		constraints.insets.right = 0;

	
		paneRightHaut1.setLayout(new BoxLayout(paneRightHaut1, BoxLayout.X_AXIS));
		paneRightHaut1.add(scrollPaneStyleTrait);
		paneRightHaut1.add(paneStylePoint);
		paneRightHaut2.setLayout(new BoxLayout(paneRightHaut2, BoxLayout.X_AXIS));
		paneRightHaut2.add(paneStyleMarqueSegment);
		paneRightHaut2.add(paneStyleFleche);
		
		
		MtgDlg.setConstraints(constraints, 0, 0, 1, 1, 100, 25,
				GridBagConstraints.NONE, GridBagConstraints.NORTH);
		paneRightHaut.add(paneRightHaut1, constraints);
		MtgDlg.setConstraints(constraints, 0, 1, 1, 1, 100, 25,
				GridBagConstraints.NONE, GridBagConstraints.NORTH);
		paneRightHaut.add(paneSlider, constraints);		
		MtgDlg.setConstraints(constraints, 0, 2, 1, 1, 100, 15,
				GridBagConstraints.NONE, GridBagConstraints.NORTH);
		paneRightHaut.add(paneRightHaut2, constraints);
		MtgDlg.setConstraints(constraints, 0, 3, 1, 1, 100, 25,
				GridBagConstraints.NONE, GridBagConstraints.NORTH);
		paneRightHaut.add(scrollPaneListeStyleMarqueAngle, constraints);
		MtgDlg.setConstraints(constraints, 0, 4, 1, 1, 100, 25,
				GridBagConstraints.NONE, GridBagConstraints.NORTH);
		paneRightHaut.add(paneStyleRemplissage, constraints);	
		MtgDlg.setConstraints(constraints, 0, 5, 1, 1, 100, 50,
				GridBagConstraints.NONE, GridBagConstraints.NORTH);
		paneRightHaut.add(paneCouleurs, constraints);
		MtgDlg.setConstraints(constraints, 0, 6, 1, 1, 100, 15,
				GridBagConstraints.NONE, GridBagConstraints.NORTH);
		paneRightHaut.add(paneTransparence, constraints);
		// Ajout version 5.0
		MtgButtonFinAction buttonFinAction = new MtgButtonFinAction(this);
		MtgDlg.setConstraints(constraints, 0, 7, 1, 1, 100, 15,
				GridBagConstraints.NONE, GridBagConstraints.NORTH);
		paneRightHaut.add(buttonFinAction, constraints);

		paneRight.add("North", paneRightHaut);
		paneInfo.removeAll();
		paneInfo.add(labelUniteAngle);
		paneInfo.add(labelNomsAuto);
		paneInfo.add(labelAideVisuelle);
		if (!pref_NomsAuto) labelNomsAuto.setVisible(false);
		if (!pref_AideVisuelle) labelAideVisuelle.setVisible(false);
		
  }
	
  public final void creeActions() {
		actionChoixFenetre = new ActionChoixFenetre(this);
		itemChoixFenetre = MtgItemMenu(actionChoixFenetre);
		actionAnnuler = new ActionAnnuler(this);
		actionRefaire = new ActionRefaire(this);
		actionFichierOuvrir = new ActionFichierOuvrir(this);
		actionTranslaterFigure = new ActionTranslaterFigure(this);
		actionZoomPlus = new ActionZoomPlus(this);
		actionZoomMoins = new ActionZoomMoins(this);
		actionCapture = new ActionCapture(this);
		actionSuppressionObjetGraphique = new ActionSuppressionObjetGraphique(this);
		actionModifierNom = new ActionModifierNom(this);
		actionAfficherObjetsNumeriques = new ActionAfficherObjetsNumeriques(this);
		actionHelpTexteFigure = new ActionHelpTexteFigure(this);
		actionSuppressionValeur = new ActionSuppressionValeur(this);
		actionPalette = new ActionPalette(this);
		actionModifierObjetGraphique = new ActionModifierObjetGraphique(this);
		actionExecuterMacro = new ActionExecuterMacro(this);
		actionGomme = new ActionGomme(this);
		actionFichierNouvelleFigure = new ActionFichierNouvelleFigure(this);
		actionFichierEnregistrer = new ActionFichierEnregistrer(this);
		actionFichierImprimer = new ActionFichierImprimer(this);
		actionModeTrace = new ActionModeTraceActivation(this);
		actionEditionCopier = new ActionEditionCopier(this);
		actionRideau = new ActionRideau(this);
		actionReafficherTout = new ActionReafficherTout(this);
		actionRelationEntreObjets = new ActionRelationEntreObjets(this);
		// Ajout version 5.0
		actionModePointAuto = new ActionModePointAuto(this);
		actionRecopierStyle = new ActionRecopierStyle(this);
		actionFichierExporterPNGAvecUnite = new ActionFichierExporterPNGAvecUnite(this);
		actionFichierNouvelleFigureAvecRepere = new ActionFichierNouvelleFigureAvecRepere(this);
		//
		actionProtocole = new ActionProtocole(this); // Ajout version4.9.9
		actionReclassementDebutObjetGraphique = new ActionReclassementDebutObjetGraphique(this);
		actionReclassementFinObjetGraphique = new ActionReclassementFinObjetGraphique(this);
		actionAugmenterTailles = new ActionAugmenterTailles(this);
		actionDiminuerTailles = new ActionDiminuerTailles(this);
		// Actions correspondants  des outils qui ne sont pas dans la barre du haut
		actionCreerPointLibre = new ActionCreerPointLibre(this);
		actionCreerMilieu = new ActionCreerMilieu(this);
		actionCreerPointLie = new ActionCreerPointLie(this);
		actionCreerDroiteAB = new ActionCreerDroiteAB(this);
		actionCreerDroiteParallele = new ActionCreerDroiteParallele(this);
		actionCreerDroitePerpendiculaire = new ActionCreerDroitePerpendiculaire(this);
		actionCreerDroiteBissectrice = new ActionCreerDroiteBissectrice(this);
		actionCreerDroiteMediatrice = new ActionCreerDroiteMediatrice(this);
		actionCreerSegment = new ActionCreerSegment(this);
		actionCreerDemiDroite = new ActionCreerDemiDroite(this);
		actionCreerIntersection = new ActionCreerIntersection(this);
		actionCreerCercleOA = new ActionCreerCercleOA(this);
		actionCreerCercleOR = new ActionCreerCercleOR(this);
		actionCreerCalcul = new ActionCreerCalculReel(this);
		actionMesureLongueur = new ActionMesureLongueur(this);
		actionMesureAbscisse = new ActionMesureAbscisse(this);
		actionMesureAngleNonOriente = new ActionMesureAngleNonOriente(this);
		actionMesureAngleOriente = new ActionMesureAngleOriente(this);
		actionRapporteur = new ActionRapporteur(this);
		actionCreerMarqueAngleNonOriente = new ActionCreerMarqueAngleNonOriente(this);
		actionCreerMarqueAngleOriente = new ActionCreerMarqueAngleOriente(this);
		actionCreerImageParTranslation = new ActionCreerImageParTranslation(this);
		actionCreerImageParTranslationParCoord = new ActionCreerImageParTranslationParCoord(this);
		actionCreerImageParSymetrieAxiale = new ActionCreerImageParSymetrieAxiale(this);
		actionCreerImageParSymetrieCentrale = new ActionCreerImageParSymetrieCentrale(this);
		actionCreerImageParHomothetie = new ActionCreerImageParHomothetie(this);
		actionCreerImageParRotation = new ActionCreerImageParRotation(this);
		actionCreerArcPetitParPointFinal = new ActionCreerArcPetitParPointFinal(this);
		actionCreerArcGrandParPointFinal = new ActionCreerArcGrandParPointFinal(this);
		actionCreerArcDirectParPointFinal = new ActionCreerArcDirectParPointFinal(this);
		actionCreerArcIndirectParPointFinal = new ActionCreerArcIndirectParPointFinal(this);
		actionCreerArcPetitParRapporteur = new ActionCreerArcPetitParRapporteur(this);
		actionCreerArcGrandParRapporteur = new ActionCreerArcGrandParRapporteur(this);
		actionCreerDroiteHorizontale = new ActionCreerDroiteHorizontale(this);
		actionCreerDroiteVerticale = new ActionCreerDroiteVerticale(this);
		actionCreerMarqueSegment = new ActionCreerMarqueSegment(this);
		actionCreerPolygone = new ActionCreerPolygone(this);
		actionCreerLigneBrisee = new ActionCreerLigneBrisee(this);
		actionCreerSurface = new ActionCreerSurface(this);
		actionCreerSurfaceLieuDroite = new ActionCreerSurfaceLieuDroite(this);
		actionCreerSurfaceLieuDeuxPoints = new ActionCreerSurfaceLieuDeuxPoints(this);
		actionCreerSurfaceDeuxLieux = new ActionCreerSurfaceDeuxLieux(this);
		actionCreerProjeteOrthogonal = new ActionCreerProjeteOrthogonal(this);
		actionCreerRepere = new ActionCreerRepere(this);
		actionCreerPointParCoordonnees = new ActionCreerPointParCoordonnees(this);
		actionCreerDroiteParCoef = new ActionCreerDroiteParCoef(this);
		actionCreerDroiteParCoef = new ActionCreerDroiteParCoef(this);
		actionCreerAffichageValeur = new ActionCreerAffichageValeur(this);
		actionCreerAffichageValeurLiePoint = new ActionCreerAffichageValeurLiePoint(this);
		actionCreerCentreGravite = new ActionCreerCentreGravite(this);
		actionCreerPointParAffixe = new ActionCreerPointParAffixe(this);
		actionCreerPointParAbscisse = new ActionCreerPointParAbscisse(this);
		actionCreerCommentaire = new ActionCreerCommentaire(this);
		actionCreerImageLibre = new ActionCreerImageLibre(this);
		actionCreerImageLieePoint = new ActionCreerImageLieePoint(this);
		actionCreerCommentaireLiePoint = new ActionCreerCommentaireLiePoint(this);
		actionCreerLatex = new ActionCreerLatex(this);
		actionCreerLatexLiePoint = new ActionCreerLatexLiePoint(this);
		actionCreerEquationDroite = new ActionCreerEquationDroite(this);
		actionCreerEquationDroiteLiePoint = new ActionCreerEquationDroiteLiePoint(this);
		actionCreerDroiteParEquation = new ActionCreerDroiteParEquation(this);
		actionCreerLieuParPointLie = new ActionCreerLieuParPointLie(this);
		actionCreerImageParSimilitude = new ActionCreerImageParSimilitude(this);
		actionCreerBarycentre = new ActionCreerBarycentre(this);
		actionCreerPointParSommeVect = new ActionCreerPointParSommeVect(this);
		actionCreerPointParMultVect = new ActionCreerPointParMultVect(this);
		actionCarreDirect = new ActionCarreDirect(this);
		actionTriangleEquilateralDirect = new ActionTriangleEquilateralDirect(this);
		actionParallelogramme = new ActionParallelogramme(this);
		actionPolygoneRegulier = new ActionPolygoneRegulier(this);
		actionCreerVecteur = new ActionCreerVecteur(this);
		actionMesureAbscisseDansRepere = new ActionMesureAbscisseDansRepere(this);
		actionMesureOrdonneeDansRepere = new ActionMesureOrdonneeDansRepere(this);
		actionMesureAffixe = new ActionMesureAffixe(this);
		actionMesureLongueurLigne = new ActionMesureLongueurLigne(this);
		actionMesureAirePolygone = new ActionMesureAirePolygone(this);
		actionMesureProduitScalaire = new ActionMesureProduitScalaire(this);	
		actionPunaiser = new ActionPunaiser(this);
		actionDepunaiser = new ActionDepunaiser(this);
		actionCreerPointLibreEnt = new ActionCreerPointLibreEnt(this);
		actionCreerFonctionReelle = new ActionCreerFonctionReelle(this);
		actionCreerCourbeFonction = new ActionCreerCourbeFonction(this);
		actionPointSurCourbeParAbscisse = new ActionPointSurCourbeParAbscisse(this);
		actionCurseur = new ActionCurseur(this);
		actionTangente = new ActionTangente(this);
		actionMesureCoefficientDirecteur = new ActionMesureCoefficientDirecteur(this);
		actionCreerPointInterieur = new ActionCreerPointInterieurPolygone(this);
		actioncreerEditeurFormule = new ActionCreerEditeurFormule(this);
		actionCreerSegmentParLong = new ActionSegmentParLong(this);
		actionHelpOnline =new ActionHelpOnline(this);
		actionChangerStyleTousPoints = new ActionChangerStyleTousPoints(this);
		actionFigureBaseCourbes = new ActionFichierNouvelleFigureBase(this, "courbeFonction");
		actionFigureBaseCourbePolaire = new ActionFichierNouvelleFigureBase(this, "courbePolaire");
		actionFigureBaseCourbeParam = new ActionFichierNouvelleFigureBase(this, "courbeParam");
		actionFigureBaseSuiteunfn = new ActionFichierNouvelleFigureBase(this, "grapheSuiteunfn");
		actionFigureBaseSuiteRecParPtLie = new ActionFichierNouvelleFigureBase(this, "grapheSuiteRecParPtLie");
		actionFigureBaseSuiteRecParPremierTerme = new ActionFichierNouvelleFigureBase(this, "grapheSuiteRecParPremierTerme");
		actionFigureBaseLoiBinomiale = new ActionFichierNouvelleFigureBase(this, "loiBinomiale");
		actionFigureBaseLoiBinomialeIntervalleConfiance = new ActionFichierNouvelleFigureBase(this, "loiBinomialeIntervalleConfiance95");
		actionFigureBaseLoiBinomialeApproximationParLoiNormale = new ActionFichierNouvelleFigureBase(this, "loiBinomialeApproximationParLoiNormale");
		actionFigureBaseLoiNormale = new ActionFichierNouvelleFigureBase(this, "loiNormale");
		actionFigureBaseLoiExponentielle = new ActionFichierNouvelleFigureBase(this, "loiExponentielle");
		actionFigureBaseLoiNormaleualpha = new ActionFichierNouvelleFigureBase(this, "loiNormaleualpha");
		actionFigureBaseLoiNormaleInverse = new ActionFichierNouvelleFigureBase(this, "loiNormaleInverse");
		actionFigureBaseEspaceCube = new ActionFichierNouvelleFigureBase(this, "espaceCube");
		actionFigureBaseEspaceParallelepipedeRect = new ActionFichierNouvelleFigureBase(this, "espaceParallelepipedeRect");
		actionFigureBaseEspaceTetraedre = new ActionFichierNouvelleFigureBase(this, "espaceTetraedre");
		actionFigureBaseEspaceTetraedreRegulier = new ActionFichierNouvelleFigureBase(this, "espaceTetraedreRegulier");
		actionFigureBaseEspaceTetraedre = new ActionFichierNouvelleFigureBase(this, "espaceTetraedre");
		actionFigureBaseEspaceCylindre = new ActionFichierNouvelleFigureBase(this, "espaceCylindre");
		actionFigureBaseEspaceConeSimple = new ActionFichierNouvelleFigureBase(this, "espaceConeSimple");
		actionFigureBaseEspaceConeDouble = new ActionFichierNouvelleFigureBase(this, "espaceConeDouble");
		actionFigureBaseEspacePyramideBaseCarre = new ActionFichierNouvelleFigureBase(this, "espacePyramideBaseCarre");
		actionFigureBaseEspacePyramideBaseRectangle = new ActionFichierNouvelleFigureBase(this, "espacePyramideBaseRectangle");
		actionFigureBaseEspacePyramideReguliereBaseCarre = new ActionFichierNouvelleFigureBase(this, "espacePyramideReguliereBaseCarre");
		actionFigureBaseEspacePrismeBaseTriangulaire = new ActionFichierNouvelleFigureBase(this, "espacePrismeBaseTriangulaire");
		actionFigureBaseEspacePrismeBaseTrapeze = new ActionFichierNouvelleFigureBase(this, "espacePrismeBaseTrapeze");
		actionFigureBaseEspaceSurfaceFonction2Var = new ActionFichierNouvelleFigureBase(this, "espaceSurfaceFonction2Var");
		actionFigureBaseEspaceRepereOrthonormal = new ActionFichierNouvelleFigureBase(this, "espaceRepereOrthonormal"); // Ajout version 5.1.2
		actionFigureBaseEspaceRepereOrthogonal = new ActionFichierNouvelleFigureBase(this, "espaceRepereOrthogonal"); // Ajout version 5.1.2
		actionFigureBaseRepere = new ActionFichierNouvelleFigureBase(this, "repereSansVecteurs");
		actionFigureBaseRepereVect = new ActionFichierNouvelleFigureBase(this, "repereAvecVecteurs");
		actionPostHtml = new ActionPostHtml(this); // Ajout version 5.0
		//
		actionCreerMarqueAngleOrienteSensDirect = new ActionCreerMarqueAngleOrienteSensDirect(this);
		actionCreerMarqueAngleOrienteSensIndirect = new ActionCreerMarqueAngleOrienteSensIndirect(this);
		actionCreerCalculComplexe = new ActionCreerCalculComplexe(this);
		actionCreerFonctionComplexe = new ActionCreerFonctionComplexe(this);
		actionCreerVariable = new ActionCreerVariable(this);
		actionCreerLieuParVariable = new ActionCreerLieuParVariable(this);
		actionCreerLieuDiscretParPointLie = new ActionCreerLieuDiscretParPointLie(this);
		actionCreerLieuDiscretParVariable = new ActionCreerLieuDiscretParVariable(this);
		actionCreerLieuObjetParPointLie = new ActionCreerLieuObjetParPointLie(this);
		actionCreerLieuObjetParVariable = new ActionCreerLieuObjetParVariable(this);
		actionCreerImageParInversion = new ActionCreerImageParInversion(this);
		actionCreerDemiPlan = new ActionCreerDemiPlan(this);
		actionCreerDerivee = new ActionCreerDerivee(this);
		actionCreerSuiteRecurrenteReelle = new ActionCreerSuiteRecurrenteReelle(this);
		actionCreerSuiteRecurrenteComplexe = new ActionCreerSuiteRecurrenteComplexe(this);
		actionCreerGrapheSuiteRecurrenteReelle = new ActionCreerGrapheSuiteRecurrenteReelle(this);
		actionCreerGrapheSuiteRecurrenteComplexe = new ActionCreerGrapheSuiteRecurrenteComplexe(this);
		actionFichierEnregistrerSous = new ActionFichierEnregistrerSous(this);
		actionFichierQuitter = new ActionFichierQuitter(this);
		actionFichierNouvelleFigureSansLongueurUnite = new ActionFichierNouvelleFigureSansLongueurUnite(this);
		actionFichierNouvelleFigureParCode = new ActionFichierNouvelleFigureParCode(this); // Ajout version 4.7.7
		actionFichierExporterPNG = new ActionFichierExporterPNG(this);
		actionFichierExporterJPEG = new ActionFichierExporterJPEG(this);
		// actionFichierExporterHTML = new ActionFichierExporterHTML(this); PLus support version 5.0
		actionFichierExporterHTML5 = new ActionFichierExporterHTML5(this);  // Ajout version 4.7.7
		actionFichierExporterTikz = new ActionFichierExporterTikz(this); // Ajout version 4.8.1 pour exportation Tikz
		actionModifierRepere = new ActionModifierRepere(this);
		actionPointSurCourbeParPointLie = new ActionPointSurCourbeParPointLie(this);
		actionCreerSolutionEquation = new ActionCreerSolutionEquation(this);
		actionCreerMaximum = new ActionCreerMaximum(this);
		actionCreerMinimum = new ActionCreerMinimum(this);
		actionCreerIntegrale = new ActionCreerIntegrale(this);
		actionCreerTestExistence = new ActionCreerTestExistence(this);
		actionCreerPartieReelle = new ActionCreerPartieReelle(this);
		actionCreerPartieImaginaire = new ActionCreerPartieImaginaire(this);
		actionCreerModule = new ActionCreerModule(this);
		actionCreerArgument = new ActionCreerArgument(this);
		actionCreerSommeIndicee = new ActionCreerSommeIndicee(this);
		actionCreerProduitIndice = new ActionCreerProduitIndice(this);
		actionCreerMacroApparition = new ActionCreerMacroApparition(this);
		actionCreerMacroDisparition = new ActionCreerMacroDisparition(this);
		actionCreerMacroAnimation = new ActionCreerMacroAnimation(this);
		actionCreerMacroAnimationAvecTrace = new ActionCreerMacroAnimationAvecTrace(this);
		actionCreerMacroAffectationValeurVariable = new ActionCreerMacroAffectationValeurVariable(this);
		actionCreerMacroModificationVariable = new ActionCreerMacroModificationVariable(this);
		actionCreerMacroJouantSon = new ActionCreerMacroJouantSon(this);
		actionCreerMacroIncrementationVariable = new ActionCreerMacroIncrementationVariable(this);
		actionCreerMacroDecrementationVariable = new ActionCreerMacroDecrementationVariable(this);
		actionCreerMacroTraceAutoParPointLie = new ActionCreerMacroTraceAutoParPointLie(this);
		actionCreerMacroTraceAutoParVariable = new ActionCreerMacroTraceAutoParVariable(this);
		actionCreerMacroAffectationPointLie = new ActionCreerMacroAffectationPointLie(this);
		actionCreerMacroClignotement = new ActionCreerMacroClignotement(this);
		actionCreerMacroAffichantTout = new ActionCreerMacroAffichantTout(this);
		actionCreerMacroPause = new ActionCreerMacroPause(this);
		actionCreerMacroBoucleAvecAnimation = new ActionCreerMacroBoucleAvecAnimation(this);
		actionCreerMacroBoucleAvecTrace = new ActionCreerMacroBoucleAvecTrace(this);
		actionCreerMacroSuiteMacros = new ActionCreerMacroSuiteMacros(this);
		actionCreerObjetDuplique = new actionCreerObjetDuplique(this);
		actionOptionsFigureEnCours = new ActionOptionsFigureEnCours(this);
		actionOptionsPreferences = new ActionOutilOptionsPreferences(this);
		actionSuppressionImageFond = new ActionSuppressionImageFond(this);
		actionImageFondVisible = new ActionImageFondVisible(this);
		actionChoixImageFond = new ActionChoixImageFond(this);
		actionMesureLongueurLigne = new ActionMesureLongueurLigne(this);
		actionCreerCentreCercle = new ActionCreerCentreCercle(this);
		actionConstructionImplementerDepuisFigure = new ActionConstructionImplementerDepuisFigure(this);
		actionAjoutObjetsMacro = new ActionAjoutObjetsMacro(this);
		actionRetraitObjetsMacro = new ActionRetraitObjetsMacro(this);
		actionModificationMacro = new ActionModifierMacro(this);
		actionMouvoirAnimationPointLie = new ActionMouvoirAnimationPointLie(this);
		actionRenommerCalcul = new ActionRenommerCalcul(this);
		actionConstructionEnregistrer = new ActionConstructionEnregistrer(this);
		actionIncorporerConstruction = new ActionConstructionIncorporer(this);
		actionModifierConstruction = new ActionConstructionModifier(this);
		actionCreerLongueurUnite = new ActionCreerLongueurUnite(this);
		actionChoixSourcesNonGraphiques = new ActionConstructionChoixSourcesNonGraphiques(this);
		actionChoixSourcesGraphiques = new ActionConstructionChoixSourcesGraphiques(this);
		actionChoixFinauxGraphiques = new ActionConstructionChoixFinauxGraphiques(this);
		actionChoixFinauxNonGraphiques = new ActionConstructionChoixFinauxNonGraphiques(this);
		actionConstructionCreer = new ActionConstructionCreer(this);
		actionConstructionReinitialiser = new ActionConstructionReinitialiser(this);
		actionConstructionImplementerDepuisFichier = new ActionConstructionImplementerDepuisFichier(this);
		actionConstructionImplementerPredefinie = new ActionConstructionImplementerPredefinie(this);
		actionConstructionSupprimer = new ActionConstructionSupprimer(this);
		actionConstructionFusionner = new ActionConstructionFusionner(this);
		actionCreationVisuelle = new ActionCreationVisuelle(this);
		actionNomsAuto = new ActionNomsAuto(this);
		actionEditerTexteFigure = new ActionEditerTexteFigure(this);
		actionCreerArcPetitParAngle = new ActionCreerArcPetitParAngle(this);
		actionCreerArcGrandParAngle = new ActionCreerArcGrandParAngle(this);
		actionCreerArcDirectParAngle = new ActionCreerArcDirectParAngle(this);
		actionCreerArcIndirectParAngle = new ActionCreerArcIndirectParAngle(this);
		actionHelpAbout = new ActionHelpAbout(this);
		actionHelpNet = new ActionHelpNet(this);
		actionHelpCodeHTML = new ActionHelpDimensions(this);
		actionHelpHelp = new ActionHelpHelp(this);
		actionSupprimerLiaison = new ActionSuppressionLiaison(this);
		actionCreerLiaison = new ActionCreerLiaison(this);
		actionSupprimerLiaisonAffichage = new ActionSuppressionLiaisonAffichage(this);
		actionCreerLiaisonAffichage = new ActionCreerLiaisonAffichage(this);
		actionReclassementFinValeur = new ActionReclassementFinValeur(this);
		actionReclassementDebutValeur = new ActionReclassementDebutValeur(this);
		actionActiverOutilPrecedent = new ActionActiverOutilPrecedent(this);
		actionCreerMacroApparitionAPartirAutreMacro = new ActionCreerMacroApparitionAPartirAutreMacro(this);
		actionCreerMacroDisparitionAPartirAutreMacro = new ActionCreerMacroDisparitionAPartirAutreMacro(this);
		actionTraceObjets = new ActionTraceObjets(this);
		actionFichierNouvelleFigurePredefinie = new ActionFichierNouvelleFigurePredefinie(this);
		actionCreationLiaisonPointInterieurPolygone = new ActionCreationLiaisonPointInterieurPolygone(this);
		actionCreationLiaisonPointInterieurCercle = new ActionCreationLiaisonPointInterieurCercle(this);
		actionCreerFonctionReelle2Var = new ActionCreerFonctionReelle2Var(this);
		actionCreerFonctionReelle3Var = new ActionCreerFonctionReelle3Var(this);
		actionCreerFonctionComplexe2Var = new ActionCreerFonctionComplexe2Var(this);
		actionCreerFonctionComplexe3Var = new ActionCreerFonctionComplexe3Var(this);
		actionOptionPersonnaliserNiveau1 = new ActionOptionPersonnaliserNiveau1(this);
		actionOptionPersonnaliserNiveau2 = new ActionOptionPersonnaliserNiveau2(this);
		actionOptionPersonnaliserNiveau3 = new ActionOptionPersonnaliserNiveau3(this);
		actionOptionPersonnaliserNiveau4 = new ActionOptionPersonnaliserNiveau4(this);
		actionCreerMacroModeTraceActivation = new ActionCreerMacroModeTraceActivation(this);
		actionCreerMacroModeTraceDesactivation = new ActionCreerMacroModeTraceDesactivation(this);
		actionMarquerPointPourTrace = new ActionMarquerPointPourTrace(this);
		actionDemarquerPointPourTrace = new ActionDemarquerPointPourTrace(this);
		actionCreerAbscisseOrigineRep = new ActionCreerAbscisseOrigineRep(this);
		actionCreerOrdonneeOrigineRep = new ActionCreerOrdonneeOrigineRep(this);
		actionCreerUnitexRep = new ActionCreerUnitexRep(this);
		actionCreerUniteyRep = new ActionCreerUniteyRep(this);
		actionAjoutGraduationSimple = new ActionAjoutGraduationSimple(this);		
		actionAjoutGraduationTrigo = new ActionAjoutGraduationTrigo(this);
		actionCreerTestEquivalence = new ActionCreerTestEquivalence(this);
		actionCreerTestEqNatOp = new ActionCreerTestEqNatOp(this);
		actionCreerDeriveePartielle = new ActionCreerDeriveePartielle(this);
		actionCreerTestDependanceVariable = new ActionCreerTestDependanceVariable(this);
		actionCreerSuiteRecurrenteComplexe2 = new ActionCreerSuiteRecurrenteComplexe2(this); // Ajout version 4.7.5
		actionCreerSuiteRecurrenteReelle2 = new ActionCreerSuiteRecurrenteReelle2(this); // Ajout version 4.7.5
		actionCreerSuiteRecurrenteComplexe3 = new ActionCreerSuiteRecurrenteComplexe3(this); // Ajout version 4.7.5
		actionCreerSuiteRecurrenteReelle3 = new ActionCreerSuiteRecurrenteReelle3(this); // Ajout version 4.7.5
		actionEditionCopierCode = new ActionEditionCopierCode(this); // Ajout version 4.7.7
		actionCreerTestFact = new ActionCreerTestFact(this); // Ajout version 4.7.9
		actionConstructionIterativeImplementerDepuisFigure = new ActionConstructionIterativeImplementerDepuisFigure(this); //Ajout version 4.8
		actionConstructionRecursiveImplementerDepuisFigure = new ActionConstructionRecursiveImplementerDepuisFigure(this); //Ajout version 4.8
		actionCreerMacroConstructionIterative = new ActionCreerMacroConstructionIterative(this); //Ajout version 4.8
		actionCreerMacroConstructionRecursive = new ActionCreerMacroConstructionRecursive(this); //Ajout version 4.8
		actionCreerObjetClone = new ActionCreerObjetClone(this); // Ajout version 4.8
		actionEditionCopierCodeTikz = new ActionEditionCopierCodeTikz(this);
		actionHTMLInfo = new ActionHTMLInfo(this);
 	
  }
  
	public final  void creeBoutons() {
		// Actions correspondant  une icne de la barre du haut
		
		buttonAnnuler = new MtgActionButton(actionAnnuler);
		buttonRefaire = new MtgActionButton(actionRefaire);
		buttonFichierOuvrir = new MtgActionButton(actionFichierOuvrir);
		buttonTranslationFigure = new MtgToggleButton(actionTranslaterFigure);
		buttonZoomPlus = new MtgActionButton(actionZoomPlus);
		buttonZoomMoins = new MtgActionButton(actionZoomMoins);
		buttonCapture = new MtgToggleButton(actionCapture);
		buttonSuppressionObjetGraphique = new MtgToggleButton(actionSuppressionObjetGraphique);
		buttonModifierNom = new MtgToggleButton(actionModifierNom);
		buttonAfficherObjetsNumeriques = new MtgToggleButton(actionAfficherObjetsNumeriques);
		buttonHelpTexteFigure = new MtgToggleButton(actionHelpTexteFigure);
		buttonSuppressionValeur = new MtgToggleButton(actionSuppressionValeur);
		buttonPalette = new MtgToggleButton(actionPalette);
		buttonModifierObjetGraphique = new MtgToggleButton(actionModifierObjetGraphique);
		buttonExecuterMacro = new MtgToggleButton(actionExecuterMacro);
		buttonGomme = new MtgToggleButton(actionGomme);
		buttonFichierNouvelleFigure = new MtgActionButton(actionFichierNouvelleFigure);
		buttonFichierEnregistrer = new MtgActionButton(actionFichierEnregistrer);
		buttonFichierImprimer = new MtgActionButton(actionFichierImprimer);
		buttonModeTrace = new MtgToggleButton(actionModeTrace);
		buttonEditionCopier = new MtgActionButton(actionEditionCopier);
		buttonRideau = new MtgToggleButton(actionRideau);
		buttonReafficherTout = new MtgActionButton(actionReafficherTout);
		buttonRelationEntreObjets = new MtgToggleButton(actionRelationEntreObjets);
		// Ajout version 5.0
		buttonModePointAuto = new MtgToggleButton(actionModePointAuto);
		buttonRecopierStyle = new MtgToggleButton(actionRecopierStyle);
		// Modification version 4.6.3 : Ajout d'un bouton dans la barre horizontale pour exporter dans un fichier png avec unit
		buttonExporterUnite = new MtgActionButton(actionFichierExporterPNGAvecUnite);
		buttonFichierNouvelleFigureAvecRepereSansVecteurs = new MtgActionButton(actionFichierNouvelleFigureAvecRepere);
		//
		buttonProtocole = new MtgToggleButton(actionProtocole);		
		// Version 5.0
		buttonReclassementDebutObjetGraphique = new MtgToggleButton(actionReclassementDebutObjetGraphique);
		buttonReclassementFinObjetGraphique = new MtgToggleButton(actionReclassementFinObjetGraphique);
		buttonAugmenterTailles = new MtgToggleButton(actionAugmenterTailles);
		buttonDiminuerTailles = new MtgToggleButton(actionDiminuerTailles);
	}

	/**
	 * Mise  jour des boutons de la barre d'outils suivant le niveau
	 * @param doc : le document auquel la barre doit s'adapter
	 */
	public void metAJourBoutonsBarreOutils(CMathGrafDoc doc) {
		// Modifi version 5.0
		paneLeft.setLayout(new BorderLayout());
		if (tabIcones != null) {
			fermeMenusHorizontaux();
			btngroup1.remove(buttonCapture);
			for (int i = 0; i < tabIcones.length; i++) btngroup1.remove(tabIcones[i]);
		};
		paneLeft.removeAll();
		// Modification version 5.0 : Il faut reconsruire l'action et le bouton de capture
		actionCapture = new ActionCapture(this);
		buttonCapture = new MtgToggleButton(actionCapture);
    buttonCapture.setBorder(BorderFactory.createCompoundBorder(
    		BorderFactory.createEtchedBorder(EtchedBorder.RAISED),BorderFactory.createCompoundBorder(
    				BorderFactory.createMatteBorder(1,8,1,8,Color.white), BorderFactory.createLineBorder(Color.white, 2))));
    buttonCapture.setSelected(true);
		//
		tabIcones = new MtgIconButton[12];
		MtgIconArrow[] tabArrow = new MtgIconArrow[12];
	
		MtgAction[] actionsPoints = {actionCreerPointLibre, actionCreerIntersection, actionCreerPointLie,actionCreerMilieu, actionCreerPointParCoordonnees,
				actionCreerPointLibreEnt, actionCreerProjeteOrthogonal,actionCreerPointInterieur, actionCreerCentreGravite, actionCreerPointParAbscisse,
				actionCreerPointParAffixe, actionCreerPointParSommeVect, actionCreerPointParMultVect, actionCreerBarycentre, actionCreerLiaison,
				actionSupprimerLiaison, actionPointSurCourbeParAbscisse, actionPunaiser, actionDepunaiser, actionMarquerPointPourTrace,
				actionDemarquerPointPourTrace};
		MtgIconButton iconeCreationPoints = new MtgIconButton(this, actionsPoints);
		tabIcones[0] = iconeCreationPoints;
		tabArrow[0] = new MtgIconArrow(iconeCreationPoints);
		MtgAction[] actionsDroites = {actionCreerDroiteAB, actionCreerDroiteParallele, actionCreerDroitePerpendiculaire,
				actionCreerDroiteBissectrice, actionCreerDroiteMediatrice, actionCreerDroiteParCoef, actionCreerDroiteParEquation,
				actionCreerDroiteHorizontale, actionCreerDroiteVerticale, actionTangente};
		MtgIconButton iconeCreationDroites = new MtgIconButton(this, actionsDroites);
		tabIcones[1] = iconeCreationDroites;
		tabArrow[1] = new MtgIconArrow(iconeCreationDroites);
		MtgAction[] actionsSegment = {actionCreerSegment, actionCreerDemiDroite, actionCreerVecteur, actionCreerSegmentParLong};
		MtgIconButton iconeCreationSeg = new MtgIconButton(this, actionsSegment);
		tabIcones[2] = iconeCreationSeg;
		tabArrow[2] = new MtgIconArrow(iconeCreationSeg);
		MtgAction[] actionsCercles = {actionCreerCercleOA, actionCreerCercleOR, actionCreerArcPetitParPointFinal, actionCreerArcGrandParPointFinal,
				actionCreerArcPetitParRapporteur, actionCreerArcGrandParRapporteur, actionCreerArcDirectParPointFinal, actionCreerArcIndirectParPointFinal,
				actionCreerArcPetitParAngle, actionCreerArcGrandParAngle, actionCreerArcDirectParAngle, actionCreerArcIndirectParAngle};
		MtgIconButton iconeCreationCercles = new MtgIconButton(this, actionsCercles);
		tabIcones[3] = iconeCreationCercles;
		tabArrow[3] = new MtgIconArrow(iconeCreationCercles);
		MtgAction[] actionsLignes = {actionCreerPolygone, actionCreerLigneBrisee, actionCarreDirect, actionTriangleEquilateralDirect,
				actionParallelogramme, actionPolygoneRegulier};
		MtgIconButton iconeCreationLignes = new MtgIconButton(this, actionsLignes);
		tabIcones[4] = iconeCreationLignes;
		tabArrow[4] = new MtgIconArrow(iconeCreationLignes);
		paneLeft.removeAll();
		MtgAction[] actionsMarques = {actionCreerMarqueSegment, actionCreerMarqueAngleNonOriente, actionCreerMarqueAngleOriente,
				actionCreerMarqueAngleOrienteSensDirect, actionCreerMarqueAngleOrienteSensIndirect};
		MtgIconButton iconeCreationMarques = new MtgIconButton(this,  actionsMarques);
		tabIcones[5] = iconeCreationMarques;
		tabArrow[5] = new MtgIconArrow(iconeCreationMarques);
		MtgAction[] actionsLieux = {actionCreerLieuParPointLie, actionCreerLieuParVariable, actionCreerLieuDiscretParPointLie,
				actionCreerLieuDiscretParVariable, actionCreerCourbeFonction, actionCreerGrapheSuiteRecurrenteReelle,
				actionCreerGrapheSuiteRecurrenteComplexe, actionCreerLieuObjetParPointLie, actionCreerLieuObjetParVariable};
		MtgIconButton iconeCreationLieux = new MtgIconButton(this, actionsLieux);
		tabIcones[6] = iconeCreationLieux;
		tabArrow[6] = new MtgIconArrow(iconeCreationLieux);
		MtgAction[] actionsTrans = {actionCreerImageParTranslation, actionCreerImageParSymetrieAxiale, actionCreerImageParSymetrieCentrale,
				actionCreerImageParRotation, actionRapporteur, actionCreerImageParTranslationParCoord, actionCreerImageParHomothetie, actionCreerImageParSimilitude};
		MtgIconButton iconeTrans = new MtgIconButton(this,  actionsTrans);
		tabIcones[7] = iconeTrans;
		tabArrow[7] = new MtgIconArrow(iconeTrans);
		MtgAction[] actionsMes = {actionMesureLongueur, actionMesureAngleNonOriente, actionMesureAngleOriente, actionMesureAbscisseDansRepere,
				actionMesureOrdonneeDansRepere, actionMesureAbscisse, actionMesureCoefficientDirecteur, actionMesureLongueurLigne,
				actionMesureAirePolygone, actionMesureProduitScalaire, actionMesureAffixe};
		MtgIconButton iconeMes = new MtgIconButton(this,  actionsMes);
		tabIcones[8] = iconeMes;
		tabArrow[8] = new MtgIconArrow(iconeMes);
		MtgAction[] actionsAff = {actionCreerAffichageValeur, actionCreerAffichageValeurLiePoint, actionCreerCommentaire,
				actionCreerCommentaireLiePoint, actionCreerLatex, actionCreerLatexLiePoint, actionCreerImageLibre,
				actionCreerImageLieePoint, actioncreerEditeurFormule, actionCreerEquationDroite, actionCreerEquationDroiteLiePoint,
				actionCreerLiaisonAffichage, actionSupprimerLiaisonAffichage};
		MtgIconButton iconAff = new MtgIconButton(this, actionsAff);
		tabIcones[9] = iconAff;
		tabArrow[9] = new MtgIconArrow(iconAff);
		MtgAction[] actionsCalc = {actionCurseur, actionCreerCalcul, actionCreerFonctionReelle, actionCreerVariable,
				actionCreerRepere, actionCreerSolutionEquation, actionCreerMaximum, actionCreerMinimum,
				actionCreerDerivee, actionCreerSuiteRecurrenteReelle, actionCreerCalculComplexe, actionCreerFonctionComplexe,
				actionCreerSuiteRecurrenteComplexe};
		MtgIconButton iconCalc = new MtgIconButton(this, actionsCalc);
		tabIcones[10] = iconCalc;
		tabArrow[10] = new MtgIconArrow(iconCalc);
		MtgAction[] actionsSurf = {actionCreerSurface, actionCreerSurfaceLieuDroite, actionCreerSurfaceLieuDeuxPoints,
				actionCreerSurfaceDeuxLieux, actionCreerDemiPlan};
		MtgIconButton iconSurf = new MtgIconButton(this, actionsSurf);
		tabIcones[11] = iconSurf;
		tabArrow[11] = new MtgIconArrow(iconSurf);
		barreGauche = new JToolBar(JToolBar.VERTICAL);
		barreDroite = new JToolBar(JToolBar.VERTICAL);
		barreGauche.setMargin(new Insets(0, 0, 0, 0));
		barreGauche.setFloatable(false);
		// barreGauche.setBorder(BorderFactory.createEmptyBorder(2, 1, 2, 1));
		barreDroite.setMargin(new Insets(0, 0, 0, 0));
		barreDroite.setFloatable(false);
		// barreDroite.setBorder(BorderFactory.createEmptyBorder(2, 1, 2, 1));
		paneLeft.add(buttonCapture, BorderLayout.NORTH);
		btngroup1.add(buttonCapture);
		for (int i = 0; i < tabIcones.length; i++) {
			btngroup1.add(tabIcones[i]);
			if (tabIcones[i].contientItemActivable()) {
				barreGauche.add(tabIcones[i]);
				barreDroite.add(tabArrow[i]);
			}
		}
		paneLeft.add(barreGauche, BorderLayout.WEST);
		paneLeft.add(barreDroite, BorderLayout.EAST);
	}
	// Ajout version 5.0
	/**
	 * Fonction activant si elle exite l'icne de la barre de gauche si elle contient une icne associe  l'action action
	 * @param action
	 */
	public final void activeActionDansBarreGauche(MtgAction action) {
		for (int i = 0; i < tabIcones.length; i++) tabIcones[i].activeAction(action);
	}
	/**
	 * Fonction ajoutant le MtgButton button  la barre d'outils horizontale  condition qu'il soit autoris
	 * par le document doc. Ajout version 4.3
	 * @param button
	 * @param doc
	 * @return true si l'item a t rajout
	 */
	private final boolean ajouteMtgButtonBarreHor(MtgActionButton button, CMathGrafDoc doc) {
		if ((doc == null) ? true : doc.itemPermis(button.action.idMenu())) {
			boolean b = docPreferencesNiveau[pref_niveau].itemPermis(button.action.idMenu());
			if (b) {
				btngroup2.add(button);
				barHor.add(button);
				return true;
			}
			else btngroup2.remove(button);
		}
		return false;
	}
	
	/**
	 * Fonction ajoutant le MtgButton button  la barre d'outils horizontale  condition qu'il soit autoris
	 * par le document doc. Ajout version 4.3.
	 * @param button
	 * @param doc
	 * @return true si l'item a t rajout
	 */	
	private final boolean ajouteMtgToggleButtonBarreHor(MtgToggleButton button, CMathGrafDoc doc) {
		if ((doc == null) ? true : doc.itemPermis(button.action.idMenu())) {
			// Si le dossier systme de MathGraph32 contient une figure nomm default.mgj on  les outils disponibles
			//  cette figure
			boolean b = (docPreferencesNiveauPredefini[pref_niveau] == null) ? true : docPreferencesNiveauPredefini[pref_niveau].itemPermis(button.action.idMenu());
			if (b) {
				// Version 4.4 : Correction d'un bug version 4.3 o le bouton de mode trace  ne fonctionnait plus
				// Modifi version 5.0
				// if (button != buttonModeTrace) btngroup1.add(button);
				if ((button != buttonModeTrace) && (button != buttonModePointAuto)) btngroup1.add(button);
				barHor.add(button);
				return true;
			}
			else {
				btngroup1.remove(button);
			}
		}
		return false;
	}	
	

	/**
	 * Ajout des boutons de la barre horizontale du haut
	 */
	public final void metAJourBoutonsBarreHorizontale(CMathGrafDoc doc) {
		boolean bseparator = false;  // Sera true si on peut rajouter un separator
		barHor.removeAll();
		bseparator |= ajouteMtgButtonBarreHor(buttonFichierNouvelleFigure,doc);
		bseparator |= ajouteMtgButtonBarreHor(buttonFichierNouvelleFigureAvecRepereSansVecteurs,doc);
		bseparator |= ajouteMtgButtonBarreHor(buttonFichierOuvrir, doc);
		bseparator |= ajouteMtgButtonBarreHor(buttonFichierEnregistrer, doc);
		bseparator |= ajouteMtgButtonBarreHor(buttonFichierImprimer, doc);
		bseparator |= ajouteMtgButtonBarreHor(buttonEditionCopier, doc);
		bseparator |= ajouteMtgButtonBarreHor(buttonExporterUnite, doc);
		if (bseparator) barHor.addSeparator();
		bseparator = false;
		
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonModifierNom, doc);
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonModifierObjetGraphique, doc); // Dplac version 5.0
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonPalette, doc);
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonRecopierStyle, doc); // Ajout version 5.0
		if (bseparator) barHor.addSeparator();
		bseparator = false;
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonProtocole, doc); // Dplac version 4.9.9
		// bseparator |= ajouteMtgToggleButtonBarreHor(buttonModifierObjetGraphique, doc); Dplac version 5.0
		// bseparator |= ajouteMtgToggleButtonBarreHor(buttonModifierFormule, doc); // Supprim version 4.9.2
		// Remplac par :
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonAfficherObjetsNumeriques, doc);
		if (bseparator) barHor.addSeparator();
		bseparator = false;
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonSuppressionObjetGraphique, doc);
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonSuppressionValeur, doc);
		// bseparator |= ajouteMtgToggleButtonBarreHor(buttonSuppressionValeur, doc);
		if (bseparator) barHor.addSeparator();
		bseparator = false;
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonGomme, doc); // null parce que toujours prsent
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonRideau, doc); // null parce que toujours prsent
		if (bseparator) barHor.addSeparator();
		bseparator = false;
		bseparator |= ajouteMtgButtonBarreHor(buttonAnnuler, doc);
		bseparator |= ajouteMtgButtonBarreHor(buttonRefaire, doc);
		if (bseparator) barHor.addSeparator();
		bseparator = false;
		bseparator |= ajouteMtgButtonBarreHor(buttonZoomPlus, doc);
		bseparator |= ajouteMtgButtonBarreHor(buttonZoomMoins, doc);
		// barHor.addSeparator();
		bseparator |= ajouteMtgButtonBarreHor(buttonReafficherTout, doc);
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonTranslationFigure, doc);
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonModeTrace, doc); // Le bouton fonctionne en bascule
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonModePointAuto, doc); // Le bouton fonctionne en bascule
		
		if (bseparator) barHor.addSeparator();
		validate(); // Pour que les barres d'outils soient mises  jour Supprim version 5.0.0.5
		bseparator = false;
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonExecuterMacro, doc);
		// Dplac verion 4.9.9
		// bseparator |= ajouteMtgToggleButtonBarreHor(buttonProtocole, doc);
		// bseparator |= ajouteMtgToggleButtonBarreHor(buttonAfficherObjetsNumeriques, doc); // Supprim version 4.9.2
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonHelpTexteFigure, doc);
		bseparator |= ajouteMtgToggleButtonBarreHor(buttonRelationEntreObjets, doc);
		// if (bseparator) barHor.addSeparator(); Modifi version 4.5.1
		// Ajout de deux boutons version 5.0 pour le reclassement d'objet graphique
		ajouteMtgToggleButtonBarreHor(buttonReclassementDebutObjetGraphique, doc);
		ajouteMtgToggleButtonBarreHor(buttonReclassementFinObjetGraphique, doc);
		// Ajout de deux boutons version 5.0 pour le reclassement d'objet graphique'augmentation ou la diminution des tailles de police
		ajouteMtgToggleButtonBarreHor(buttonAugmenterTailles, doc);
		ajouteMtgToggleButtonBarreHor(buttonDiminuerTailles, doc);

	}

	/**
	 * Renvoie un item de menu associ  l'action action Lors du suvol de l'item
	 * la barre d'information affichera l'information relative  l'outil s'il en
	 * existe un associ Utilise un inset pour que les items ne soient pas trop
	 * colls les uns aux autres
	 * 
	 * @param action
	 * @return
	 */
	MtgMenuItem MtgItemMenu(MtgAction action) {
		MtgMenuItem item = new MtgMenuItem(action);
		return item;
	}

	MtgMenu MtgMenu(String ch) {
		MtgMenu menu = new MtgMenu(ch, this);
		return menu;
	}

	/**
	 * Ajout du menu  la menuBar en fonction des items permis ou interdits
	 * par le document actif
	 */
	public void creeMenu() {
		menuFenetre.setText(getString("menuFenetre")); // Pour s'adapter  la langue
		// A revoir pour optimisation
		if (menuFichier != null)
			menuBar.remove(menuFichier);
		if (menuEdition != null)
			menuBar.remove(menuEdition);
		if (menuCreer != null)
			menuBar.remove(menuCreer);
		if (menuCalculs != null)
			menuBar.remove(menuCalculs);
		if (menuModifier != null)
			menuBar.remove(menuModifier);
		if (menuMouvoir != null)
			menuBar.remove(menuMouvoir);
		if (menuMacros != null)
			menuBar.remove(menuMacros);
		if (menuFenetre != null)
			menuBar.remove(menuFenetre);
		if (menuHelp != null)
			menuBar.remove(menuHelp);
		if (menuOptions != null)
			menuBar.remove(menuOptions);
		if (menuConstructions != null)
			menuBar.remove(menuConstructions);
		menuFichier = creeMenuFichier();
		menuBar.add(menuFichier);
		menuEdition = creeMenuEdition();
		menuBar.add(menuEdition);
		menuCreer = creeMenuCreer();
		menuBar.add(menuCreer);
		menuModifier = creeMenuModifier();
		menuBar.add(menuModifier);
		menuMouvoir = creeMenuMouvoir();
		menuBar.add(menuMouvoir);
		menuCalculs = creeMenuCalculs();
		menuBar.add(menuCalculs);
		menuMacros = creeMenuMacros();
		menuBar.add(menuMacros);
		menuBar.add(menuFenetre);
		menuOptions = creeMenuOptions();
		menuBar.add(menuOptions);
		menuConstructions = creeMenuConstructions();
		menuBar.add(menuConstructions);
		menuHelp = creeMenuHelp();
		menuBar.add(menuHelp);

		
		menuFichierInitial = creeMenuFichier();
		menuEditionInitial = creeMenuEdition();
		menuCreerInitial = creeMenuCreer();
		menuCalculsInitial = creeMenuCalculs();
		menuModifierInitial = creeMenuModifier();
		menuMouvoirInitial = creeMenuMouvoir();
		menuMacrosInitial = creeMenuMacros();
		menuHelpInitial = creeMenuHelp();
		menuOptionsInitial = creeMenuOptions();
		menuConstructionsInitial = creeMenuConstructions();
		

		setJMenuBar(menuBar);

	}
	
	public MtgMenu creeMenuFichier() {
		MtgMenu menuFichier = MtgMenu(getString("menuFichier"));
		JMenuItem menuItem = MtgItemMenu(actionFichierNouvelleFigure);
		menuFichier.add(menuItem);
		menuItem = MtgItemMenu(actionFichierNouvelleFigureSansLongueurUnite);
		menuFichier.add(menuItem);
		JMenu menuNouvelleFigureAvec = MtgMenu(getString("menuNouvelleFigureAvec"));
		menuFichier.add(menuNouvelleFigureAvec);
		menuItem = MtgItemMenu(actionFichierNouvelleFigureAvecRepere);
		menuNouvelleFigureAvec.add(menuItem);
		menuItem = MtgItemMenu(actionFichierNouvelleFigurePredefinie);
		menuNouvelleFigureAvec.add(menuItem);
		menuItem = MtgItemMenu(actionFigureBaseRepere);
		menuNouvelleFigureAvec.add(menuItem);
		menuItem = MtgItemMenu(actionFigureBaseRepereVect);
		menuNouvelleFigureAvec.add(menuItem);
		JMenu menuCourbes = MtgMenu(getString("courbes"));
		menuNouvelleFigureAvec.add(menuCourbes);
		menuItem = MtgItemMenu(actionFigureBaseCourbes);
		menuCourbes.add(menuItem);
		menuItem = MtgItemMenu(actionFigureBaseCourbePolaire);
		menuCourbes.add(menuItem);
		menuItem = MtgItemMenu(actionFigureBaseCourbeParam);
		menuCourbes.add(menuItem);
		JMenu menuSuites = MtgMenu(getString("grapheSuite"));
		menuNouvelleFigureAvec.add(menuSuites);
		menuItem = MtgItemMenu(actionFigureBaseSuiteunfn);
		menuSuites.add(menuItem);
		menuItem = MtgItemMenu(actionFigureBaseSuiteRecParPtLie);
		menuSuites.add(menuItem);
		menuItem = MtgItemMenu(actionFigureBaseSuiteRecParPremierTerme);
		menuSuites.add(menuItem);
		JMenu menuProbas = MtgMenu(getString("probas"));
		menuNouvelleFigureAvec.add(menuProbas);
		menuItem = MtgItemMenu(actionFigureBaseLoiBinomiale);
		menuProbas.add(menuItem);
		menuItem = MtgItemMenu(actionFigureBaseLoiBinomialeIntervalleConfiance);
		menuProbas.add(menuItem);
		menuItem = MtgItemMenu(actionFigureBaseLoiBinomialeApproximationParLoiNormale);
		menuProbas.add(menuItem);
		menuItem = MtgItemMenu(actionFigureBaseLoiExponentielle);
		menuProbas.add(menuItem);
		menuItem = MtgItemMenu(actionFigureBaseLoiNormale);
		menuProbas.add(menuItem);
		menuItem = MtgItemMenu(actionFigureBaseLoiNormaleualpha);
		menuProbas.add(menuItem);
		menuItem = MtgItemMenu(actionFigureBaseLoiNormaleInverse);
		menuProbas.add(menuItem);
		JMenu menu3D = MtgMenu(getString("figure3D"));
		menuNouvelleFigureAvec.add(menu3D);
		menuItem = MtgItemMenu(actionFigureBaseEspaceCube);
		menu3D.add(menuItem);
		menuItem = MtgItemMenu(actionFigureBaseEspaceParallelepipedeRect);
		menu3D.add(menuItem);
		menuItem = MtgItemMenu(actionFigureBaseEspaceTetraedre);
		menu3D.add(menuItem);
		menuItem = MtgItemMenu(actionFigureBaseEspaceTetraedreRegulier);
		menu3D.add(menuItem);
		menuItem = MtgItemMenu(actionFigureBaseEspaceCylindre);
		menu3D.add(menuItem);
		menuItem = MtgItemMenu(actionFigureBaseEspaceConeSimple);
		menu3D.add(menuItem);
		menuItem = MtgItemMenu(actionFigureBaseEspaceConeDouble);
		menu3D.add(menuItem);
		menuItem = MtgItemMenu(actionFigureBaseEspacePyramideBaseCarre);
		menu3D.add(menuItem);
		menuItem = MtgItemMenu(actionFigureBaseEspacePyramideBaseRectangle);
		menu3D.add(menuItem);
		menuItem = MtgItemMenu(actionFigureBaseEspacePyramideReguliereBaseCarre);
		menu3D.add(menuItem);
		menuItem = MtgItemMenu(actionFigureBaseEspacePrismeBaseTriangulaire);
		menu3D.add(menuItem);
		menuItem = MtgItemMenu(actionFigureBaseEspacePrismeBaseTrapeze);
		menu3D.add(menuItem);
		menuItem = MtgItemMenu(actionFigureBaseEspaceSurfaceFonction2Var);
		menu3D.add(menuItem);
		// Ajout version 5.1.2
		menuItem = MtgItemMenu(actionFigureBaseEspaceRepereOrthonormal);
		menu3D.add(menuItem);
		menuItem = MtgItemMenu(actionFigureBaseEspaceRepereOrthogonal);
		menu3D.add(menuItem);
		//		
		// Ajout version 4.7.7
		menuItem = MtgItemMenu(actionFichierNouvelleFigureParCode);
		menuFichier.add(menuItem);
		//
		menuFichier.add(new JSeparator());
		menuItem = MtgItemMenu(actionFichierOuvrir);
		menuFichier.add(menuItem);
		menuItem = MtgItemMenu(actionFichierEnregistrer);
		menuFichier.add(menuItem);
		if (!url.isEmpty()) {
			menuItem = MtgItemMenu(actionPostHtml);
			menuFichier.add(menuItem);			
		}
		menuItem = MtgItemMenu(actionFichierEnregistrerSous);
		menuFichier.add(menuItem);
		menuFichier.add(new JSeparator());
		menuItem = MtgItemMenu(actionFichierImprimer);
		menuFichier.add(menuItem);
		menuFichier.add(new JSeparator());
		JMenu menuFichierExporter = MtgMenu(getString("menuFichierExporter"));
		menuFichier.add(menuFichierExporter);
		menuItem = MtgItemMenu(actionFichierExporterPNG);
		menuFichierExporter.add(menuItem);
		menuItem = MtgItemMenu(actionFichierExporterJPEG);
		menuFichierExporter.add(menuItem);
		menuItem = MtgItemMenu(actionFichierExporterPNGAvecUnite);
		menuFichier.add(menuItem);
		/* Plus support version 5.0
		menuItem = MtgItemMenu(actionFichierExporterHTML);
		menuFichierExporter.add(menuItem);
		*/
		menuItem = MtgItemMenu(actionFichierExporterHTML5);  // Ajout version 4.7.7
		menuFichierExporter.add(menuItem);
		menuItem = MtgItemMenu(actionFichierExporterTikz);  // Ajout version 4.8.1
		menuFichier.add(menuItem);
		menuFichier.add(new JSeparator());
		menuItem = MtgItemMenu(actionFichierQuitter);
		menuFichier.add(menuItem);
		return menuFichier;
	}

	public MtgMenu creeMenuEdition() {
		MtgMenu menuEdition = MtgMenu(getString("menuEdition"));
		menuItemEditionAnnuler = MtgItemMenu(actionAnnuler);
		menuEdition.add(menuItemEditionAnnuler);
		menuItemEditionRefaire = MtgItemMenu(actionRefaire);
		menuEdition.add(menuItemEditionRefaire);
		menuEdition.add(new JSeparator());
		MtgMenuItem menuItem = MtgItemMenu(actionActiverOutilPrecedent);
		menuEdition.add(menuItem);
		menuItem = MtgItemMenu(actionEditionCopier);
		menuEdition.add(menuItem);
		// Ajout version 4.7.7
		menuItem = MtgItemMenu(actionEditionCopierCode);
		menuEdition.add(menuItem);
		//
		// Ajout version 4.8.1
		menuItem = MtgItemMenu(actionEditionCopierCodeTikz);
		menuEdition.add(menuItem);
		//
		menuEdition.add(new JSeparator());
		JMenu menuReclassementDebut = MtgMenu(getString("menuReclassementDebut"));
		menuEdition.add(menuReclassementDebut);
		menuItem = MtgItemMenu(actionReclassementDebutObjetGraphique);
		menuReclassementDebut.add(menuItem);
		menuItem = MtgItemMenu(actionReclassementDebutValeur);
		menuReclassementDebut.add(menuItem);
		JMenu menuReclassementFin = MtgMenu(getString("menuReclassementFin"));
		menuEdition.add(menuReclassementFin);
		menuItem = MtgItemMenu(actionReclassementFinObjetGraphique);
		menuReclassementFin.add(menuItem);
		menuItem = MtgItemMenu(actionReclassementFinValeur);
		menuReclassementFin.add(menuItem);

		menuEdition.add(new JSeparator());
		menuItem = MtgItemMenu(actionProtocole); // Ajout version 4.9.9
		menuEdition.add(menuItem);
		menuEdition.add(new JSeparator()); // Ajout version 4.9.9
		menuItem = MtgItemMenu(actionRelationEntreObjets);
		menuEdition.add(menuItem);
		menuItem = MtgItemMenu(actionEditerTexteFigure);
		menuEdition.add(menuItem);
		return menuEdition;
	}
	
	public MtgMenu creeMenuCreer() {
		MtgMenu menuCreer = MtgMenu(getString("menuCreer"));
		JMenu menuCreerPoint = MtgMenu(getString("menuCreerPoint"));
		menuCreer.add(menuCreerPoint);
		MtgMenuItem menuItem = MtgItemMenu(actionCreerPointLibre);
		menuCreerPoint.add(menuItem);
		menuItem = MtgItemMenu(actionCreerPointLibreEnt);
		menuCreerPoint.add(menuItem);
		menuItem = MtgItemMenu(actionCreerPointLie);
		menuCreerPoint.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMilieu);
		menuCreerPoint.add(menuItem);
		menuItem = MtgItemMenu(actionCreerCentreGravite);
		menuCreerPoint.add(menuItem);
		menuItem = MtgItemMenu(actionCreerProjeteOrthogonal);
		menuCreerPoint.add(menuItem);
		menuItem = MtgItemMenu(actionCreerPointParCoordonnees);
		menuCreerPoint.add(menuItem);
		menuItem = MtgItemMenu(actionCreerPointParAbscisse);
		menuCreerPoint.add(menuItem);
		menuItem = MtgItemMenu(actionCreerPointParSommeVect);
		menuCreerPoint.add(menuItem);
		menuItem = MtgItemMenu(actionCreerPointParMultVect);
		menuCreerPoint.add(menuItem);
		menuItem = MtgItemMenu(actionCreerBarycentre);
		menuCreerPoint.add(menuItem);
		menuItem = MtgItemMenu(actionCreerPointInterieur);
		menuCreerPoint.add(menuItem);
		menuItem = MtgItemMenu(actionCreerCentreCercle);
		menuCreerPoint.add(menuItem);
		menuItem = MtgItemMenu(actionCreerPointParAffixe);
		menuCreerPoint.add(menuItem);
		JMenu menuCreerPointSurCourbe = MtgMenu(getString("menuCreerPointSurCourbe"));
		menuCreerPoint.add(menuCreerPointSurCourbe);
		menuItem = MtgItemMenu(actionPointSurCourbeParAbscisse);
		menuCreerPointSurCourbe.add(menuItem);
		menuItem = MtgItemMenu(actionPointSurCourbeParPointLie);
		menuCreerPointSurCourbe.add(menuItem);		
		JMenu menuCreerDroite = MtgMenu(getString("menuCreerDroite"));
		menuCreer.add(menuCreerDroite);
		menuItem = MtgItemMenu(actionCreerDroiteAB);
		menuCreerDroite.add(menuItem);
		menuItem = MtgItemMenu(actionCreerDroiteParallele);
		menuCreerDroite.add(menuItem);
		menuItem = MtgItemMenu(actionCreerDroitePerpendiculaire);
		menuCreerDroite.add(menuItem);
		menuItem = MtgItemMenu(actionCreerDroiteMediatrice);
		menuCreerDroite.add(menuItem);
		menuItem = MtgItemMenu(actionCreerDroiteBissectrice);
		menuCreerDroite.add(menuItem);
		menuItem = MtgItemMenu(actionCreerDroiteHorizontale);
		menuCreerDroite.add(menuItem);
		menuItem = MtgItemMenu(actionCreerDroiteVerticale);
		menuCreerDroite.add(menuItem);
		menuItem = MtgItemMenu(actionCreerDroiteParCoef);
		menuCreerDroite.add(menuItem);
		menuItem = MtgItemMenu(actionCreerDroiteParEquation);	
		menuCreerDroite.add(menuItem);
		menuItem = MtgItemMenu(actionTangente);
		menuCreerDroite.add(menuItem);
		/*
		menuItem = MtgItemMenu(actionCreerSegment);
		menuCreer.add(menuItem);
		*/
		JMenu menuCreerSegment = MtgMenu(getString("menuCreerSegment"));
		menuCreer.add(menuCreerSegment);
		menuItem = MtgItemMenu(actionCreerSegment);
		menuCreerSegment.add(menuItem);
		menuItem = MtgItemMenu(actionCreerSegmentParLong);
		menuCreerSegment.add(menuItem);		
		menuItem = MtgItemMenu(actionCreerDemiDroite);
		menuCreer.add(menuItem);
		menuItem = MtgItemMenu(actionCreerVecteur);
		menuCreer.add(menuItem);

		JMenu menuCreerCercle = MtgMenu(getString("menuCreerCercle"));
		menuCreer.add(menuCreerCercle);
		menuItem = MtgItemMenu(actionCreerCercleOA);
		menuCreerCercle.add(menuItem);
		menuItem = MtgItemMenu(actionCreerCercleOR);
		menuCreerCercle.add(menuItem);

		JMenu menuCreerArcDeCercle = MtgMenu(getString("menuCreerArcDeCercle"));
		menuCreer.add(menuCreerArcDeCercle);
		JMenu menuCreerArcDeCercleParParPointFinal = MtgMenu(getString("menuCreerArcDeCercleParPointFinal"));
		menuCreerArcDeCercle.add(menuCreerArcDeCercleParParPointFinal);
		menuItem = MtgItemMenu(actionCreerArcPetitParPointFinal);
		menuCreerArcDeCercleParParPointFinal.add(menuItem);
		menuItem = MtgItemMenu(actionCreerArcGrandParPointFinal);
		menuCreerArcDeCercleParParPointFinal.add(menuItem);
		menuItem = MtgItemMenu(actionCreerArcDirectParPointFinal);
		menuCreerArcDeCercleParParPointFinal.add(menuItem);
		menuItem = MtgItemMenu(actionCreerArcIndirectParPointFinal);
		menuCreerArcDeCercleParParPointFinal.add(menuItem);
		JMenu menuCreerArcDeCercleParParAngle = MtgMenu(getString("menuCreerArcDeCercleParAngle"));
		menuCreerArcDeCercle.add(menuCreerArcDeCercleParParAngle);
		menuItem = MtgItemMenu(actionCreerArcPetitParAngle);
		menuCreerArcDeCercleParParAngle.add(menuItem);
		menuItem = MtgItemMenu(actionCreerArcGrandParAngle);
		menuCreerArcDeCercleParParAngle.add(menuItem);
		menuItem = MtgItemMenu(actionCreerArcDirectParAngle);
		menuCreerArcDeCercleParParAngle.add(menuItem);
		menuItem = MtgItemMenu(actionCreerArcIndirectParAngle);
		menuCreerArcDeCercleParParAngle.add(menuItem);
		JMenu menuCreerArcDeCercleParParRapporteur = MtgMenu(getString("menuCreerArcDeCercleParRapporteur"));
		menuCreerArcDeCercle.add(menuCreerArcDeCercleParParRapporteur);
		menuItem = MtgItemMenu(actionCreerArcPetitParRapporteur);
		menuCreerArcDeCercleParParRapporteur.add(menuItem);
		menuItem = MtgItemMenu(actionCreerArcGrandParRapporteur);
		menuCreerArcDeCercleParParRapporteur.add(menuItem);
		
		JMenu menuCreerImageParTransformation = MtgMenu(getString("menuCreerImageParTransformation"));
		menuCreer.add(menuCreerImageParTransformation);
		menuItem = MtgItemMenu(actionCreerImageParTranslation);
		menuCreerImageParTransformation.add(menuItem);
		menuItem = MtgItemMenu(actionCreerImageParTranslationParCoord);
		menuCreerImageParTransformation.add(menuItem);
		menuItem = MtgItemMenu(actionCreerImageParSymetrieAxiale);
		menuCreerImageParTransformation.add(menuItem);
		menuItem = MtgItemMenu(actionCreerImageParSymetrieCentrale);
		menuCreerImageParTransformation.add(menuItem);
		JMenu menuCreerLigneOuPolygone = MtgMenu(getString("menuLigneOuPolygone"));
		menuCreer.add(menuCreerLigneOuPolygone);
		menuItem = MtgItemMenu(actionCreerPolygone);
		menuCreerLigneOuPolygone.add(menuItem);
		menuItem = MtgItemMenu(actionPolygoneRegulier);
		menuCreerLigneOuPolygone.add(menuItem);
		menuItem = MtgItemMenu(actionCreerLigneBrisee);
		menuCreerLigneOuPolygone.add(menuItem);
		menuItem = MtgItemMenu(actionParallelogramme);
		menuCreerLigneOuPolygone.add(menuItem);
		menuItem = MtgItemMenu(actionCarreDirect);
		menuCreerLigneOuPolygone.add(menuItem);
		menuItem = MtgItemMenu(actionTriangleEquilateralDirect);
		menuCreerLigneOuPolygone.add(menuItem);
		menuItem = MtgItemMenu(actionCreerImageParRotation);
		menuCreerImageParTransformation.add(menuItem);
		menuItem = MtgItemMenu(actionCreerImageParHomothetie);
		menuCreerImageParTransformation.add(menuItem);
		menuItem = MtgItemMenu(actionCreerImageParSimilitude);
		menuCreerImageParTransformation.add(menuItem);
		menuItem = MtgItemMenu(actionCreerImageParInversion);
		menuCreerImageParTransformation.add(menuItem);
		menuItem = MtgItemMenu(actionRapporteur);
		menuCreerImageParTransformation.add(menuItem);

		menuItem = MtgItemMenu(actionCurseur);
		menuCreer.add(menuItem);
		menuCreer.add(new JSeparator());
		menuItem = MtgItemMenu(actionCreerCourbeFonction);
		menuCreer.add(menuItem);
		JMenu menuCreerLieuDePointsRelies = MtgMenu(getString("menuCreerLieuDePointsRelies"));
		menuCreer.add(menuCreerLieuDePointsRelies);
		menuItem = MtgItemMenu(actionCreerLieuParPointLie);
		menuCreerLieuDePointsRelies.add(menuItem);
		menuItem = MtgItemMenu(actionCreerLieuParVariable);
		menuCreerLieuDePointsRelies.add(menuItem);
		JMenu menuCreerLieuDePointsNonRelies = MtgMenu(getString("menuCreerLieuDePointsNonRelies"));
		menuCreer.add(menuCreerLieuDePointsNonRelies);
		menuItem = MtgItemMenu(actionCreerLieuDiscretParPointLie);
		menuCreerLieuDePointsNonRelies.add(menuItem);
		menuItem = MtgItemMenu(actionCreerLieuDiscretParVariable);
		menuCreerLieuDePointsNonRelies.add(menuItem);
		JMenu menuCreerLieuObjet = MtgMenu(getString("menuCreerLieuObjet"));
		menuCreer.add(menuCreerLieuObjet);
		menuItem = MtgItemMenu(actionCreerLieuObjetParPointLie);
		menuCreerLieuObjet.add(menuItem);
		menuItem = MtgItemMenu(actionCreerLieuObjetParVariable);
		menuCreerLieuObjet.add(menuItem);

		JMenu menuCreerGrapheSuiteRecurrente = MtgMenu(getString("menuCreerGrapheSuiteRecurrente"));
		menuCreer.add(menuCreerGrapheSuiteRecurrente);
		menuItem = MtgItemMenu(actionCreerGrapheSuiteRecurrenteReelle);
		menuCreerGrapheSuiteRecurrente.add(menuItem);
		menuItem = MtgItemMenu(actionCreerGrapheSuiteRecurrenteComplexe);
		menuCreerGrapheSuiteRecurrente.add(menuItem);

		JMenu menuCreerSurface = MtgMenu(getString("menuCreerSurface"));
		menuCreer.add(menuCreerSurface);
		menuItem = MtgItemMenu(actionCreerSurface);
		menuCreerSurface.add(menuItem);
		menuItem = MtgItemMenu(actionCreerSurfaceDeuxLieux);
		menuCreerSurface.add(menuItem);
		menuItem = MtgItemMenu(actionCreerSurfaceLieuDroite);
		menuCreerSurface.add(menuItem);
		menuItem = MtgItemMenu(actionCreerSurfaceLieuDeuxPoints);
		menuCreerSurface.add(menuItem);
		menuItem = MtgItemMenu(actionCreerDemiPlan);
		menuCreerSurface.add(menuItem);
		menuCreer.add(new JSeparator());

		JMenu menuCreerMarqueAngle = MtgMenu(getString("menuCreerMarqueAngle"));
		menuCreer.add(menuCreerMarqueAngle);
		menuItem = MtgItemMenu(actionCreerMarqueAngleNonOriente);
		menuCreerMarqueAngle.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMarqueAngleOriente);
		menuCreerMarqueAngle.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMarqueAngleOrienteSensDirect);
		menuCreerMarqueAngle.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMarqueAngleOrienteSensIndirect);
		menuCreerMarqueAngle.add(menuItem);

		menuItem = MtgItemMenu(actionCreerMarqueSegment);
		menuCreer.add(menuItem);

		menuCreer.add(new JSeparator());
		JMenu menuAffichage = MtgMenu(getString("menuAffichage"));
		menuCreer.add(menuAffichage);
		JMenu menuCreerAffichageValeur = MtgMenu(getString("menuCreerAffichageValeur"));
		menuAffichage.add(menuCreerAffichageValeur);
		menuItem = MtgItemMenu(actionCreerAffichageValeur);
		menuCreerAffichageValeur.add(menuItem);
		menuItem = MtgItemMenu(actionCreerAffichageValeurLiePoint);
		menuCreerAffichageValeur.add(menuItem);
		JMenu menuCreerCommentaire = MtgMenu(getString("menuCreerCommentaire"));
		menuAffichage.add(menuCreerCommentaire);
		menuItem = MtgItemMenu(actionCreerCommentaire);
		menuCreerCommentaire.add(menuItem);
		menuItem = MtgItemMenu(actionCreerCommentaireLiePoint);
		menuCreerCommentaire.add(menuItem);
		JMenu menuCreerLatex = MtgMenu(getString("menuCreerLatex"));
		menuAffichage.add(menuCreerLatex);
		menuItem = MtgItemMenu(actionCreerLatex);
		menuCreerLatex.add(menuItem);
		menuItem = MtgItemMenu(actionCreerLatexLiePoint);
		menuCreerLatex.add(menuItem);
		JMenu menuCreerEquationDroite = MtgMenu(getString("equationDroiteDlg1"));
		menuAffichage.add(menuCreerEquationDroite);
		menuItem = MtgItemMenu(actionCreerEquationDroite);
		menuCreerEquationDroite.add(menuItem);
		menuItem = MtgItemMenu(actionCreerEquationDroiteLiePoint);
		menuCreerEquationDroite.add(menuItem);

		
		JMenu menuCreerAffichageImage = MtgMenu(getString("menuCreerAffichageImage"));
		menuAffichage.add(menuCreerAffichageImage);
		menuItem = MtgItemMenu(actionCreerImageLibre);
		menuCreerAffichageImage.add(menuItem);
		menuItem = MtgItemMenu(actionCreerImageLieePoint);
		menuCreerAffichageImage.add(menuItem);
		menuItem = MtgItemMenu(actioncreerEditeurFormule);
		menuAffichage.add(menuItem);

		menuCreer.add(new JSeparator());
		menuItem = MtgItemMenu(actionCreerIntersection);
		menuCreer.add(menuItem);
		menuCreer.add(new JSeparator());
		menuItem = MtgItemMenu(actionCreerRepere);
		menuCreer.add(menuItem);
		menuCreer.add(new JSeparator());
		menuItem = MtgItemMenu(actionCreerObjetDuplique);
		menuCreer.add(menuItem);
		menuItem = MtgItemMenu(actionCreerObjetClone);
		menuCreer.add(menuItem);
		menuCreer.add(new JSeparator());
		menuItem = MtgItemMenu(actionCreerLongueurUnite);
		menuCreer.add(menuItem);
		return menuCreer;
	}
	
	public MtgMenu creeMenuModifier() {
		MtgMenu menuModifier = MtgMenu(getString("menuModifier"));
		MtgMenuItem menuItem = MtgItemMenu(actionSuppressionObjetGraphique);
		menuModifier.add(menuItem);
		menuItem = MtgItemMenu(actionSuppressionValeur);
		menuModifier.add(menuItem);
		menuModifier.add(new JSeparator());
		menuItem = MtgItemMenu(actionModifierObjetGraphique);
		menuModifier.add(menuItem);
		/* Supprim version 4.9.2
		menuItem = MtgItemMenu(actionModifierFormule);
		menuModifier.add(menuItem);
		*/
		menuModifier.add(new JSeparator());
		menuItem = MtgItemMenu(actionModifierRepere);
		menuModifier.add(menuItem);
		// Ajout version 4.5
		JMenu menuGraduation = MtgMenu(getString("menuAjouterGraduation"));
		menuModifier.add(menuGraduation);	
		menuItem = MtgItemMenu(actionAjoutGraduationSimple);
		menuGraduation.add(menuItem);
		menuItem = MtgItemMenu(actionAjoutGraduationTrigo);
		menuGraduation.add(menuItem);
		//
		menuModifier.add(new JSeparator());
		menuItem = MtgItemMenu(actionAugmenterTailles); // Ajout version 4.3
		menuModifier.add(menuItem);
		menuItem = MtgItemMenu(actionDiminuerTailles); // Ajout version 4.3
		menuModifier.add(menuItem);
		menuItem = MtgItemMenu(actionChangerStyleTousPoints); // Ajout version 4.3
		menuModifier.add(menuItem);
		//
		menuModifier.add(new JSeparator());
		menuItem = MtgItemMenu(actionCapture); // Ajout version 4.3
		menuModifier.add(menuItem);
		menuItem = MtgItemMenu(actionModifierNom); // Ajout version 4.3
		menuModifier.add(menuItem);
		menuItem = MtgItemMenu(actionPalette); // Ajout version 4.3
		menuModifier.add(menuItem);
		menuItem = MtgItemMenu(actionRecopierStyle); // Ajout version 5.0
		menuModifier.add(menuItem);
		menuItem = MtgItemMenu(actionGomme);
		menuModifier.add(menuItem);
		menuItem = MtgItemMenu(actionRideau);
		menuModifier.add(menuItem);
		// Ajout version 4.4 : deux outils pour marquer et dmarquer un point pour la trace
		menuItem = MtgItemMenu(actionMarquerPointPourTrace);
		menuModifier.add(menuItem);
		menuItem = MtgItemMenu(actionDemarquerPointPourTrace);
		menuModifier.add(menuItem);
		menuItem = MtgItemMenu(actionReafficherTout);
		menuModifier.add(menuItem);	
		menuItem = MtgItemMenu(actionTranslaterFigure);
		menuModifier.add(menuItem);	
		menuModifier.add(new JSeparator());
		JMenu menuModifierCreationLiaison = MtgMenu(getString("menuModifierCreationLiaison"));
		menuModifier.add(menuModifierCreationLiaison);
		menuItem = MtgItemMenu(actionCreerLiaison);
		menuModifierCreationLiaison.add(menuItem);
		menuItem = MtgItemMenu(actionCreerLiaisonAffichage);
		menuModifierCreationLiaison.add(menuItem);
		menuItem = MtgItemMenu(actionCreationLiaisonPointInterieurPolygone);
		menuModifierCreationLiaison.add(menuItem);
		menuItem = MtgItemMenu(actionCreationLiaisonPointInterieurCercle);
		menuModifierCreationLiaison.add(menuItem);
		
		JMenu menuModifierSuppressionLiaison = MtgMenu(getString("menuModifierSuppressionLiaison"));
		menuModifier.add(menuModifierSuppressionLiaison);
		menuItem = MtgItemMenu(actionSupprimerLiaison);
		menuModifierSuppressionLiaison.add(menuItem);
		menuItem = MtgItemMenu(actionSupprimerLiaisonAffichage);
		menuModifierSuppressionLiaison.add(menuItem);
		menuModifier.add(new JSeparator());
		menuItem = MtgItemMenu(actionPunaiser);
		menuModifier.add(menuItem);
		menuItem = MtgItemMenu(actionDepunaiser);
		menuModifier.add(menuItem);
		menuModifier.add(new JSeparator());
		menuItem = MtgItemMenu(actionZoomPlus); // Ajout version 4.3
		menuModifier.add(menuItem);
		menuItem = MtgItemMenu(actionZoomMoins);
		menuModifier.add(menuItem);
		menuModifier.add(new JSeparator());
		return menuModifier;
	}
	
	public MtgMenu creeMenuMouvoir() {
		MtgMenu menuMouvoir = MtgMenu(getString("menuMouvoir"));
		MtgMenuItem menuItem = MtgItemMenu(actionMouvoirAnimationPointLie);
		menuMouvoir.add(menuItem);
		menuItem = MtgItemMenu(actionCapture);
		menuMouvoir.add(menuItem);
		menuItem = MtgItemMenu(actionTraceObjets);
		menuMouvoir.add(menuItem);
		return menuMouvoir;
	}
	
	public MtgMenu creeMenuCalculs() {
		MtgMenu menuCalculs = MtgMenu(getString("menuCalculs"));
		MtgMenuItem menuItem = MtgItemMenu(actionCreerVariable);
		menuCalculs.add(menuItem);
		JMenu menuCalculDansR = MtgMenu(getString("menuCalculDansR"));
		menuCalculs.add(menuCalculDansR);
		menuItem = MtgItemMenu(actionCreerCalcul);
		menuCalculDansR.add(menuItem);
		menuItem = MtgItemMenu(actionCreerFonctionReelle);
		menuCalculDansR.add(menuItem);
		menuItem = MtgItemMenu(actionCreerFonctionReelle2Var);
		menuCalculDansR.add(menuItem);
		menuItem = MtgItemMenu(actionCreerFonctionReelle3Var);
		menuCalculDansR.add(menuItem);

		menuItem = MtgItemMenu(actionCreerDerivee);
		menuCalculDansR.add(menuItem);
		menuItem = MtgItemMenu(actionCreerSolutionEquation);
		menuCalculDansR.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMaximum);
		menuCalculDansR.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMinimum);
		menuCalculDansR.add(menuItem);
		menuItem = MtgItemMenu(actionCreerIntegrale);
		menuCalculDansR.add(menuItem);
		menuItem = MtgItemMenu(actionCreerTestExistence);
		menuCalculDansR.add(menuItem);
		menuItem = MtgItemMenu(actionCreerSuiteRecurrenteReelle);
		menuCalculDansR.add(menuItem);
		// Ajout version 4.7.5
		menuItem = MtgItemMenu(actionCreerSuiteRecurrenteReelle2);
		menuCalculDansR.add(menuItem);
		menuItem = MtgItemMenu(actionCreerSuiteRecurrenteReelle3);
		menuCalculDansR.add(menuItem);
		//
		
		JMenu menuCalculAvance = MtgMenu(getString("avance"));
		menuCalculDansR.add(menuCalculAvance);
		menuItem = MtgItemMenu(actionCreerDeriveePartielle);
		menuCalculAvance.add(menuItem);
		menuItem = MtgItemMenu(actionCreerTestEquivalence);
		menuCalculAvance.add(menuItem);
		menuItem = MtgItemMenu(actionCreerTestFact);
		menuCalculAvance.add(menuItem);
		menuItem = MtgItemMenu(actionCreerTestDependanceVariable);
		menuCalculAvance.add(menuItem);
		menuItem = MtgItemMenu(actionCreerTestEqNatOp);
		menuCalculAvance.add(menuItem);
		menuItem = MtgItemMenu(actionCreerSommeIndicee);
		menuCalculAvance.add(menuItem);
		menuItem = MtgItemMenu(actionCreerProduitIndice);
		menuCalculAvance.add(menuItem);
		menuCalculDansR.add(new JSeparator());
		menuItem = MtgItemMenu(actionCreerPartieReelle);
		menuCalculDansR.add(menuItem);
		menuItem = MtgItemMenu(actionCreerPartieImaginaire);
		menuCalculDansR.add(menuItem);
		menuItem = MtgItemMenu(actionCreerModule);
		menuCalculDansR.add(menuItem);
		menuItem = MtgItemMenu(actionCreerArgument);
		menuCalculDansR.add(menuItem);
		
		JMenu menuCalculDansC = MtgMenu(getString("menuCalculDansC"));
		menuCalculs.add(menuCalculDansC);
		menuItem = MtgItemMenu(actionCreerCalculComplexe);
		menuCalculDansC.add(menuItem);
		menuItem = MtgItemMenu(actionCreerFonctionComplexe);
		menuCalculDansC.add(menuItem);
		menuItem = MtgItemMenu(actionCreerFonctionComplexe2Var);
		menuCalculDansC.add(menuItem);
		menuItem = MtgItemMenu(actionCreerFonctionComplexe3Var);
		menuCalculDansC.add(menuItem);
		menuItem = MtgItemMenu(actionCreerSuiteRecurrenteComplexe);
		menuCalculDansC.add(menuItem);
		// Ajout version 4.7.5
		menuItem = MtgItemMenu(actionCreerSuiteRecurrenteComplexe2);
		menuCalculDansC.add(menuItem);
		menuItem = MtgItemMenu(actionCreerSuiteRecurrenteComplexe3);
		menuCalculDansC.add(menuItem);
		//
		menuCalculs.add(new JSeparator());

		JMenu menuCalculMesurer = MtgMenu(getString("menuCalculMesurer"));
		menuCalculs.add(menuCalculMesurer);
		menuItem = MtgItemMenu(actionMesureLongueur);
		menuCalculMesurer.add(menuItem);
		menuItem = MtgItemMenu(actionMesureAbscisse);
		menuCalculMesurer.add(menuItem);
		menuItem = MtgItemMenu(actionMesureAngleNonOriente);
		menuCalculMesurer.add(menuItem);
		menuItem = MtgItemMenu(actionMesureAngleOriente);
		menuCalculMesurer.add(menuItem);
		menuItem = MtgItemMenu(actionMesureAbscisseDansRepere);
		menuCalculMesurer.add(menuItem);
		menuItem = MtgItemMenu(actionMesureOrdonneeDansRepere);
		menuCalculMesurer.add(menuItem);
		menuItem = MtgItemMenu(actionMesureCoefficientDirecteur);
		menuCalculMesurer.add(menuItem);
		menuItem = MtgItemMenu(actionMesureAffixe);
		menuCalculMesurer.add(menuItem);
		menuItem = MtgItemMenu(actionMesureProduitScalaire);
		menuCalculMesurer.add(menuItem);
		menuItem = MtgItemMenu(actionMesureLongueurLigne);
		menuCalculMesurer.add(menuItem);
		menuItem = MtgItemMenu(actionMesureAirePolygone);
		menuCalculMesurer.add(menuItem);
		menuCalculs.add(new JSeparator());
		menuItem = MtgItemMenu(actionRenommerCalcul);
		menuCalculs.add(menuItem);
		menuItem = MtgItemMenu(actionAfficherObjetsNumeriques);
		menuCalculs.add(menuItem);
		// Ajouts version 4.5
		menuCalculs.add(new JSeparator());
		JMenu menuInfosRepere = MtgMenu(getString("menuInfoRepere"));
		menuCalculs.add(menuInfosRepere);
		menuItem = MtgItemMenu(actionCreerAbscisseOrigineRep);
		menuInfosRepere.add(menuItem);
		menuItem = MtgItemMenu(actionCreerOrdonneeOrigineRep);
		menuInfosRepere.add(menuItem);
		menuItem = MtgItemMenu(actionCreerUnitexRep);
		menuInfosRepere.add(menuItem);
		menuItem = MtgItemMenu(actionCreerUniteyRep);
		menuInfosRepere.add(menuItem);
		return menuCalculs;
	}
	
	public MtgMenu creeMenuMacros() {
		MtgMenu menuMacros = MtgMenu(getString("menuMacros"));
		JMenu menuMacrosNouvelleMacro = MtgMenu(getString("menuMacrosNouvelleMacro"));
		menuMacros.add(menuMacrosNouvelleMacro);
		MtgMenuItem menuItem = MtgItemMenu(actionCreerMacroApparition);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroApparitionAPartirAutreMacro);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroDisparition);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroDisparitionAPartirAutreMacro);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroAnimation);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroAnimationAvecTrace);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroAffectationValeurVariable);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroModificationVariable);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroJouantSon);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroIncrementationVariable);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroDecrementationVariable);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroTraceAutoParPointLie);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroTraceAutoParVariable);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroAffectationPointLie);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroClignotement);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroAffichantTout);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroPause);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroBoucleAvecAnimation);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroBoucleAvecTrace);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroSuiteMacros);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroModeTraceActivation);
		menuMacrosNouvelleMacro.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroModeTraceDesactivation);
		menuMacrosNouvelleMacro.add(menuItem);
		menuMacrosNouvelleMacro.add(new JSeparator());
		JMenu menuMacrosConstruction = MtgMenu(getString("menuMacroConstruction"));
		menuItem = MtgItemMenu(actionCreerMacroConstructionIterative);
		menuMacrosConstruction.add(menuItem);
		menuItem = MtgItemMenu(actionCreerMacroConstructionRecursive);
		menuMacrosConstruction.add(menuItem);
		menuMacrosNouvelleMacro.add(menuMacrosConstruction);
		

		menuItem = MtgItemMenu(actionExecuterMacro);
		menuMacros.add(menuItem);
		menuMacros.add(new JSeparator());
		menuItem = MtgItemMenu(actionModificationMacro);
		menuMacros.add(menuItem);
		menuMacros.add(new JSeparator());
		menuItem = MtgItemMenu(actionAjoutObjetsMacro);
		menuMacros.add(menuItem);
		menuItem = MtgItemMenu(actionRetraitObjetsMacro);
		menuMacros.add(menuItem);
		return menuMacros;
	}
	
	public MtgMenu creeMenuHelp() {
		MtgMenu menuHelp = MtgMenu(getString("menuHelp"));
		MtgMenuItem menuItem = MtgItemMenu(actionHelpAbout);
		menuHelp.add(menuItem);
		menuItem = MtgItemMenu(actionHelpNet);
		menuHelp.add(menuItem);
		menuItem = MtgItemMenu(actionHelpHelp);
		menuHelp.add(menuItem);
		menuItem = MtgItemMenu(actionHelpOnline);
		menuHelp.add(menuItem);		
		menuItem = MtgItemMenu(actionHelpCodeHTML);
		menuHelp.add(menuItem);
		menuItem = MtgItemMenu(actionHelpTexteFigure);
		menuHelp.add(menuItem);
		// Ajout version 4.9.2
		menuItem = MtgItemMenu(actionHTMLInfo);
		menuHelp.add(menuItem);
		return menuHelp;
	}
	
	public MtgMenu creeMenuOptions() {
		MtgMenu menuOptions = MtgMenu(getString("menuOptions"));
		MtgMenuItem menuItem = MtgItemMenu(actionOptionsFigureEnCours);
		menuOptions.add(menuItem);
		menuItem = MtgItemMenu(actionOptionsPreferences);
		menuOptions.add(menuItem);
		menuOptions.add(new JSeparator());
		menuItem = MtgItemMenu(actionChoixImageFond);
		menuOptions.add(menuItem);
		menuItem = MtgItemMenu(actionSuppressionImageFond);
		menuOptions.add(menuItem);
		cbImageFondVisible = new MtgCheckBoxMenuItem(actionImageFondVisible, pref_NomsAuto);
		menuOptions.add(cbImageFondVisible);
		menuOptions.add(new JSeparator());		
		cbAideVisuelle = new MtgCheckBoxMenuItem(actionCreationVisuelle,
				pref_AideVisuelle);
		cbNomsAuto = new MtgCheckBoxMenuItem(actionNomsAuto, pref_NomsAuto);
		menuOptions.add(cbAideVisuelle);
		menuOptions.add(cbNomsAuto);
		cbModeTrace = new MtgCheckBoxMenuItem(actionModeTrace, false); // Mode trace dsactiv au dmarrage
		cbModeCreationPointAuto = new MtgCheckBoxMenuItem(actionModePointAuto, pref_creationPointsAuto); // Ajout version 5.0
		menuOptions.add(cbModeTrace);
		menuOptions.add(cbModeCreationPointAuto);
		JMenu menuPersonnaliserNiveaux = MtgMenu(getString("personnalisationNiveau"));
		// Version 4.3 : Il faut adapter les intituls de ces 4 menus suivant le fait qu'on utilise ou pas les
		// niveaux personnaliss.
		menuItem = MtgItemMenu(actionOptionPersonnaliserNiveau1);
		menuItem.setText((descripteurNiveauPersonnalise[4] != null) ? descripteurNiveauPersonnalise[4] : descripteurNiveau[4]);
		menuPersonnaliserNiveaux.add(menuItem);
		menuItem = MtgItemMenu(actionOptionPersonnaliserNiveau2);
		menuItem.setText((descripteurNiveauPersonnalise[3] != null) ? descripteurNiveauPersonnalise[3] : descripteurNiveau[3]);
		menuPersonnaliserNiveaux.add(menuItem);
		menuItem = MtgItemMenu(actionOptionPersonnaliserNiveau3);
		menuItem.setText((descripteurNiveauPersonnalise[2] != null) ? descripteurNiveauPersonnalise[2] : descripteurNiveau[2]);
		menuPersonnaliserNiveaux.add(menuItem);
		menuItem = MtgItemMenu(actionOptionPersonnaliserNiveau4);
		menuItem.setText((descripteurNiveauPersonnalise[1] != null) ? descripteurNiveauPersonnalise[1] : descripteurNiveau[1]);
		menuPersonnaliserNiveaux.add(menuItem);
		menuOptions.add(menuPersonnaliserNiveaux);		
		return menuOptions;
	}
	
	public MtgMenu creeMenuConstructions() {
		MtgMenu menuConstructions = MtgMenu(getString("menuConstructions"));
		MtgMenuItem menuItem = MtgItemMenu(actionConstructionImplementerDepuisFigure);
		menuConstructions.add(menuItem);
		menuItem = MtgItemMenu(actionConstructionImplementerDepuisFichier);
		menuConstructions.add(menuItem);
		menuItem = MtgItemMenu(actionConstructionImplementerPredefinie);
		menuConstructions.add(menuItem);
		// Ajout version 4.8
		menuItem = MtgItemMenu(actionConstructionIterativeImplementerDepuisFigure);
		menuConstructions.add(menuItem);	
		menuItem = MtgItemMenu(actionConstructionRecursiveImplementerDepuisFigure);
		menuConstructions.add(menuItem);	
		//
		menuConstructions.add(new JSeparator());
		menuItem = MtgItemMenu(actionConstructionEnregistrer);
		menuConstructions.add(menuItem);
		menuItem = MtgItemMenu(actionIncorporerConstruction);
		menuConstructions.add(menuItem);
		menuItem = MtgItemMenu(actionConstructionSupprimer);
		menuConstructions.add(menuItem);
		menuItem = MtgItemMenu(actionModifierConstruction);
		menuConstructions.add(menuItem);
		menuItem = MtgItemMenu(actionConstructionFusionner);
		menuConstructions.add(menuItem);
		menuConstructions.add(new JSeparator());
		MtgMenu menuConstructionSources = new MtgMenu(getString("MenuConstructionsChoixSources"), this);
		menuConstructions.add(menuConstructionSources);
		menuItem = MtgItemMenu(actionChoixSourcesNonGraphiques);
		menuConstructionSources.add(menuItem);
		menuItem = MtgItemMenu(actionChoixSourcesGraphiques);
		menuConstructionSources.add(menuItem);
		MtgMenu menuConstructionFinaux = new MtgMenu(getString("MenuConstructionsChoixFinaux"), this);
		menuConstructions.add(menuConstructionFinaux);
		menuItem = MtgItemMenu(actionChoixFinauxNonGraphiques);
		menuConstructionFinaux.add(menuItem);
		menuItem = MtgItemMenu(actionChoixFinauxGraphiques);
		menuConstructionFinaux.add(menuItem);
		menuItem = MtgItemMenu(actionConstructionCreer);
		menuConstructions.add(menuItem);
		menuItem = MtgItemMenu(actionConstructionReinitialiser);
		menuConstructions.add(menuItem);
		return menuConstructions;
	}

	// Modifi version 4.3
	/**
	 * Adaptation du menu au document en cours et en mme temps, s'il est prsent
	 * au document spcifiant les outils disponibles pour le niveau de fonctionnement choisi
	 * @param doc
	 */
	public void adapteMenuADocument(CMathGrafDoc doc) {
		cbImageFondVisible.setSelected(doc.imageFondVisible);
		menuFichier.adapteADocument(doc, docPreferencesNiveau[pref_niveau]);
		menuEdition.adapteADocument(doc, docPreferencesNiveau[pref_niveau]);
		menuCreer.adapteADocument(doc, docPreferencesNiveau[pref_niveau]);
		menuCalculs.adapteADocument(doc, docPreferencesNiveau[pref_niveau]);
		menuModifier.adapteADocument(doc, docPreferencesNiveau[pref_niveau]);
		menuMacros.adapteADocument(doc, docPreferencesNiveau[pref_niveau]);
		menuConstructions.adapteADocument(doc, docPreferencesNiveau[pref_niveau]);
	}

	/**
	 * Adaptation des icnes et du menu au document doc
	 * @param doc
	 */
	public void metAJourIconesEtMenus(CMathGrafDoc doc) {
		adapteMenuADocument(docPreferencesNiveau[pref_niveau]);
		adapteMenuADocument(doc);
		btngroup1 = new ButtonGroup();
		btngroup2 = new ButtonGroup();
		metAJourBoutonsBarreOutils(doc); // A revoir version 4.3
		metAJourBoutonsBarreHorizontale(doc);
		repaint();
	}

	/**
	 * Fonction renvoyant le syle de trait actif
	 * Modifi version 5.0. Renvoie un objet StyleTrait et plus un entier
	 * @return
	 */
	public StyleTrait styleTraitActif() {
		return new StyleTrait(listeStyleTrait.styleTrait(), epaisseurTraitActif());
	}
	
	// Ajout version 5.0
	public int epaisseurTraitActif() {
		return sliderEpaisseur.getValue();
	}

	/**
	 * Fonction renvoyant le style de point actif
	 * 
	 * @return
	 */
	public int stylePointActif() {
		return stylePointActif;
	}

	/**
	 * Fonction renvoyant le style de remplissage actif
	 * 
	 * @return
	 */
	public int styleRemplissageActif() {
		return styleRemplissageActif;
	}

	/**
	 * Fonction renvoyant le style de marque de segment actif
	 * 
	 * @return
	 */
	public int styleMarqueSegmentActif() {
		return styleMarqueSegmentActif;
	}

	/**
	 * Fonction renvoyant le style de flche actif pour les vecteurs et arcs
	 * orients
	 * 
	 * @return
	 */
	public int styleFlecheActif() {
		return styleFlecheActif;
	}
	
	/**
	 * Fonction renvoyant le style de marque d'angle actif
	 * @return
	 */
	public int styleMarqueAngleActif() {
		return styleMarqueAngleActif;
	}

	/**
	 * Fonctio prparant les variables xfenetre et yfenetre pour un affichage
	 * d'une nouvelle fenetre MtgInternalFrame
	 * 
	 * @param tailleFen
	 * @return renvoie true si le positonnement est possible dans une sous fentre
	 *         et false si la fentre a besoin d'tre maximise
	 */
	public boolean preparePositionPourFenenetre(Dimension tailleFen) {
		Dimension size = mtgDesktopPane.getSize();
		if ((size.width <= tailleFen.width) || (size.height <= tailleFen.height)) {
			xfenetre = 0;
			yfenetre = 0;
			return false;
		} else {
			int k = MtgFrame.screenIsHighResolution ? 2 : 1;
			xfenetre += 10*k;
			yfenetre += 10*k;
			int x = xfenetre + tailleFen.width;
			int y = yfenetre + tailleFen.height;
			if (x > size.width)
				xfenetre = 0;
			if (y > size.height)
				yfenetre = 0;
			return true;
		}
	}

	/**
	 * Renvoie un pinteur sur le cadre correspondant au chemin d'accs et null sinon
	 * @param path
	 * @return
	 */
	public MtgInternalFrame cadreParChemin(String path) {
		JInternalFrame[] tableauCadres = mtgDesktopPane.getAllFrames();
		for (int i = 0; i < tableauCadres.length; i++) {
			MtgInternalFrame fr = (MtgInternalFrame) tableauCadres[i];
			if (fr.path != null) {
				if (fr.path.equals(path))
					return fr;
			}
		}
		return null;
	}

	/**
	 * Proprits par dfaut du logiciel
	 * @return
	 */
	public Properties defaultProperties() {
		Properties prop = new Properties();
		// Unit d'angle pour les nouvelles figures
		prop.setProperty("angleUnit", "degree");
		// Nombre de points utiliss quand on utilise le menu Mouvoir Animer un
		// point li
		prop.setProperty("moveNbPoints", "100");
		// Frquence utilise quand on utilise le menu Mouvoir Animer un point li
		prop.setProperty("moveFreq", "20");
		// / Animation cyclique ou non
		prop.setProperty("moveCyclic", "true");
		// Rpertoire par dfaut pour l'ouverture des fichiers
		prop.setProperty("fileDirectory", "");
		// Rpertoire par dfaut pour l'ouverture des constructions
		prop.setProperty("constDirectory", "");
		// Niveau defonctionnement
		prop.setProperty("level", "1");
		// Aide  la cration visuelle
		prop.setProperty("visualHelp", "true");
		// Dcalage des affichages libres lors des translations
		prop.setProperty("FreeDisplaysTranslation", "true");
		// Noms automatiques
		prop.setProperty("NomsAuto", "false");
		// Indice de taiile de caractres pour les noms
		// Version 5.0 : On tient compte des crans haute dfinition
		int size = (int) (coefMultForSystemFonts*13);
		prop.setProperty("IndexFontSizeNames", Integer.toString(size));
		// Indice de taile de caractres pour les noms
		// Version 5.0 : On tient compte des crans haute dfinition
		size = (int) (coefMultForSystemFonts*12);
		prop.setProperty("IndexFontSizeDisplays", Integer.toString(size));
		// Nombre de colonnes pour la barre d'outils de gauche
		// Modifi version 4.6
		// prop.setProperty("NbColToolbar", "5");
		// prop.setProperty("NbColToolbar", "6"); Supprim verson 5.0
		// Petites ou grandes icones
		// Modifi version 4.6 : Grandes icnes par dfaut
		// prop.setProperty("littleIconSize", "true");
		prop.setProperty("littleIconSize", "false");
		// Ajout pour version 2.0
		prop.setProperty("transparency", "20");
		// Ajout pour version 3.2.4
		prop.setProperty("personalizedLevel", "false");
		// Version 3.2.4 : quatre boolens servant  savoir si on doit afficher les mesures de longuers et angles sur la figure
		// Par dfaut c'est le cas seulement pour les deux premiers niveaux (lmentaire et collge)
		prop.setProperty("measuresDisplay1", "true");
		prop.setProperty("measuresDisplay2", "true");
		prop.setProperty("measuresDisplay3", "true");
		prop.setProperty("measuresDisplay4", "true");
		prop.setProperty("antialiasing", "true");
		prop.setProperty("clipboardImagesRatio", "1"); // Ajout version 4.6
		// Version 4.6.4
		prop.setProperty("thicknessDecrease", "false");
		// Version 4.7.5.2
		// Pour version OLPC utiliser la ligne suivante :
		// prop.setProperty("addSystemFont", "7");
		// Modifi version 5.0
		prop.setProperty("addSystemFont", "0"); // Nouveau version 4.5.2. Agrandissement par dfaut des polices systme.
		// Ajout version 5.0
		prop.setProperty("newInterface", "true");
		prop.setProperty("creationPointsAuto", "true"); // Si true cration de points libres lors de clics sur emplacement vide
		//
		// Version 5.0 : On mmorise pour la version Webstart si les fichiers d'aide et de construction ont bien t chargs en ligne
		// lors du premier dmarrage. Si ce n'est pas le cas on ressayera lors des dmarrages suivants
		prop.setProperty("versionLoaded", "none");
		return prop;
	}

	private void set_pref_langage_Systeme() {
		// On regarde quelle est la langue du systme
		try {
			String language = System.getProperty("user.language");
			if (language.equals("fr")) pref_langage = "fra";
			else {
				if (language.equals("es")) pref_langage = "esp";
				else pref_langage = "eng"; // Anglais par dfaut si langue pas supporte par MathGraph32
			}
		}
		catch (Exception e) {
			pref_langage = "eng";
		}
	}
	
	/**
	 * Chargement des prfrences de fonctionnement  partir du fichier de prfrences
	 */
	private void chargePreferences() {
		// On charge les prfrences de l'application
		// properties = new Properties(defaultProperties()); Modifi version 3.7
		// Essai
		// extractConstructions();
		properties = defaultProperties();
		String fs = System.getProperty("file.separator");
		String path;
		// Obligation d'intercepter exceptions du type SecuriteException sur certains systme
		// Version 5.0 : Pour que les versions WebStart et non webstart pussent communiquer, un dossier MathGraph32WebStart est
		// utilis pour la version WebStart
		try {
			String se = System.getProperty("os.name");
			if (se.startsWith("Linux")) path = System.getProperty("user.home") + fs +".config" + fs + "MathGraph32WebStart";
			else if (se.startsWith("Mac")) path = System.getProperty("user.home") + fs + ".MathGraph32WebStart";
			else path = System.getProperty("user.home") + fs + "MathGraph32WebStart";
		}
		catch (Exception e) {
			path = System.getProperty("user.home") + fs + "MathGraph32WebStart";
		}
		String ch = path + fs + "MathGraph32Properties";
		File file = new File(ch);
		if (file.exists()) {
			FileInputStream in;
			try {
				in = new FileInputStream(file);
				try {
					properties.load(in);
					in.close();
				} catch (IOException e) {
					properties = new Properties(defaultProperties());
					set_pref_langage_Systeme();
				}
			} catch (FileNotFoundException e) {
				properties = new Properties(defaultProperties());
				set_pref_langage_Systeme();
			}
			pref_langage = properties.getProperty("langage");
			if (pref_langage == null) set_pref_langage_Systeme();
		}
		else set_pref_langage_Systeme();
		// Version 3.3 : Si le programme tourne pour la premire fois c'est ici qu'il faut mettre cette ligne
		// et non plus dans defaultProperties
		properties.setProperty("langage", pref_langage);
  	if (pref_langage.startsWith("fr" )) {
   		locale = new Locale("fr");
  	}
  	else
  	if (pref_langage.startsWith("es" )) {
  		locale = new Locale("es");
  	}
  	else
  	if (pref_langage.startsWith("en" )) {
  		locale = new Locale("en");
  	}
  	messages = ResourceBundle.getBundle("textes.messages", locale);
		
		// Ajout version 5.0 WebStart : On regarde le numro de version (s'il existe).
  	// S'il n'est pas gal au numro de version actuel, on extrait tous les fichiers de contructios
  	// prdfinies pour les mettres dans le dossier MathGraph32 du rpertoire utilisateur.
		// Version Webstart 5.0 : On regarde si les fichiers d'aide et de construction en ligne ont bien t chargs.
		// Si ce n'est pas le cas on tentera une nouvele fois de le faire.
		versionLoaded = properties.getProperty("versionLoaded");
  	String version = properties.getProperty("version");
  	if ((version == null) || (!version.equals(MtgFrame.numeroVersion)) || versionLoaded.equals("none")) extractConstructions();
  	//
		String ch1 = properties.getProperty("angleUnit");
		if (ch1.equals("degree"))
			pref_angleUnit = UniteAngle.Degre;
		else
			pref_angleUnit = UniteAngle.Radian;
		pref_moveNbPoints = Integer
				.parseInt(properties.getProperty("moveNbPoints"));
		pref_moveFreq = Integer.parseInt(properties.getProperty("moveFreq"));
		pref_moveCyclic = Boolean
				.parseBoolean(properties.getProperty("moveCyclic"));
		pref_fileDirectory = properties.getProperty("fileDirectory");
		pref_constDirectory = properties.getProperty("constDirectory");
		pref_niveau = Integer.parseInt(properties.getProperty("level"));
		pref_AideVisuelle = Boolean.parseBoolean(properties
				.getProperty("visualHelp"));
		pref_decalageAffichagesLibres = Boolean.parseBoolean(properties
				.getProperty("FreeDisplaysTranslation"));
		pref_NomsAuto = Boolean.parseBoolean(properties.getProperty("NomsAuto"));
		// Modifi version 5.0 : pref_indiceTaillePoliceNom devient maintenant la taille relle de police
		pref_indiceTaillePoliceNom = Integer.parseInt(properties.getProperty("IndexFontSizeNames"));
		if (pref_indiceTaillePoliceNom < 10) pref_indiceTaillePoliceNom = 16; // Rajout version 5.0
		// Modifi version 5.0 : pref_IndiceTaillePoliceAffichages devient maintenant la taille relle de police
		pref_IndiceTaillePoliceAffichages = Integer.parseInt(properties.getProperty("IndexFontSizeDisplays"));
		if (pref_IndiceTaillePoliceAffichages < 10) pref_IndiceTaillePoliceAffichages = 13; // Rajout version 5.0
		// Nouveau version 3.2.4 : personnalisation possible des niveaux (ivers  cause d'un mauvais choix initial
		pref_NiveauPersonnalise = Boolean.parseBoolean(properties.getProperty("personalizedLevel"));
		pref_affichageMesuresNiveau[1] = Boolean.parseBoolean(properties.getProperty("measuresDisplay1"));
		pref_affichageMesuresNiveau[2] = Boolean.parseBoolean(properties.getProperty("measuresDisplay2"));
		pref_affichageMesuresNiveau[3] = Boolean.parseBoolean(properties.getProperty("measuresDisplay3"));
		pref_affichageMesuresNiveau[4] = Boolean.parseBoolean(properties.getProperty("measuresDisplay4"));

		// On charge aussi le document qui spcifie les outils permis ou interdits nomm
		// default1.mgj pour le niveau lmentaire
		// default2.mgj pour le niveau collge
		// default3.mgj pour le niveau avanc sans complexes
		// default4.mgj pour le niveau avanc avec complexesation
		// On le cherche dans le dossier contenant le jar excutable.
		// Version 3.2.4 : on initialise les chanes de descripteur de niveau personnalis qui seront ventuellement
		// modifies dans chargePreferences
		descripteurNiveauPredefini[1] = getString("niveauAvanceAvecComplexes");
		descripteurNiveauPredefini[2] = getString("niveauAvanceSansComplexes");
		descripteurNiveauPredefini[3] = getString("niveauMoyen");
		descripteurNiveauPredefini[4] = getString("niveauElementaire");
		//
		// Version 4.5.1 : Possibilit de choisir d'utiliser le mode antialiasint activ ou non
		pref_antialiasing = Boolean.parseBoolean(properties.getProperty("antialiasing"));
		// Ajout version 4.7.5.2;
		pref_addSystemFont = Integer.parseInt(properties.getProperty("addSystemFont"));
		// Modification version 4.3 : Les documents donant les outils permis par niveaux sont chargs dans des tableaux
		for (int i = 1; i <= 4; i++) {
			// Modifi version 5.0
			// docPreferencesNiveauPredefini[i] = chargeDocumentDansDossierMtgFrame("Help" + pref_langage + "/default" + Integer.toString(5-i) + ".mgj");
			docPreferencesNiveauPredefini[i] = chargeDocumentDansDossierMtgFrame(pref_langage + "/default" + Integer.toString(5-i) + ".mgj");
			docPreferencesNiveauPersonnalise[i] = chargeDocumentDansDossierUtilisateur("level" + Integer.toString(5-i) + ".mgj");
		} 
		// Ajout version 4.6
		pref_ClipboardImagesRatio = Double.parseDouble(properties.getProperty("clipboardImagesRatio"));
		// Ajout version 5.0
		pref_creationPointsAuto = Boolean.parseBoolean(properties.getProperty("creationPointsAuto"));
		//
		setDocPreferencesNiveau();
		setDescripteursNiveau();
	}

	/**
	 * Sauvegarde des prfrences de fonctionnement dans le fichier de prfrences
	 */
	private void sauvePreferences() {
		// Ajout version 5.0 WebStart
		// On sauve le numro de version pour savoir s'il faut ou non extraire  les fichiers de construction
		properties.setProperty("version", MtgFrame.numeroVersion);
		// On sauvegarde aussi versionLoaded qui sert  savoir si les fichiers d'aide et de construction en ligne
		// ont bien pu tre tlchargs lors de la premire utilisation.
		properties.setProperty("versionLoaded", versionLoaded);
		// Les quatre prfrences qui ne se font pas via le menu Preferences
		properties.setProperty("NomsAuto", Boolean.toString(pref_NomsAuto));
		properties.setProperty("visualHelp", Boolean.toString(pref_AideVisuelle));
		properties.setProperty("IndexFontSizeNames", Integer.toString(pref_indiceTaillePoliceNom));
		properties.setProperty("IndexFontSizeDisplays", Integer.toString(pref_IndiceTaillePoliceAffichages));
		// Ajout version 4.6
		properties.setProperty("clipboardImagesRatio", glob.Kernel.chaineNombre(pref_ClipboardImagesRatio, 2));
		properties.setProperty("addSystemFont", Integer.toString(pref_addSystemFont)); // Ajout version 4.7.5.2
		// Ajout version 5.0
		properties.setProperty("creationPointsAuto", Boolean.toString(pref_creationPointsAuto));
		//
		String fs = System.getProperty("file.separator");
		String path;
		try {
			// Version 5.0 : Pour que les versions WebStart et non webstart pussent communiquer, un dossier MathGraph32WebStart est
			// utilis pour la version WebStart
			String se = System.getProperty("os.name");
			if (se.startsWith("Linux")) path = System.getProperty("user.home") + fs +".config" + fs + "MathGraph32WebStart";
			else if (se.startsWith("Mac")) path = System.getProperty("user.home") + fs + ".MathGraph32WebStart";
			else path = System.getProperty("user.home") + fs + "MathGraph32WebStart";
		}
		catch (Exception e) {
			path = System.getProperty("user.home") + fs + "MathGraph32WebStart";
		}
		String ch = path + fs + "MathGraph32Properties";
		File filePath = new File(path);
		File file = new File(ch);
		FileOutputStream out;
		try {
			if (!filePath.exists())
				filePath.mkdir();
			out = new FileOutputStream(file);
			try {
				properties.store(out, "Mathgraph32 Properties");
				out.close();
			} catch (IOException e1) {
			}
		} catch (FileNotFoundException e2) {
		}
	}
	

	/**
	 * Gnration automatique d'un nom quand la gnration automatique de noms de
	 * points est active
	 * 
	 * @return
	 */
	public String genereNomPourPoint() {
		if (pref_NomsAuto)
			return cadreActif().listePr.genereNomPourPoint(false);
		else
			return "";
	}

	
  // Spcifique version WebStart. On fait comme pour les applets.
	public final CMathGrafDoc chargeDocumentDansDossierMtgFrame(String nomFichier) {
		BufferedInputStream buff = null;
		CMathGrafDoc doc = new CMathGrafDoc(this);
		try {
			URL adresse = MtgFrame.class.getResource(nomFichier);
			try {
				buff = new BufferedInputStream(adresse.openStream());
				MathGraphDataInputStream dataStream = new MathGraphDataInputStream(buff);
				doc.chargeDepuisFlux(dataStream); // Modifi version 4.4.1
			} finally {
				buff.close();
			}
		} catch (Exception e) {
		}
		return doc;
	}

	/**
	 * Nouveau version 5.0 : Charge un document dans le dossier de l'utilisateur (celui ou est MathGraph32.properties)
	 * C'est ici qu'on mettra maintenant les figures MathGraph32 servant  dfinir les niveaux personnaliss.
	 * @param nomFichier
	 * @return null si fiichier ps trouv, sinon la figure MathGraph32 correspondante
	 */
	public final CMathGrafDoc chargeDocumentDansDossierUtilisateur(String nomFichier) {
		String fs = System.getProperty("file.separator");
		String path;
		// Obligation d'intercepter exceptions du type SecuriteException sur certains systme
		try {
			String se = System.getProperty("os.name");
			if (se.startsWith("Linux")) path = System.getProperty("user.home") + fs +".config" + fs + "MathGraph32WebStart";
			else if (se.startsWith("Mac")) path = System.getProperty("user.home") + fs + ".MathGraph32WebStart";
			else path = System.getProperty("user.home") + fs + "MathGraph32WebStart";
		}
		catch (Exception e) {
			path = System.getProperty("user.home") + fs + "MathGraph32WebStart";
		}
		String ch = path + fs + nomFichier;
		File file = new File(ch);
		CMathGrafDoc doc = new CMathGrafDoc(this);
		if (file.exists()) {
			FileInputStream in;
			try {
				in = new FileInputStream(file);
 				MathGraphDataInputStream dataStream = new MathGraphDataInputStream(in);
 				try{
					doc.chargeDepuisFlux(dataStream); // Modifi version 4.4.1
					in.close();
 				}
 				catch (IOException e) {
 					return null;
				}
			} 
			catch (FileNotFoundException e) {
				return null;
			}
			return doc;
		}
		else return null;
	}

  /**
   * Fournit le chemin d'accs au jar excutable (avec le / de fin)
   */
  public final String pathJar() {
    String path = "/" + MtgFrame.class.getName().replace('.', '/') + ".class";
    URL url = getClass().getResource(path);
    try {
			path = URLDecoder.decode(url.toString(), "UTF-8");
		} catch (UnsupportedEncodingException e2) {
			path = "";
			return null;
		}
		if (path.length() != 0) {
	    int index = path.lastIndexOf("/");
	    path = path.substring(0, index);
	    
	    // On enlve le jar:file: du dbut si ncessaire
	    if (path.startsWith("jar:file:"))
	    {
	      index = path.indexOf("!");
	      path = path.substring(9, index);
	    }
	    // On remonte de un niveau
	    index = path.lastIndexOf("/");
	    path = path.substring(0, index + 1);
	    if (path.startsWith("file:/")) path = path.substring(6);
		}
		return path;
  }
  
  //Ajout version 4.3
	/**
	 * Chargement d'une construction prdfinie dans le dossier constructions situ dans le dossier contenant MtgFrame
	 * @param nomFichier
	 * @return
	 */
  public final CPrototype chargeConstructionDansDossierMtgFrame(String nomFichier) {
		CPrototype prot = new CPrototype();
		try {
			URL adresse = MtgFrame.class.getResource("constructions/" + nomFichier);
			BufferedInputStream buff = new BufferedInputStream(adresse.openStream());
			MathGraphDataInputStream dataStream = new MathGraphDataInputStream(buff);
			prot.read(dataStream);
			buff.close(); // Ajout version 5.0
		} catch (Exception e) {};
		return prot;
	}
  
  // Ajout version 4.3
  /**
   * On tablit les chaines de caratres charges de dcrire chaque niveau
   */
  public void setDescripteursNiveau() {
  	for (int i = 1; i <= 4; i++) {
	  	if (docPreferencesNiveauPersonnalise[i] == null) descripteurNiveauPersonnalise[i] = descripteurNiveauPredefini[i];
	  	else {
				// On charge les 80 premiers caractres du document contenant les outils personnaliss pour ce niveau
				String ch2 = docPreferencesNiveauPersonnalise[i].TexteFigure;
				int len = ch2.length();
				if (len > 80) ch2 = ch2.substring(0, 79);
				// On retire les ventuels retours  la ligne
				int lf = ch2.indexOf("\n");
				if (lf != -1) ch2 = ch2.substring(0, lf);
				descripteurNiveauPersonnalise[i] = ch2;
	  	}
  		if (pref_NiveauPersonnalise) {
	  		if (docPreferencesNiveauPersonnalise[i] == null) {
	  			descripteurNiveau[i] = descripteurNiveauPredefini[i];
	  		}
	  		else {
					descripteurNiveau[i] = descripteurNiveauPersonnalise[i];
	  		}
  		}
  		else {
  			descripteurNiveau[i] = descripteurNiveauPredefini[i];
  		}
  	}
  }
  
  public void setDocPreferencesNiveau() {
  	for (int i = 1; i <= 4; i++) {
			if (pref_NiveauPersonnalise) {
				if (docPreferencesNiveauPersonnalise[i] == null) {
					docPreferencesNiveau[i] = docPreferencesNiveauPredefini[i];
				}
				else {
					docPreferencesNiveau[i] = docPreferencesNiveauPersonnalise[i];
				}
			}
			else docPreferencesNiveau[i] = docPreferencesNiveauPredefini[i];
  	}
  }
  
  public static String getString(String key) {
  	try {
  		return messages.getString(key);
  	}
  	catch (Exception e) {
			return "!" + key + "!";
		}
  }
  
  /**
   * Fonction mettant  zros les menus en fonction de la langue systme
   */
  public final void setLanguage() {
  	if (pref_langage.startsWith("fr" )) {
   		locale = new Locale("fr");
  	}
  	else
  	if (pref_langage.startsWith("es" )) {
  		locale = new Locale("es");
  	}
  	else
  	if (pref_langage.startsWith("en" )) {
  		locale = new Locale("en");
  	}
  	messages = ResourceBundle.getBundle("textes.messages", locale);
  	creeActions();
  	creeBoutons();
  	creeMenu();
		// MtgInternalFrame cadre = cadreActif();
		// metAJourIconesEtMenus((cadre != null)  ? cadre.document : null);
  	
  }

	public void componentHidden(ComponentEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void componentMoved(ComponentEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void componentResized(ComponentEvent e) {
		// Ligne suivante pour viter dmarrage yrs lent sous Linux ou cette fonction peut-tre appele plusieurs fois "en  mme temps"
		if (premierDemEnCours)		return;
		// La ligne suivante est pour assurer la compatibilit avec Linux car cette fonction est aussi
		// appele quand Linux rduit l'appli en icne
		if ((getContentPane().getWidth() <=200) || (getContentPane().getHeight() <= 200 )) return;
		premierDemEnCours = true;
		CMathGrafDoc doc = null;
		MtgInternalFrame cadre = null;
		if (!barreOutilsGaucheEtablie) {
			if (url.isEmpty()) {
				if (path.isEmpty()) {
					doc = new CMathGrafDoc(this);
					doc.listePrincipale.ajouteConstantePi();
					doc.listePrincipale.ajouteSegmentLongueurUnite(MtgInternalFrame.largeurNouvelleFigure, MtgInternalFrame.hauteurNouvelleFigure);
					doc.listePrincipale.uniteAngle = pref_angleUnit;
					cadre = new MtgInternalFrame(this, "MathGraph"
							+ Integer.toString(instanceNouvelleFenetre()), doc,
							MtgInternalFrame.largeurNouvelleFigure, MtgInternalFrame.hauteurNouvelleFigure);
					cadre.initialiseListes();
				}
				else {
					File file = new File(path);
					if (file != null) {
						try {
							int ind = path.lastIndexOf(File.separatorChar);
							FileInputStream fis = new FileInputStream(file);
							MathGraphDataInputStream inps = new MathGraphDataInputStream(fis);
							try {
								String titre = path.substring(ind + 1);
								doc = new CMathGrafDoc(this);
								doc.path = path;
								doc.chargeDepuisFlux(inps);
								inps.close();
								cadre = new MtgInternalFrame(this, "MathGraph"
										+ Integer.toString(instanceNouvelleFenetre()), doc, doc.sizeWindowx, doc.sizeWindowy);
								cadre.setTitle(titre);
								cadre.path = file.getAbsolutePath();
							} catch (IOException ioerr) {
								dialogueAvertissement(getString("ch35"));
								return;
							}
						} catch (FileNotFoundException err) {
							dialogueAvertissement(getString("ch35"));
							return;
						}
					}
				}
			}
			else {
				// Cas ou MathGraph32 est ouvert via JNLP avec une URL passe en paramtre
				if (path.isEmpty()) { // Cas o en dmarre en jnlp depuis une figure vierge
					doc = new CMathGrafDoc(this);
					doc.listePrincipale.uniteAngle = pref_angleUnit;
					doc.listePrincipale.ajouteConstantePi();
					doc.listePrincipale.ajouteSegmentLongueurUnite(MtgInternalFrame.largeurNouvelleFigure,MtgInternalFrame.hauteurNouvelleFigure);
				}
				else {
					ByteArrayInputStream ba = new ByteArrayInputStream(Kernel.base64Decode(path));
					MathGraphDataInputStream inps = new MathGraphDataInputStream(ba);
					doc = new CMathGrafDoc(this);
					try {
						doc.chargeDepuisFlux(inps);
						inps.close();
					} catch (IOException e1) {
						dialogueAvertissement(getString("ch35"));
						e1.printStackTrace();
					}
				}
				cadre = new MtgInternalFrame(this, "MathGraph" + Integer.toString(instanceNouvelleFenetre()), doc, doc.sizeWindowx, doc.sizeWindowy);

			}
			try {
				cadre.gestionnaire.initialise();
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			// metAJourIconesEtMenus(cadre.document);
			mtgDesktopPane.add(cadre);
			mtgDesktopPane.getDesktopManager().activateFrame(cadre);
			// La ligne suivante est ncessaire pour Linux car sinon
			// MtgInternalFrame.internalFrameActivated n'est pas appel
			// et la figure n'est visible qu'aprs un clic ou une action sur la figure
			// validate(); // Dplac plus bas.
			try {
				cadre.setSelected(true);
			} catch (java.beans.PropertyVetoException err) {
			};
			// On cre un item de menu dans le menu Fenetres pour activer cete fentre
			MtgMenuFileItem item = new MtgMenuFileItem(cadre);
			menuFenetre.add(item);
			creeActions();
			creeBoutons();
			metAJourBoutonsBarreHorizontale(cadreActif().document);
			creeMenu();
			activeOutilCapture(); // Modifi version 4.4.1.2
			if (cadreActif() != null) {
				metAJourBoutonsBarreOutils(cadreActif().document);
				// validate();
			}
			else {
				metAJourBoutonsBarreOutils(docPreferencesNiveau[pref_niveau]);
				// validate();
			}
			// validate(); Supprim version 5.0.0.5
			barreOutilsGaucheEtablie = true;
			// Ajout version 5.0
			buttonModePointAuto.setSelected(pref_creationPointsAuto);
		}
	}
	
	public void componentShown(ComponentEvent e) {
	}
	
	// Ajout version 4.7.5.2
	public void adaptePoliceSysteme() {
		Object key, value;
		if (pref_addSystemFont == 0) return;
		try {
	  	UIDefaults dd = UIManager.getDefaults();
	  	for (Enumeration<Object> e = dd.keys() ; e.hasMoreElements() ;) {
  	    key = e.nextElement();
        value = UIManager.get(key);
        if (value != null && value instanceof javax.swing.plaf.FontUIResource) {
  	    		Font font = UIManager.getFont(key);
  	    		// Font font2 = font.deriveFont((float)(font.getSize()+pref_addSystemFont));
  	    		// Font fon3 = new Font("Arial",Font.PLAIN,20);
  	    		// Modifi version 4.7.7 car certains caractres unicodes taient reprsents par des carrs vides dans les JTextField
  	    		// Font font2 = new Font("Arial",Font.PLAIN,font.getSize()+pref_addSystemFont);
  	    		// Font font2 = new Font(null,Font.PLAIN,font.getSize()+pref_addSystemFont); Modifi version 5.0
  	    		// Modifi version 5.0 pour ne pas avoir une trop grand police dans les JTextField et JTextArea
  	    		String ch = key.toString();
  	    		Font font2 = font.deriveFont((float)(font.getSize() + pref_addSystemFont));
  	        UIManager.put(key,font2);
  	        if (ch.equals("TextField.font")) {
  	        	indicationArea.setFont(font2);
  	        	paneSouth.validate();
  	        }
  	    }
	  	}
		}
		catch (Exception e) {};

	}
	/**
	 * Fonction montrant ou cachant la barre d'outils de gauche
	 * @param visible
	 */
	public final void montreBarreOutil(boolean visible) {
		paneLeft.setVisible(visible);
	}
	
	public void fermeMenusHorizontauxSauf(MtgIconButton btn) {
		for (int i = 0; i < tabIcones.length; i++) {
			if (tabIcones[i] != btn) {
				tabIcones[i].fermeMenu();
			}
		}		
	}
	
	/**
	 * Fonction dermant tou sles menus horizontaux drouls s'il y en a (il ne leut y en avoir qu'un seul).
	 */
	public final boolean fermeMenusHorizontaux() {
		boolean b = false;
		for (int i = 0; i < tabIcones.length; i++) {
			b = b || tabIcones[i].fermeMenu();
		}
		return b;
	}
	
	/**
	 * Ajout version 5.0
	 * Ferme les meus horizontaux s'il y en a des drouls et active l'icne de l'outil actif.
	 * Appel dans MtgFigurePanel.mousePressed.
	 * Utili : Si par exemple l'outil actif est l'outil de capture et qu'on a double cliqu sur l'icne des points puis ensuite cliqu
	 * sur la figure, on n' pas chang d'outil et il faut donc ractiver l'icne de capture
	 */
	public final void fermeMenusHorizontauxEtSelectIconeActive() {
		boolean b = fermeMenusHorizontaux();
		if (b) {
			if (outilActif == outilCapture) buttonCapture.setSelected(true);
			else {
				for (int i = 0; i < tabIcones.length; i++) {
					if (tabIcones[i].action.outilAssocie == outilActif) {
						tabIcones[i].setSelected(true);
						break;
					}
				}
				}
		}
	}

	/**
	 * Fonction renvoyant l'indice ce l'icne qui a un emnu horizontal droul et -1 s'il n'y en a aps
	 * @return
	 */
	public final int indiceMenuHorizontal() {
		for (int i = 0; i < tabIcones.length; i++) {
			MtgIconButton ib = tabIcones[i];
			if ((ib.menu != null) && (ib.menu.isVisible())) return i;
		}		
		return -1;	
	}
	
	public void setStyleTrait(StyleTrait style) {
		listeStyleTrait.setSelectedIndex(style.style);
		sliderEpaisseur.setValue(style.epaisseur);
	}
	
	public void setColor(Color color) {
		paneTransparence.donneCouleurActive(color);
	}
	public void setStyleRemplissage(int styleRemp) {
		paneStyleRemplissage.donneStyle(styleRemp);
	}
	public final void activeOutilPalette() {
		outilActif = outilPalette;
		buttonPalette.setSelected(true);
		MtgInternalFrame cadre = cadreActif();
		if (cadre != null) {
			outilActif.select();
			cadre.requestFocus();
		}
	}

	public final void extractConstructions() {
		WarningDlg dlg = new WarningDlg(this, getString("messInstall"));
		dlg.setLocationRelativeTo(this);
    dlg.setVisible(true);
		// Recherche d'aord du dossier user o se fera l'extraction
		String fs = System.getProperty("file.separator");
		String path;
		try {
			String se = System.getProperty("os.name");
			if (se.startsWith("Linux")) path = System.getProperty("user.home") + fs +".config" + fs + "MathGraph32WebStart";
			else if (se.startsWith("Mac")) path = System.getProperty("user.home") + fs + ".MathGraph32WebStart";
			else path = System.getProperty("user.home") + fs + "MathGraph32WebStart";
		}
		catch (Exception e) {
			path = System.getProperty("user.home") + fs + "MathGraph32WebStart";
		}
		// String ch = path + fs + "MathGraph32Properties";
		// File filePath = new File(path);
		/*
		CodeSource src = MtgFrame.class.getProtectionDomain().getCodeSource();
    URL url = src.getLocation();
    String pathOfJar;
    try {
			pathOfJar = URLDecoder.decode(url.toString(), "UTF-8") + "MathGraph32.jar";
			pathOfJar = pathOfJar.replaceAll(" ", "%20");
			if (pathOfJar.endsWith(fs)) pathOfJar += "MathGraph32.jar";
		} catch (UnsupportedEncodingException e) {
			messageErreurTransfert();
			dlg.dispose();
			e.printStackTrace();
			return;
		}
    if (pathOfJar.startsWith("file:/")) pathOfJar = pathOfJar.substring(6);
    */
		String[] st = {"fra","eng","esp"};
		// On commence par dtruire les sous-dossiers qui contiennent les fichiers d'aide en chm et les constructions prdfinies s'ils exxistent
		for (int i = 0; i < st.length; i++) {
			File f = new File(path + fs + st[i]);
			removeDirectory(f);
		}
		JarFile jar;
		try {
			URL url = new URL("jar:http://www.mathgraph32.org/ftp/jaws/MathGraph32Files.jar!/");
			JarURLConnection jarConnection = (JarURLConnection) url.openConnection();
			jar = jarConnection.getJarFile();
			Enumeration<JarEntry> enumEntries = jar.entries();
			while (enumEntries.hasMoreElements()) {
		    JarEntry file = (java.util.jar.JarEntry) enumEntries.nextElement();
		    String name = file.getName().replace("/", fs);
		    boolean b = false;
		    // for (int j = 0; j < st.length; j++) b = b || name.startsWith( st[j] + fs + "Constructions") ||
		    // 		name.startsWith( st[j] + fs + "HTML");
		    for (int j = 0; j < st.length; j++) b = b || name.startsWith( st[j] + fs);

		    if (b) {
			    File f = new File(path + fs + name ); // Pour crer le dossier coontenant le fichier s'il n'existe pas
			    f.getParentFile().mkdirs();
			    InputStream is = jar.getInputStream(file);
			    BufferedInputStream bis = new BufferedInputStream(is);
			    BufferedOutputStream fos = new BufferedOutputStream(new FileOutputStream(f));
			    while (is.available() > 0) {
			        fos.write(is.read());
			    }
			    fos.close();
			    is.close();
			    bis.close();
		    }
			}
			versionLoaded = MtgFrame.numeroVersion;
		} catch (IOException e) {
			messageErreurTransfert();
			e.printStackTrace();
			dlg.dispose();
			versionLoaded = "none";
		}
		dlg.dispose(); 
	}
	
	public boolean removeDirectory(File directory) {
	  if (directory == null)
	    return false;
	  if (!directory.exists())
	    return true;
	  if (!directory.isDirectory())
	    return false;

	  String[] list = directory.list();

	  // Certaines JVMs renvoient null pour File.list() si le dossier est vide
	  if (list != null) {
	    for (int i = 0; i < list.length; i++) {
	      File entry = new File(directory, list[i]);
	      if (entry.isDirectory())
	      {
	        if (!removeDirectory(entry))
	          return false;
	      }
	      else
	      {
	        if (!entry.delete())
	          return false;
	      }
	    }
	  }
	  return directory.delete();
	}
	
	private final void messageErreurTransfert() {
		JOptionPane.showMessageDialog(null, getString("messErreurInstall"), getString("avertissement"), JOptionPane.WARNING_MESSAGE);
	}

	public final boolean screenIsHighResolution() {
		// Ne pas utiliser la ligne suivante qui sous Linux avec multi crans ralentit normment le dmarrage du logiciel
		// GraphicsDevice gd = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice();
		GraphicsDevice gd = getGraphicsConfiguration().getDevice();
		int width = gd.getDisplayMode().getWidth();
    // Dimension dim = Toolkit.getDefaultToolkit().getScreenSize(); Abandonn Incompatible mode multi-cran sous Linux
		// int width = dim.width;
		return (width >= 2500);
	}
	
	/**
	 * Fonction renvoyant un coefficient servant  adapter la taillle des polices systmes utilises  la rsolution horizontale de l'cant
	 * @return
	 */
	public final double coefMultForSystemFonts() {
		GraphicsDevice gd = getGraphicsConfiguration().getDevice();
		int height = gd.getDisplayMode().getHeight();
		if (height >= 1600) return height/800d;
		else
		if (height >= 1000) return height/700d;
		else
		if (height >= 800) return height/800d;
			else return height/900d;
	}

	public final double coefMultForDlgBoxIcons() {
		GraphicsDevice gd = getGraphicsConfiguration().getDevice();
		int width = gd.getDisplayMode().getWidth();
		int height = gd.getDisplayMode().getHeight();
		double k1 =  screenIsHighResolution() ? (double)height/768d : (double)height/900d;
		double k2 = screenIsHighResolution() ? (double)width/1366d : (double)width/1600d;
		return Math.min(k1, k2);
	}
	
	public final double coefMultVertIcons() {
		int h = this.getContentPane().getHeight();
		double k = (double)h/(72*17); // 13 icones plus la barre du haut + 4 par scurit. 64 pixels normalement plus 8 de scurit
		if (k >= 2.2) return 2.2; else return k;
	}
	
	public final double coefMultHorIcons() {
		int h = this.getContentPane().getWidth();
		// double d = 30 ;
		double k =(double)h/(72*36); // 32 icnes horizontales plus 6 par scurit 64 pixels normalement plus 8 de scurit
		if (k > 2.2) return 2.2; else return k;
	}
	
	/**
	 * Ajout version 5.0.
	 * Renvoie un coefficient qui servira  agrandir les marques de segment, d'angles et les points
	 * pour les crans de haute rsolution
	 * @param dim
	 * @return
	 */
	public static final double coefMultForPointsAndMarks(int width, int height) {
		// On considre une figure d'aire 800000 comme la norme et au-del on applique un coefficient
		// proportionnel  l'aire de la figure
		// Modification version 5.0.0.5  : On n'augmente la taille des points au del d'une certaine taille que si l'cran est haute rsolution
			double d = Math.sqrt((double)(width*height)/800000d);
			if (d < 1) return 1; else if (d > 2.5) return 2.5; else return d;
	}
	
	/**
	 * Ajout version 5.0.
	 * Renvoie un coefficient qui servira  agrandir les fontes
	 * pour les crans de haute rsolution.
	 * @param dim
	 * @return
	 */
	public static final double coefMultForFonts(int width, int height) {
		// On considre une figure d'aire 800000 comme la norme et au-del on applique un coefficient
		// proportionnel  l'aire de la figure
		double d = Math.sqrt((double)(width*height)/800000d);
		if (d < 1) return 1; else if(d>1.75) return 1.75; else return d;
	}
}